@@ -6,21 +6,25 @@
  * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2010 University of Houston. All rights reserved.
  * Copyright (c) 2007-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc. All rights reserved.
- * Copyright (c) 2012-2014 Los Alamos National Security, LLC.
- * Copyright (c) 2011-2013 INRIA.  All rights reserved.
+ * Copyright (c) 2012-2015 Los Alamos National Security, LLC.
+ *                         All rights reserved.
+ * Copyright (c) 2011-2013 Inria.  All rights reserved.
  * Copyright (c) 2011-2013 Universite Bordeaux 1
  *                         All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -36,24 +40,24 @@
 #include "ompi/runtime/params.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/attribute/attribute.h"
-#include "ompi/mca/dpm/dpm.h"
+#include "ompi/dpm/dpm.h"
 #include "ompi/memchecker.h"
 
 /*
 ** Table for Fortran <-> C communicator handle conversion
 ** Also used by P2P code to lookup communicator based
 ** on cid.
-** 
+**
 */
-opal_pointer_array_t ompi_mpi_communicators; 
-opal_pointer_array_t ompi_comm_f_to_c_table;
+opal_pointer_array_t ompi_mpi_communicators = {{0}};
+opal_pointer_array_t ompi_comm_f_to_c_table = {{0}};
 
-ompi_predefined_communicator_t  ompi_mpi_comm_world;
-ompi_predefined_communicator_t  ompi_mpi_comm_self;
-ompi_predefined_communicator_t  ompi_mpi_comm_null;
-ompi_communicator_t  *ompi_mpi_comm_parent;
+ompi_predefined_communicator_t  ompi_mpi_comm_world = {{{0}}};
+ompi_predefined_communicator_t  ompi_mpi_comm_self = {{{0}}};
+ompi_predefined_communicator_t  ompi_mpi_comm_null = {{{0}}};
+ompi_communicator_t  *ompi_mpi_comm_parent = NULL;
 
-ompi_predefined_communicator_t *ompi_mpi_comm_world_addr = 
+ompi_predefined_communicator_t *ompi_mpi_comm_world_addr =
     &ompi_mpi_comm_world;
 ompi_predefined_communicator_t *ompi_mpi_comm_self_addr =
     &ompi_mpi_comm_self;
@@ -68,7 +72,7 @@ OBJ_CLASS_INSTANCE(ompi_communicator_t, opal_object_t,
                    ompi_comm_destruct);
 
 /* This is the counter for the number of communicators, which contain
-   process with more than one jobid. This counter is a usefull 
+   process with more than one jobid. This counter is a usefull
    shortcut for finalize and abort. */
 int ompi_comm_num_dyncomm=0;
 
@@ -81,7 +85,7 @@ int ompi_comm_init(void)
     size_t size;
 
     /* Setup communicator array */
-    OBJ_CONSTRUCT(&ompi_mpi_communicators, opal_pointer_array_t); 
+    OBJ_CONSTRUCT(&ompi_mpi_communicators, opal_pointer_array_t);
     if( OPAL_SUCCESS != opal_pointer_array_init(&ompi_mpi_communicators, 0,
                                                 OMPI_FORTRAN_HANDLE_MAX, 64) ) {
         return OMPI_ERROR;
@@ -98,12 +102,26 @@ int ompi_comm_init(void)
     OBJ_CONSTRUCT(&ompi_mpi_comm_world, ompi_communicator_t);
     assert(ompi_mpi_comm_world.comm.c_f_to_c_index == 0);
     group = OBJ_NEW(ompi_group_t);
-    group->grp_proc_pointers = ompi_proc_world(&size);
-    group->grp_proc_count    = (int)size;
+
+    size = ompi_process_info.num_procs;
+    group->grp_proc_pointers = (ompi_proc_t **) calloc (size, sizeof (ompi_proc_t *));
+    group->grp_proc_count = size;
+
+    for (size_t i = 0 ; i < size ; ++i) {
+        opal_process_name_t name = {.vpid = i, .jobid = OMPI_PROC_MY_NAME->jobid};
+        /* look for existing ompi_proc_t that matches this name */
+        group->grp_proc_pointers[i] = (ompi_proc_t *) ompi_proc_lookup (name);
+        if (NULL == group->grp_proc_pointers[i]) {
+            /* set sentinel value */
+            group->grp_proc_pointers[i] = (ompi_proc_t *) ompi_proc_name_to_sentinel (name);
+        } else {
+            OBJ_RETAIN (group->grp_proc_pointers[i]);
+        }
+    }
+
     OMPI_GROUP_SET_INTRINSIC (group);
     OMPI_GROUP_SET_DENSE (group);
     ompi_set_group_rank(group, ompi_proc_local());
-    ompi_group_increment_proc_count (group);
 
     ompi_mpi_comm_world.comm.c_contextid    = 0;
     ompi_mpi_comm_world.comm.c_id_start_index = 4;
@@ -167,7 +185,7 @@ int ompi_comm_init(void)
     assert(ompi_mpi_comm_null.comm.c_f_to_c_index == 2);
     ompi_mpi_comm_null.comm.c_local_group  = &ompi_mpi_group_null.group;
     ompi_mpi_comm_null.comm.c_remote_group = &ompi_mpi_group_null.group;
-    OBJ_RETAIN(&ompi_mpi_group_null.group); 
+    OBJ_RETAIN(&ompi_mpi_group_null.group);
     OBJ_RETAIN(&ompi_mpi_group_null.group);
 
     ompi_mpi_comm_null.comm.c_contextid    = 2;
@@ -210,9 +228,9 @@ ompi_communicator_t *ompi_comm_allocate ( int local_size, int remote_size )
         new_comm->c_remote_group = ompi_group_allocate (remote_size);
         new_comm->c_flags |= OMPI_COMM_INTER;
     } else {
-        /* 
-         * simplifies some operations (e.g. p2p), if 
-         * we can always use the remote group 
+        /*
+         * simplifies some operations (e.g. p2p), if
+         * we can always use the remote group
          */
         new_comm->c_remote_group = new_comm->c_local_group;
         OBJ_RETAIN(new_comm->c_remote_group);
@@ -224,7 +242,7 @@ ompi_communicator_t *ompi_comm_allocate ( int local_size, int remote_size )
     return new_comm;
 }
 
-int ompi_comm_finalize(void) 
+int ompi_comm_finalize(void)
 {
     int max, i;
     ompi_communicator_t *comm;
@@ -233,7 +251,7 @@ int ompi_comm_finalize(void)
     OBJ_DESTRUCT( &ompi_mpi_comm_self );
 
     /* disconnect all dynamic communicators */
-    ompi_dpm.dyn_finalize();
+    ompi_dpm_dyn_finalize();
 
     /* Free the attributes on comm world. This is not done in the
      * destructor as we delete attributes in ompi_comm_free (which
@@ -258,22 +276,22 @@ int ompi_comm_finalize(void)
         OBJ_DESTRUCT (ompi_mpi_comm_parent);
 
         /* Please note, that the we did increase the reference count
-           for ompi_mpi_comm_null, ompi_mpi_group_null, and 
-           ompi_mpi_errors_are_fatal in ompi_comm_init because of 
-           ompi_mpi_comm_parent.  In case a 
+           for ompi_mpi_comm_null, ompi_mpi_group_null, and
+           ompi_mpi_errors_are_fatal in ompi_comm_init because of
+           ompi_mpi_comm_parent.  In case a
            parent communicator is really created, the ref. counters
-           for these objects are decreased again by one. However, in a 
+           for these objects are decreased again by one. However, in a
            static scenario, we should ideally decrease the ref. counter
-           for these objects by one here. The problem just is, that 
+           for these objects by one here. The problem just is, that
            if the app had a parent_comm, and this has been freed/disconnected,
-           ompi_comm_parent points again to ompi_comm_null, the reference count 
+           ompi_comm_parent points again to ompi_comm_null, the reference count
            for these objects has not been increased again.
            So the point is, if ompi_mpi_comm_parent == &ompi_mpi_comm_null
            we do not know whether we have to decrease the ref count for
            those three objects or not. Since this is a constant, non-increasing
-           amount of memory, we stick with the current solution for now, 
+           amount of memory, we stick with the current solution for now,
            namely don't do anything.
-        */  
+        */
     }
 
     /* Shut down MPI_COMM_NULL */
@@ -407,7 +425,6 @@ static void ompi_comm_destruct(ompi_communicator_t* comm)
     }
 
     if (NULL != comm->c_local_group) {
-        ompi_group_decrement_proc_count (comm->c_local_group);
         OBJ_RELEASE ( comm->c_local_group );
         comm->c_local_group = NULL;
         if ( OMPI_COMM_IS_INTRA(comm) ) {
@@ -420,7 +437,6 @@ static void ompi_comm_destruct(ompi_communicator_t* comm)
     }
 
     if (NULL != comm->c_remote_group) {
-        ompi_group_decrement_proc_count (comm->c_remote_group);
         OBJ_RELEASE ( comm->c_remote_group );
         comm->c_remote_group = NULL;
     }
@@ -439,12 +455,10 @@ static void ompi_comm_destruct(ompi_communicator_t* comm)
     }
 
     /* reset the ompi_comm_f_to_c_table entry */
-    if ( MPI_UNDEFINED != comm->c_f_to_c_index && 
+    if ( MPI_UNDEFINED != comm->c_f_to_c_index &&
          NULL != opal_pointer_array_get_item(&ompi_comm_f_to_c_table,
                                              comm->c_f_to_c_index)) {
         opal_pointer_array_set_item ( &ompi_comm_f_to_c_table,
                                       comm->c_f_to_c_index, NULL);
     }
-
-    return;
 }
