@@ -1,24 +1,27 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
-/* 
+/*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 University of Houston. All rights reserved.
  * Copyright (c) 2007      Cisco Systems, Inc. All rights reserved.
  * Copyright (c) 2012      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2012-2013 Inria.  All rights reserved.
+ * Copyright (c) 2013-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -34,91 +37,95 @@ int ompi_group_free ( ompi_group_t **group )
     ompi_group_t *l_group;
 
     l_group = (ompi_group_t *) *group;
-    ompi_group_decrement_proc_count (l_group);
     OBJ_RELEASE(l_group);
 
     *group = MPI_GROUP_NULL;
     return OMPI_SUCCESS;
 }
 
-int ompi_group_translate_ranks ( ompi_group_t *group1, 
+int ompi_group_translate_ranks ( ompi_group_t *group1,
                                  int n_ranks, const int *ranks1,
-                                 ompi_group_t *group2, 
-                                 int *ranks2) 
+                                 ompi_group_t *group2,
+                                 int *ranks2)
 {
-    int rank, proc, proc2;
-    struct ompi_proc_t *proc1_pointer, *proc2_pointer;
-
     if ( MPI_GROUP_EMPTY == group1 || MPI_GROUP_EMPTY == group2 ) {
-        for (proc = 0; proc < n_ranks ; proc++) {
+        for (int proc = 0; proc < n_ranks ; ++proc) {
             ranks2[proc] = MPI_UNDEFINED;
         }
         return MPI_SUCCESS;
     }
 
-    /* 
+#if OMPI_GROUP_SPARSE
+    /*
      * If we are translating from a parent to a child that uses the sparse format
-     * or vice versa, we use the translate ranks function corresponding to the 
-     * format used. Generally, all these functions require less time than the 
-     * original method that loops over the processes of both groups till we 
+     * or vice versa, we use the translate ranks function corresponding to the
+     * format used. Generally, all these functions require less time than the
+     * original method that loops over the processes of both groups till we
      * find a match.
      */
-    if( group1->grp_parent_group_ptr == group2 ) { /* from child to parent */ 
-        if(OMPI_GROUP_IS_SPORADIC(group1)) { 
-            return ompi_group_translate_ranks_sporadic_reverse 
-                (group1,n_ranks,ranks1,group2,ranks2); 
+    if( group1->grp_parent_group_ptr == group2 ) { /* from child to parent */
+        if(OMPI_GROUP_IS_SPORADIC(group1)) {
+            return ompi_group_translate_ranks_sporadic_reverse
+                (group1,n_ranks,ranks1,group2,ranks2);
         }
         else if(OMPI_GROUP_IS_STRIDED(group1)) {
-            return ompi_group_translate_ranks_strided_reverse 
-                (group1,n_ranks,ranks1,group2,ranks2); 
+            return ompi_group_translate_ranks_strided_reverse
+                (group1,n_ranks,ranks1,group2,ranks2);
         }
         else if(OMPI_GROUP_IS_BITMAP(group1)) {
-            return ompi_group_translate_ranks_bmap_reverse 
-                (group1,n_ranks,ranks1,group2,ranks2); 
+            return ompi_group_translate_ranks_bmap_reverse
+                (group1,n_ranks,ranks1,group2,ranks2);
         }
 
+        /* unknown sparse group type */
+        assert (0);
     }
-    else if( group2->grp_parent_group_ptr == group1 ) { /* from parent to child*/
-        if(OMPI_GROUP_IS_SPORADIC(group2)) { 
-            return ompi_group_translate_ranks_sporadic 
-                (group1,n_ranks,ranks1,group2,ranks2); 
+
+    if( group2->grp_parent_group_ptr == group1 ) { /* from parent to child*/
+        if(OMPI_GROUP_IS_SPORADIC(group2)) {
+            return ompi_group_translate_ranks_sporadic
+                (group1,n_ranks,ranks1,group2,ranks2);
         }
         else if(OMPI_GROUP_IS_STRIDED(group2)) {
-            return ompi_group_translate_ranks_strided 
-                (group1,n_ranks,ranks1,group2,ranks2); 
+            return ompi_group_translate_ranks_strided
+                (group1,n_ranks,ranks1,group2,ranks2);
         }
         else if(OMPI_GROUP_IS_BITMAP(group2)) {
-            return ompi_group_translate_ranks_bmap 
-                (group1,n_ranks,ranks1,group2,ranks2); 
+            return ompi_group_translate_ranks_bmap
+                (group1,n_ranks,ranks1,group2,ranks2);
         }
-	
+
+        /* unknown sparse group type */
+        assert (0);
     }
-    else {
-        /* loop over all ranks */
-        for (proc = 0; proc < n_ranks; proc++) {
-            rank=ranks1[proc];
-            if ( MPI_PROC_NULL == rank) {
-                ranks2[proc] = MPI_PROC_NULL;
+#endif
+
+    /* loop over all ranks */
+    for (int proc = 0; proc < n_ranks; ++proc) {
+        struct ompi_proc_t *proc1_pointer, *proc2_pointer;
+        int rank = ranks1[proc];
+
+        if ( MPI_PROC_NULL == rank) {
+            ranks2[proc] = MPI_PROC_NULL;
+            continue;
+        }
+
+        proc1_pointer = ompi_group_get_proc_ptr_raw (group1, rank);
+        /* initialize to no "match" */
+        ranks2[proc] = MPI_UNDEFINED;
+        for (int proc2 = 0; proc2 < group2->grp_proc_count; ++proc2) {
+            proc2_pointer = ompi_group_get_proc_ptr_raw (group2, proc2);
+            if ( proc1_pointer == proc2_pointer) {
+                ranks2[proc] = proc2;
+                break;
             }
-            else {
-                proc1_pointer = ompi_group_peer_lookup(group1 ,rank);
-                /* initialize to no "match" */
-                ranks2[proc] = MPI_UNDEFINED;
-                for (proc2 = 0; proc2 < group2->grp_proc_count; proc2++) {
-                    proc2_pointer= ompi_group_peer_lookup(group2, proc2);
-                    if ( proc1_pointer == proc2_pointer) {
-                        ranks2[proc] = proc2;
-                        break;
-                    }
-                }  /* end proc2 loop */
-            } /* end proc loop */
-        }
-    }
+        }  /* end proc2 loop */
+    } /* end proc loop */
 
     return MPI_SUCCESS;
 }
 
-int ompi_group_dump (ompi_group_t* group) 
+int ompi_group_dump (ompi_group_t* group)
 {
     int i;
     int new_rank;
@@ -165,33 +172,14 @@ int ompi_group_dump (ompi_group_t* group)
     return OMPI_SUCCESS;
 }
 
-/* 
- * This is the function that iterates through the sparse groups to the dense group
- * to reach the process pointer
- */
-ompi_proc_t* ompi_group_get_proc_ptr (ompi_group_t* group , int rank) 
-{ 
-    int ranks1,ranks2;
-    do {
-        if(OMPI_GROUP_IS_DENSE(group)) {
-            return group->grp_proc_pointers[rank];
-        }
-        ranks1 = rank;
-        ompi_group_translate_ranks( group, 1, &ranks1,
-                                    group->grp_parent_group_ptr,&ranks2);
-        rank = ranks2;
-        group = group->grp_parent_group_ptr;
-    } while (1);
-}
-
-int ompi_group_minloc ( int list[] , int length ) 
-{ 
+int ompi_group_minloc ( int list[] , int length )
+{
     int i,index,min;
     min = list[0];
     index = 0;
-    
-    for (i=0 ; i<length ; i++) { 
-        if (min > list[i] && list[i] != -1) { 
+
+    for (i=0 ; i<length ; i++) {
+        if (min > list[i] && list[i] != -1) {
             min = list[i];
             index = i;
         }
@@ -212,12 +200,12 @@ int ompi_group_incl(ompi_group_t* group, int n, const int *ranks, ompi_group_t *
         len[1] = ompi_group_calc_strided  ( n ,ranks );
         len[2] = ompi_group_calc_sporadic ( n ,ranks );
         len[3] = ompi_group_calc_bmap     ( n , group->grp_proc_count ,ranks );
-        
+
         /* determin minimum length */
         method = ompi_group_minloc ( len, 4 );
     }
 #endif
-    
+
     switch (method)
         {
         case 0:
@@ -247,19 +235,19 @@ int ompi_group_excl(ompi_group_t* group, int n, const int *ranks, ompi_group_t *
     if (0 < (group->grp_proc_count - n)) {
         ranks_included = (int *)malloc( (group->grp_proc_count-n)*(sizeof(int)));
 
-        for (i=0 ; i<group->grp_proc_count ; i++) { 
-            for(j=0 ; j<n ; j++) { 
+        for (i=0 ; i<group->grp_proc_count ; i++) {
+            for(j=0 ; j<n ; j++) {
                 if(ranks[j] == i) {
                     break;
                 }
             }
-            if (j==n) { 
+            if (j==n) {
                 ranks_included[k] = i;
                 k++;
             }
         }
     }
-    
+
     result = ompi_group_incl(group, k, ranks_included, new_group);
 
     if (NULL != ranks_included) {
@@ -270,8 +258,8 @@ int ompi_group_excl(ompi_group_t* group, int n, const int *ranks, ompi_group_t *
 }
 
 int ompi_group_range_incl(ompi_group_t* group, int n_triplets, int ranges[][3],
-                          ompi_group_t **new_group) 
-{ 
+                          ompi_group_t **new_group)
+{
     int j,k;
     int *ranks_included=NULL;
     int index,first_rank,last_rank,stride;
@@ -280,12 +268,12 @@ int ompi_group_range_incl(ompi_group_t* group, int n_triplets, int ranges[][3],
     count = 0;
     /* determine the number of included processes for the range-incl-method */
     k = 0;
-    for(j=0 ; j<n_triplets ; j++) { 
-	
+    for(j=0 ; j<n_triplets ; j++) {
+
         first_rank = ranges[j][0];
         last_rank = ranges[j][1];
         stride = ranges[j][2];
-	
+
         if (first_rank < last_rank) {
             /* positive stride */
             index = first_rank;
@@ -293,8 +281,8 @@ int ompi_group_range_incl(ompi_group_t* group, int n_triplets, int ranges[][3],
                 count ++;
                 k++;
                 index += stride;
-            }                   /* end while loop */	    
-        } 
+            }                   /* end while loop */
+        }
         else if (first_rank > last_rank) {
             /* negative stride */
             index = first_rank;
@@ -315,7 +303,7 @@ int ompi_group_range_incl(ompi_group_t* group, int n_triplets, int ranges[][3],
     }
     /* determine the list of included processes for the range-incl-method */
     k = 0;
-    for(j=0 ; j<n_triplets ; j++) { 
+    for(j=0 ; j<n_triplets ; j++) {
 
         first_rank = ranges[j][0];
         last_rank = ranges[j][1];
@@ -328,8 +316,8 @@ int ompi_group_range_incl(ompi_group_t* group, int n_triplets, int ranges[][3],
                 ranks_included[k] = index;
                 k++;
                 index += stride;
-            }                   /* end while loop */	    
-        } 
+            }                   /* end while loop */
+        }
         else if (first_rank > last_rank) {
             /* negative stride */
             index = first_rank;
@@ -355,9 +343,9 @@ int ompi_group_range_incl(ompi_group_t* group, int n_triplets, int ranges[][3],
 }
 
 int ompi_group_range_excl(ompi_group_t* group, int n_triplets, int ranges[][3],
-                          ompi_group_t **new_group) 
+                          ompi_group_t **new_group)
 {
-    
+
     int j,k,i;
     int *ranks_included=NULL, *ranks_excluded=NULL;
     int index,first_rank,last_rank,stride,count,result;
@@ -366,7 +354,7 @@ int ompi_group_range_excl(ompi_group_t* group, int n_triplets, int ranges[][3],
 
     /* determine the number of excluded processes for the range-excl-method */
     k = 0;
-    for(j=0 ; j<n_triplets ; j++) { 
+    for(j=0 ; j<n_triplets ; j++) {
         first_rank = ranges[j][0];
         last_rank = ranges[j][1];
         stride = ranges[j][2];
@@ -377,8 +365,8 @@ int ompi_group_range_excl(ompi_group_t* group, int n_triplets, int ranges[][3],
             while (index <= last_rank) {
                 count ++;
                 index += stride;
-            }                   /* end while loop */	    
-        } 
+            }                   /* end while loop */
+        }
         else if (first_rank > last_rank) {
             /* negative stride */
             index = first_rank;
@@ -398,7 +386,7 @@ int ompi_group_range_excl(ompi_group_t* group, int n_triplets, int ranges[][3],
     /* determine the list of included processes for the range-excl-method */
     k = 0;
     i = 0;
-    for(j=0 ; j<n_triplets ; j++) { 
+    for(j=0 ; j<n_triplets ; j++) {
         first_rank = ranges[j][0];
         last_rank = ranges[j][1];
         stride = ranges[j][2];
@@ -410,8 +398,8 @@ int ompi_group_range_excl(ompi_group_t* group, int n_triplets, int ranges[][3],
                 ranks_excluded[i] = index;
                 i++;
                 index += stride;
-            }                   /* end while loop */	    
-        } 
+            }                   /* end while loop */
+        }
         else if (first_rank > last_rank) {
             /* negative stride */
             index = first_rank;
@@ -430,11 +418,11 @@ int ompi_group_range_excl(ompi_group_t* group, int n_triplets, int ranges[][3],
     if (0 != (group->grp_proc_count - count)) {
         ranks_included = (int *)malloc( (group->grp_proc_count - count)*(sizeof(int)));
     }
-    for (j=0 ; j<group->grp_proc_count ; j++) { 
-        for(index=0 ; index<i ; index++) { 
+    for (j=0 ; j<group->grp_proc_count ; j++) {
+        for(index=0 ; index<i ; index++) {
             if(ranks_excluded[index] == j) break;
         }
-        if (index == i) { 
+        if (index == i) {
             ranks_included[k] = j;
             k++;
         }
@@ -453,7 +441,7 @@ int ompi_group_range_excl(ompi_group_t* group, int n_triplets, int ranges[][3],
 }
 
 int ompi_group_intersection(ompi_group_t* group1,ompi_group_t* group2,
-                            ompi_group_t **new_group) 
+                            ompi_group_t **new_group)
 {
     int proc1,proc2,k, result;
     int *ranks_included=NULL;
@@ -463,34 +451,19 @@ int ompi_group_intersection(ompi_group_t* group1,ompi_group_t* group2,
     group1_pointer=(ompi_group_t *)group1;
     group2_pointer=(ompi_group_t *)group2;
 
-    /* determine the number of included processes for the incl-method */
     k = 0;
-    for (proc1 = 0; proc1 < group1_pointer->grp_proc_count; proc1++) {
-        proc1_pointer = ompi_group_peer_lookup (group1_pointer , proc1);
-        
-        /* check to see if this proc is in group2 */
-    
-        for (proc2 = 0; proc2 < group2_pointer->grp_proc_count; proc2++) {
-            proc2_pointer = ompi_group_peer_lookup (group2_pointer , proc2);
-
-            if( proc1_pointer == proc2_pointer ) {
-                k++;
-                break;
-            }
-        }  /* end proc2 loop */
-    }  /* end proc1 loop */
-    
-    if (0 != k) {
-        ranks_included = (int *)malloc( k*(sizeof(int)));
+    /* allocate the max required memory */
+    ranks_included = (int *)malloc(group1_pointer->grp_proc_count*(sizeof(int)));
+    if (NULL == ranks_included) {
+        return MPI_ERR_NO_MEM;
     }
-
     /* determine the list of included processes for the incl-method */
     k = 0;
     for (proc1 = 0; proc1 < group1_pointer->grp_proc_count; proc1++) {
         proc1_pointer = ompi_group_peer_lookup (group1_pointer , proc1);
 
         /* check to see if this proc is in group2 */
-    
+
         for (proc2 = 0; proc2 < group2_pointer->grp_proc_count; proc2++) {
             proc2_pointer = ompi_group_peer_lookup (group2_pointer ,proc2);
 
@@ -503,7 +476,7 @@ int ompi_group_intersection(ompi_group_t* group1,ompi_group_t* group2,
     }  /* end proc1 loop */
 
     result = ompi_group_incl(group1, k, ranks_included, new_group);
-    
+
     if (NULL != ranks_included) {
         free(ranks_included);
     }
@@ -580,3 +553,23 @@ int ompi_group_compare(ompi_group_t *group1,
 
     return return_value;
 }
+
+bool ompi_group_have_remote_peers (ompi_group_t *group)
+{
+    for (int i = 0 ; i < group->grp_proc_count ; ++i) {
+        ompi_proc_t *proc = NULL;
+#if OMPI_GROUP_SPARSE
+        proc = ompi_group_peer_lookup (group, i);
+#else
+        if (ompi_proc_is_sentinel (group->grp_proc_pointers[i])) {
+            return true;
+        }
+        proc = group->grp_proc_pointers[i];
+#endif
+        if (!OPAL_PROC_ON_LOCAL_NODE(proc->super.proc_flags)) {
+            return true;
+        }
+    }
+
+    return false;
+}
