@@ -1,10 +1,12 @@
 #!/usr/bin/env perl
 #
 # Copyright (c) 2011-2014 Cisco Systems, Inc.  All rights reserved.
+# Copyright (c) 2016      Research Organization for Information Science
+#                         and Technology (RIST). All rights reserved.
 # $COPYRIGHT$
-# 
+#
 # Additional copyrights may follow
-# 
+#
 # $HEADER$
 #
 
@@ -51,7 +53,7 @@ sub write_file {
             $need_write = 1;
         }
     }
-    
+
     if ($need_write) {
         open(FILE_OUT, ">$filename_out") || die "Couldn't open $filename_out";
         print FILE_OUT $str;
@@ -203,7 +205,7 @@ $io_handles->{MPI_FILE_NULL} = 0;
 my $constants;
 
 $constants->{MPI_VERSION} = 3;
-$constants->{MPI_SUBVERSION} = 0;
+$constants->{MPI_SUBVERSION} = 1;
 
 $constants->{MPI_ANY_SOURCE} = -1;
 $constants->{MPI_ANY_TAG} = -1;
@@ -227,6 +229,12 @@ $constants->{MPI_UNIVERSE_SIZE} = 6;
 $constants->{MPI_WIN_BASE} = 7;
 $constants->{MPI_WIN_SIZE} = 8;
 $constants->{MPI_WIN_DISP_UNIT} = 9;
+$constants->{MPI_WIN_CREATE_FLAVOR} = 10;
+$constants->{MPI_WIN_MODEL} = 11;
+$constants->{MPI_WIN_FLAVOR_CREATE} = 1;
+$constants->{MPI_WIN_FLAVOR_ALLOCATE} = 2;
+$constants->{MPI_WIN_FLAVOR_DYNAMIC} = 3;
+$constants->{MPI_WIN_FLAVOR_SHARED} = 4;
 
 $constants->{MPI_BSEND_OVERHEAD} = 128;
 $constants->{MPI_ORDER_C} = 0;
@@ -305,14 +313,29 @@ $constants->{MPI_ERR_SPAWN} = 50;
 $constants->{MPI_ERR_UNSUPPORTED_DATAREP} = 51;
 $constants->{MPI_ERR_UNSUPPORTED_OPERATION} = 52;
 $constants->{MPI_ERR_WIN} = 53;
+# these error codes will never be returned by a fortran function
+# since there are no fortran bindings for MPI_T
+$constants->{MPI_T_ERR_MEMORY} = 54;
+$constants->{MPI_T_ERR_NOT_INITIALIZED} = 55;
+$constants->{MPI_T_ERR_CANNOT_INIT} = 56;
+$constants->{MPI_T_ERR_INVALID_INDEX} = 57;
+$constants->{MPI_T_ERR_INVALID_ITEM} = 58;
+$constants->{MPI_T_ERR_INVALID_HANDLE} = 59;
+$constants->{MPI_T_ERR_OUT_OF_HANDLES} = 60;
+$constants->{MPI_T_ERR_OUT_OF_SESSIONS} = 61;
+$constants->{MPI_T_ERR_INVALID_SESSION} = 62;
+$constants->{MPI_T_ERR_CVAR_SET_NOT_NOW} = 63;
+$constants->{MPI_T_ERR_CVAR_SET_NEVER} = 64;
+$constants->{MPI_T_ERR_PVAR_NO_STARTSTOP} = 65;
+$constants->{MPI_T_ERR_PVAR_NO_WRITE} = 66;
+$constants->{MPI_T_ERR_PVAR_NO_ATOMIC} = 67;
 $constants->{MPI_ERR_RMA_RANGE} = 68;
 $constants->{MPI_ERR_RMA_ATTACH} = 69;
 $constants->{MPI_ERR_RMA_FLAVOR} = 70;
 $constants->{MPI_ERR_RMA_SHARED} = 71;
+$constants->{MPI_T_ERR_INVALID} = 72;
 $constants->{MPI_ERR_LASTCODE} = 92;
 
-$constants->{MPI_ERR_SYSRESOURCE} = -2;
-
 $constants->{MPI_IDENT} = 0;
 $constants->{MPI_CONGRUENT} = 1;
 $constants->{MPI_SIMILAR} = 2;
@@ -339,6 +362,18 @@ $constants->{MPI_COMBINER_RESIZED} = 17;
 $constants->{MPI_COMBINER_HINDEXED_BLOCK} = 18;
 
 $constants->{MPI_COMM_TYPE_SHARED} = 0;
+$constants->{OMPI_COMM_TYPE_HWTHREAD} = 1;
+$constants->{OMPI_COMM_TYPE_CORE} = 2;
+$constants->{OMPI_COMM_TYPE_L1CACHE} = 3;
+$constants->{OMPI_COMM_TYPE_L2CACHE} = 4;
+$constants->{OMPI_COMM_TYPE_L3CACHE} = 5;
+$constants->{OMPI_COMM_TYPE_SOCKET} = 6;
+$constants->{OMPI_COMM_TYPE_NUMA} = 7;
+$constants->{OMPI_COMM_TYPE_NODE} = 0;
+$constants->{OMPI_COMM_TYPE_BOARD} = 8;
+$constants->{OMPI_COMM_TYPE_HOST} = 9;
+$constants->{OMPI_COMM_TYPE_CU} = 10;
+$constants->{OMPI_COMM_TYPE_CLUSTER} = 11;
 
 #----------------------------------------------------------------------------
 
@@ -402,13 +437,13 @@ sub write_fortran_file {
     write_file($file, $header);
 }
 
-write_fortran_file($header, $handles, 
+write_fortran_file($header, $handles,
                    "$topdir/ompi/include/mpif-handles.h");
-write_fortran_file($header, $constants, 
+write_fortran_file($header, $constants,
                    "$topdir/ompi/include/mpif-constants.h");
-write_fortran_file($header, $io_handles, 
+write_fortran_file($header, $io_handles,
                    "$topdir/ompi/include/mpif-io-handles.h");
-write_fortran_file($header, $io_constants, 
+write_fortran_file($header, $io_constants,
                    "$topdir/ompi/include/mpif-io-constants.h");
 
 #----------------------------------------------------------------------------
