@@ -2,19 +2,19 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2016 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2014      Research Organization for Information Science
+ * Copyright (c) 2014-2015 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -32,50 +32,6 @@
 #include "coll_basic.h"
 
 
-/*
- *	allgather_intra
- *
- *	Function:	- allgather using other MPI collections
- *	Accepts:	- same as MPI_Allgather()
- *	Returns:	- MPI_SUCCESS or error code
- */
-int
-mca_coll_basic_allgather_intra(void *sbuf, int scount,
-                               struct ompi_datatype_t *sdtype, void *rbuf,
-                               int rcount, struct ompi_datatype_t *rdtype,
-                               struct ompi_communicator_t *comm,
-                               mca_coll_base_module_t *module)
-{
-    int err;
-    ptrdiff_t lb, extent;
-
-    /* Handle MPI_IN_PLACE (see explanantion in reduce.c for how to
-       allocate temp buffer) -- note that rank 0 can use IN_PLACE
-       natively, and we can just alias the right position in rbuf
-       as sbuf and avoid using a temporary buffer if gather is
-       implemented correctly */
-    if (MPI_IN_PLACE == sbuf && 0 != ompi_comm_rank(comm)) {
-       ompi_datatype_get_extent(rdtype, &lb, &extent);
-       sbuf = ((char*) rbuf) + (ompi_comm_rank(comm) * extent * rcount);
-       sdtype = rdtype;
-       scount = rcount;
-    } 
-
-    /* Gather and broadcast. */
-
-    err = comm->c_coll.coll_gather(sbuf, scount, sdtype, rbuf, rcount,
-                                   rdtype, 0, comm, comm->c_coll.coll_gather_module);
-    if (MPI_SUCCESS == err) {
-        err = comm->c_coll.coll_bcast(rbuf, rcount * ompi_comm_size(comm), 
-                                      rdtype, 0, comm, comm->c_coll.coll_bcast_module);
-    }
-
-    /* All done */
-
-    return err;
-}
-
-
 /*
  *	allgather_inter
  *
@@ -84,19 +40,18 @@ mca_coll_basic_allgather_intra(void *sbuf, int scount,
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-mca_coll_basic_allgather_inter(void *sbuf, int scount,
+mca_coll_basic_allgather_inter(const void *sbuf, int scount,
                                struct ompi_datatype_t *sdtype,
                                void *rbuf, int rcount,
                                struct ompi_datatype_t *rdtype,
                                struct ompi_communicator_t *comm,
                                mca_coll_base_module_t *module)
 {
-    int rank, root = 0, size, rsize, err, i;
+    int rank, root = 0, size, rsize, err, i, line;
     char *tmpbuf = NULL, *ptmp;
     ptrdiff_t rlb, slb, rextent, sextent, incr;
     ompi_request_t *req;
-    mca_coll_basic_module_t *basic_module = (mca_coll_basic_module_t*) module;
-    ompi_request_t **reqs = basic_module->mccb_reqs;
+    ompi_request_t **reqs = NULL;
 
     rank = ompi_comm_rank(comm);
     size = ompi_comm_size(comm);
@@ -105,7 +60,7 @@ mca_coll_basic_allgather_inter(void *sbuf, int scount,
     /* Algorithm:
      * - a gather to the root in remote group (simultaniously executed,
      * thats why we cannot use coll_gather).
-     * - exchange the temp-results between two roots 
+     * - exchange the temp-results between two roots
      * - inter-bcast (again simultanious).
      */
 
@@ -115,35 +70,29 @@ mca_coll_basic_allgather_inter(void *sbuf, int scount,
         err = MCA_PML_CALL(send(sbuf, scount, sdtype, root,
                                 MCA_COLL_BASE_TAG_ALLGATHER,
                                 MCA_PML_BASE_SEND_STANDARD, comm));
-        if (OMPI_SUCCESS != err) {
-            return err;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
     } else {
         /* receive a msg. from all other procs. */
         err = ompi_datatype_get_extent(rdtype, &rlb, &rextent);
-        if (OMPI_SUCCESS != err) {
-            return err;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
         err = ompi_datatype_get_extent(sdtype, &slb, &sextent);
-        if (OMPI_SUCCESS != err) {
-            return err;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
+
+        /* Get a requests arrays of the right size */
+        reqs = coll_base_comm_get_reqs(module->base_data, rsize + 1);
+        if( NULL == reqs ) { line = __LINE__; err = OMPI_ERR_OUT_OF_RESOURCE; goto exit; }
 
         /* Do a send-recv between the two root procs. to avoid deadlock */
         err = MCA_PML_CALL(isend(sbuf, scount, sdtype, 0,
                                  MCA_COLL_BASE_TAG_ALLGATHER,
                                  MCA_PML_BASE_SEND_STANDARD,
                                  comm, &reqs[rsize]));
-        if (OMPI_SUCCESS != err) {
-            return err;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
 
         err = MCA_PML_CALL(irecv(rbuf, rcount, rdtype, 0,
                                  MCA_COLL_BASE_TAG_ALLGATHER, comm,
                                  &reqs[0]));
-        if (OMPI_SUCCESS != err) {
-            return err;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
 
         incr = rextent * rcount;
         ptmp = (char *) rbuf + incr;
@@ -151,55 +100,41 @@ mca_coll_basic_allgather_inter(void *sbuf, int scount,
             err = MCA_PML_CALL(irecv(ptmp, rcount, rdtype, i,
                                      MCA_COLL_BASE_TAG_ALLGATHER,
                                      comm, &reqs[i]));
-            if (MPI_SUCCESS != err) {
-                return err;
-            }
+            if (MPI_SUCCESS != err) { line = __LINE__; goto exit; }
         }
 
         err = ompi_request_wait_all(rsize + 1, reqs, MPI_STATUSES_IGNORE);
-        if (OMPI_SUCCESS != err) {
-            return err;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
 
         /* Step 2: exchange the resuts between the root processes */
         tmpbuf = (char *) malloc(scount * size * sextent);
-        if (NULL == tmpbuf) {
-            return err;
-        }
+        if (NULL == tmpbuf) { line = __LINE__; err = OMPI_ERR_OUT_OF_RESOURCE; goto exit; }
 
         err = MCA_PML_CALL(isend(rbuf, rsize * rcount, rdtype, 0,
                                  MCA_COLL_BASE_TAG_ALLGATHER,
                                  MCA_PML_BASE_SEND_STANDARD, comm, &req));
-        if (OMPI_SUCCESS != err) {
-            goto exit;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
 
         err = MCA_PML_CALL(recv(tmpbuf, size * scount, sdtype, 0,
                                 MCA_COLL_BASE_TAG_ALLGATHER, comm,
                                 MPI_STATUS_IGNORE));
-        if (OMPI_SUCCESS != err) {
-            goto exit;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
 
         err = ompi_request_wait( &req, MPI_STATUS_IGNORE);
-        if (OMPI_SUCCESS != err) {
-            goto exit;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
     }
 
 
-    /* Step 3: bcast the data to the remote group. This 
-     * happens in both groups simultaniously, thus we can 
-     * not use coll_bcast (this would deadlock). 
+    /* Step 3: bcast the data to the remote group. This
+     * happens in both groups simultaneously, thus we can
+     * not use coll_bcast (this would deadlock).
      */
     if (rank != root) {
         /* post the recv */
         err = MCA_PML_CALL(recv(rbuf, rsize * rcount, rdtype, 0,
                                 MCA_COLL_BASE_TAG_ALLGATHER, comm,
                                 MPI_STATUS_IGNORE));
-        if (OMPI_SUCCESS != err) {
-            goto exit;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
 
     } else {
         /* Send the data to every other process in the remote group
@@ -209,19 +144,19 @@ mca_coll_basic_allgather_inter(void *sbuf, int scount,
                                      MCA_COLL_BASE_TAG_ALLGATHER,
                                      MCA_PML_BASE_SEND_STANDARD,
                                      comm, &reqs[i - 1]));
-            if (OMPI_SUCCESS != err) {
-                goto exit;
-            }
-
+            if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
         }
 
         err = ompi_request_wait_all(rsize - 1, reqs, MPI_STATUSES_IGNORE);
-        if (OMPI_SUCCESS != err) {
-            goto exit;
-        }
+        if (OMPI_SUCCESS != err) { line = __LINE__; goto exit; }
     }
 
   exit:
+    if( MPI_SUCCESS != err ) {
+        OPAL_OUTPUT( (ompi_coll_base_framework.framework_output,"%s:%4d\tError occurred %d, rank %2d",
+                      __FILE__, line, err, rank) );
+        if( NULL != reqs ) ompi_coll_base_free_reqs(reqs, rsize+1);
+    }
     if (NULL != tmpbuf) {
         free(tmpbuf);
     }
