@@ -3,10 +3,10 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2016 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -14,13 +14,13 @@
  * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2013      FUJITSU LIMITED.  All rights reserved.
- * Copyright (c) 2014      Research Organization for Information Science
+ * Copyright (c) 2014-2015 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2014      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -36,14 +36,13 @@
 
 
 static int
-mca_coll_basic_alltoallw_intra_inplace(void *rbuf, int *rcounts, const int *rdisps,
+mca_coll_basic_alltoallw_intra_inplace(const void *rbuf, const int *rcounts, const int *rdisps,
                                        struct ompi_datatype_t * const *rdtypes,
                                        struct ompi_communicator_t *comm,
                                        mca_coll_base_module_t *module)
 {
-    mca_coll_basic_module_t *basic_module = (mca_coll_basic_module_t*) module;
     int i, j, size, rank, err=MPI_SUCCESS, max_size;
-    MPI_Request *preq;
+    ompi_request_t **preq, **reqs = NULL;
     char *tmp_buffer;
     ptrdiff_t ext;
 
@@ -71,6 +70,9 @@ mca_coll_basic_alltoallw_intra_inplace(void *rbuf, int *rcounts, const int *rdis
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
+    reqs = coll_base_comm_get_reqs( module->base_data, 2);
+    if( NULL == reqs ) { err = OMPI_ERR_OUT_OF_RESOURCE; goto error_hndl; }
+
     /* in-place alltoallw slow algorithm (but works) */
     for (i = 0 ; i < size ; ++i) {
         size_t msg_size_i;
@@ -82,7 +84,7 @@ mca_coll_basic_alltoallw_intra_inplace(void *rbuf, int *rcounts, const int *rdis
             msg_size_j *= rcounts[j];
 
             /* Initiate all send/recv to/from others. */
-            preq = basic_module->mccb_reqs;
+            preq = reqs;
 
             if (i == rank && msg_size_j != 0) {
                 /* Copy the data into the temporary buffer */
@@ -119,17 +121,19 @@ mca_coll_basic_alltoallw_intra_inplace(void *rbuf, int *rcounts, const int *rdis
             }
 
             /* Wait for the requests to complete */
-            err = ompi_request_wait_all (2, basic_module->mccb_reqs, MPI_STATUSES_IGNORE);
+            err = ompi_request_wait_all (2, reqs, MPI_STATUSES_IGNORE);
             if (MPI_SUCCESS != err) { goto error_hndl; }
-
-            /* Free the requests. */
-            mca_coll_basic_free_reqs(basic_module->mccb_reqs, 2);
         }
     }
 
  error_hndl:
     /* Free the temporary buffer */
     free (tmp_buffer);
+    if( MPI_SUCCESS != err ) {  /* Free the requests. */
+        if( NULL != reqs ) {
+            ompi_coll_base_free_reqs(reqs, 2);
+        }
+    }
 
     /* All done */
 
@@ -145,22 +149,16 @@ mca_coll_basic_alltoallw_intra_inplace(void *rbuf, int *rcounts, const int *rdis
  *	Returns:	- MPI_SUCCESS or an MPI error code
  */
 int
-mca_coll_basic_alltoallw_intra(void *sbuf, int *scounts, int *sdisps,
-                               struct ompi_datatype_t **sdtypes,
-                               void *rbuf, int *rcounts, int *rdisps,
-                               struct ompi_datatype_t **rdtypes,
+mca_coll_basic_alltoallw_intra(const void *sbuf, const int *scounts, const int *sdisps,
+                               struct ompi_datatype_t * const *sdtypes,
+                               void *rbuf, const int *rcounts, const int *rdisps,
+                               struct ompi_datatype_t * const *rdtypes,
                                struct ompi_communicator_t *comm,
                                mca_coll_base_module_t *module)
 {
-    int i;
-    int size;
-    int rank;
-    int err;
-    char *psnd;
-    char *prcv;
-    int nreqs;
-    MPI_Request *preq;
-    mca_coll_basic_module_t *basic_module = (mca_coll_basic_module_t*) module;
+    int i, size, rank, err, nreqs;
+    char *psnd, *prcv;
+    ompi_request_t **preq, **reqs;
 
     /* Initialize. */
     if (MPI_IN_PLACE == sbuf) {
@@ -191,7 +189,8 @@ mca_coll_basic_alltoallw_intra(void *sbuf, int *scounts, int *sdisps,
     /* Initiate all send/recv to/from others. */
 
     nreqs = 0;
-    preq = basic_module->mccb_reqs;
+    reqs = preq = coll_base_comm_get_reqs(module->base_data, 2 * size);
+    if( NULL == reqs ) { return OMPI_ERR_OUT_OF_RESOURCE; }
 
     /* Post all receives first -- a simple optimization */
 
@@ -199,7 +198,7 @@ mca_coll_basic_alltoallw_intra(void *sbuf, int *scounts, int *sdisps,
         size_t msg_size;
         ompi_datatype_type_size(rdtypes[i], &msg_size);
         msg_size *= rcounts[i];
-       
+
         if (i == rank || 0 == msg_size)
             continue;
 
@@ -209,8 +208,7 @@ mca_coll_basic_alltoallw_intra(void *sbuf, int *scounts, int *sdisps,
                                       preq++));
         ++nreqs;
         if (MPI_SUCCESS != err) {
-            mca_coll_basic_free_reqs(basic_module->mccb_reqs,
-                                     nreqs);
+            ompi_coll_base_free_reqs(reqs, nreqs);
             return err;
         }
     }
@@ -232,15 +230,14 @@ mca_coll_basic_alltoallw_intra(void *sbuf, int *scounts, int *sdisps,
                                       preq++));
         ++nreqs;
         if (MPI_SUCCESS != err) {
-            mca_coll_basic_free_reqs(basic_module->mccb_reqs,
-                                     nreqs);
+            ompi_coll_base_free_reqs(reqs, nreqs);
             return err;
         }
     }
 
     /* Start your engines.  This will never return an error. */
 
-    MCA_PML_CALL(start(nreqs, basic_module->mccb_reqs));
+    MCA_PML_CALL(start(nreqs, reqs));
 
     /* Wait for them all.  If there's an error, note that we don't care
      * what the error was -- just that there *was* an error.  The PML
@@ -249,15 +246,11 @@ mca_coll_basic_alltoallw_intra(void *sbuf, int *scounts, int *sdisps,
      * So free them anyway -- even if there was an error, and return the
      * error after we free everything. */
 
-    err = ompi_request_wait_all(nreqs, basic_module->mccb_reqs,
-                                MPI_STATUSES_IGNORE);
-
-    /* Free the requests. */
-
-    mca_coll_basic_free_reqs(basic_module->mccb_reqs, nreqs);
+    err = ompi_request_wait_all(nreqs, reqs, MPI_STATUSES_IGNORE);
+    /* Free the requests in all cases as they are persistent */
+    ompi_coll_base_free_reqs(reqs, nreqs);
 
     /* All done */
-
     return err;
 }
 
@@ -270,35 +263,31 @@ mca_coll_basic_alltoallw_intra(void *sbuf, int *scounts, int *sdisps,
  *	Returns:	- MPI_SUCCESS or an MPI error code
  */
 int
-mca_coll_basic_alltoallw_inter(void *sbuf, int *scounts, int *sdisps,
-                               struct ompi_datatype_t **sdtypes,
-                               void *rbuf, int *rcounts, int *rdisps,
-                               struct ompi_datatype_t **rdtypes,
+mca_coll_basic_alltoallw_inter(const void *sbuf, const int *scounts, const int *sdisps,
+                               struct ompi_datatype_t * const *sdtypes,
+                               void *rbuf, const int *rcounts, const int *rdisps,
+                               struct ompi_datatype_t * const *rdtypes,
                                struct ompi_communicator_t *comm,
                                mca_coll_base_module_t *module)
 {
-    int i;
-    int size;
-    int err;
-    char *psnd;
-    char *prcv;
-    int nreqs;
-    MPI_Request *preq;
-    mca_coll_basic_module_t *basic_module = (mca_coll_basic_module_t*) module;
+    int i, size, err, nreqs;
+    char *psnd, *prcv;
+    ompi_request_t **preq, **reqs;
 
     /* Initialize. */
     size = ompi_comm_remote_size(comm);
 
     /* Initiate all send/recv to/from others. */
     nreqs = 0;
-    preq = basic_module->mccb_reqs;
+    reqs = preq = coll_base_comm_get_reqs(module->base_data, 2 * size);
+    if( NULL == reqs ) { return OMPI_ERR_OUT_OF_RESOURCE; }
 
     /* Post all receives first -- a simple optimization */
     for (i = 0; i < size; ++i) {
         size_t msg_size;
         ompi_datatype_type_size(rdtypes[i], &msg_size);
         msg_size *= rcounts[i];
-       
+
         if (0 == msg_size)
             continue;
 
@@ -308,8 +297,7 @@ mca_coll_basic_alltoallw_inter(void *sbuf, int *scounts, int *sdisps,
                                       comm, preq++));
         ++nreqs;
         if (OMPI_SUCCESS != err) {
-            mca_coll_basic_free_reqs(basic_module->mccb_reqs,
-                                     nreqs);
+            ompi_coll_base_free_reqs(reqs, nreqs);
             return err;
         }
     }
@@ -330,14 +318,13 @@ mca_coll_basic_alltoallw_inter(void *sbuf, int *scounts, int *sdisps,
                                       preq++));
         ++nreqs;
         if (OMPI_SUCCESS != err) {
-            mca_coll_basic_free_reqs(basic_module->mccb_reqs,
-                                     nreqs);
+            ompi_coll_base_free_reqs(reqs, nreqs);
             return err;
         }
     }
 
     /* Start your engines.  This will never return an error. */
-    MCA_PML_CALL(start(nreqs, basic_module->mccb_reqs));
+    MCA_PML_CALL(start(nreqs, reqs));
 
     /* Wait for them all.  If there's an error, note that we don't care
      * what the error was -- just that there *was* an error.  The PML
@@ -345,11 +332,10 @@ mca_coll_basic_alltoallw_inter(void *sbuf, int *scounts, int *sdisps,
      * i.e., by the end of this call, all the requests are free-able.
      * So free them anyway -- even if there was an error, and return the
      * error after we free everything. */
-    err = ompi_request_wait_all(nreqs, basic_module->mccb_reqs,
-                                MPI_STATUSES_IGNORE);
+    err = ompi_request_wait_all(nreqs, reqs, MPI_STATUSES_IGNORE);
 
-    /* Free the requests. */
-    mca_coll_basic_free_reqs(basic_module->mccb_reqs, nreqs);
+    /* Free the requests in all cases as they are persistent */
+    ompi_coll_base_free_reqs(reqs, nreqs);
 
     /* All done */
     return err;
