@@ -5,15 +5,17 @@
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2010 University of Houston. All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -24,7 +26,9 @@
 
 #include "mpi.h"
 #include "ompi/constants.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
+#include "ompi/request/request.h"
+#include "ompi/communicator/communicator.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/coll/base/coll_tags.h"
@@ -37,7 +41,7 @@
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-mca_coll_inter_allgather_inter(void *sbuf, int scount,
+mca_coll_inter_allgather_inter(const void *sbuf, int scount,
                                struct ompi_datatype_t *sdtype,
                                void *rbuf, int rcount,
                                struct ompi_datatype_t *rdtype,
@@ -54,20 +58,20 @@ mca_coll_inter_allgather_inter(void *sbuf, int scount,
     rsize = ompi_comm_remote_size(comm);
 
     /* Perform the gather locally at the root */
-    err = ompi_ddt_get_extent(sdtype, &slb, &sextent);
+    err = ompi_datatype_get_extent(sdtype, &slb, &sextent);
     if (OMPI_SUCCESS != err) {
 	return OMPI_ERROR;
     }
-    
+
     if ( scount > 0 ) {
 	incr = sextent * scount;
-	ptmp = (char*)malloc(size * incr); 
+	ptmp = (char*)malloc(size * incr);
 	if (NULL == ptmp) {
 	    return OMPI_ERR_OUT_OF_RESOURCE;
 	}
 
-	err = comm->c_local_comm->c_coll.coll_gather(sbuf, scount, sdtype, 
-						     ptmp, scount, sdtype, 
+	err = comm->c_local_comm->c_coll.coll_gather(sbuf, scount, sdtype,
+						     ptmp, scount, sdtype,
 						     0, comm->c_local_comm,
 						     comm->c_local_comm->c_coll.coll_gather_module);
 	if (OMPI_SUCCESS != err) {
@@ -99,7 +103,7 @@ mca_coll_inter_allgather_inter(void *sbuf, int scount,
     }
     /* bcast the message to all the local processes */
     if ( rcount > 0 ) {
-	err = comm->c_local_comm->c_coll.coll_bcast(rbuf, rcount*rsize, rdtype, 
+	err = comm->c_local_comm->c_coll.coll_bcast(rbuf, rcount*rsize, rdtype,
 						    root, comm->c_local_comm,
 						    comm->c_local_comm->c_coll.coll_bcast_module);
 	if (OMPI_SUCCESS != err) {
