@@ -2,26 +2,26 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2014 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 #include "ompi_config.h"
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 
 #include "opal/datatype/opal_convertor.h"
 #include "opal/sys/atomic.h"
@@ -35,16 +35,16 @@
 /*
  * Local functions
  */
-static int reduce_inorder(void *sbuf, void* rbuf, int count, 
-                          struct ompi_datatype_t *dtype, 
-                          struct ompi_op_t *op, 
+static int reduce_inorder(const void *sbuf, void* rbuf, int count,
+                          struct ompi_datatype_t *dtype,
+                          struct ompi_op_t *op,
                           int root, struct ompi_communicator_t *comm,
                           mca_coll_base_module_t *module);
 #define WANT_REDUCE_NO_ORDER 0
 #if WANT_REDUCE_NO_ORDER
-static int reduce_no_order(void *sbuf, void* rbuf, int count, 
-                           struct ompi_datatype_t *dtype, 
-                           struct ompi_op_t *op, 
+static int reduce_no_order(const void *sbuf, void* rbuf, int count,
+                           struct ompi_datatype_t *dtype,
+                           struct ompi_op_t *op,
                            int root, struct ompi_communicator_t *comm,
                            mca_coll_base_module_t *module);
 #endif
@@ -52,21 +52,21 @@ static int reduce_no_order(void *sbuf, void* rbuf, int count,
 /*
  * Useful utility routine
  */
-#if !defined(__WINDOWS__) && !defined(__CYGWIN__)
+#if !defined(min)
 static inline int min(int a, int b)
 {
     return (a < b) ? a : b;
 }
-#endif  /* !defined(__WINDOWS__) */
+#endif
 
 /**
  * Shared memory reduction.
  *
  * Simply farms out to the associative or non-associative functions.
  */
-int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count, 
-                             struct ompi_datatype_t *dtype, 
-                             struct ompi_op_t *op, 
+int mca_coll_sm_reduce_intra(const void *sbuf, void* rbuf, int count,
+                             struct ompi_datatype_t *dtype,
+                             struct ompi_op_t *op,
                              int root, struct ompi_communicator_t *comm,
                              mca_coll_base_module_t *module)
 {
@@ -90,25 +90,25 @@ int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count,
         return sm_module->previous_reduce(sbuf, rbuf, count,
                                           dtype, op, root, comm,
                                           sm_module->previous_reduce_module);
-    } 
+    }
 #if WANT_REDUCE_NO_ORDER
     else {
         /* Lazily enable the module the first time we invoke a
            collective on it */
         if (!sm_module->enabled) {
-            if (OMPI_SUCCESS != 
+            if (OMPI_SUCCESS !=
                 (ret = ompi_coll_sm_lazy_enable(module, comm))) {
                 return ret;
             }
         }
-        
+
         if (!ompi_op_is_intrinsic(op) ||
             (ompi_op_is_intrinsic(op) && !ompi_op_is_float_assoc(op) &&
              0 != (dtype->flags & OMPI_DATATYPE_FLAG_DATA_FLOAT))) {
-            return reduce_inorder(sbuf, rbuf, count, dtype, op, 
+            return reduce_inorder(sbuf, rbuf, count, dtype, op,
                                   root, comm, module);
         } else {
-            return reduce_no_order(sbuf, rbuf, count, dtype, op, 
+            return reduce_no_order(sbuf, rbuf, count, dtype, op,
                                    root, comm, module);
         }
     }
@@ -119,7 +119,7 @@ int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count,
         if (!sm_module->enabled) {
             int ret;
 
-            if (OMPI_SUCCESS != 
+            if (OMPI_SUCCESS !=
                 (ret = ompi_coll_sm_lazy_enable(module, comm))) {
                 return ret;
             }
@@ -139,7 +139,7 @@ int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count,
  * (result operation 3), etc.
  *
  * Root's algorithm:
- * 
+ *
  * If our datatype is "friendly" (i.e., the representation of the
  * buffer is the same packed as it is unpacked), then the root doesn't
  * need a temporary buffer -- we can combine the operands directly
@@ -154,7 +154,7 @@ int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count,
  * called, we know that the datattype is smaller than the max size of
  * a fragment, so this is definitely possible)
  *
- * 2. loop over all the processes -- 0 to (comm_size-1).  
+ * 2. loop over all the processes -- 0 to (comm_size-1).
  * For process 0:
  * - if the root==0, copy the *entire* buffer (i.e., don't copy
  *   fragment by fragment -- might as well copy the entire thing) the
@@ -173,9 +173,9 @@ int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count,
  */
 
 
-static int reduce_inorder(void *sbuf, void* rbuf, int count, 
-                          struct ompi_datatype_t *dtype, 
-                          struct ompi_op_t *op, 
+static int reduce_inorder(const void *sbuf, void* rbuf, int count,
+                          struct ompi_datatype_t *dtype,
+                          struct ompi_op_t *op,
                           int root, struct ompi_communicator_t *comm,
                           mca_coll_base_module_t *module)
 {
@@ -223,7 +223,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
     /*********************************************************************
      * Root
      *********************************************************************/
-    
+
     if (root == rank) {
         opal_convertor_t rtb_convertor, rbuf_convertor;
         char *reduce_temp_buffer, *free_buffer, *reduce_target;
@@ -239,7 +239,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
            representation is not the same, then we need to get a
            receive convertor and a temporary buffer to receive
            into. */
-        
+
         if (ompi_datatype_is_contiguous_memory_layout(dtype, count)) {
             reduce_temp_buffer = free_buffer = NULL;
         } else {
@@ -265,23 +265,23 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                entire user buffer) -- we only need to be able to hold
                "segment_ddt_count" instances (i.e., the number of
                instances that can be held in a single fragment) */
-            
-            free_buffer = (char*)malloc(true_extent + 
+
+            free_buffer = (char*)malloc(true_extent +
                                         (segment_ddt_count - 1) * extent);
             if (NULL == free_buffer) {
                 return OMPI_ERR_OUT_OF_RESOURCE;
             }
-            reduce_temp_buffer = free_buffer - lb;
-            
+            reduce_temp_buffer = free_buffer - true_lb;
+
             /* Trickery here: we use a potentially smaller count than
                the user count -- use the largest count that is <=
                user's count that will fit within a single segment. */
-            
-            if (OMPI_SUCCESS != 
+
+            if (OMPI_SUCCESS !=
                 (ret = opal_convertor_copy_and_prepare_for_recv(
                                        ompi_mpi_local_convertor,
                                        &(dtype->super),
-                                       segment_ddt_count, 
+                                       segment_ddt_count,
                                        reduce_temp_buffer,
                                        0,
                                        &rtb_convertor))) {
@@ -291,11 +291,11 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
 
             /* See if we need the rbuf_convertor */
             if (size - 1 != rank) {
-                if (OMPI_SUCCESS != 
+                if (OMPI_SUCCESS !=
                     (ret = opal_convertor_copy_and_prepare_for_recv(
                                        ompi_mpi_local_convertor,
                                        &(dtype->super),
-                                       count, 
+                                       count,
                                        rbuf,
                                        0,
                                        &rbuf_convertor))) {
@@ -319,16 +319,16 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                 }
                 return OMPI_ERR_OUT_OF_RESOURCE;
             }
-            sbuf = inplace_temp - lb;
+            sbuf = inplace_temp - true_lb;
             ompi_datatype_copy_content_same_ddt(dtype, count, (char *) sbuf, (char *) rbuf);
         } else {
             inplace_temp = NULL;
         }
-               
+
         /* Main loop over receiving / reducing fragments */
 
         do {
-            flag_num = (data->mcb_operation_count % 
+            flag_num = (data->mcb_operation_count %
                         mca_coll_sm_component.sm_comm_num_in_use_flags);
             FLAG_SETUP(flag_num, flag, data);
             FLAG_WAIT_FOR_IDLE(flag, reduce_root_flag_label);
@@ -336,10 +336,10 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
             ++data->mcb_operation_count;
 
             /* Loop over all the segments in this set */
-            
-            segment_num = 
+
+            segment_num =
                 flag_num * mca_coll_sm_component.sm_segs_per_inuse_flag;
-            max_segment_num = 
+            max_segment_num =
                 (flag_num + 1) * mca_coll_sm_component.sm_segs_per_inuse_flag;
             reduce_target = (((char*) rbuf) + (frag_num * extent * segment_ddt_count));
             do {
@@ -364,7 +364,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                                                reduce_target, (char*)sbuf);
                         }
                     }
-                } 
+                }
 
                 /* Process (size-1) is not the root */
                 else {
@@ -372,19 +372,19 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                        like any other non-root process */
                     index = &(data->mcb_data_index[segment_num]);
                     PARENT_WAIT_FOR_NOTIFY_SPECIFIC(size - 1, rank, index, max_data, reduce_root_parent_label1);
-                        
+
                     /* If the datatype is contiguous, just copy it
                        straight to the reduce_target */
                     if (NULL == free_buffer) {
                         memcpy(reduce_target, ((char*)index->mcbmi_data) +
                                (size - 1) * mca_coll_sm_component.sm_fragment_size, max_data);
-                    } 
+                    }
                     /* If the datatype is noncontiguous, use the
                        rbuf_convertor to unpack it straight to the
                        rbuf */
                     else {
                         max_data = segment_ddt_bytes;
-                        COPY_FRAGMENT_OUT(rbuf_convertor, size - 1, index, 
+                        COPY_FRAGMENT_OUT(rbuf_convertor, size - 1, index,
                                           iov, max_data);
                     }
                 }
@@ -395,12 +395,14 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                 for (peer = size - 2; peer >= 0; --peer) {
 
                     /* Handle the case where the source is this
-                       process.  In this case, we
+                       process (which, by definition, excludes the
+                       sbuf_copied_to_rbuf case because that can
+                       *only* happen when root==0).  In this case, we
                        don't need to wait for the peer (i.e., me) to
                        copy into shmem -- just reduce directly from my
                        sbuf. */
                     if (rank == peer) {
-                        ompi_op_reduce(op, 
+                        ompi_op_reduce(op,
                                        ((char *) sbuf) +
                                        frag_num * extent * segment_ddt_count,
                                        reduce_target,
@@ -413,42 +415,42 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                        the segment into shmem. */
                     else {
                         index = &(data->mcb_data_index[segment_num]);
-                        PARENT_WAIT_FOR_NOTIFY_SPECIFIC(peer, rank, 
+                        PARENT_WAIT_FOR_NOTIFY_SPECIFIC(peer, rank,
                                                         index, max_data, reduce_root_parent_label2);
-                        
+
                         /* If we don't need an extra buffer, then do the
                            reduction operation on the fragment straight
                            from the shmem. */
-                        
+
                         if (NULL == free_buffer) {
                             ompi_op_reduce(op,
-                                           (index->mcbmi_data + 
+                                           (index->mcbmi_data +
                                             (peer * mca_coll_sm_component.sm_fragment_size)),
-                                           reduce_target, 
+                                           reduce_target,
                                            min(count_left, segment_ddt_count),
                                            dtype);
                         }
-                        
+
                         /* Otherwise, unpack the fragment to the temporary
                            buffer and then do the reduction from there */
-                        
+
                         else {
                             /* Unpack the fragment into my temporary
                                buffer */
                             max_data = segment_ddt_bytes;
-                            COPY_FRAGMENT_OUT(rtb_convertor, peer, index, 
+                            COPY_FRAGMENT_OUT(rtb_convertor, peer, index,
                                               iov, max_data);
                             opal_convertor_set_position(&rtb_convertor, &zero);
-                            
+
                             /* Do the reduction on this fragment */
                             ompi_op_reduce(op, reduce_temp_buffer,
-                                           reduce_target, 
+                                           reduce_target,
                                            min(count_left, segment_ddt_count),
                                            dtype);
                         }
                     } /* whether this process was me or not */
                 } /* loop over all proceses */
-                
+
                 /* We've iterated through all the processes -- now we
                    move on to the next segment */
 
@@ -464,7 +466,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
         } while (bytes < total_size);
 
         /* Kill the convertor, if we had one */
-        
+
         if (NULL != free_buffer) {
             OBJ_DESTRUCT(&rtb_convertor);
             OBJ_DESTRUCT(&rbuf_convertor);
@@ -485,11 +487,11 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
 
         opal_convertor_t sbuf_convertor;
         OBJ_CONSTRUCT(&sbuf_convertor, opal_convertor_t);
-        if (OMPI_SUCCESS != 
-            (ret = 
+        if (OMPI_SUCCESS !=
+            (ret =
              opal_convertor_copy_and_prepare_for_send(ompi_mpi_local_convertor,
                                                       &(dtype->super),
-                                                      count, 
+                                                      count,
                                                       sbuf,
                                                       0,
                                                       &sbuf_convertor))) {
@@ -497,9 +499,9 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
         }
 
         /* Loop over sending fragments to the root */
-        
+
         do {
-            flag_num = (data->mcb_operation_count % 
+            flag_num = (data->mcb_operation_count %
                         mca_coll_sm_component.sm_comm_num_in_use_flags);
 
             /* Wait for the root to mark this set of segments as
@@ -510,9 +512,9 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
 
             /* Loop over all the segments in this set */
 
-            segment_num = 
+            segment_num =
                 flag_num * mca_coll_sm_component.sm_segs_per_inuse_flag;
-            max_segment_num = 
+            max_segment_num =
                 (flag_num + 1) * mca_coll_sm_component.sm_segs_per_inuse_flag;
             do {
                 index = &(data->mcb_data_index[segment_num]);
@@ -525,7 +527,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
 
                 /* Wait for the write to absolutely complete */
                 opal_atomic_wmb();
-                
+
                 /* Tell my parent (always the reduction root -- we're
                    ignoring the mcb_tree parent/child relationships
                    here) that this fragment is ready */
@@ -556,9 +558,9 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
  * This function performs the reduction in whatever order the operands
  * arrive.
  */
-static int reduce_no_order(void *sbuf, void* rbuf, int count, 
-                           struct ompi_datatype_t *dtype, 
-                           struct ompi_op_t *op, 
+static int reduce_no_order(const void *sbuf, void* rbuf, int count,
+                           struct ompi_datatype_t *dtype,
+                           struct ompi_op_t *op,
                            int root, struct ompi_communicator_t *comm,
                            mca_coll_base_module_t *module)
 {
