@@ -1,11 +1,14 @@
 /*
  * Copyright (c) 2011-2013 Sandia National Laboratories.  All rights reserved.
+ * Copyright (c) 2014      The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -18,8 +21,6 @@
 #include "osc_portals4.h"
 #include "osc_portals4_request.h"
 
-#include "ompi/mca/mtl/portals4/mtl_portals4_endpoint.h"
-
 
 static int
 ompi_osc_portals4_get_op(struct ompi_op_t *op, ptl_op_t *ptl_op)
@@ -196,8 +197,6 @@ ompi_osc_portals4_rput(const void *origin_addr,
     ptl_process_t peer = ompi_osc_portals4_get_peer(module, target);
     size_t length;
     size_t offset;
-    ptl_handle_md_t md_h;
-    void *md_base;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "rput: 0x%lx, %d, %s, %d, %d, %d, %s, 0x%lx",
@@ -219,7 +218,7 @@ ompi_osc_portals4_rput(const void *origin_addr,
                     "MPI_Rput: transfer of non-contiguous memory is not currently supported.\n");
         return OMPI_ERR_NOT_SUPPORTED;
     } else {
-        opal_atomic_add_64(&module->opcount, 1);
+        (void)opal_atomic_add_64(&module->opcount, 1);
         request->ops_expected = 1;
         ret = ompi_datatype_type_size(origin_dt, &length);
         if (OMPI_SUCCESS != ret) {
@@ -227,9 +226,8 @@ ompi_osc_portals4_rput(const void *origin_addr,
             return ret;
         }
         length *= origin_count;
-        ompi_osc_portals4_get_md(origin_addr, module->req_md_h, &md_h, &md_base);
-        ret = PtlPut(md_h,
-                     (ptl_size_t) ((char*) origin_addr - (char*) md_base),
+        ret = PtlPut(module->req_md_h,
+                     (ptl_size_t) origin_addr,
                      length,
                      PTL_ACK_REQ,
                      peer,
@@ -266,8 +264,6 @@ ompi_osc_portals4_rget(void *origin_addr,
     ptl_process_t peer = ompi_osc_portals4_get_peer(module, target);
     size_t length;
     size_t offset;
-    ptl_handle_md_t md_h;
-    void *md_base;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "rget: 0x%lx, %d, %s, %d, %d, %d, %s, 0x%lx",
@@ -289,7 +285,7 @@ ompi_osc_portals4_rget(void *origin_addr,
                     "MPI_Rget: transfer of non-contiguous memory is not currently supported.\n");
         return OMPI_ERR_NOT_SUPPORTED;
     } else {
-        opal_atomic_add_64(&module->opcount, 1);
+        (void)opal_atomic_add_64(&module->opcount, 1);
         request->ops_expected = 1;
         ret = ompi_datatype_type_size(origin_dt, &length);
         if (OMPI_SUCCESS != ret) {
@@ -297,9 +293,8 @@ ompi_osc_portals4_rget(void *origin_addr,
             return ret;
         }
         length *= origin_count;
-        ompi_osc_portals4_get_md(origin_addr, module->req_md_h, &md_h, &md_base);
-        ret = PtlGet(md_h,
-                     (ptl_size_t) ((char*) origin_addr - (char*) md_base),
+        ret = PtlGet(module->req_md_h,
+                     (ptl_size_t) origin_addr,
                      length,
                      peer,
                      module->pt_idx,
@@ -337,8 +332,6 @@ ompi_osc_portals4_raccumulate(const void *origin_addr,
     size_t offset;
     ptl_op_t ptl_op;
     ptl_datatype_t ptl_dt;
-    ptl_handle_md_t md_h;
-    void *md_base;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "raccumulate: 0x%lx, %d, %s, %d, %d, %d, %s, %s 0x%lx",
@@ -371,16 +364,15 @@ ompi_osc_portals4_raccumulate(const void *origin_addr,
         length *= origin_count;
         sent = 0;
 
-        ompi_osc_portals4_get_md(origin_addr, module->req_md_h, &md_h, &md_base);
-        md_offset = ((char*) origin_addr - (char*) md_base);
+        md_offset = (ptl_size_t) origin_addr;
 
         do {
             size_t msg_length = MIN(module->atomic_max, length - sent);
-            opal_atomic_add_64(&module->opcount, 1);
+            (void)opal_atomic_add_64(&module->opcount, 1);
             request->ops_expected++;
 
             if (MPI_REPLACE == op) {
-                ret = PtlPut(md_h,
+                ret = PtlPut(module->req_md_h,
                              md_offset + sent,
                              msg_length,
                              PTL_ACK_REQ,
@@ -397,7 +389,7 @@ ompi_osc_portals4_raccumulate(const void *origin_addr,
                 ret = ompi_osc_portals4_get_op(op, &ptl_op);
                 if (OMPI_SUCCESS != ret) return ret;
 
-                ret = PtlAtomic(md_h,
+                ret = PtlAtomic(module->req_md_h,
                                 offset + sent,
                                 msg_length,
                                 PTL_ACK_REQ,
@@ -426,14 +418,14 @@ int
 ompi_osc_portals4_rget_accumulate(const void *origin_addr,
                                   int origin_count,
                                   struct ompi_datatype_t *origin_dt,
-                                  void *result_addr, 
-                                  int result_count, 
+                                  void *result_addr,
+                                  int result_count,
                                   struct ompi_datatype_t *result_dt,
-                                  int target, 
-                                  MPI_Aint target_disp, 
+                                  int target,
+                                  MPI_Aint target_disp,
                                   int target_count,
-                                  struct ompi_datatype_t *target_dt, 
-                                  struct ompi_op_t *op, 
+                                  struct ompi_datatype_t *target_dt,
+                                  struct ompi_op_t *op,
                                   struct ompi_win_t *win,
                                   struct ompi_request_t **ompi_req)
 {
@@ -474,8 +466,6 @@ ompi_osc_portals4_rget_accumulate(const void *origin_addr,
         sent = 0;
 
         if (MPI_REPLACE == op) {
-            ptl_handle_md_t result_md_h, origin_md_h;
-            void *result_md_base, *origin_md_base;
             ptl_size_t result_md_offset, origin_md_offset;
 
             ret = ompi_datatype_type_size(origin_dt, &length);
@@ -483,22 +473,25 @@ ompi_osc_portals4_rget_accumulate(const void *origin_addr,
                 OMPI_OSC_PORTALS4_REQUEST_RETURN(request);
                 return ret;
             }
+            ret = ompi_osc_portals4_get_dt(origin_dt, &ptl_dt);
+            if (OMPI_SUCCESS != ret) {
+                OMPI_OSC_PORTALS4_REQUEST_RETURN(request);
+                return ret;
+            }
             length *= origin_count;
 
-            ompi_osc_portals4_get_md(result_addr, module->req_md_h, &result_md_h, &result_md_base);
-            result_md_offset = ((char*) result_addr - (char*) result_md_base);
-            ompi_osc_portals4_get_md(origin_addr, module->md_h, &origin_md_h, &origin_md_base);
-            origin_md_offset = ((char*) origin_addr - (char*) origin_md_base);
+            result_md_offset = (ptl_size_t) result_addr;
+            origin_md_offset = (ptl_size_t) origin_addr;
 
             do {
                 size_t msg_length = MIN(module->fetch_atomic_max, length - sent);
 
-                opal_atomic_add_64(&module->opcount, 1);
+                (void)opal_atomic_add_64(&module->opcount, 1);
                 request->ops_expected++;
 
-                ret = PtlSwap(result_md_h,
+                ret = PtlSwap(module->req_md_h,
                               result_md_offset + sent,
-                              origin_md_h,
+                              module->md_h,
                               origin_md_offset + sent,
                               msg_length,
                               peer,
@@ -513,8 +506,6 @@ ompi_osc_portals4_rget_accumulate(const void *origin_addr,
                 sent += msg_length;
             } while (sent < length);
         } else if (MPI_NO_OP == op) {
-            ptl_handle_md_t md_h;
-            void *md_base;
             ptl_size_t md_offset;
 
             ret = ompi_datatype_type_size(target_dt, &length);
@@ -524,16 +515,15 @@ ompi_osc_portals4_rget_accumulate(const void *origin_addr,
             }
             length *= target_count;
 
-            ompi_osc_portals4_get_md(result_addr, module->req_md_h, &md_h, &md_base);
-            md_offset = ((char*) result_addr - (char*) md_base);
+            md_offset = (ptl_size_t) result_addr;
 
             do {
                 size_t msg_length = MIN(module->fetch_atomic_max, length - sent);
 
-                opal_atomic_add_64(&module->opcount, 1);
+                (void)opal_atomic_add_64(&module->opcount, 1);
                 request->ops_expected++;
 
-                ret = PtlGet(md_h,
+                ret = PtlGet(module->req_md_h,
                              md_offset + sent,
                              msg_length,
                              peer,
@@ -544,8 +534,6 @@ ompi_osc_portals4_rget_accumulate(const void *origin_addr,
                 sent += msg_length;
             } while (sent < length);
         } else {
-            ptl_handle_md_t result_md_h, origin_md_h;
-            void *result_md_base, *origin_md_base;
             ptl_size_t result_md_offset, origin_md_offset;
 
             ret = ompi_datatype_type_size(origin_dt, &length);
@@ -555,10 +543,8 @@ ompi_osc_portals4_rget_accumulate(const void *origin_addr,
             }
             length *= origin_count;
 
-            ompi_osc_portals4_get_md(result_addr, module->req_md_h, &result_md_h, &result_md_base);
-            result_md_offset = ((char*) result_addr - (char*) result_md_base);
-            ompi_osc_portals4_get_md(origin_addr, module->md_h, &origin_md_h, &origin_md_base);
-            origin_md_offset = ((char*) origin_addr - (char*) origin_md_base);
+            result_md_offset = (ptl_size_t) result_addr;
+            origin_md_offset = (ptl_size_t) origin_addr;
 
             ret = ompi_osc_portals4_get_dt(origin_dt, &ptl_dt);
             if (OMPI_SUCCESS != ret) return ret;
@@ -569,12 +555,12 @@ ompi_osc_portals4_rget_accumulate(const void *origin_addr,
             do {
                 size_t msg_length = MIN(module->fetch_atomic_max, length - sent);
 
-                opal_atomic_add_64(&module->opcount, 1);
+                (void)opal_atomic_add_64(&module->opcount, 1);
                 request->ops_expected++;
 
-                ret = PtlFetchAtomic(result_md_h,
+                ret = PtlFetchAtomic(module->req_md_h,
                                      result_md_offset + sent,
-                                     origin_md_h,
+                                     module->md_h,
                                      origin_md_offset + sent,
                                      msg_length,
                                      peer,
@@ -614,8 +600,6 @@ ompi_osc_portals4_put(const void *origin_addr,
     ptl_process_t peer = ompi_osc_portals4_get_peer(module, target);
     size_t length;
     size_t offset;
-    ptl_handle_md_t md_h;
-    void *md_base;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "put: 0x%lx, %d, %s, %d, %d, %d, %s, 0x%lx",
@@ -632,15 +616,14 @@ ompi_osc_portals4_put(const void *origin_addr,
                     "MPI_Put: transfer of non-contiguous memory is not currently supported.\n");
         return OMPI_ERR_NOT_SUPPORTED;
     } else {
-        opal_atomic_add_64(&module->opcount, 1);
+        (void)opal_atomic_add_64(&module->opcount, 1);
         ret = ompi_datatype_type_size(origin_dt, &length);
         if (OMPI_SUCCESS != ret) {
             return ret;
         }
         length *= origin_count;
-        ompi_osc_portals4_get_md(origin_addr, module->md_h, &md_h, &md_base);
-        ret = PtlPut(md_h,
-                     (ptl_size_t) ((char*) origin_addr - (char*) md_base),
+        ret = PtlPut(module->md_h,
+                     (ptl_size_t) origin_addr,
                      length,
                      PTL_ACK_REQ,
                      peer,
@@ -674,8 +657,6 @@ ompi_osc_portals4_get(void *origin_addr,
     ptl_process_t peer = ompi_osc_portals4_get_peer(module, target);
     size_t length;
     size_t offset;
-    ptl_handle_md_t md_h;
-    void *md_base;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "get: 0x%lx, %d, %s, %d, %d, %d, %s, 0x%lx",
@@ -692,15 +673,14 @@ ompi_osc_portals4_get(void *origin_addr,
                     "MPI_Get: transfer of non-contiguous memory is not currently supported.\n");
         return OMPI_ERR_NOT_SUPPORTED;
     } else {
-        opal_atomic_add_64(&module->opcount, 1);
+        (void)opal_atomic_add_64(&module->opcount, 1);
         ret = ompi_datatype_type_size(origin_dt, &length);
         if (OMPI_SUCCESS != ret) {
             return ret;
         }
         length *= origin_count;
-        ompi_osc_portals4_get_md(origin_addr, module->md_h, &md_h, &md_base);
-        ret = PtlGet(md_h,
-                     (ptl_size_t) ((char*) origin_addr - (char*) md_base),
+        ret = PtlGet(module->md_h,
+                     (ptl_size_t) origin_addr,
                      length,
                      peer,
                      module->pt_idx,
@@ -735,8 +715,6 @@ ompi_osc_portals4_accumulate(const void *origin_addr,
     size_t offset;
     ptl_op_t ptl_op;
     ptl_datatype_t ptl_dt;
-    ptl_handle_md_t md_h;
-    void *md_base;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "accumulate: 0x%lx, %d, %s, %d, %d, %d, %s, %s, 0x%lx",
@@ -763,15 +741,14 @@ ompi_osc_portals4_accumulate(const void *origin_addr,
         length *= origin_count;
         sent = 0;
 
-        ompi_osc_portals4_get_md(origin_addr, module->md_h, &md_h, &md_base);
-        md_offset = ((char*) origin_addr - (char*) md_base);
+        md_offset = (ptl_size_t) origin_addr;
 
         do {
             size_t msg_length = MIN(module->atomic_max, length - sent);
-            opal_atomic_add_64(&module->opcount, 1);
+            (void)opal_atomic_add_64(&module->opcount, 1);
 
             if (MPI_REPLACE == op) {
-                ret = PtlPut(md_h,
+                ret = PtlPut(module->md_h,
                              md_offset + sent,
                              msg_length,
                              PTL_ACK_REQ,
@@ -788,7 +765,7 @@ ompi_osc_portals4_accumulate(const void *origin_addr,
                 ret = ompi_osc_portals4_get_op(op, &ptl_op);
                 if (OMPI_SUCCESS != ret) return ret;
 
-                ret = PtlAtomic(md_h,
+                ret = PtlAtomic(module->md_h,
                                 md_offset + sent,
                                 msg_length,
                                 PTL_ACK_REQ,
@@ -816,14 +793,14 @@ int
 ompi_osc_portals4_get_accumulate(const void *origin_addr,
                                  int origin_count,
                                  struct ompi_datatype_t *origin_dt,
-                                 void *result_addr, 
-                                 int result_count, 
+                                 void *result_addr,
+                                 int result_count,
                                  struct ompi_datatype_t *result_dt,
-                                 int target, 
-                                 MPI_Aint target_disp, 
+                                 int target,
+                                 MPI_Aint target_disp,
                                  int target_count,
                                  struct ompi_datatype_t *target_dt,
-                                 struct ompi_op_t *op, 
+                                 struct ompi_op_t *op,
                                  struct ompi_win_t *win)
 {
     int ret;
@@ -857,29 +834,29 @@ ompi_osc_portals4_get_accumulate(const void *origin_addr,
     } else {
         sent = 0;
         if (MPI_REPLACE == op) {
-            ptl_handle_md_t result_md_h, origin_md_h;
-            void *result_md_base, *origin_md_base;
             ptl_size_t result_md_offset, origin_md_offset;
 
             ret = ompi_datatype_type_size(origin_dt, &length);
             if (OMPI_SUCCESS != ret) {
                 return ret;
             }
+            ret = ompi_osc_portals4_get_dt(origin_dt, &ptl_dt);
+            if (OMPI_SUCCESS != ret) {
+                return ret;
+            }
             length *= origin_count;
 
-            ompi_osc_portals4_get_md(result_addr, module->md_h, &result_md_h, &result_md_base);
-            result_md_offset = ((char*) result_addr - (char*) result_md_base);
-            ompi_osc_portals4_get_md(origin_addr, module->md_h, &origin_md_h, &origin_md_base);
-            origin_md_offset = ((char*) origin_addr - (char*) origin_md_base);
+            result_md_offset = (ptl_size_t) result_addr;
+            origin_md_offset = (ptl_size_t) origin_addr;
 
             do {
                 size_t msg_length = MIN(module->fetch_atomic_max, length - sent);
 
-                opal_atomic_add_64(&module->opcount, 1);
+                (void)opal_atomic_add_64(&module->opcount, 1);
 
-                ret = PtlSwap(result_md_h,
+                ret = PtlSwap(module->md_h,
                               result_md_offset + sent,
-                              origin_md_h,
+                              module->md_h,
                               origin_md_offset + sent,
                               msg_length,
                               peer,
@@ -894,8 +871,6 @@ ompi_osc_portals4_get_accumulate(const void *origin_addr,
                 sent += msg_length;
             } while (sent < length);
         } else if (MPI_NO_OP == op) {
-            ptl_handle_md_t md_h;
-            void *md_base;
             ptl_size_t md_offset;
 
             ret = ompi_datatype_type_size(target_dt, &length);
@@ -904,15 +879,14 @@ ompi_osc_portals4_get_accumulate(const void *origin_addr,
             }
             length *= target_count;
 
-            ompi_osc_portals4_get_md(result_addr, module->md_h, &md_h, &md_base);
-            md_offset = ((char*) result_addr - (char*) md_base);
+            md_offset = (ptl_size_t) result_addr;
 
             do {
                 size_t msg_length = MIN(module->fetch_atomic_max, length - sent);
 
-                opal_atomic_add_64(&module->opcount, 1);
+                (void)opal_atomic_add_64(&module->opcount, 1);
 
-                ret = PtlGet(md_h,
+                ret = PtlGet(module->md_h,
                              md_offset + sent,
                              msg_length,
                              peer,
@@ -923,9 +897,7 @@ ompi_osc_portals4_get_accumulate(const void *origin_addr,
                 sent += msg_length;
             } while (sent < length);
         } else {
-            ptl_handle_md_t result_md_h, origin_md_h;
-            void *result_md_base, *origin_md_base;
-            ptl_size_t result_md_offset, origin_md_offset; 
+            ptl_size_t result_md_offset, origin_md_offset;
 
             ret = ompi_datatype_type_size(origin_dt, &length);
             if (OMPI_SUCCESS != ret) {
@@ -933,10 +905,8 @@ ompi_osc_portals4_get_accumulate(const void *origin_addr,
             }
             length *= origin_count;
 
-            ompi_osc_portals4_get_md(result_addr, module->md_h, &result_md_h, &result_md_base);
-            result_md_offset = ((char*) result_addr - (char*) result_md_base);
-            ompi_osc_portals4_get_md(origin_addr, module->md_h, &origin_md_h, &origin_md_base);
-            origin_md_offset = ((char*) origin_addr - (char*) origin_md_base);
+            result_md_offset = (ptl_size_t) result_addr;
+            origin_md_offset = (ptl_size_t) origin_addr;
 
             ret = ompi_osc_portals4_get_dt(origin_dt, &ptl_dt);
             if (OMPI_SUCCESS != ret) return ret;
@@ -948,11 +918,11 @@ ompi_osc_portals4_get_accumulate(const void *origin_addr,
             do {
                 size_t msg_length = MIN(module->fetch_atomic_max, length - sent);
 
-                opal_atomic_add_64(&module->opcount, 1);
+                (void)opal_atomic_add_64(&module->opcount, 1);
 
-                ret = PtlFetchAtomic(result_md_h,
+                ret = PtlFetchAtomic(module->md_h,
                                      result_md_offset + sent,
-                                     origin_md_h,
+                                     module->md_h,
                                      origin_md_offset + sent,
                                      msg_length,
                                      peer,
@@ -991,13 +961,11 @@ ompi_osc_portals4_compare_and_swap(const void *origin_addr,
     size_t length;
     size_t offset;
     ptl_datatype_t ptl_dt;
-    ptl_handle_md_t result_md_h, origin_md_h;
-    void *result_md_base, *origin_md_base;
     ptl_size_t result_md_offset, origin_md_offset;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "compare_and_swap: 0x%lx, 0x%lx, 0x%lx, %s, %d, %d, 0x%lx",
-                         (unsigned long) origin_addr, 
+                         (unsigned long) origin_addr,
                          (unsigned long) compare_addr,
                          (unsigned long) result_addr,
                          dt->name, target, (int) target_disp,
@@ -1011,18 +979,16 @@ ompi_osc_portals4_compare_and_swap(const void *origin_addr,
     ret = ompi_datatype_type_size(dt, &length);
     if (OMPI_SUCCESS != ret) return ret;
 
-    assert(length < module->fetch_atomic_max);
+    assert(length <= module->fetch_atomic_max);
 
-    ompi_osc_portals4_get_md(result_addr, module->md_h, &result_md_h, &result_md_base);
-    result_md_offset = ((char*) result_addr - (char*) result_md_base);
-    ompi_osc_portals4_get_md(origin_addr, module->md_h, &origin_md_h, &origin_md_base);
-    origin_md_offset = ((char*) origin_addr - (char*) origin_md_base);
+    result_md_offset = (ptl_size_t) result_addr;
+    origin_md_offset = (ptl_size_t) origin_addr;
 
-    opal_atomic_add_64(&module->opcount, 1);
+    (void)opal_atomic_add_64(&module->opcount, 1);
 
-    ret = PtlSwap(result_md_h,
+    ret = PtlSwap(module->md_h,
                   result_md_offset,
-                  origin_md_h,
+                  module->md_h,
                   origin_md_offset,
                   length,
                   peer,
@@ -1062,7 +1028,7 @@ ompi_osc_portals4_fetch_and_op(const void *origin_addr,
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "fetch_and_op: 0x%lx, 0x%lx, %s, %d, %d, %s, 0x%lx",
-                         (unsigned long) origin_addr, 
+                         (unsigned long) origin_addr,
                          (unsigned long) result_addr,
                          dt->name, target, (int) target_disp,
                          op->o_name,
@@ -1076,23 +1042,19 @@ ompi_osc_portals4_fetch_and_op(const void *origin_addr,
     ret = ompi_datatype_type_size(dt, &length);
     if (OMPI_SUCCESS != ret) return ret;
 
-    assert(length < module->fetch_atomic_max);
+    assert(length <= module->fetch_atomic_max);
 
-    opal_atomic_add_64(&module->opcount, 1);
+    (void)opal_atomic_add_64(&module->opcount, 1);
 
     if (MPI_REPLACE == op) {
-        ptl_handle_md_t result_md_h, origin_md_h;
-        void *result_md_base, *origin_md_base;
         ptl_size_t result_md_offset, origin_md_offset;
 
-        ompi_osc_portals4_get_md(result_addr, module->md_h, &result_md_h, &result_md_base);
-        result_md_offset = ((char*) result_addr - (char*) result_md_base);
-        ompi_osc_portals4_get_md(origin_addr, module->md_h, &origin_md_h, &origin_md_base);
-        origin_md_offset = ((char*) origin_addr - (char*) origin_md_base);
+        result_md_offset = (ptl_size_t) result_addr;
+        origin_md_offset = (ptl_size_t) origin_addr;
 
-        ret = PtlSwap(result_md_h,
+        ret = PtlSwap(module->md_h,
                       result_md_offset,
-                      origin_md_h,
+                      module->md_h,
                       origin_md_offset,
                       length,
                       peer,
@@ -1105,14 +1067,11 @@ ompi_osc_portals4_fetch_and_op(const void *origin_addr,
                       PTL_SWAP,
                       ptl_dt);
     } else if (MPI_NO_OP == op) {
-        ptl_handle_md_t md_h;
-        void *md_base;
         ptl_size_t md_offset;
 
-        ompi_osc_portals4_get_md(result_addr, module->md_h, &md_h, &md_base);
-        md_offset = ((char*) result_addr - (char*) md_base);
+        md_offset = (ptl_size_t) result_addr;
 
-        ret = PtlGet(md_h,
+        ret = PtlGet(module->md_h,
                      md_offset,
                      length,
                      peer,
@@ -1121,21 +1080,17 @@ ompi_osc_portals4_fetch_and_op(const void *origin_addr,
                      offset,
                      NULL);
     } else {
-        ptl_handle_md_t result_md_h, origin_md_h;
-        void *result_md_base, *origin_md_base;
-        ptl_size_t result_md_offset, origin_md_offset; 
+        ptl_size_t result_md_offset, origin_md_offset;
 
         ret = ompi_osc_portals4_get_op(op, &ptl_op);
         if (OMPI_SUCCESS != ret) return ret;
 
-        ompi_osc_portals4_get_md(result_addr, module->md_h, &result_md_h, &result_md_base);
-        result_md_offset = ((char*) result_addr - (char*) result_md_base);
-        ompi_osc_portals4_get_md(origin_addr, module->md_h, &origin_md_h, &origin_md_base);
-        origin_md_offset = ((char*) origin_addr - (char*) origin_md_base);
+        result_md_offset = (ptl_size_t) result_addr;
+        origin_md_offset = (ptl_size_t) origin_addr;
 
-        ret = PtlFetchAtomic(result_md_h,
+        ret = PtlFetchAtomic(module->md_h,
                              result_md_offset,
-                             origin_md_h,
+                             module->md_h,
                              origin_md_offset,
                              length,
                              peer,
