@@ -8,12 +8,13 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2014 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2007-2016 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2009-2011 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2012-2013 Sandia National Laboratories.  All rights reserved.
  * Copyright (c) 2014-2015 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,23 +22,19 @@
  * $HEADER$
  */
 
-#include "ompi_config.h"
-
 #include "osc_pt2pt.h"
 #include "osc_pt2pt_header.h"
 #include "osc_pt2pt_data_move.h"
 #include "osc_pt2pt_frag.h"
 #include "osc_pt2pt_request.h"
 
-#include "opal/threads/condition.h"
-#include "opal/threads/mutex.h"
 #include "opal/util/arch.h"
-#include "opal/util/output.h"
 #include "opal/sys/atomic.h"
 #include "opal/align.h"
 
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/pml/base/pml_base_sendreq.h"
+#include "opal/mca/btl/btl.h"
 #include "ompi/mca/osc/base/osc_base_obj_convert.h"
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/op/op.h"
@@ -80,10 +77,6 @@ static void osc_pt2pt_accumulate_data_destructor (osc_pt2pt_accumulate_data_t *a
     if (acc_data->datatype) {
         OBJ_RELEASE(acc_data->datatype);
     }
-
-    if (acc_data->op) {
-        OBJ_RELEASE(acc_data->op);
-    }
 }
 
 OBJ_CLASS_DECLARATION(osc_pt2pt_accumulate_data_t);
@@ -215,13 +208,13 @@ static inline int datatype_buffer_length (ompi_datatype_t *datatype, int count)
  * to a target) before this is sent.
  */
 int ompi_osc_pt2pt_control_send (ompi_osc_pt2pt_module_t *module, int target,
-                               void *data, size_t len)
+                                 void *data, size_t len)
 {
     ompi_osc_pt2pt_frag_t *frag;
     char *ptr;
     int ret;
 
-    ret = ompi_osc_pt2pt_frag_alloc(module, target, len, &frag, &ptr);
+    ret = ompi_osc_pt2pt_frag_alloc(module, target, len, &frag, &ptr, false, true);
     if (OPAL_LIKELY(OMPI_SUCCESS == ret)) {
         memcpy (ptr, data, len);
 
@@ -406,7 +399,7 @@ static inline int process_put_long(ompi_osc_pt2pt_module_t* module, int source,
     ret = ompi_osc_pt2pt_component_irecv (module, target,
                                          put_header->count,
                                          datatype, source,
-                                         put_header->tag,
+                                         tag_to_target(put_header->tag),
                                          module->comm);
     if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
         OPAL_OUTPUT_VERBOSE((1, ompi_osc_base_framework.framework_output,
@@ -491,6 +484,7 @@ static int osc_pt2pt_get_post_send (ompi_osc_pt2pt_module_t *module, void *sourc
                                    ompi_datatype_t *datatype, int peer, int tag)
 {
     struct osc_pt2pt_get_post_send_cb_data_t *data;
+    int ret;
 
     data = malloc (sizeof (*data));
     if (OPAL_UNLIKELY(NULL == data)) {
@@ -502,8 +496,14 @@ static int osc_pt2pt_get_post_send (ompi_osc_pt2pt_module_t *module, void *sourc
      * in an active target epoch) */
     data->peer = (tag & 0x1) ? peer : MPI_PROC_NULL;
 
-    return ompi_osc_pt2pt_isend_w_cb (source, count, datatype, peer, tag, module->comm,
+    /* data will be freed by the callback */
+    ret = ompi_osc_pt2pt_isend_w_cb (source, count, datatype, peer, tag, module->comm,
                                      osc_pt2pt_get_post_send_cb, (void *) data);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
+        free (data);
+    }
+
+    return ret;
 }
 
 /**
@@ -535,7 +535,8 @@ static inline int process_get (ompi_osc_pt2pt_module_t* module, int target,
     }
 
     /* send get data */
-    ret = osc_pt2pt_get_post_send (module, source, get_header->count, datatype, target, get_header->tag);
+    ret = osc_pt2pt_get_post_send (module, source, get_header->count, datatype,
+                                  target, tag_to_origin(get_header->tag));
 
     OBJ_RELEASE(datatype);
 
@@ -568,7 +569,7 @@ static inline int osc_pt2pt_accumulate_buffer (void *target, void *source, size_
     }
 
 #if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
-    if (proc->proc_arch != ompi_proc_local()->proc_arch) {
+    if (proc->super.proc_arch != ompi_proc_local()->super.proc_arch) {
         ompi_datatype_t *primitive_datatype = NULL;
         uint32_t primitive_count;
         size_t buflen;
@@ -641,7 +642,6 @@ static int osc_pt2pt_accumulate_allocate (ompi_osc_pt2pt_module_t *module, int p
     acc_data->datatype = datatype;
     OBJ_RETAIN(datatype);
     acc_data->op = op;
-    OBJ_RETAIN(op);
     acc_data->request_count = request_count;
 
     *acc_data_out = acc_data;
@@ -709,7 +709,7 @@ static int accumulate_cb (ompi_request_t *request)
 static int ompi_osc_pt2pt_acc_op_queue (ompi_osc_pt2pt_module_t *module, ompi_osc_pt2pt_header_t *header, int source,
                                        char *data, size_t data_len, ompi_datatype_t *datatype)
 {
-    ompi_osc_pt2pt_peer_t *peer = module->peers + source;
+    ompi_osc_pt2pt_peer_t *peer = ompi_osc_pt2pt_peer_lookup (module, source);
     osc_pt2pt_pending_acc_t *pending_acc;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
@@ -810,8 +810,6 @@ static int ompi_osc_pt2pt_acc_start (ompi_osc_pt2pt_module_t *module, int source
     ret = osc_pt2pt_accumulate_buffer (target, data, data_len, proc, acc_header->count,
                                       datatype, op);
 
-    OBJ_RELEASE(op);
-
     ompi_osc_pt2pt_accumulate_unlock (module);
 
     return ret;
@@ -851,9 +849,9 @@ static int ompi_osc_pt2pt_acc_long_start (ompi_osc_pt2pt_module_t *module, int s
 
     do {
         if (op == &ompi_mpi_op_replace.op) {
-            ret = ompi_osc_pt2pt_irecv_w_cb (target, acc_header->count, datatype, source,
-                                            acc_header->tag, module->comm, NULL,
-                                            replace_cb, module);
+            ret = ompi_osc_pt2pt_irecv_w_cb (target, acc_header->count, datatype,
+                                            source, tag_to_target(acc_header->tag), module->comm,
+                                            NULL, replace_cb, module);
             break;
         }
 
@@ -874,21 +872,20 @@ static int ompi_osc_pt2pt_acc_long_start (ompi_osc_pt2pt_module_t *module, int s
         }
 
         ret = osc_pt2pt_accumulate_allocate (module, source, target, buffer, buflen, proc, acc_header->count,
-                                            datatype, op, 1, &acc_data);
+                                             datatype, op, 1, &acc_data);
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             free (buffer);
             break;
         }
 
-        ret = ompi_osc_pt2pt_irecv_w_cb (buffer, primitive_count, primitive_datatype, source,
-                                        acc_header->tag, module->comm, NULL, accumulate_cb, acc_data);
+        ret = ompi_osc_pt2pt_irecv_w_cb (buffer, primitive_count, primitive_datatype,
+                                        source, tag_to_target(acc_header->tag), module->comm,
+                                        NULL, accumulate_cb, acc_data);
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             OBJ_RELEASE(acc_data);
         }
     } while (0);
 
-    OBJ_RELEASE(op);
-
     if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
         ompi_osc_pt2pt_accumulate_unlock (module);
     }
@@ -931,15 +928,14 @@ static int ompi_osc_pt2pt_gacc_start (ompi_osc_pt2pt_module_t *module, int sourc
             break;
         }
 
-        ret = ompi_osc_pt2pt_isend_w_cb (target, acc_header->count, datatype, source, acc_header->tag,
-                                        module->comm, accumulate_cb, acc_data);
+        ret = ompi_osc_pt2pt_isend_w_cb (target, acc_header->count, datatype,
+                                        source, tag_to_origin(acc_header->tag), module->comm,
+                                        accumulate_cb, acc_data);
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             OBJ_RELEASE(acc_data);
         }
     } while (0);
 
-    OBJ_RELEASE(op);
-
     if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
         ompi_osc_pt2pt_accumulate_unlock (module);
     }
@@ -982,12 +978,6 @@ static int ompi_osc_gacc_long_start (ompi_osc_pt2pt_module_t *module, int source
     buflen = datatype_buffer_length (datatype, acc_header->count);
 
     do {
-        buffer = malloc (buflen);
-        if (OPAL_UNLIKELY(NULL == buffer)) {
-            ret = OMPI_ERR_OUT_OF_RESOURCE;
-            break;
-        }
-
         ret = ompi_osc_base_get_primitive_type_info (datatype, &primitive_datatype, &primitive_count);
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             break;
@@ -995,21 +985,30 @@ static int ompi_osc_gacc_long_start (ompi_osc_pt2pt_module_t *module, int source
 
         primitive_count *= acc_header->count;
 
-        ret = osc_pt2pt_accumulate_allocate (module, source, target, buffer, buflen, proc, acc_header->count,
-                                            datatype, op, 2, &acc_data);
-        if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
+        buffer = malloc (buflen);
+        if (OPAL_UNLIKELY(NULL == buffer)) {
+            ret = OMPI_ERR_OUT_OF_RESOURCE;
             break;
         }
 
-        ret = ompi_osc_pt2pt_irecv_w_cb (buffer, acc_header->count, datatype, source, acc_header->tag,
-                                        module->comm, &recv_request, accumulate_cb, acc_data);
+        ret = osc_pt2pt_accumulate_allocate (module, source, target, buffer, buflen, proc, acc_header->count,
+                                             datatype, op, 2, &acc_data);
+        if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
+            free (buffer);
+            break;
+        }
+
+        ret = ompi_osc_pt2pt_irecv_w_cb (buffer, acc_header->count, datatype,
+                                        source, tag_to_target(acc_header->tag), module->comm,
+                                        &recv_request, accumulate_cb, acc_data);
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             OBJ_RELEASE(acc_data);
             break;
         }
 
-        ret = ompi_osc_pt2pt_isend_w_cb (target, primitive_count, primitive_datatype, source, acc_header->tag,
-                                        module->comm, accumulate_cb, acc_data);
+        ret = ompi_osc_pt2pt_isend_w_cb (target, primitive_count, primitive_datatype,
+                                        source, tag_to_origin(acc_header->tag), module->comm,
+                                        accumulate_cb, acc_data);
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             /* cancel the receive and free the accumulate data */
             ompi_request_cancel (recv_request);
@@ -1018,8 +1017,6 @@ static int ompi_osc_gacc_long_start (ompi_osc_pt2pt_module_t *module, int source
         }
     } while (0);
 
-    OBJ_RELEASE(op);
-
     if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
         ompi_osc_pt2pt_accumulate_unlock (module);
     }
@@ -1063,8 +1060,8 @@ static int ompi_osc_pt2pt_cswap_start (ompi_osc_pt2pt_module_t *module, int sour
 
     do {
         /* no reason to do a non-blocking send here */
-        ret = MCA_PML_CALL(send(target, 1, datatype, source, cswap_header->tag, MCA_PML_BASE_SEND_STANDARD,
-                                module->comm));
+        ret = MCA_PML_CALL(send(target, 1, datatype, source, tag_to_origin(cswap_header->tag),
+                                MCA_PML_BASE_SEND_STANDARD, module->comm));
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             break;
         }
@@ -1334,20 +1331,8 @@ static inline int process_cswap (ompi_osc_pt2pt_module_t *module, int source,
 static inline int process_complete (ompi_osc_pt2pt_module_t *module, int source,
                                     ompi_osc_pt2pt_header_complete_t *complete_header)
 {
-    OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
-                         "osc pt2pt:  process_complete got complete message from %d. expected fragment count %d. "
-                         "current signal count %d. current incomming count: %d",
-                         source, complete_header->frag_count, module->active_incoming_frag_signal_count,
-                         module->active_incoming_frag_count));
-
     /* the current fragment is not part of the frag_count so we need to add it here */
-    OPAL_THREAD_ADD32((int32_t *) &module->active_incoming_frag_signal_count,
-                      complete_header->frag_count + 1);
-
-
-    if (0 == OPAL_THREAD_ADD32((int32_t *) &module->num_complete_msgs, 1)) {
-        opal_condition_broadcast (&module->cond);
-    }
+    osc_pt2pt_incoming_complete (module, source, complete_header->frag_count + 1);
 
     return sizeof (*complete_header);
 }
@@ -1358,7 +1343,7 @@ static inline int process_complete (ompi_osc_pt2pt_module_t *module, int source,
 static inline int process_flush (ompi_osc_pt2pt_module_t *module, int source,
                                  ompi_osc_pt2pt_header_flush_t *flush_header)
 {
-    ompi_osc_pt2pt_peer_t *peer = module->peers + source;
+    ompi_osc_pt2pt_peer_t *peer = ompi_osc_pt2pt_peer_lookup (module, source);
     int ret;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
@@ -1392,7 +1377,7 @@ static inline int process_flush (ompi_osc_pt2pt_module_t *module, int source,
 static inline int process_unlock (ompi_osc_pt2pt_module_t *module, int source,
                                   ompi_osc_pt2pt_header_unlock_t *unlock_header)
 {
-    ompi_osc_pt2pt_peer_t *peer = module->peers + source;
+    ompi_osc_pt2pt_peer_t *peer = ompi_osc_pt2pt_peer_lookup (module, source);
     int ret;
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
@@ -1524,8 +1509,9 @@ static int process_large_datatype_request (ompi_osc_pt2pt_module_t *module, int
     memcpy (ddt_buffer->header, header, header_len);
 
     ret = ompi_osc_pt2pt_irecv_w_cb ((void *)((uintptr_t) ddt_buffer->header + header_len),
-                                    ddt_len, MPI_BYTE, source, tag, module->comm, NULL,
-                                    process_large_datatype_request_cb, ddt_buffer);
+                                    ddt_len, MPI_BYTE,
+                                    source, tag_to_target(tag), module->comm,
+                                    NULL, process_large_datatype_request_cb, ddt_buffer);
     if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
         OBJ_RELEASE(ddt_buffer);
         return ret;
@@ -1636,8 +1622,6 @@ static int ompi_osc_pt2pt_callback (ompi_request_t *request)
     size_t incoming_length = request->req_status._ucount;
     int source = request->req_status.MPI_SOURCE;
 
-    OPAL_THREAD_UNLOCK(&ompi_request_lock);
-
     assert(incoming_length >= sizeof(ompi_osc_pt2pt_header_base_t));
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
@@ -1650,10 +1634,11 @@ static int ompi_osc_pt2pt_callback (ompi_request_t *request)
         process_frag(module, (ompi_osc_pt2pt_frag_header_t *) base_header);
 
         /* only data fragments should be included in the completion counters */
-        mark_incoming_completion (module, (base_header->base.flags & OMPI_OSC_PT2PT_HDR_FLAG_PASSIVE_TARGET) ? source : MPI_PROC_NULL);
+        mark_incoming_completion (module, (base_header->base.flags & OMPI_OSC_PT2PT_HDR_FLAG_PASSIVE_TARGET) ?
+                                  source : MPI_PROC_NULL);
         break;
     case OMPI_OSC_PT2PT_HDR_TYPE_POST:
-        (void) osc_pt2pt_incoming_post (module, source);
+        osc_pt2pt_incoming_post (module, source);
         break;
     case OMPI_OSC_PT2PT_HDR_TYPE_LOCK_ACK:
         ompi_osc_pt2pt_process_lock_ack(module, (ompi_osc_pt2pt_header_lock_ack_t *) base_header);
@@ -1675,11 +1660,10 @@ static int ompi_osc_pt2pt_callback (ompi_request_t *request)
 
     osc_pt2pt_gc_clean (module);
 
-    /* put this request on the garbage colletion list */
-    osc_pt2pt_gc_add_request (module, request);
     ompi_osc_pt2pt_frag_start_receive (module);
 
-    OPAL_THREAD_LOCK(&ompi_request_lock);
+    /* put this request on the garbage colletion list */
+    osc_pt2pt_gc_add_request (module, request);
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "finished posting receive request"));
@@ -1689,6 +1673,7 @@ static int ompi_osc_pt2pt_callback (ompi_request_t *request)
 
 int ompi_osc_pt2pt_frag_start_receive (ompi_osc_pt2pt_module_t *module)
 {
+    module->frag_request = MPI_REQUEST_NULL;
     return ompi_osc_pt2pt_irecv_w_cb (module->incoming_buffer, mca_osc_pt2pt_component.buffer_size + sizeof (ompi_osc_pt2pt_frag_header_t),
                                      MPI_BYTE, OMPI_ANY_SOURCE, OSC_PT2PT_FRAG_TAG, module->comm, &module->frag_request,
                                      ompi_osc_pt2pt_callback, module);
@@ -1702,33 +1687,6 @@ int ompi_osc_pt2pt_component_irecv (ompi_osc_pt2pt_module_t *module, void *buf,
                                      osc_pt2pt_incoming_req_complete, module);
 }
 
-
-static int
-isend_completion_cb(ompi_request_t *request)
-{
-    ompi_osc_pt2pt_module_t *module =
-        (ompi_osc_pt2pt_module_t*) request->req_complete_cb_data;
-
-    OPAL_OUTPUT_VERBOSE((10, ompi_osc_base_framework.framework_output,
-                         "isend_completion_cb called"));
-
-    mark_outgoing_completion(module);
-
-    /* put this request on the garbage colletion list */
-    osc_pt2pt_gc_add_request (module, request);
-
-    return OMPI_SUCCESS;
-}
-
-
-int ompi_osc_pt2pt_component_isend (ompi_osc_pt2pt_module_t *module, const void *buf,
-                                    size_t count, struct ompi_datatype_t *datatype,
-                                    int dest, int tag, struct ompi_communicator_t *comm)
-{
-    return ompi_osc_pt2pt_isend_w_cb (buf, count, datatype, dest, tag, comm,
-                                     isend_completion_cb, module);
-}
-
 int ompi_osc_pt2pt_isend_w_cb (const void *ptr, int count, ompi_datatype_t *datatype, int target, int tag,
                               ompi_communicator_t *comm, ompi_request_complete_fn_t cb, void *ctx)
 {
@@ -1775,11 +1733,14 @@ int ompi_osc_pt2pt_irecv_w_cb (void *ptr, int count, ompi_datatype_t *datatype,
 
     request->req_complete_cb = cb;
     request->req_complete_cb_data = ctx;
-    if (request_out) {
+
+    ret = MCA_PML_CALL(start(1, &request));
+    if (request_out && MPI_REQUEST_NULL != request) {
         *request_out = request;
     }
 
-    ret = MCA_PML_CALL(start(1, &request));
+    OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
+                         "osc pt2pt: pml start returned %d. state: %d", ret, request->req_state));
 
     return ret;
 }
