@@ -3,10 +3,10 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2009 The University of Tennessee and The University
+ * Copyright (c) 2004-2016 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -15,9 +15,9 @@
  * Copyright (c) 2013-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -33,13 +33,15 @@
 #include "pml_ob1_recvreq.h"
 #include "pml_ob1_rdmafrag.h"
 #include "pml_ob1_recvfrag.h"
-#include "ompi/mca/bml/base/base.h" 
+#include "ompi/mca/bml/base/base.h"
 #include "pml_ob1_component.h"
-#include "ompi/mca/allocator/base/base.h"
+#include "opal/mca/allocator/base/base.h"
 #include "opal/mca/base/mca_base_pvar.h"
+#include "opal/runtime/opal_params.h"
+#include "opal/mca/btl/base/base.h"
 
 OBJ_CLASS_INSTANCE( mca_pml_ob1_pckt_pending_t,
-                    ompi_free_list_item_t,
+                    opal_free_list_item_t,
                     NULL,
                     NULL );
 
@@ -52,38 +54,36 @@ mca_pml_ob1_component_init( int* priority, bool enable_progress_threads,
 static int mca_pml_ob1_component_fini(void);
 int mca_pml_ob1_output = 0;
 static int mca_pml_ob1_verbose = 0;
+bool mca_pml_ob1_matching_protection = false;
 
 mca_pml_base_component_2_0_0_t mca_pml_ob1_component = {
-
     /* First, the mca_base_component_t struct containing meta
        information about the component itself */
 
-    {
-      MCA_PML_BASE_VERSION_2_0_0,
-    
-      "ob1", /* MCA component name */
-      OMPI_MAJOR_VERSION,  /* MCA component major version */
-      OMPI_MINOR_VERSION,  /* MCA component minor version */
-      OMPI_RELEASE_VERSION,  /* MCA component release version */
-      mca_pml_ob1_component_open,  /* component open */
-      mca_pml_ob1_component_close, /* component close */
-      NULL,
-      mca_pml_ob1_component_register
+    .pmlm_version = {
+        MCA_PML_BASE_VERSION_2_0_0,
+
+        .mca_component_name = "ob1",
+        .mca_component_major_version = OMPI_MAJOR_VERSION,
+        .mca_component_minor_version = OMPI_MINOR_VERSION,
+        .mca_component_release_version = OMPI_RELEASE_VERSION,
+        .mca_open_component = mca_pml_ob1_component_open,
+        .mca_close_component = mca_pml_ob1_component_close,
+        .mca_register_component_params = mca_pml_ob1_component_register
     },
-    {
+    .pmlm_data = {
         /* The component is checkpoint ready */
         MCA_BASE_METADATA_PARAM_CHECKPOINT
     },
 
-    mca_pml_ob1_component_init,  /* component init */
-    mca_pml_ob1_component_fini   /* component finalize */
-    
+    .pmlm_init = mca_pml_ob1_component_init,
+    .pmlm_finalize = mca_pml_ob1_component_fini,
 };
 
 void *mca_pml_ob1_seg_alloc( struct mca_mpool_base_module_t* mpool,
                              size_t* size,
                              mca_mpool_base_registration_t** registration);
- 
+
 void mca_pml_ob1_seg_free( struct mca_mpool_base_module_t* mpool,
                            void* segment );
 
@@ -146,9 +146,12 @@ static int mca_pml_ob1_get_unex_msgq_size (const struct mca_base_pvar_t *pvar, v
     int i;
 
     for (i = 0 ; i < comm_size ; ++i) {
-        pml_proc = pml_comm->procs + i;
-
-        values[i] = opal_list_get_size (&pml_proc->unexpected_frags);
+        pml_proc = pml_comm->procs[i];
+        if (pml_proc) {
+            values[i] = opal_list_get_size (&pml_proc->unexpected_frags);
+        } else {
+            values[i] = 0;
+        }
     }
 
     return OMPI_SUCCESS;
@@ -164,9 +167,13 @@ static int mca_pml_ob1_get_posted_recvq_size (const struct mca_base_pvar_t *pvar
     int i;
 
     for (i = 0 ; i < comm_size ; ++i) {
-        pml_proc = pml_comm->procs + i;
+        pml_proc = pml_comm->procs[i];
 
-        values[i] = opal_list_get_size (&pml_proc->specific_receives);
+        if (pml_proc) {
+            values[i] = opal_list_get_size (&pml_proc->specific_receives);
+        } else {
+            values[i] = 0;
+        }
     }
 
     return OMPI_SUCCESS;
@@ -193,7 +200,7 @@ static int mca_pml_ob1_component_register(void)
     mca_pml_ob1_param_register_int("max_send_per_range", 4, &mca_pml_ob1.max_send_per_range);
 
     mca_pml_ob1_param_register_uint("unexpected_limit", 128, &mca_pml_ob1.unexpected_limit);
- 
+
     mca_pml_ob1.allocator_name = "bucket";
     (void) mca_base_component_var_register(&mca_pml_ob1_component.pmlm_version, "allocator",
                                            "Name of allocator component for unexpected messages",
@@ -220,7 +227,7 @@ static int mca_pml_ob1_component_open(void)
     mca_pml_ob1_output = opal_output_open(NULL);
     opal_output_set_verbosity(mca_pml_ob1_output, mca_pml_ob1_verbose);
 
-    mca_pml_ob1.enabled = false; 
+    mca_pml_ob1.enabled = false;
     return mca_base_framework_open(&ompi_bml_base_framework, 0);
 }
 
@@ -239,7 +246,7 @@ static int mca_pml_ob1_component_close(void)
 
 
 static mca_pml_base_module_t*
-mca_pml_ob1_component_init( int* priority, 
+mca_pml_ob1_component_init( int* priority,
                             bool enable_progress_threads,
                             bool enable_mpi_threads )
 {
@@ -264,16 +271,32 @@ mca_pml_ob1_component_init( int* priority,
         return NULL;
     }
 
-    if(OMPI_SUCCESS != mca_bml_base_init( enable_progress_threads, 
+    if(OMPI_SUCCESS != mca_bml_base_init( enable_progress_threads,
                                           enable_mpi_threads)) {
         return NULL;
     }
 
+    /* check if any btls do not support dynamic add_procs */
+    mca_btl_base_selected_module_t* selected_btl;
+    OPAL_LIST_FOREACH(selected_btl, &mca_btl_base_modules_initialized, mca_btl_base_selected_module_t) {
+        mca_btl_base_module_t *btl = selected_btl->btl_module;
+
+        if (btl->btl_flags & MCA_BTL_FLAGS_BTL_PROGRESS_THREAD_ENABLED) {
+            mca_pml_ob1_matching_protection = true;
+        }
+
+        if (btl->btl_flags & MCA_BTL_FLAGS_SINGLE_ADD_PROCS) {
+            mca_pml_ob1.super.pml_flags |= MCA_PML_BASE_FLAG_REQUIRE_WORLD;
+            break;
+        }
+
+    }
+
     /* Set this here (vs in component_open()) because
-       ompi_mpi_leave_pinned* may have been set after MCA params were
+       opal_leave_pinned* may have been set after MCA params were
        read (e.g., by the openib btl) */
-    mca_pml_ob1.leave_pinned = (1 == ompi_mpi_leave_pinned);
-    mca_pml_ob1.leave_pinned_pipeline = (int) ompi_mpi_leave_pinned_pipeline;
+    mca_pml_ob1.leave_pinned = (1 == opal_leave_pinned);
+    mca_pml_ob1.leave_pinned_pipeline = (int) opal_leave_pinned_pipeline;
 
     return &mca_pml_ob1.super;
 }
@@ -287,18 +310,18 @@ int mca_pml_ob1_component_fini(void)
         return rc;
 
     if(!mca_pml_ob1.enabled)
-        return OMPI_SUCCESS; /* never selected.. return success.. */  
+        return OMPI_SUCCESS; /* never selected.. return success.. */
     mca_pml_ob1.enabled = false;  /* not anymore */
 
     /* return the static receive/send requests to the respective free list and
      * let the free list handle destruction. */
     if( NULL != mca_pml_ob1_recvreq ) {
-        OMPI_FREE_LIST_RETURN_MT (&mca_pml_base_recv_requests, (ompi_free_list_item_t *) mca_pml_ob1_recvreq);
+        opal_free_list_return (&mca_pml_base_recv_requests, (opal_free_list_item_t *) mca_pml_ob1_recvreq);
         mca_pml_ob1_recvreq = NULL;
     }
 
     if( NULL != mca_pml_ob1_sendreq ) {
-        OMPI_FREE_LIST_RETURN_MT (&mca_pml_base_send_requests, (ompi_free_list_item_t *) mca_pml_ob1_sendreq);
+        opal_free_list_return (&mca_pml_base_send_requests, (opal_free_list_item_t *) mca_pml_ob1_sendreq);
         mca_pml_ob1_sendreq = NULL;
     }
 
@@ -339,11 +362,11 @@ int mca_pml_ob1_component_fini(void)
 
 void *mca_pml_ob1_seg_alloc( struct mca_mpool_base_module_t* mpool,
                              size_t* size,
-                             mca_mpool_base_registration_t** registration) { 
+                             mca_mpool_base_registration_t** registration) {
     return malloc(*size);
 }
 
 void mca_pml_ob1_seg_free( struct mca_mpool_base_module_t* mpool,
-                           void* segment ) { 
+                           void* segment ) {
     free(segment);
 }
