@@ -2,19 +2,22 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2013 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2009 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2009 University of Houston.  All rights reserved.
+ * Copyright (c) 2012-2013 Inria.  All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -28,21 +31,18 @@
 #include "ompi/proc/proc.h"
 #include "ompi/memchecker.h"
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_Intercomm_merge = PMPI_Intercomm_merge
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/c/profile/defines.h"
+#define MPI_Intercomm_merge PMPI_Intercomm_merge
 #endif
 
-#define INTERCOMM_MERGE_TAG 1010
-
 static const char FUNC_NAME[] = "MPI_Intercomm_merge";
 
 
 int MPI_Intercomm_merge(MPI_Comm intercomm, int high,
-                        MPI_Comm *newcomm) 
+                        MPI_Comm *newcomm)
 {
     ompi_communicator_t *newcomp=MPI_COMM_NULL;
     ompi_proc_t **procs=NULL;
@@ -52,27 +52,24 @@ int MPI_Intercomm_merge(MPI_Comm intercomm, int high,
     int rc=MPI_SUCCESS;
     int thigh = high;
     ompi_group_t *new_group_pointer;
-    
 
     MEMCHECKER(
         memchecker_comm(intercomm);
     );
 
     if ( MPI_PARAM_CHECK ) {
-        OMPI_ERR_INIT_FINALIZE(FUNC_NAME); 
+        OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
         if (ompi_comm_invalid ( intercomm ) ||
-             !( intercomm->c_flags & OMPI_COMM_INTER ) ) 
+             !( intercomm->c_flags & OMPI_COMM_INTER ) )
             return OMPI_ERRHANDLER_INVOKE ( MPI_COMM_WORLD, MPI_ERR_COMM,
                                             FUNC_NAME);
 
         if ( NULL == newcomm )
-            return OMPI_ERRHANDLER_INVOKE ( intercomm, MPI_ERR_ARG, 
+            return OMPI_ERRHANDLER_INVOKE ( intercomm, MPI_ERR_ARG,
                                             FUNC_NAME);
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     local_size  = ompi_comm_size ( intercomm );
     remote_size = ompi_comm_remote_size ( intercomm );
     total_size  = local_size + remote_size;
@@ -81,7 +78,7 @@ int MPI_Intercomm_merge(MPI_Comm intercomm, int high,
         rc = MPI_ERR_INTERN;
         goto exit;
     }
-    
+
     first = ompi_comm_determine_first ( intercomm, thigh );
     if ( MPI_UNDEFINED == first ) {
         rc = MPI_ERR_INTERN;
@@ -103,7 +100,7 @@ int MPI_Intercomm_merge(MPI_Comm intercomm, int high,
                          NULL,                     /* remote_procs */
                          NULL,                     /* attrs */
                          intercomm->error_handler, /* error handler*/
-                         NULL,                     /* topo mpodule */
+                         false,                    /* don't copy the topo */
                          new_group_pointer,        /* local group */
                          NULL                      /* remote group */
                          );
@@ -115,12 +112,11 @@ int MPI_Intercomm_merge(MPI_Comm intercomm, int high,
         goto exit;
     }
 
-    ompi_group_decrement_proc_count(new_group_pointer);
     OBJ_RELEASE(new_group_pointer);
     new_group_pointer = MPI_GROUP_NULL;
 
     /* Determine context id. It is identical to f_2_c_handle */
-    rc = ompi_comm_nextcid ( newcomp,              /* new comm */ 
+    rc = ompi_comm_nextcid ( newcomp,              /* new comm */
                              intercomm,            /* old comm */
                              NULL,                 /* bridge comm */
                              NULL,                 /* local leader */
@@ -132,7 +128,7 @@ int MPI_Intercomm_merge(MPI_Comm intercomm, int high,
     }
 
     /* activate communicator and init coll-module */
-    rc = ompi_comm_activate( &newcomp,             /* new comm */ 
+    rc = ompi_comm_activate( &newcomp,             /* new comm */
                              intercomm,            /* old comm */
                              NULL,                 /* bridge comm */
                              NULL,                 /* local leader */
@@ -144,7 +140,6 @@ int MPI_Intercomm_merge(MPI_Comm intercomm, int high,
     }
 
  exit:
-    OPAL_CR_EXIT_LIBRARY();
 
     if ( NULL != procs ) {
         free ( procs );
