@@ -5,19 +5,19 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights
- *                         reserved. 
- * Copyright (c) 2014      Research Organization for Information Science
+ *                         reserved.
+ * Copyright (c) 2014-2015 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 #include "ompi_config.h"
@@ -31,18 +31,17 @@
 #include "ompi/request/request.h"
 #include "ompi/memchecker.h"
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_Startall = PMPI_Startall
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/c/profile/defines.h"
+#define MPI_Startall PMPI_Startall
 #endif
 
 static const char FUNC_NAME[] = "MPI_Startall";
 
 
-int MPI_Startall(int count, MPI_Request requests[]) 
+int MPI_Startall(int count, MPI_Request requests[])
 {
     int i;
     int ret = OMPI_SUCCESS;
@@ -74,7 +73,6 @@ int MPI_Startall(int count, MPI_Request requests[])
         OMPI_ERRHANDLER_CHECK(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
     }
 
-    OPAL_CR_ENTER_LIBRARY();
 
     for (i = 0; i < count; ++i) {
         if (OMPI_REQUEST_NOOP == requests[i]->req_type) {
@@ -89,9 +87,9 @@ int MPI_Startall(int count, MPI_Request requests[])
             }
         }
     }
+
     ret = MCA_PML_CALL(start(count, requests));
 
-    OPAL_CR_EXIT_LIBRARY();
     return ret;
 }
 
