@@ -1,12 +1,12 @@
 // -*- c++ -*-
-// 
+//
 // Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
 //                         University Research and Technology
 //                         Corporation.  All rights reserved.
 // Copyright (c) 2004-2005 The University of Tennessee and The University
 //                         of Tennessee Research Foundation.  All rights
 //                         reserved.
-// Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+// Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
 //                         University of Stuttgart.  All rights reserved.
 // Copyright (c) 2004-2005 The Regents of the University of California.
 //                         All rights reserved.
@@ -14,9 +14,9 @@
 // Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
 // Copyright (c) 2011      FUJITSU LIMITED.  All rights reserved.
 // $COPYRIGHT$
-// 
+//
 // Additional copyrights may follow
-// 
+//
 // $HEADER$
 
 #include "mpicxx.h"
@@ -74,7 +74,7 @@ const Datatype DOUBLE_INT(MPI_DOUBLE_INT);
 const Datatype LONG_INT(MPI_LONG_INT);
 const Datatype TWOINT(MPI_2INT);
 const Datatype SHORT_INT(MPI_SHORT_INT);
-const Datatype LONG_DOUBLE_INT(MPI_LONG_DOUBLE);
+const Datatype LONG_DOUBLE_INT(MPI_LONG_DOUBLE_INT);
 
 #if OMPI_BUILD_FORTRAN_BINDINGS
 // elementary datatype (Fortran)
@@ -151,7 +151,7 @@ Comm_Null    COMM_NULL;
 const Datatype     DATATYPE_NULL = MPI_DATATYPE_NULL;
 Request      REQUEST_NULL = MPI_REQUEST_NULL;
 const Op           OP_NULL = MPI_OP_NULL;
-const Errhandler   ERRHANDLER_NULL;  
+const Errhandler   ERRHANDLER_NULL;
 #if OMPI_PROVIDE_MPI_FILE_INTERFACE
 const File FILE_NULL = MPI_FILE_NULL;
 #endif
