@@ -6,22 +6,25 @@
  * Copyright (c) 2004-2014 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2015 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ *                         reserved.
  * Copyright (c) 2006-2009 University of Houston. All rights reserved.
  * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
- * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2012-2013 Inria.  All rights reserved.
+ * Copyright (c) 2014-2015 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2015 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  *
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -51,12 +54,18 @@
 #include "opal/util/show_help.h"
 #include "opal/runtime/opal.h"
 #include "opal/mca/event/event.h"
+#include "opal/mca/allocator/base/base.h"
+#include "opal/mca/rcache/base/base.h"
+#include "opal/mca/rcache/rcache.h"
+#include "opal/mca/mpool/base/base.h"
+#include "opal/mca/btl/base/base.h"
+#include "opal/mca/pmix/pmix.h"
+#include "opal/util/timings.h"
 
 #include "ompi/constants.h"
 #include "ompi/mpi/fortran/base/constants.h"
 #include "ompi/runtime/mpiruntime.h"
 #include "ompi/runtime/params.h"
-#include "ompi/runtime/ompi_module_exchange.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/info/info.h"
 #include "ompi/errhandler/errcode.h"
@@ -68,10 +77,6 @@
 #include "ompi/mca/op/base/base.h"
 #include "ompi/file/file.h"
 #include "ompi/attribute/attribute.h"
-#include "ompi/mca/allocator/base/base.h"
-#include "ompi/mca/rcache/base/base.h"
-#include "ompi/mca/rcache/rcache.h"
-#include "ompi/mca/mpool/base/base.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/bml/bml.h"
 #include "ompi/mca/pml/base/base.h"
@@ -85,25 +90,16 @@
 #include "ompi/debuggers/debuggers.h"
 #include "ompi/proc/proc.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
-#include "ompi/mca/dpm/base/base.h"
-#include "ompi/mca/pubsub/base/base.h"
+#include "ompi/dpm/dpm.h"
 #include "ompi/mpiext/mpiext.h"
 
-#if OPAL_ENABLE_FT_CR == 1
-#include "ompi/mca/crcp/crcp.h"
-#include "ompi/mca/crcp/base/base.h"
-#endif
-#include "ompi/runtime/ompi_cr.h"
-
-#if defined(MEMORY_LINUX_PTMALLOC2) && MEMORY_LINUX_PTMALLOC2
-#include "opal/mca/memory/linux/memory_linux.h"
+#include "opal/mca/memory/base/base.h"
 /* So this sucks, but with OPAL in its own library that is brought in
    implicity from libmpi, there are times when the malloc initialize
    hook in the memory component doesn't work.  So we have to do it
    from here, since any MPI code is going to call MPI_Init... */
-OPAL_DECLSPEC void (*__malloc_initialize_hook) (void) = 
-    opal_memory_linux_malloc_init_hook;
-#endif
+OPAL_DECLSPEC void (*__malloc_initialize_hook) (void) =
+    opal_memory_base_malloc_init_hook;
 
 /* This is required for the boundaries of the hash tables used to store
  * the F90 types returned by the MPI_Type_create_f90_XXX functions.
@@ -121,10 +117,12 @@ const char ompi_version_string[] = OMPI_IDENT_STRING;
  * Global variables and symbols for the MPI layer
  */
 
-bool ompi_mpi_init_started = false;
-bool ompi_mpi_initialized = false;
-bool ompi_mpi_finalized = false;
-int32_t ompi_mpi_finalize_started = 0;
+opal_mutex_t ompi_mpi_bootstrap_mutex = OPAL_MUTEX_STATIC_INIT;
+volatile bool ompi_mpi_init_started = false;
+volatile bool ompi_mpi_initialized = false;
+volatile bool ompi_mpi_finalize_started = false;
+volatile bool ompi_mpi_finalized = false;
+volatile bool ompi_rte_initialized = false;
 
 bool ompi_mpi_thread_multiple = false;
 int ompi_mpi_thread_requested = MPI_THREAD_SINGLE;
@@ -132,8 +130,6 @@ int ompi_mpi_thread_provided = MPI_THREAD_SINGLE;
 
 opal_thread_t *ompi_mpi_main_thread = NULL;
 
-bool ompi_warn_on_fork;
-
 /*
  * These variables are for the MPI F08 bindings (F08 must bind Fortran
  * varaiables to symbols; it cannot bind Fortran variables to the
@@ -171,31 +167,6 @@ struct ompi_status_public_t *ompi_mpi_status_ignore_addr =
 struct ompi_status_public_t *ompi_mpi_statuses_ignore_addr =
     (ompi_status_public_t *) 0;
 
-#if OPAL_HAVE_POSIX_THREADS
-static bool fork_warning_issued = false;
-static bool atfork_called = false;
-
-static void warn_fork_cb(void)
-{
-    if (ompi_mpi_initialized && !ompi_mpi_finalized && !fork_warning_issued) {
-        opal_show_help("help-mpi-runtime.txt", "mpi_init:warn-fork", true,
-                       ompi_process_info.nodename, getpid(),
-                       ompi_mpi_comm_world.comm.c_my_rank);
-        fork_warning_issued = true;
-    }
-}
-#endif
-
-void ompi_warn_fork(void)
-{
-#if OPAL_HAVE_POSIX_THREADS
-    if (ompi_warn_on_fork && !atfork_called) {
-        pthread_atfork(warn_fork_cb, NULL, NULL);
-        atfork_called = true;
-    }
-#endif
-}
-
 /*
  * These variables are here, rather than under ompi/mpi/c/foo.c
  * because it is not sufficient to have a .c file that only contains
@@ -261,7 +232,7 @@ MPI_Fint *MPI_F_STATUSES_IGNORE = NULL;
 
    The values are *NOT* initialized.  We do not use the values of
    these constants; only their addresses (because they're always
-   passed by reference by Fortran).  
+   passed by reference by Fortran).
 
    Initializing upon instantiation these can reveal size and/or
    alignment differences between Fortran and C (!) which can cause
@@ -280,37 +251,14 @@ MPI_Fint *MPI_F_STATUSES_IGNORE = NULL;
    ompi/include/mpif-common.h.
  */
 
-#define INST(type, upper_case, lower_case, single_u, double_u)   \
-type lower_case; \
-type upper_case; \
-type single_u;  \
-type double_u
-
-INST(int, MPI_FORTRAN_BOTTOM, mpi_fortran_bottom,
-     mpi_fortran_bottom_, mpi_fortran_bottom__);
-INST(int, MPI_FORTRAN_IN_PLACE, mpi_fortran_in_place,
-     mpi_fortran_in_place_, mpi_fortran_in_place__);
-INST(int, MPI_FORTRAN_UNWEIGHTED, mpi_fortran_unweighted,
-     mpi_fortran_unweighted_, mpi_fortran_unweighted__);
-INST(int, MPI_FORTRAN_WEIGHTS_EMPTY, mpi_fortran_weights_empty,
-     mpi_fortran_weights_empty_, mpi_fortran_weights_empty__);
-INST(char *, MPI_FORTRAN_ARGV_NULL, mpi_fortran_argv_null,
-     mpi_fortran_argv_null_, mpi_fortran_argv_null__);
-INST(char *, MPI_FORTRAN_ARGVS_NULL, mpi_fortran_argvs_null,
-     mpi_fortran_argvs_null_, mpi_fortran_argvs_null__);
-INST(int *, MPI_FORTRAN_ERRCODES_IGNORE, mpi_fortran_errcodes_ignore,
-     mpi_fortran_errcodes_ignore_, mpi_fortran_errcodes_ignore__);
-INST(int *, MPI_FORTRAN_STATUS_IGNORE, mpi_fortran_status_ignore,
-     mpi_fortran_status_ignore_, mpi_fortran_status_ignore__);
-INST(int *, MPI_FORTRAN_STATUSES_IGNORE, mpi_fortran_statuses_ignore,
-      mpi_fortran_statuses_ignore_, mpi_fortran_statuses_ignore__);
+#include "mpif-c-constants.h"
 
 /*
  * Hash tables for MPI_Type_create_f90* functions
  */
-opal_hash_table_t ompi_mpi_f90_integer_hashtable;
-opal_hash_table_t ompi_mpi_f90_real_hashtable;
-opal_hash_table_t ompi_mpi_f90_complex_hashtable;
+opal_hash_table_t ompi_mpi_f90_integer_hashtable = {{0}};
+opal_hash_table_t ompi_mpi_f90_real_hashtable = {{0}};
+opal_hash_table_t ompi_mpi_f90_complex_hashtable = {{0}};
 
 /*
  * Per MPI-2:9.5.3, MPI_REGISTER_DATAREP is a memory leak.  There is
@@ -319,12 +267,43 @@ opal_hash_table_t ompi_mpi_f90_complex_hashtable;
  * de-registered during MPI_FINALIZE so that memory-tracking debuggers
  * don't show Open MPI as leaking memory.
  */
-opal_list_t ompi_registered_datareps;
+opal_list_t ompi_registered_datareps = {{0}};
 
-bool ompi_enable_timing;
+bool ompi_enable_timing = false, ompi_enable_timing_ext = false;
 extern bool ompi_mpi_yield_when_idle;
 extern int ompi_mpi_event_tick_rate;
 
+/**
+ * Static functions used to configure the interactions between the OPAL and
+ * the runtime.
+ */
+static char*
+_process_name_print_for_opal(const opal_process_name_t procname)
+{
+    ompi_process_name_t* rte_name = (ompi_process_name_t*)&procname;
+    return OMPI_NAME_PRINT(rte_name);
+}
+
+static int
+_process_name_compare(const opal_process_name_t p1, const opal_process_name_t p2)
+{
+    ompi_process_name_t* o1 = (ompi_process_name_t*)&p1;
+    ompi_process_name_t* o2 = (ompi_process_name_t*)&p2;
+    return ompi_rte_compare_name_fields(OMPI_RTE_CMP_ALL, o1, o2);
+}
+
+static int _convert_string_to_process_name(opal_process_name_t *name,
+                                           const char* name_string)
+{
+    return ompi_rte_convert_string_to_process_name(name, name_string);
+}
+
+static int _convert_process_name_to_string(char** name_string,
+                                          const opal_process_name_t *name)
+{
+    return ompi_rte_convert_process_name_to_string(name_string, name);
+}
+
 
 void ompi_mpi_thread_level(int requested, int *provided)
 {
@@ -353,7 +332,7 @@ void ompi_mpi_thread_level(int requested, int *provided)
         ompi_mpi_main_thread = opal_thread_get_self();
     }
 
-    ompi_mpi_thread_multiple = (ompi_mpi_thread_provided == 
+    ompi_mpi_thread_multiple = (ompi_mpi_thread_provided ==
                                 MPI_THREAD_MULTIPLE);
 }
 
@@ -375,42 +354,97 @@ static int ompi_register_mca_variables(void)
                                  MCA_BASE_VAR_SCOPE_READONLY,
                                  &ompi_enable_timing);
 
+    ompi_enable_timing_ext = false;
+    (void) mca_base_var_register("ompi", "ompi", NULL, "timing_ext",
+                                 "Request that critical timing loops be measured",
+                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                 OPAL_INFO_LVL_9,
+                                 MCA_BASE_VAR_SCOPE_READONLY,
+                                 &ompi_enable_timing_ext);
     return OMPI_SUCCESS;
 }
 
+static void fence_release(int status, void *cbdata)
+{
+    volatile bool *active = (volatile bool*)cbdata;
+    *active = false;
+}
+
 int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
 {
     int ret;
     ompi_proc_t** procs;
     size_t nprocs;
     char *error = NULL;
-    struct timeval ompistart, ompistop;
-    bool rte_setup = false;
-    ompi_rte_collective_t *coll;
     char *cmd=NULL, *av=NULL;
+    volatile bool active;
+    OPAL_TIMING_DECLARE(tm);
+    OPAL_TIMING_INIT_EXT(&tm, OPAL_TIMING_GET_TIME_OF_DAY);
 
     /* bitflag of the thread level support provided. To be used
      * for the modex in order to work in heterogeneous environments. */
-    uint8_t threadlevel_bf; 
+    uint8_t threadlevel_bf;
 
-    /* Indicate that we have *started* MPI_INIT*.  MPI_FINALIZE has
-       something sorta similar in a static local variable in
-       ompi_mpi_finalize(). */
+    /* Ensure that we were not already initialized or finalized.
+
+       This lock is held for the duration of ompi_mpi_init() and
+       ompi_mpi_finalize().  Hence, if we get it, then no other thread
+       is inside the critical section (and we don't have to check the
+       *_started bool variables). */
+    opal_mutex_lock(&ompi_mpi_bootstrap_mutex);
+    if (ompi_mpi_finalized) {
+        opal_show_help("help-mpi-runtime.txt",
+                       "mpi_init: already finalized", true);
+        opal_mutex_unlock(&ompi_mpi_bootstrap_mutex);
+        return MPI_ERR_OTHER;
+    } else if (ompi_mpi_initialized) {
+        opal_show_help("help-mpi-runtime.txt",
+                       "mpi_init: invoked multiple times", true);
+        opal_mutex_unlock(&ompi_mpi_bootstrap_mutex);
+        return MPI_ERR_OTHER;
+    }
+
+    /* Indicate that we have *started* MPI_INIT* */
     ompi_mpi_init_started = true;
 
-    /* Setup enough to check get/set MCA params */
+    /* Figure out the final MPI thread levels.  If we were not
+       compiled for support for MPI threads, then don't allow
+       MPI_THREAD_MULTIPLE.  Set this stuff up here early in the
+       process so that other components can make decisions based on
+       this value. */
 
+    ompi_mpi_thread_level(requested, provided);
+
+    /* Setup enough to check get/set MCA params */
     if (OPAL_SUCCESS != (ret = opal_init_util(&argc, &argv))) {
         error = "ompi_mpi_init: opal_init_util failed";
         goto error;
     }
 
+    /* If thread support was enabled, then setup OPAL to allow for them. This must be done
+     * early to prevent a race condition that can occur with orte_init(). */
+    if (*provided != MPI_THREAD_SINGLE) {
+        opal_set_using_threads(true);
+    }
+
+    /* Convince OPAL to use our naming scheme */
+    opal_process_name_print = _process_name_print_for_opal;
+    opal_compare_proc = _process_name_compare;
+    opal_convert_string_to_process_name = _convert_string_to_process_name;
+    opal_convert_process_name_to_string = _convert_process_name_to_string;
+    opal_proc_for_name = ompi_proc_for_name;
+
     /* Register MCA variables */
     if (OPAL_SUCCESS != (ret = ompi_register_mca_variables())) {
         error = "ompi_mpi_init: ompi_register_mca_variables failed";
         goto error;
     }
 
+    if (OPAL_SUCCESS != (ret = opal_arch_set_fortran_logical_size(sizeof(ompi_fortran_logical_t)))) {
+        error = "ompi_mpi_init: opal_arch_set_fortran_logical_size failed";
+        goto error;
+    }
+
     /* _After_ opal_init_util() but _before_ orte_init(), we need to
        set an MCA param that tells libevent that it's ok to use any
        mechanism in libevent that is available on this platform (e.g.,
@@ -437,9 +471,7 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         mca_base_var_set_value(ret, allvalue, 4, MCA_BASE_VAR_SOURCE_DEFAULT, NULL);
     }
 
-    if (ompi_enable_timing && 0 == OMPI_PROC_MY_NAME->vpid) {
-        gettimeofday(&ompistart, NULL);
-    }
+    OPAL_TIMING_MSTART((&tm,"time from start to completion of rte_init"));
 
     /* if we were not externally started, then we need to setup
      * some envars so the MPI_INFO_ENV can get the cmd name
@@ -465,24 +497,16 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
     }
     /* no select is required as this is a static framework */
 
-    /* Setup RTE - note that we are an MPI process  */
+    /* Setup RTE */
     if (OMPI_SUCCESS != (ret = ompi_rte_init(NULL, NULL))) {
         error = "ompi_mpi_init: ompi_rte_init failed";
         goto error;
     }
-    rte_setup = true;
-    
+    ompi_rte_initialized = true;
+
     /* check for timing request - get stop time and report elapsed time if so */
-    if (ompi_enable_timing && 0 == OMPI_PROC_MY_NAME->vpid) {
-        gettimeofday(&ompistop, NULL);
-        opal_output(0, "ompi_mpi_init [%ld]: time from start to completion of rte_init %ld usec",
-                    (long)OMPI_PROC_MY_NAME->vpid,
-                    (long int)((ompistop.tv_sec - ompistart.tv_sec)*1000000 +
-                               (ompistop.tv_usec - ompistart.tv_usec)));
-        gettimeofday(&ompistart, NULL);
-    }
+    OPAL_TIMING_MNEXT((&tm,"time from completion of rte_init to modex"));
 
-#if OPAL_HAVE_HWLOC
     /* if hwloc is available but didn't get setup for some
      * reason, do so now
      */
@@ -492,7 +516,6 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
             goto error;
         }
     }
-#endif
 
     /* Register the default errhandler callback - RTE will ignore if it
      * doesn't support this capability
@@ -500,29 +523,20 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
     ompi_rte_register_errhandler(ompi_errhandler_runtime_callback,
                                  OMPI_RTE_ERRHANDLER_LAST);
 
-    /* Figure out the final MPI thread levels.  If we were not
-       compiled for support for MPI threads, then don't allow
-       MPI_THREAD_MULTIPLE.  Set this stuff up here early in the
-       process so that other components can make decisions based on
-       this value. */
-
-    ompi_mpi_thread_level(requested, provided);
 
     /* determine the bitflag belonging to the threadlevel_support provided */
     memset ( &threadlevel_bf, 0, sizeof(uint8_t));
     OMPI_THREADLEVEL_SET_BITFLAG ( ompi_mpi_thread_provided, threadlevel_bf );
 
-    /* If thread support was enabled, then setup OPAL to allow for
-       them. */
-    if (*provided != MPI_THREAD_SINGLE) {
-        opal_set_using_threads(true);
-    }
-
+#if OMPI_ENABLE_THREAD_MULTIPLE
     /* add this bitflag to the modex */
-    if ( OMPI_SUCCESS != (ret = ompi_modex_send_string("MPI_THREAD_LEVEL", &threadlevel_bf, sizeof(uint8_t)))) {
+    OPAL_MODEX_SEND_STRING(ret, OPAL_PMIX_GLOBAL,
+                           "MPI_THREAD_LEVEL", &threadlevel_bf, sizeof(uint8_t));
+    if (OPAL_SUCCESS != ret) {
         error = "ompi_mpi_init: modex send thread level";
         goto error;
     }
+#endif
 
     /* initialize datatypes. This step should be done early as it will
      * create the local convertor and local arch used in the proc
@@ -547,8 +561,8 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         error = "ompi_op_base_open() failed";
         goto error;
     }
-    if (OMPI_SUCCESS != 
-        (ret = ompi_op_base_find_available(OMPI_ENABLE_PROGRESS_THREADS,
+    if (OMPI_SUCCESS !=
+        (ret = ompi_op_base_find_available(OPAL_ENABLE_PROGRESS_THREADS,
                                            ompi_mpi_thread_multiple))) {
         error = "ompi_op_base_find_available() failed";
         goto error;
@@ -560,15 +574,15 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
 
     /* Open up MPI-related MCA components */
 
-    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_allocator_base_framework, 0))) {
+    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&opal_allocator_base_framework, 0))) {
         error = "mca_allocator_base_open() failed";
         goto error;
     }
-    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_rcache_base_framework, 0))) {
+    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&opal_rcache_base_framework, 0))) {
         error = "mca_rcache_base_open() failed";
         goto error;
     }
-    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_mpool_base_framework, 0))) {
+    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&opal_mpool_base_framework, 0))) {
         error = "mca_mpool_base_open() failed";
         goto error;
     }
@@ -576,6 +590,10 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         error = "mca_bml_base_open() failed";
         goto error;
     }
+    if (OMPI_SUCCESS != (ret = mca_bml_base_init (1, ompi_mpi_thread_multiple))) {
+        error = "mca_bml_base_init() failed";
+        goto error;
+    }
     if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_pml_base_framework, 0))) {
         error = "mca_pml_base_open() failed";
         goto error;
@@ -590,13 +608,6 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
-#if OPAL_ENABLE_FT_CR == 1
-    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_crcp_base_framework, 0))) {
-        error = "ompi_crcp_base_open() failed";
-        goto error;
-    }
-#endif
-
     /* In order to reduce the common case for MPI apps (where they
        don't use MPI-2 IO or MPI-1 topology functions), the io and
        topo frameworks are initialized lazily, at the first use of
@@ -605,84 +616,62 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
 
     /* Select which MPI components to use */
 
-    if (OMPI_SUCCESS != 
-        (ret = mca_mpool_base_init(OMPI_ENABLE_PROGRESS_THREADS,
+    if (OMPI_SUCCESS !=
+        (ret = mca_mpool_base_init(OPAL_ENABLE_PROGRESS_THREADS,
                                    ompi_mpi_thread_multiple))) {
         error = "mca_mpool_base_init() failed";
         goto error;
     }
 
-    if (OMPI_SUCCESS != 
-        (ret = mca_pml_base_select(OMPI_ENABLE_PROGRESS_THREADS,
+    if (OMPI_SUCCESS !=
+        (ret = mca_pml_base_select(OPAL_ENABLE_PROGRESS_THREADS,
                                    ompi_mpi_thread_multiple))) {
         error = "mca_pml_base_select() failed";
         goto error;
     }
 
     /* check for timing request - get stop time and report elapsed time if so */
-    if (ompi_enable_timing && 0 == OMPI_PROC_MY_NAME->vpid) {
-        gettimeofday(&ompistop, NULL);
-        opal_output(0, "ompi_mpi_init[%ld]: time from completion of rte_init to modex %ld usec",
-                    (long)OMPI_PROC_MY_NAME->vpid,
-                    (long int)((ompistop.tv_sec - ompistart.tv_sec)*1000000 +
-                               (ompistop.tv_usec - ompistart.tv_usec)));
-        gettimeofday(&ompistart, NULL);
-    }
-    
-    /* exchange connection info - this function also acts as a barrier
-     * as it will not return until the exchange is complete
-     */
-    coll = OBJ_NEW(ompi_rte_collective_t);
-    coll->id = ompi_process_info.peer_modex;
-    coll->active = true;
-    if (OMPI_SUCCESS != (ret = ompi_rte_modex(coll))) {
-        error = "rte_modex failed";
-        goto error;
-    }
-    /* wait for modex to complete - this may be moved anywhere in mpi_init
-     * so long as it occurs prior to calling a function that needs
-     * the modex info!
-     */
-    OMPI_WAIT_FOR_COMPLETION(coll->active);
-    OBJ_RELEASE(coll);
+    OPAL_TIMING_MNEXT((&tm,"time to execute modex"));
 
-    if (ompi_enable_timing && 0 == OMPI_PROC_MY_NAME->vpid) {
-        gettimeofday(&ompistop, NULL);
-        opal_output(0, "ompi_mpi_init[%ld]: time to execute modex %ld usec",
-                    (long)OMPI_PROC_MY_NAME->vpid,
-                    (long int)((ompistop.tv_sec - ompistart.tv_sec)*1000000 +
-                               (ompistop.tv_usec - ompistart.tv_usec)));
-        gettimeofday(&ompistart, NULL);
+    /* exchange connection info - this function may also act as a barrier
+     * if data exchange is required. The modex occurs solely across procs
+     * in our job. If a barrier is required, the "modex" function will
+     * perform it internally */
+    active = true;
+    opal_pmix.commit();
+    if (!opal_pmix_base_async_modex) {
+        if (NULL != opal_pmix.fence_nb) {
+            opal_pmix.fence_nb(NULL, opal_pmix_collect_all_data,
+                               fence_release, (void*)&active);
+            OMPI_WAIT_FOR_COMPLETION(active);
+        } else {
+            opal_pmix.fence(NULL, opal_pmix_collect_all_data);
+        }
     }
 
+    OPAL_TIMING_MNEXT((&tm,"time from modex to first barrier"));
+
     /* select buffered send allocator component to be used */
     if( OMPI_SUCCESS !=
-	(ret = mca_pml_base_bsend_init(ompi_mpi_thread_multiple))) {
+        (ret = mca_pml_base_bsend_init(ompi_mpi_thread_multiple))) {
         error = "mca_pml_base_bsend_init() failed";
         goto error;
     }
 
-    if (OMPI_SUCCESS != 
-        (ret = mca_coll_base_find_available(OMPI_ENABLE_PROGRESS_THREADS,
+    if (OMPI_SUCCESS !=
+        (ret = mca_coll_base_find_available(OPAL_ENABLE_PROGRESS_THREADS,
                                             ompi_mpi_thread_multiple))) {
         error = "mca_coll_base_find_available() failed";
         goto error;
     }
 
-    if (OMPI_SUCCESS != 
-        (ret = ompi_osc_base_find_available(OMPI_ENABLE_PROGRESS_THREADS,
+    if (OMPI_SUCCESS !=
+        (ret = ompi_osc_base_find_available(OPAL_ENABLE_PROGRESS_THREADS,
                                             ompi_mpi_thread_multiple))) {
         error = "ompi_osc_base_find_available() failed";
         goto error;
     }
 
-#if OPAL_ENABLE_FT_CR == 1
-    if (OMPI_SUCCESS != (ret = ompi_crcp_base_select() ) ) {
-        error = "ompi_crcp_base_select() failed";
-        goto error;
-    }
-#endif
-
     /* io and topo components are not selected here -- see comment
        above about the io and topo frameworks being loaded lazily */
 
@@ -715,13 +704,13 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         error = "ompi_mpi_errcode_init() failed";
         goto error;
     }
-    
+
     /* initialize internal error codes */
     if (OMPI_SUCCESS != (ret = ompi_errcode_intern_init())) {
         error = "ompi_errcode_intern_init() failed";
         goto error;
     }
-     
+
     /* initialize groups  */
     if (OMPI_SUCCESS != (ret = ompi_group_init())) {
         error = "ompi_group_init() failed";
@@ -767,10 +756,21 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
-    /* add all ompi_proc_t's to PML */
-    if (NULL == (procs = ompi_proc_world(&nprocs))) {
-        error = "ompi_proc_world() failed";
-        goto error;
+    /* some btls/mtls require we call add_procs with all procs in the job.
+     * since the btls/mtls have no visibility here it is up to the pml to
+     * convey this requirement */
+    if (mca_pml_base_requires_world ()) {
+        if (NULL == (procs = ompi_proc_world (&nprocs))) {
+            error = "ompi_proc_get_allocated () failed";
+            goto error;
+        }
+    } else {
+        /* add all allocated ompi_proc_t's to PML (below the add_procs limit this
+         * behaves identically to ompi_proc_world ()) */
+        if (NULL == (procs = ompi_proc_get_allocated (&nprocs))) {
+            error = "ompi_proc_get_allocated () failed";
+            goto error;
+        }
     }
     ret = MCA_PML_CALL(add_procs(procs, nprocs));
     free(procs);
@@ -778,7 +778,7 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
        Otherwise, if we got some other failure, fall through to print
        a generic message. */
     if (OMPI_ERR_UNREACH == ret) {
-        opal_show_help("help-mpi-runtime",
+        opal_show_help("help-mpi-runtime.txt",
                        "mpi_init:startup:pml-add-procs-fail", true);
         error = NULL;
         goto error;
@@ -794,49 +794,35 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
      * Dump all MCA parameters if requested
      */
     if (ompi_mpi_show_mca_params) {
-        ompi_show_all_mca_params(ompi_mpi_comm_world.comm.c_my_rank, 
-                                 nprocs, 
+        ompi_show_all_mca_params(ompi_mpi_comm_world.comm.c_my_rank,
+                                 nprocs,
                                  ompi_process_info.nodename);
     }
 
     /* Do we need to wait for a debugger? */
     ompi_rte_wait_for_debugger();
 
-    /* check for timing request - get stop time and report elapsed
-       time if so, then start the clock again */
-    if (ompi_enable_timing && 0 == OMPI_PROC_MY_NAME->vpid) {
-        gettimeofday(&ompistop, NULL);
-        opal_output(0, "ompi_mpi_init[%ld]: time from modex to first barrier %ld usec",
-                    (long)OMPI_PROC_MY_NAME->vpid,
-                    (long int)((ompistop.tv_sec - ompistart.tv_sec)*1000000 +
-                               (ompistop.tv_usec - ompistart.tv_usec)));
-        gettimeofday(&ompistart, NULL);
-    }
+    /* Next timing measurement */
+    OPAL_TIMING_MNEXT((&tm,"time to execute barrier"));
 
-    /* wait for everyone to reach this point */
-    coll = OBJ_NEW(ompi_rte_collective_t);
-    coll->id = ompi_process_info.peer_init_barrier;
-    coll->active = true;
-    if (OMPI_SUCCESS != (ret = ompi_rte_barrier(coll))) {
-        error = "rte_barrier failed";
-        goto error;
+    /* wait for everyone to reach this point - this is a hard
+     * barrier requirement at this time, though we hope to relax
+     * it at a later point */
+    active = true;
+    opal_pmix.commit();
+    if (NULL != opal_pmix.fence_nb) {
+        opal_pmix.fence_nb(NULL, opal_pmix_collect_all_data,
+                           fence_release, (void*)&active);
+        OMPI_WAIT_FOR_COMPLETION(active);
+    } else {
+        opal_pmix.fence(NULL, opal_pmix_collect_all_data);
     }
-    /* wait for barrier to complete */
-    OMPI_WAIT_FOR_COMPLETION(coll->active);
-    OBJ_RELEASE(coll);
 
     /* check for timing request - get stop time and report elapsed
        time if so, then start the clock again */
-    if (ompi_enable_timing && 0 == OMPI_PROC_MY_NAME->vpid) {
-        gettimeofday(&ompistop, NULL);
-        opal_output(0, "ompi_mpi_init[%ld]: time to execute barrier %ld usec",
-                    (long)OMPI_PROC_MY_NAME->vpid,
-                    (long int)((ompistop.tv_sec - ompistart.tv_sec)*1000000 +
-                               (ompistop.tv_usec - ompistart.tv_usec)));
-        gettimeofday(&ompistart, NULL);
-    }
+    OPAL_TIMING_MNEXT((&tm,"time from barrier to complete mpi_init"));
 
-#if OMPI_ENABLE_PROGRESS_THREADS == 0
+#if OPAL_ENABLE_PROGRESS_THREADS == 0
     /* Start setting up the event engine for MPI operations.  Don't
        block in the event library, so that communications don't take
        forever between procs in the dynamic code.  This will increase
@@ -845,7 +831,7 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
        latency. */
     opal_progress_set_event_flag(OPAL_EVLOOP_NONBLOCK);
 #endif
-    
+
     /* wire up the mpi interface, if requested.  Do this after the
        non-block switch for non-TCP performance.  Do before the
        polling change as anyone with a complex wire-up is going to be
@@ -855,39 +841,24 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
-    /* Setup the publish/subscribe (PUBSUB) framework */
-    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_pubsub_base_framework, 0))) {
-        error = "mca_pubsub_base_open() failed";
-        goto error;
-    }
-    if (OMPI_SUCCESS != (ret = ompi_pubsub_base_select())) {
-        error = "ompi_pubsub_base_select() failed";
-        goto error;
-    }
-    
-    /* Setup the dynamic process management (DPM) framework */
-    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_dpm_base_framework, 0))) {
-        error = "ompi_dpm_base_open() failed";
-        goto error;
-    }
-    if (OMPI_SUCCESS != (ret = ompi_dpm_base_select())) {
-        error = "ompi_dpm_base_select() failed";
+    /* Setup the dynamic process management (DPM) subsystem */
+    if (OMPI_SUCCESS != (ret = ompi_dpm_init())) {
+        error = "ompi_dpm_init() failed";
         goto error;
     }
 
-    /* Determine the overall threadlevel support of all processes 
-       in MPI_COMM_WORLD. This has to be done before calling 
+    /* Determine the overall threadlevel support of all processes
+       in MPI_COMM_WORLD. This has to be done before calling
        coll_base_comm_select, since some of the collective components
        e.g. hierarch, might create subcommunicators. The threadlevel
        requested by all processes is required in order to know
        which cid allocation algorithm can be used. */
-    if ( OMPI_SUCCESS != 
-	 ( ret = ompi_comm_cid_init ())) {
-	error = "ompi_mpi_init: ompi_comm_cid_init failed";
-	goto error;
+    if (OMPI_SUCCESS != ( ret = ompi_comm_cid_init ())) {
+        error = "ompi_mpi_init: ompi_comm_cid_init failed";
+        goto error;
     }
 
-    /* Init coll for the comms. This has to be after dpm_base_select, 
+    /* Init coll for the comms. This has to be after dpm_base_select,
        (since dpm.mark_dyncomm is not set in the communicator creation
        function else), but before dpm.dyncom_init, since this function
        might require collective for the CID allocation. */
@@ -897,42 +868,30 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
-    if (OMPI_SUCCESS != 
+    if (OMPI_SUCCESS !=
         (ret = mca_coll_base_comm_select(MPI_COMM_SELF))) {
         error = "mca_coll_base_comm_select(MPI_COMM_SELF) failed";
         goto error;
     }
 
-
-    
     /* Check whether we have been spawned or not.  We introduce that
        at the very end, since we need collectives, datatypes, ptls
        etc. up and running here.... */
-    if (OMPI_SUCCESS != (ret = ompi_dpm.dyn_init())) {
-        error = "ompi_comm_dyn_init() failed";
+    if (OMPI_SUCCESS != (ret = ompi_dpm_dyn_init())) {
+        error = "ompi_dpm_dyn_init() failed";
         goto error;
     }
 
-    /*
-     * Startup the Checkpoint/Restart Mech.
-     * Note: Always do this so tools don't hang when
-     * in a non-checkpointable build
-     */
-    if (OMPI_SUCCESS != (ret = ompi_cr_init())) {
-        error = "ompi_cr_init";
-        goto error;
-    }
-
-    /* Undo OPAL calling opal_progress_event_users_increment() during 
-       opal_init, to get better latency when not using TCP.  Do 
-       this *after* dyn_init, as dyn init uses lots of RTE 
-       communication and we don't want to hinder the performance of 
-       that code. */ 
-    opal_progress_event_users_decrement(); 
+    /* Undo OPAL calling opal_progress_event_users_increment() during
+       opal_init, to get better latency when not using TCP.  Do
+       this *after* dyn_init, as dyn init uses lots of RTE
+       communication and we don't want to hinder the performance of
+       that code. */
+    opal_progress_event_users_decrement();
 
     /* see if yield_when_idle was specified - if so, use it */
     opal_progress_set_yield_when_idle(ompi_mpi_yield_when_idle);
-    
+
     /* negative value means use default - just don't do anything */
     if (ompi_mpi_event_tick_rate >= 0) {
         opal_progress_set_event_poll_rate(ompi_mpi_event_tick_rate);
@@ -954,17 +913,11 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         /* Only print a message if one was not already printed */
         if (NULL != error) {
             const char *err_msg = opal_strerror(ret);
-            /* If RTE was not setup yet, don't use opal_show_help */
-            if (rte_setup) {
-                opal_show_help("help-mpi-runtime",
-                               "mpi_init:startup:internal-failure", true,
-                               "MPI_INIT", "MPI_INIT", error, err_msg, ret);
-            } else {
-                opal_show_help("help-mpi-runtime",
-                               "mpi_init:startup:internal-failure", true,
-                               "MPI_INIT", "MPI_INIT", error, err_msg, ret);
-            }
+            opal_show_help("help-mpi-runtime.txt",
+                           "mpi_init:startup:internal-failure", true,
+                           "MPI_INIT", "MPI_INIT", error, err_msg, ret);
         }
+        opal_mutex_unlock(&ompi_mpi_bootstrap_mutex);
         return ret;
     }
 
@@ -987,14 +940,13 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
 
     ompi_mpi_initialized = true;
 
-    /* check for timing request - get stop time and report elapsed time if so */
-    if (ompi_enable_timing && 0 == OMPI_PROC_MY_NAME->vpid) {
-        gettimeofday(&ompistop, NULL);
-        opal_output(0, "ompi_mpi_init[%ld]: time from barrier to complete mpi_init %ld usec",
-                    (long)OMPI_PROC_MY_NAME->vpid,
-                    (long int)((ompistop.tv_sec - ompistart.tv_sec)*1000000 +
-                               (ompistop.tv_usec - ompistart.tv_usec)));
-    }
+    /* Finish last measurement, output results
+     * and clear timing structure */
+    OPAL_TIMING_MSTOP(&tm);
+    OPAL_TIMING_DELTAS(ompi_enable_timing, &tm);
+    OPAL_TIMING_REPORT(ompi_enable_timing_ext, &tm);
+    OPAL_TIMING_RELEASE(&tm);
 
+    opal_mutex_unlock(&ompi_mpi_bootstrap_mutex);
     return MPI_SUCCESS;
 }
