@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,43 +6,33 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
-#ifndef OMPI_SYS_ARCH_ATOMIC_H
-#define OMPI_SYS_ARCH_ATOMIC_H 1
+#ifndef OPAL_SYS_ARCH_ATOMIC_H
+#define OPAL_SYS_ARCH_ATOMIC_H 1
 
 /*
  * On powerpc ...
  */
 
-#if OPAL_WANT_SMP_LOCKS
-
 #define MB()  __asm__ __volatile__ ("sync" : : : "memory")
 #define RMB() __asm__ __volatile__ ("lwsync" : : : "memory")
 #define WMB() __asm__ __volatile__ ("eieio" : : : "memory")
 #define SMP_SYNC  "sync \n\t"
 #define SMP_ISYNC "\n\tisync"
 
-#else
-
-#define MB()
-#define RMB()
-#define WMB()
-#define SMP_SYNC  ""
-#define SMP_ISYNC
-
-#endif
-
 
 /**********************************************************************
  *
@@ -51,14 +42,18 @@
 #define OPAL_HAVE_ATOMIC_MEM_BARRIER 1
 
 #define OPAL_HAVE_ATOMIC_CMPSET_32 1
+#define OPAL_HAVE_ATOMIC_SWAP_32 1
+#define OPAL_HAVE_ATOMIC_LLSC_32 1
 
 #define OPAL_HAVE_ATOMIC_MATH_32 1
 #define OPAL_HAVE_ATOMIC_ADD_32 1
 #define OPAL_HAVE_ATOMIC_SUB_32 1
 
 
-#if (OPAL_ASSEMBLY_ARCH == OMPI_POWERPC64) || OPAL_ASM_SUPPORT_64BIT
+#if (OPAL_ASSEMBLY_ARCH == OPAL_POWERPC64) || OPAL_ASM_SUPPORT_64BIT
 #define OPAL_HAVE_ATOMIC_CMPSET_64 1
+#define OPAL_HAVE_ATOMIC_SWAP_64 1
+#define OPAL_HAVE_ATOMIC_LLSC_64 1
 #endif
 
 
@@ -67,7 +62,7 @@
  * Memory Barriers
  *
  *********************************************************************/
-#if OMPI_GCC_INLINE_ASSEMBLY
+#if OPAL_GCC_INLINE_ASSEMBLY
 
 static inline
 void opal_atomic_mb(void)
@@ -89,7 +84,7 @@ void opal_atomic_wmb(void)
     WMB();
 }
 
-#elif OMPI_XLC_INLINE_ASSEMBLY /* end OMPI_GCC_INLINE_ASSEMBLY */
+#elif OPAL_XLC_INLINE_ASSEMBLY /* end OPAL_GCC_INLINE_ASSEMBLY */
 
 /* Yeah, I don't know who thought this was a reasonable syntax for
  * inline assembly.  Do these because they are used so often and they
@@ -116,7 +111,7 @@ void opal_atomic_wmb(void)
  * Atomic math operations
  *
  *********************************************************************/
-#if OMPI_GCC_INLINE_ASSEMBLY
+#if OPAL_GCC_INLINE_ASSEMBLY
 
 #ifdef __xlC__
 /* work-around bizzare xlc bug in which it sign-extends
@@ -145,6 +140,32 @@ static inline int opal_atomic_cmpset_32(volatile int32_t *addr,
    return (ret == oldval);
 }
 
+static inline int32_t opal_atomic_ll_32 (volatile int32_t *addr)
+{
+   int32_t ret;
+
+   __asm__ __volatile__ ("lwarx   %0, 0, %1  \n\t"
+                         : "=&r" (ret)
+                         : "r" (addr)
+                         );
+   return ret;
+}
+
+static inline int opal_atomic_sc_32 (volatile int32_t *addr, int32_t newval)
+{
+    int32_t ret, foo;
+
+    __asm__ __volatile__ ("   stwcx.  %4, 0, %3  \n\t"
+                          "   li      %0,0       \n\t"
+                          "   bne-    1f         \n\t"
+                          "   ori     %0,%0,1    \n\t"
+                          "1:"
+                          : "=r" (ret), "=m" (*addr), "=r" (foo)
+                          : "r" (addr), "r" (newval)
+                          : "cc", "memory");
+    return ret;
+}
+
 /* these two functions aren't inlined in the non-gcc case because then
    there would be two function calls (since neither cmpset_32 nor
    atomic_?mb can be inlined).  Instead, we "inline" them by hand in
@@ -169,12 +190,26 @@ static inline int opal_atomic_cmpset_rel_32(volatile int32_t *addr,
     return opal_atomic_cmpset_32(addr, oldval, newval);
 }
 
-#endif /* OMPI_GCC_INLINE_ASSEMBLY */
+static inline int32_t opal_atomic_swap_32(volatile int32_t *addr, int32_t newval)
+{
+    int32_t ret;
+
+    __asm__ __volatile__ ("1: lwarx   %0, 0, %2  \n\t"
+                          "   stwcx.  %3, 0, %2  \n\t"
+                          "   bne-    1b         \n\t"
+                          : "=&r" (ret), "=m" (*addr)
+                          : "r" (addr), "r" (newval)
+                          : "cc", "memory");
+
+   return ret;
+}
+
+#endif /* OPAL_GCC_INLINE_ASSEMBLY */
 
 
-#if (OPAL_ASSEMBLY_ARCH == OMPI_POWERPC64)
+#if (OPAL_ASSEMBLY_ARCH == OPAL_POWERPC64)
 
-#if  OMPI_GCC_INLINE_ASSEMBLY
+#if  OPAL_GCC_INLINE_ASSEMBLY
 static inline int opal_atomic_cmpset_64(volatile int64_t *addr,
                                         int64_t oldval, int64_t newval)
 {
@@ -190,10 +225,36 @@ static inline int opal_atomic_cmpset_64(volatile int64_t *addr,
                          : "=&r" (ret), "=m" (*addr)
                          : "r" (addr), "r" (oldval), "r" (newval), "m" (*addr)
                          : "cc", "memory");
-    
+
    return (ret == oldval);
 }
 
+static inline int64_t opal_atomic_ll_64(volatile int64_t *addr)
+{
+   int64_t ret;
+
+   __asm__ __volatile__ ("ldarx   %0, 0, %1  \n\t"
+                         : "=&r" (ret)
+                         : "r" (addr)
+                         :);
+   return ret;
+}
+
+static inline int opal_atomic_sc_64(volatile int64_t *addr, int64_t newval)
+{
+    int32_t ret, foo;
+
+    __asm__ __volatile__ ("   stdcx.  %4, 0, %3  \n\t"
+                          "   li      %0,0       \n\t"
+                          "   bne-    1f         \n\t"
+                          "   ori     %0,%0,1    \n\t"
+                          "1:"
+                          : "=r" (ret), "=m" (*addr), "=r" (foo)
+                          : "r" (addr), "r" (newval)
+                          : "cc", "memory");
+    return ret;
+}
+
 /* these two functions aren't inlined in the non-gcc case because then
    there would be two function calls (since neither cmpset_64 nor
    atomic_?mb can be inlined).  Instead, we "inline" them by hand in
@@ -218,16 +279,30 @@ static inline int opal_atomic_cmpset_rel_64(volatile int64_t *addr,
     return opal_atomic_cmpset_64(addr, oldval, newval);
 }
 
-#endif /* OMPI_GCC_INLINE_ASSEMBLY */
+static inline int64_t opal_atomic_swap_64(volatile int64_t *addr, int64_t newval)
+{
+   int64_t ret;
 
-#elif (OPAL_ASSEMBLY_ARCH == OMPI_POWERPC32) && OPAL_ASM_SUPPORT_64BIT
+   __asm__ __volatile__ ("1: ldarx   %0, 0, %2  \n\t"
+                         "   stdcx.  %3, 0, %2  \n\t"
+                         "   bne-    1b         \n\t"
+                         : "=&r" (ret), "=m" (*addr)
+                         : "r" (addr), "r" (newval)
+                         : "cc", "memory");
+
+   return ret;
+}
+
+#endif /* OPAL_GCC_INLINE_ASSEMBLY */
+
+#elif (OPAL_ASSEMBLY_ARCH == OPAL_POWERPC32) && OPAL_ASM_SUPPORT_64BIT
 
 #ifndef ll_low /* GLIBC provides these somewhere, so protect */
 #define ll_low(x)       *(((unsigned int*)&(x))+0)
 #define ll_high(x)      *(((unsigned int*)&(x))+1)
 #endif
 
-#if  OMPI_GCC_INLINE_ASSEMBLY
+#if  OPAL_GCC_INLINE_ASSEMBLY
 
 static inline int opal_atomic_cmpset_64(volatile int64_t *addr,
                                         int64_t oldval, int64_t newval)
@@ -258,10 +333,10 @@ static inline int opal_atomic_cmpset_64(volatile int64_t *addr,
                          "subfic r9,r5,0        \n\t"
                          "adde %0,r9,r5         \n\t"
                          : "=&r" (ret)
-                         : "r"OPAL_ASM_ADDR(addr), 
+                         : "r"OPAL_ASM_ADDR(addr),
                            "m"(oldval), "m"(newval)
                          : "r4", "r5", "r9", "cc", "memory");
-    
+
      return ret;
 }
 
@@ -289,12 +364,12 @@ static inline int opal_atomic_cmpset_rel_64(volatile int64_t *addr,
     return opal_atomic_cmpset_64(addr, oldval, newval);
 }
 
-#endif /* OMPI_GCC_INLINE_ASSEMBLY */
+#endif /* OPAL_GCC_INLINE_ASSEMBLY */
 
 #endif /* OPAL_ASM_SUPPORT_64BIT */
 
 
-#if OMPI_GCC_INLINE_ASSEMBLY
+#if OPAL_GCC_INLINE_ASSEMBLY
 
 static inline int32_t opal_atomic_add_32(volatile int32_t* v, int inc)
 {
@@ -330,6 +405,6 @@ static inline int32_t opal_atomic_sub_32(volatile int32_t* v, int dec)
 }
 
 
-#endif /* OMPI_GCC_INLINE_ASSEMBLY */
+#endif /* OPAL_GCC_INLINE_ASSEMBLY */
 
-#endif /* ! OMPI_SYS_ARCH_ATOMIC_H */
+#endif /* ! OPAL_SYS_ARCH_ATOMIC_H */
