@@ -5,7 +5,7 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2010 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2010 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -13,10 +13,12 @@
  * Copyright (c) 2009-2013 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  *
  * This file is included at the bottom of opal_config.h, and is
@@ -28,7 +30,7 @@
  * need to #ifndef/#endif protection here.
  */
 
-#ifndef OPAL_CONFIG_H 
+#ifndef OPAL_CONFIG_H
 #error "opal_config_bottom.h should only be included from opal_config.h"
 #endif
 
@@ -36,15 +38,15 @@
  * If we build a static library, Visual C define the _LIB symbol. In the
  * case of a shared library _USERDLL get defined.
  *
- * OMPI_BUILDING and _LIB define how ompi_config.h
+ * OMPI_BUILDING and _LIB define how opal_config.h
  * handles configuring all of Open MPI's "compatibility" code.  Both
- * constants will always be defined by the end of ompi_config.h.
+ * constants will always be defined by the end of opal_config.h.
  *
  * OMPI_BUILDING affects how much compatibility code is included by
- * ompi_config.h.  It will always be 1 or 0.  The user can set the
- * value before including either mpi.h or ompi_config.h and it will be
- * respected.  If ompi_config.h is included before mpi.h, it will
- * default to 1.  If mpi.h is included before ompi_config.h, it will
+ * opal_config.h.  It will always be 1 or 0.  The user can set the
+ * value before including either mpi.h or opal_config.h and it will be
+ * respected.  If opal_config.h is included before mpi.h, it will
+ * default to 1.  If mpi.h is included before opal_config.h, it will
  * default to 0.
  */
 #ifndef OMPI_BUILDING
@@ -54,7 +56,7 @@
 /*
  * Flex is trying to include the unistd.h file. As there is no configure
  * option or this, the flex generated files will try to include the file
- * even on platforms without unistd.h (such as Windows). Therefore, if we
+ * even on platforms without unistd.h. Therefore, if we
  * know this file is not available, we can prevent flex from including it.
  */
 #ifndef HAVE_UNISTD_H
@@ -68,9 +70,6 @@
  *
  **********************************************************************/
 
-/* Do we have posix or solaris thread lib */
-#define OPAL_HAVE_THREADS (OPAL_HAVE_POSIX_THREADS)
-
 /*
  * BEGIN_C_DECLS should be used at the beginning of your declarations,
  * so that C++ compilers don't mangle their names.  Use END_C_DECLS at
@@ -242,10 +241,6 @@
 #    define OPAL_MODULE_DECLSPEC
 #  endif
 
-/*
- * Do we have <stdint.h>?
- */
-#ifdef HAVE_STDINT_H
 #if !defined(__STDC_LIMIT_MACROS) && (defined(c_plusplus) || defined (__cplusplus))
 /* When using a C++ compiler, the max / min value #defines for std
    types are only included if __STDC_LIMIT_MACROS is set before
@@ -253,10 +248,7 @@
 #define __STDC_LIMIT_MACROS
 #endif
 #include "opal_config.h"
-#include <stdint.h>
-#else
 #include "opal_stdint.h"
-#endif
 
 /***********************************************************************
  *
@@ -344,13 +336,8 @@ typedef unsigned char bool;
 /*
  * Set the compile-time path-separator on this system and variable separator
  */
-#ifdef __WINDOWS__
-#define OPAL_PATH_SEP "\\"
-#define OPAL_ENV_SEP  ';'
-#else
 #define OPAL_PATH_SEP "/"
 #define OPAL_ENV_SEP  ':'
-#endif
 
 
 /*
@@ -362,13 +349,13 @@ typedef unsigned char bool;
  * 2. In the OMPI C++ bindings: we do not want them
  * 3. In the OMPI C++ executables: we do want them
  *
- * So for 1, everyone must include <ompi_config.h> first.  For 2, the
- * C++ bindings will never include <ompi_config.h> -- they will only
- * include <mpi.h>, which includes <ompi_config.h>, but after
+ * So for 1, everyone must include <opal_config.h> first.  For 2, the
+ * C++ bindings will never include <opal_config.h> -- they will only
+ * include <mpi.h>, which includes <opal_config.h>, but after
  * setting OMPI_BUILDING to 0  For 3, it's the same as 1 -- just include
- * <ompi_config.h> first.
+ * <opal_config.h> first.
  *
- * Give code that needs to include ompi_config.h but really can't have
+ * Give code that needs to include opal_config.h but really can't have
  * this stuff enabled (like the memory manager code) a way to turn us
  * off
  */
@@ -452,9 +439,9 @@ typedef unsigned char bool;
  * are no htonl and friends.  If that's the case, provide stubs.  I
  * would hope we never find a platform that doesn't have these macros
  * and would want to talk to the outside world... On other platforms
- * (like Windows) we fail to detect them correctly.
+ * we fail to detect them correctly.
  */
-#if !defined(__WINDOWS__) && !defined(HAVE_UNIX_BYTESWAP)
+#if !defined(HAVE_UNIX_BYTESWAP)
 static inline uint32_t htonl(uint32_t hostvar) { return hostvar; }
 static inline uint32_t ntohl(uint32_t netvar) { return netvar; }
 static inline uint16_t htons(uint16_t hostvar) { return hostvar; }
@@ -472,35 +459,13 @@ static inline uint16_t ntohs(uint16_t netvar) { return netvar; }
 #define __func__ __FILE__
 #endif
 
-/**
- * Because of the way we're using the opal_object inside Open MPI (i.e.
- * dynamic resolution at run-time to derive all objects from the basic
- * type), on Windows we have to build everything on C++ mode, simply
- * because the C mode does not support dynamic resolution in DLL. Therefore,
- * no automatic conversion is allowed. All types have to be explicitly casted
- * or the compiler generate an error. This is true even for the void* type. As
- * we use void* to silence others compilers in the resolution of the addr member
- * of the iovec structure, we have now to find a way around. The simplest solution
- * is to define a special type for this field (just for casting). It can be
- * set to void* on all platforms with the exception of windows where it has to be
- * char*.
- */
-#if defined(__WINDOWS__)
-#define IOVBASE_TYPE  char
-#else
 #define IOVBASE_TYPE  void
-#endif  /* defined(__WINDOWS__) */
 
 /**
  * If we generate our own bool type, we need a special way to cast the result
- * in such a way to keep the compilers silent. Right now, th only compiler who
- * complain about int to bool conversions is the Microsoft compiler.
+ * in such a way to keep the compilers silent.
  */
-#if defined(__WINDOWS__)
-#  define OPAL_INT_TO_BOOL(VALUE)  ((VALUE) != 0 ? true : false)
-#else
 #  define OPAL_INT_TO_BOOL(VALUE)  (bool)(VALUE)
-#endif  /* defined(__WINDOWS__) */
 
 /**
  * Top level define to check 2 things: a) if we want ipv6 support, and
@@ -598,7 +563,7 @@ static inline uint16_t ntohs(uint16_t netvar) { return netvar; }
    want to protect others from the autoconf/automake-generated
    PACKAGE_<foo> macros in opal_config.h.  We can't put these undef's
    directly in opal_config.h because they'll be turned into #defines'
-   via autoconf.  
+   via autoconf.
 
    So put them here in case any only else includes OMPI/ORTE/OPAL's
    config.h files. */
