@@ -1,7 +1,8 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2013-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2015 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -75,7 +76,7 @@ int mca_base_pvar_find (const char *project, const char *framework, const char *
         return OPAL_ERROR;
     }
 
-    ret = mca_base_pvar_find_by_name (full_name, &index);
+    ret = mca_base_pvar_find_by_name (full_name, MCA_BASE_PVAR_CLASS_ANY, &index);
     free (full_name);
 
     /* NTH: should we verify the name components match the returned variable? */
@@ -83,8 +84,9 @@ int mca_base_pvar_find (const char *project, const char *framework, const char *
     return (OPAL_SUCCESS != ret) ? ret : index;
 }
 
-int mca_base_pvar_find_by_name (const char *full_name, int *index)
+int mca_base_pvar_find_by_name (const char *full_name, int var_class, int *index)
 {
+    mca_base_pvar_t *pvar;
     void *tmp;
     int rc;
 
@@ -94,6 +96,15 @@ int mca_base_pvar_find_by_name (const char *full_name, int *index)
         return rc;
     }
 
+    rc = mca_base_pvar_get_internal ((int)(uintptr_t) tmp, &pvar, false);
+    if (OPAL_SUCCESS != rc) {
+        return rc;
+    }
+
+    if (MCA_BASE_PVAR_CLASS_ANY != var_class && pvar->var_class != var_class) {
+        return OPAL_ERR_NOT_FOUND;
+    }
+
     *index = (int)(uintptr_t) tmp;
 
     return OPAL_SUCCESS;
@@ -259,8 +270,8 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
 
         do {
             /* generate the variable's full name */
-            mca_base_var_generate_full_name4 (NULL, framework, component, name, &pvar->name);
-            if (NULL == pvar->name) {
+            ret = mca_base_var_generate_full_name4 (NULL, framework, component, name, &pvar->name);
+            if (OPAL_SUCCESS != ret) {
                 ret = OPAL_ERR_OUT_OF_RESOURCE;
                 break;
             }
@@ -316,7 +327,7 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
     if (!(flags & MCA_BASE_PVAR_FLAG_READONLY)) {
         pvar->set_value = set_value ? set_value : mca_base_pvar_default_set_value;
     }
-        
+
     pvar->ctx        = ctx;
     pvar->pvar_index = pvar_count;
 
@@ -446,7 +457,7 @@ int mca_base_pvar_handle_alloc (mca_base_pvar_session_t *session, int index, voi
         }
 
         pvar_handle->obj_handle = obj_handle;
-        pvar_handle->pvar = pvar; 
+        pvar_handle->pvar = pvar;
 
         *handle = pvar_handle;
 
@@ -491,7 +502,7 @@ int mca_base_pvar_handle_alloc (mca_base_pvar_session_t *session, int index, voi
                 ret = OPAL_ERR_OUT_OF_RESOURCE;
                 break;
             }
-       
+
             pvar_handle->last_value = calloc (*count, datatype_size);
             if (NULL == pvar_handle->last_value) {
                 ret = OPAL_ERR_OUT_OF_RESOURCE;
@@ -698,7 +709,7 @@ int mca_base_pvar_handle_write_value (mca_base_pvar_handle_t *handle, const void
     if (OPAL_SUCCESS != ret) {
         return ret;
     }
-    
+
     memmove (handle->current_value, value, handle->count * var_type_sizes[handle->pvar->type]);
 
     return OPAL_SUCCESS;
@@ -915,12 +926,12 @@ static void mca_base_pvar_destructor (mca_base_pvar_t *pvar)
 OBJ_CLASS_INSTANCE(mca_base_pvar_t, opal_object_t, mca_base_pvar_contructor, mca_base_pvar_destructor);
 
 /* mca_base_pvar_session_t class */
-static void ompi_mpi_pvar_session_constructor (mca_base_pvar_session_t *session)
+static void opal_mpi_pvar_session_constructor (mca_base_pvar_session_t *session)
 {
     OBJ_CONSTRUCT(&session->handles, opal_list_t);
 }
 
-static void ompi_mpi_pvar_session_destructor (mca_base_pvar_session_t *session)
+static void opal_mpi_pvar_session_destructor (mca_base_pvar_session_t *session)
 {
     mca_base_pvar_handle_t *handle, *next;
 
@@ -934,8 +945,8 @@ static void ompi_mpi_pvar_session_destructor (mca_base_pvar_session_t *session)
     OBJ_DESTRUCT(&session->handles);
 }
 
-OBJ_CLASS_INSTANCE(mca_base_pvar_session_t, opal_object_t, ompi_mpi_pvar_session_constructor,
-                   ompi_mpi_pvar_session_destructor);
+OBJ_CLASS_INSTANCE(mca_base_pvar_session_t, opal_object_t, opal_mpi_pvar_session_constructor,
+                   opal_mpi_pvar_session_destructor);
 
 /* mca_base_pvar_handle_t class */
 static void mca_base_pvar_handle_constructor (mca_base_pvar_handle_t *handle)
@@ -964,7 +975,9 @@ static void mca_base_pvar_handle_destructor (mca_base_pvar_handle_t *handle)
     }
 
     /* remove this handle from the pvar list */
-    opal_list_remove_item (&handle->pvar->bound_handles, &handle->list2);
+    if (handle->pvar) {
+        opal_list_remove_item (&handle->pvar->bound_handles, &handle->list2);
+    }
 
     OBJ_DESTRUCT(&handle->list2);
 
