@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,18 +6,20 @@
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2011-2012 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
+ * Copyright (c) 2012-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * Copyright (c) 2013-2014 Intel, Inc. All rights reserved.
+ * Copyright (c) 2015-2016 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -54,10 +57,10 @@ hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,
 {
     hwloc_obj_type_t obj_type = HWLOC_OBJ_CORE;
     hwloc_obj_t obj;
-    
+
     /* hwloc isn't able to find cores on all platforms.  Example:
        PPC64 running RHEL 5.4 (linux kernel 2.6.18) only reports NUMA
-       nodes and PU's.  Fine.  
+       nodes and PU's.  Fine.
 
        However, note that hwloc_get_obj_by_type() will return NULL in
        2 (effectively) different cases:
@@ -68,7 +71,7 @@ hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,
        So first we have to see if we can find *any* cores by looking
        for the 0th core.  If we find it, then try to find the Nth
        core.  Otherwise, try to find the Nth PU. */
-    if (NULL == hwloc_get_obj_by_type(topo, HWLOC_OBJ_CORE, 0)) {
+    if (opal_hwloc_use_hwthreads_as_cpus || (NULL == hwloc_get_obj_by_type(topo, HWLOC_OBJ_CORE, 0))) {
         obj_type = HWLOC_OBJ_PU;
     }
 
@@ -88,10 +91,10 @@ hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,
         }
         return obj;
     }
-            
+
     opal_output_verbose(5, opal_hwloc_base_framework.framework_output,
                         "Searching for %d LOGICAL PU", lid);
-    
+
     /* Now do the actual lookup. */
     obj = hwloc_get_obj_by_type(topo, obj_type, lid);
     OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
@@ -199,27 +202,32 @@ int opal_hwloc_base_filter_cpus(hwloc_topology_t topo)
 
 static void fill_cache_line_size(void)
 {
-    int i = 0;
+    int i = 0, cache_level = 2;
     unsigned size;
     hwloc_obj_t obj;
     bool found = false;
 
     /* Look for the smallest L2 cache size */
     size = 4096;
-    while (1) {
-        obj = opal_hwloc_base_get_obj_by_type(opal_hwloc_topology,
-                                              HWLOC_OBJ_CACHE, 2,
-                                              i, OPAL_HWLOC_LOGICAL);
-        if (NULL == obj) {
-            break;
-        } else { 
-            found = true;
-            if (NULL != obj->attr &&
-                size > obj->attr->cache.linesize) {
-                size = obj->attr->cache.linesize;
+    while (cache_level > 0 && !found) {
+        i=0;
+        while (1) {
+            obj = opal_hwloc_base_get_obj_by_type(opal_hwloc_topology,
+                                                  HWLOC_OBJ_CACHE, cache_level,
+                                                  i, OPAL_HWLOC_LOGICAL);
+            if (NULL == obj) {
+                --cache_level;
+                break;
+            } else {
+                if (NULL != obj->attr &&
+                    obj->attr->cache.linesize > 0 &&
+                    size > obj->attr->cache.linesize) {
+                    size = obj->attr->cache.linesize;
+                    found = true;
+                }
             }
+            ++i;
         }
-        ++i;
     }
 
     /* If we found an L2 cache size in the hwloc data, save it in
@@ -239,7 +247,7 @@ int opal_hwloc_base_get_topology(void)
 
     if (NULL == opal_hwloc_base_topo_file) {
         if (0 != hwloc_topology_init(&opal_hwloc_topology) ||
-            0 != hwloc_topology_set_flags(opal_hwloc_topology, 
+            0 != hwloc_topology_set_flags(opal_hwloc_topology,
                                           (HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM |
                                            HWLOC_TOPOLOGY_FLAG_IO_DEVICES)) ||
             0 != hwloc_topology_load(opal_hwloc_topology)) {
@@ -257,13 +265,12 @@ int opal_hwloc_base_get_topology(void)
     /* fill opal_cache_line_size global with the smallest L1 cache
        line size */
     fill_cache_line_size();
+
     return rc;
 }
 
 int opal_hwloc_base_set_topology(char *topofile)
 {
-    hwloc_obj_t obj;
-    unsigned j, k;
     struct hwloc_topology_support *support;
     int rc;
 
@@ -298,30 +305,7 @@ int opal_hwloc_base_set_topology(char *topofile)
                              "hwloc:base:set_topology failed to load"));
         return OPAL_ERR_NOT_SUPPORTED;
     }
-    /* remove the hostname from the topology. Unfortunately, hwloc
-     * decided to add the source hostname to the "topology", thus
-     * rendering it unusable as a pure topological description. So
-     * we remove that information here.
-     */
-    obj = hwloc_get_root_obj(opal_hwloc_topology);
-    for (k=0; k < obj->infos_count; k++) {
-        if (NULL == obj->infos[k].name ||
-            NULL == obj->infos[k].value) {
-            continue;
-        }
-        if (0 == strncmp(obj->infos[k].name, "HostName", strlen("HostName"))) {
-            free(obj->infos[k].name);
-            free(obj->infos[k].value);
-            /* left justify the array */
-            for (j=k; j < obj->infos_count-1; j++) {
-                obj->infos[j] = obj->infos[j+1];
-            }
-            obj->infos[obj->infos_count-1].name = NULL;
-            obj->infos[obj->infos_count-1].value = NULL;
-            obj->infos_count--;
-            break;
-        }
-    }
+
     /* unfortunately, hwloc does not include support info in its
      * xml output :-(( We default to assuming it is present as
      * systems that use this option are likely to provide
@@ -396,8 +380,8 @@ void opal_hwloc_base_get_local_cpuset(void)
         }
 
         /* get the cpus we are bound to */
-        if (hwloc_get_cpubind(opal_hwloc_topology, 
-                              opal_hwloc_my_cpuset, 
+        if (hwloc_get_cpubind(opal_hwloc_topology,
+                              opal_hwloc_my_cpuset,
                               HWLOC_CPUBIND_PROCESS) < 0) {
             /* we are not bound - use the root's available cpuset */
             root = hwloc_get_root_obj(opal_hwloc_topology);
@@ -508,6 +492,13 @@ static void df_search_cores(hwloc_obj_t obj, unsigned int *cnt)
             obj->userdata = (void*)data;
         }
         if (NULL == opal_hwloc_base_cpu_set) {
+            if (!hwloc_bitmap_intersects(obj->cpuset, obj->allowed_cpuset)) {
+                /*
+                 * do not count not allowed cores (e.g. cores with zero allowed PU)
+                 * if SMT is enabled, do count cores with at least one allowed hwthread
+                 */
+                return;
+            }
             data->npus = 1;
         }
         *cnt += data->npus;
@@ -776,7 +767,7 @@ static hwloc_obj_t df_search(hwloc_topology_t topo,
             return obj;
         }
     }
-    
+
     return NULL;
 }
 
@@ -881,7 +872,7 @@ static hwloc_obj_t df_search_min_bound(hwloc_topology_t topo,
             data = OBJ_NEW(opal_hwloc_obj_data_t);
             start->userdata = data;
         }
-            
+
         OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
                              "hwloc:base:min_bound_under_obj object %s:%u nbound %u min %u",
                              hwloc_obj_type_string(target), start->logical_index,
@@ -909,7 +900,7 @@ static hwloc_obj_t df_search_min_bound(hwloc_topology_t topo,
             break;
         }
     }
-    
+
     return save;
 }
 
@@ -1075,7 +1066,7 @@ static int socket_to_cpu_set(char *cpus,
         res = opal_hwloc_base_get_available_cpus(topo, obj);
         hwloc_bitmap_or(cpumask, cpumask, res);
         break;
-                
+
     case 2:  /* range of sockets was given */
         lower_range = atoi(range[0]);
         upper_range = atoi(range[1]);
@@ -1115,7 +1106,7 @@ static int socket_core_to_cpu_set(char *socket_core_list,
 
     socket_core = opal_argv_split(socket_core_list, ':');
     socket_id = atoi(socket_core[0]);
-    
+
     /* get the object for this socket id */
     if (NULL == (socket = opal_hwloc_base_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0,
                                                           socket_id, rtype))) {
@@ -1159,6 +1150,9 @@ static int socket_core_to_cpu_set(char *socket_core_list,
                     if (NULL == (core = df_search(topo, socket, obj_type, 0,
                                                   core_id, OPAL_HWLOC_AVAILABLE,
                                                   &idx, NULL))) {
+                        opal_argv_free(list);
+                        opal_argv_free(range);
+                        opal_argv_free(socket_core);
                         return OPAL_ERR_NOT_FOUND;
                     }
                     /* get the cpus */
@@ -1167,7 +1161,7 @@ static int socket_core_to_cpu_set(char *socket_core_list,
                 }
                 opal_argv_free(list);
                 break;
-                
+
             case 2:  /* range of core id's was given */
                 opal_output_verbose(5, opal_hwloc_base_framework.framework_output,
                                     "range of cores given: start %s stop %s",
@@ -1180,6 +1174,8 @@ static int socket_core_to_cpu_set(char *socket_core_list,
                     if (NULL == (core = df_search(topo, socket, obj_type, 0,
                                                   core_id, OPAL_HWLOC_AVAILABLE,
                                                   &idx, NULL))) {
+                        opal_argv_free(range);
+                        opal_argv_free(socket_core);
                         return OPAL_ERR_NOT_FOUND;
                     }
                     /* get the cpus */
@@ -1188,7 +1184,7 @@ static int socket_core_to_cpu_set(char *socket_core_list,
                     hwloc_bitmap_or(cpumask, cpumask, res);
                 }
                 break;
-                
+
             default:
                 opal_argv_free(range);
                 opal_argv_free(socket_core);
@@ -1222,14 +1218,14 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
     if (NULL == slot_str || 0 == strlen(slot_str)) {
         return OPAL_ERR_BAD_PARAM;
     }
-    
+
     opal_output_verbose(5, opal_hwloc_base_framework.framework_output,
                         "slot assignment: slot_list == %s",
                         slot_str);
 
     /* split at ';' */
     item = opal_argv_split(slot_str, ';');
-    
+
     /* start with a clean mask */
     hwloc_bitmap_zero(cpumask);
     /* loop across the items and accumulate the mask */
@@ -1301,6 +1297,7 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
                             opal_argv_free(range);
                             opal_argv_free(item);
                             opal_argv_free(rngs);
+                            opal_argv_free(list);
                             return OPAL_ERR_SILENT;
                         }
                         /* get the available cpus for that object */
@@ -1310,7 +1307,7 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
                     }
                     opal_argv_free(list);
                     break;
-                    
+
                 case 2:  /* range of core id's was given */
                     lower_range = atoi(range[0]);
                     upper_range = atoi(range[1]);
@@ -1328,13 +1325,14 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
                         hwloc_bitmap_or(cpumask, cpumask, pucpus);
                     }
                     break;
-                
+
                 default:
                     opal_argv_free(range);
                     opal_argv_free(item);
                     opal_argv_free(rngs);
                     return OPAL_ERROR;
                 }
+                opal_argv_free(range);
             }
             opal_argv_free(rngs);
         }
@@ -1359,7 +1357,7 @@ opal_hwloc_locality_t opal_hwloc_base_get_relative_locality(hwloc_topology_t top
      * NOTE: we may alter that latter part as hwloc's ability to
      * sense multi-cu, multi-cluster systems grows
      */
-    locality = OPAL_PROC_ON_NODE;
+    locality = OPAL_PROC_ON_NODE | OPAL_PROC_ON_HOST | OPAL_PROC_ON_CU | OPAL_PROC_ON_CLUSTER;
 
     /* if either cpuset is NULL, then that isn't bound */
     if (NULL == cpuset1 || NULL == cpuset2) {
@@ -1407,25 +1405,25 @@ opal_hwloc_locality_t opal_hwloc_base_get_relative_locality(hwloc_topology_t top
                 shared = true;
                 switch(obj->type) {
                 case HWLOC_OBJ_NODE:
-                    locality = OPAL_PROC_ON_NUMA;
+                    locality |= OPAL_PROC_ON_NUMA;
                     break;
                 case HWLOC_OBJ_SOCKET:
-                    locality = OPAL_PROC_ON_SOCKET;
+                    locality |= OPAL_PROC_ON_SOCKET;
                     break;
                 case HWLOC_OBJ_CACHE:
                     if (3 == obj->attr->cache.depth) {
-                        locality = OPAL_PROC_ON_L3CACHE;
+                        locality |= OPAL_PROC_ON_L3CACHE;
                     } else if (2 == obj->attr->cache.depth) {
-                        locality = OPAL_PROC_ON_L2CACHE;
+                        locality |= OPAL_PROC_ON_L2CACHE;
                     } else {
-                        locality = OPAL_PROC_ON_L1CACHE;
+                        locality |= OPAL_PROC_ON_L1CACHE;
                     }
                     break;
                 case HWLOC_OBJ_CORE:
-                    locality = OPAL_PROC_ON_CORE;
+                    locality |= OPAL_PROC_ON_CORE;
                     break;
                 case HWLOC_OBJ_PU:
-                    locality = OPAL_PROC_ON_HWTHREAD;
+                    locality |= OPAL_PROC_ON_HWTHREAD;
                     break;
                 default:
                     /* just ignore it */
@@ -1515,7 +1513,7 @@ static char *hwloc_getline(FILE *fp)
 	   buff = strdup(input);
 	   return buff;
     }
-    
+
     return NULL;
 }
 
@@ -1664,6 +1662,7 @@ static char *bitmap2rangestr(int bitmap)
                 /* A range just ended; output it */
                 if (!first) {
                     strncat(ret, ",", sizeof(ret) - strlen(ret) - 1);
+                } else {
                     first = false;
                 }
 
@@ -1709,29 +1708,26 @@ static char *bitmap2rangestr(int bitmap)
 /*
  * Make a map of socket/core/hwthread tuples
  */
-static int build_map(int *num_sockets_arg, int *num_cores_arg, 
+static int build_map(int *num_sockets_arg, int *num_cores_arg,
                      hwloc_cpuset_t cpuset, int ***map, hwloc_topology_t topo)
 {
-    static int num_sockets = -1, num_cores = -1;
+    int num_sockets, num_cores;
     int socket_index, core_index, pu_index;
     hwloc_obj_t socket, core, pu;
     int **data;
 
-    /* Find out how many sockets we have (cached so that we don't have
-       to look this up every time) */
-    if (num_sockets < 0) {
-        num_sockets = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_SOCKET);
-        /* some systems (like the iMac) only have one
-         * socket and so don't report a socket
-         */
-        if (0 == num_sockets) {
-            num_sockets = 1;
-        }
-        /* Lazy: take the total number of cores that we have in the
-           topology; that'll be more than the max number of cores
-           under any given socket */
-        num_cores = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_CORE);
+    /* Find out how many sockets we have */
+    num_sockets = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_SOCKET);
+    /* some systems (like the iMac) only have one
+     * socket and so don't report a socket
+     */
+    if (0 == num_sockets) {
+        num_sockets = 1;
     }
+    /* Lazy: take the total number of cores that we have in the
+       topology; that'll be more than the max number of cores
+       under any given socket */
+    num_cores = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_CORE);
     *num_sockets_arg = num_sockets;
     *num_cores_arg = num_cores;
 
@@ -1753,11 +1749,11 @@ static int build_map(int *num_sockets_arg, int *num_cores_arg,
        the socket/core/pu triples */
     for (pu_index = 0,
              pu = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                      cpuset, HWLOC_OBJ_PU, 
+                                                      cpuset, HWLOC_OBJ_PU,
                                                       pu_index);
          NULL != pu;
          pu = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                  cpuset, HWLOC_OBJ_PU, 
+                                                  cpuset, HWLOC_OBJ_PU,
                                                   ++pu_index)) {
         /* Go upward and find the core this PU belongs to */
         core = pu;
@@ -1800,7 +1796,7 @@ int opal_hwloc_base_cset2str(char *str, int len,
     int ret, socket_index, core_index;
     char tmp[BUFSIZ];
     const int stmp = sizeof(tmp) - 1;
-    int **map;
+    int **map=NULL;
     hwloc_obj_t root;
     opal_hwloc_topo_data_t *sum;
 
@@ -1815,19 +1811,19 @@ int opal_hwloc_base_cset2str(char *str, int len,
     root = hwloc_get_root_obj(topo);
     if (NULL == root->userdata) {
         opal_hwloc_base_filter_cpus(topo);
-    }
-    sum = (opal_hwloc_topo_data_t*)root->userdata;
-    if (NULL == sum->available) {
-       return OPAL_ERROR;
-    }
-    if (0 != hwloc_bitmap_isincluded(sum->available, cpuset)) {
-        return OPAL_ERR_NOT_BOUND;
+    } else {
+        sum = (opal_hwloc_topo_data_t*)root->userdata;
+        if (NULL == sum->available) {
+           return OPAL_ERROR;
+        }
+        if (0 != hwloc_bitmap_isincluded(sum->available, cpuset)) {
+            return OPAL_ERR_NOT_BOUND;
+        }
     }
 
     if (OPAL_SUCCESS != (ret = build_map(&num_sockets, &num_cores, cpuset, &map, topo))) {
         return ret;
     }
-
     /* Iterate over the data matrix and build up the string */
     first = true;
     for (socket_index = 0; socket_index < num_sockets; ++socket_index) {
@@ -1838,21 +1834,25 @@ int opal_hwloc_base_cset2str(char *str, int len,
                 }
                 first = false;
 
-                snprintf(tmp, stmp, "socket %d[core %d[hwt %s]]", 
+                snprintf(tmp, stmp, "socket %d[core %d[hwt %s]]",
                          socket_index, core_index,
                          bitmap2rangestr(map[socket_index][core_index]));
                 strncat(str, tmp, len - strlen(str));
             }
         }
     }
-    free(map[0]);
-    free(map);
+    if (NULL != map) {
+        if (NULL != map[0]) {
+            free(map[0]);
+        }
+        free(map);
+    }
 
     return OPAL_SUCCESS;
 }
 
 /*
- * Make a prettyprint string for a cset in a map format.  
+ * Make a prettyprint string for a cset in a map format.
  * Example: [B./..]
  * Key:  [] - signifies socket
  *        / - divider between cores
@@ -1881,29 +1881,30 @@ int opal_hwloc_base_cset2mapstr(char *str, int len,
     root = hwloc_get_root_obj(topo);
     if (NULL == root->userdata) {
         opal_hwloc_base_filter_cpus(topo);
-    }
-    sum = (opal_hwloc_topo_data_t*)root->userdata;
-    if (NULL == sum->available) {
-       return OPAL_ERROR;
-    }
-    if (0 != hwloc_bitmap_isincluded(sum->available, cpuset)) {
-        return OPAL_ERR_NOT_BOUND;
+    } else {
+        sum = (opal_hwloc_topo_data_t*)root->userdata;
+        if (NULL == sum->available) {
+           return OPAL_ERROR;
+        }
+        if (0 != hwloc_bitmap_isincluded(sum->available, cpuset)) {
+            return OPAL_ERR_NOT_BOUND;
+        }
     }
 
     /* Iterate over all existing sockets */
     for (socket = hwloc_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0);
-         NULL != socket; 
+         NULL != socket;
          socket = socket->next_cousin) {
         strncat(str, "[", len - strlen(str));
 
         /* Iterate over all existing cores in this socket */
         core_index = 0;
         for (core = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                        socket->cpuset, 
+                                                        socket->cpuset,
                                                         HWLOC_OBJ_CORE, core_index);
-             NULL != core; 
+             NULL != core;
              core = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                        socket->cpuset, 
+                                                        socket->cpuset,
                                                         HWLOC_OBJ_CORE, ++core_index)) {
             if (core_index > 0) {
                 strncat(str, "/", len - strlen(str));
@@ -1912,11 +1913,11 @@ int opal_hwloc_base_cset2mapstr(char *str, int len,
             /* Iterate over all existing PUs in this core */
             pu_index = 0;
             for (pu = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                          core->cpuset, 
+                                                          core->cpuset,
                                                           HWLOC_OBJ_PU, pu_index);
-                 NULL != pu; 
+                 NULL != pu;
                  pu = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                          core->cpuset, 
+                                                          core->cpuset,
                                                           HWLOC_OBJ_PU, ++pu_index)) {
 
                 /* Is this PU in the cpuset? */
@@ -1935,8 +1936,8 @@ int opal_hwloc_base_cset2mapstr(char *str, int len,
 
 static int dist_cmp_fn (opal_list_item_t **a, opal_list_item_t **b)
 {
-    orte_rmaps_numa_node_t *aitem = *((orte_rmaps_numa_node_t **) a);
-    orte_rmaps_numa_node_t *bitem = *((orte_rmaps_numa_node_t **) b);
+    opal_rmaps_numa_node_t *aitem = *((opal_rmaps_numa_node_t **) a);
+    opal_rmaps_numa_node_t *bitem = *((opal_rmaps_numa_node_t **) b);
 
     if (aitem->dist_from_closed > bitem->dist_from_closed) {
         return 1;
@@ -1952,7 +1953,7 @@ static void sort_by_dist(hwloc_topology_t topo, char* device_name, opal_list_t *
     hwloc_obj_t device_obj = NULL;
     hwloc_obj_t obj = NULL, root = NULL;
     const struct hwloc_distances_s* distances;
-    orte_rmaps_numa_node_t *numa_node;
+    opal_rmaps_numa_node_t *numa_node;
     int close_node_index;
     float latency;
     unsigned int j;
@@ -2009,7 +2010,7 @@ static void sort_by_dist(hwloc_topology_t topo, char* device_name, opal_list_t *
                 /* fill list of numa nodes */
                 for (j = 0; j < distances->nbobjs; j++) {
                     latency = distances->latency[close_node_index + distances->nbobjs * j];
-                    numa_node = OBJ_NEW(orte_rmaps_numa_node_t);
+                    numa_node = OBJ_NEW(opal_rmaps_numa_node_t);
                     numa_node->index = j;
                     numa_node->dist_from_closed = latency;
                     opal_list_append(sorted_list, &numa_node->super);
@@ -2022,15 +2023,15 @@ static void sort_by_dist(hwloc_topology_t topo, char* device_name, opal_list_t *
     }
 }
 
-static int find_devices(hwloc_topology_t topo, char* device_name) 
+static int find_devices(hwloc_topology_t topo, char** device_name)
 {
     hwloc_obj_t device_obj = NULL;
     int count = 0;
     for (device_obj = hwloc_get_obj_by_type(topo, HWLOC_OBJ_OS_DEVICE, 0); device_obj; device_obj = hwloc_get_next_osdev(topo, device_obj)) {
         if (device_obj->attr->osdev.type == HWLOC_OBJ_OSDEV_OPENFABRICS) {
             count++;
-            free(device_name);
-            device_name = strdup(device_obj->name);
+            free(*device_name);
+            *device_name = strdup(device_obj->name);
         }
     }
     return count;
@@ -2042,7 +2043,7 @@ int opal_hwloc_get_sorted_numa_list(hwloc_topology_t topo, char* device_name, op
     opal_list_item_t *item;
     opal_hwloc_summary_t *sum;
     opal_hwloc_topo_data_t *data;
-    orte_rmaps_numa_node_t *numa, *copy_numa;
+    opal_rmaps_numa_node_t *numa, *copy_numa;
     int count;
 
     obj = hwloc_get_root_obj(topo);
@@ -2056,9 +2057,9 @@ int opal_hwloc_get_sorted_numa_list(hwloc_topology_t topo, char* device_name, op
                 item = opal_list_get_next(item)) {
             sum = (opal_hwloc_summary_t*)item;
             if (HWLOC_OBJ_NODE == sum->type) {
-                if (opal_list_get_size(&sum->sorted_by_dist_list) > 0) { 
-                    OPAL_LIST_FOREACH(numa, &(sum->sorted_by_dist_list), orte_rmaps_numa_node_t) {
-                        copy_numa = OBJ_NEW(orte_rmaps_numa_node_t);
+                if (opal_list_get_size(&sum->sorted_by_dist_list) > 0) {
+                    OPAL_LIST_FOREACH(numa, &(sum->sorted_by_dist_list), opal_rmaps_numa_node_t) {
+                        copy_numa = OBJ_NEW(opal_rmaps_numa_node_t);
                         copy_numa->index = numa->index;
                         copy_numa->dist_from_closed = numa->dist_from_closed;
                         opal_list_append(sorted_list, &copy_numa->super);
@@ -2068,18 +2069,22 @@ int opal_hwloc_get_sorted_numa_list(hwloc_topology_t topo, char* device_name, op
                     /* don't already know it - go get it */
                     /* firstly we check if we need to autodetect OpenFabrics  devices or we have the specified one */
                     if (!strcmp(device_name, "auto")) {
-                        count = find_devices(topo, device_name);
-                       if (count > 1) {
-                           return count;
-                       }
+                        count = find_devices(topo, &device_name);
+                        if (count > 1) {
+                            free(device_name);
+                            return count;
+                        }
                     }
-                    if (!device_name || (strlen(device_name) == 0)) {
+                    if (!device_name) {
+                        return OPAL_ERR_NOT_FOUND;
+                    } else if (strlen(device_name) == 0) {
+                        free(device_name);
                         return OPAL_ERR_NOT_FOUND;
                     }
                     sort_by_dist(topo, device_name, sorted_list);
                     /* store this info in summary object for later usage */
-                    OPAL_LIST_FOREACH(numa, sorted_list, orte_rmaps_numa_node_t) {
-                        copy_numa = OBJ_NEW(orte_rmaps_numa_node_t);
+                    OPAL_LIST_FOREACH(numa, sorted_list, opal_rmaps_numa_node_t) {
+                        copy_numa = OBJ_NEW(opal_rmaps_numa_node_t);
                         copy_numa->index = numa->index;
                         copy_numa->dist_from_closed = numa->dist_from_closed;
                         opal_list_append(&(sum->sorted_by_dist_list), &copy_numa->super);
@@ -2098,7 +2103,7 @@ char* opal_hwloc_base_get_topo_signature(hwloc_topology_t topo)
     char *sig=NULL, *arch=NULL;
     hwloc_obj_t obj;
     unsigned i;
-    
+
     nnuma = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_NODE, 0, OPAL_HWLOC_AVAILABLE);
     nsocket = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_SOCKET, 0, OPAL_HWLOC_AVAILABLE);
     nl3 = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_CACHE, 3, OPAL_HWLOC_AVAILABLE);
