@@ -14,6 +14,7 @@
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2010-2014 Los Alamos National Security, LLC.
  *                         All rights reserved.
+ * Copyright (c) 2016      University of Houston. All rights reserved.
  *
  * $COPYRIGHT$
  *
@@ -37,19 +38,16 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif /* HAVE_SYS_TYPES_H */
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif /* HAVE_STRING_H */
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif /* HAVE_NETDB_H */
-#ifdef HAVE_TIME_H
 #include <time.h>
-#endif /* HAVE_NETDB_H */
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif /* HAVE_SYS_STAT_H */
 
+#include "opal_stdint.h"
 #include "opal/constants.h"
 #include "opal/util/alfg.h"
 #include "opal/util/output.h"
@@ -89,6 +87,12 @@ segment_unlink(opal_shmem_ds_t *ds_buf);
 static int
 module_finalize(void);
 
+static int
+enough_space(const char *filename,
+             size_t space_req,
+             uint64_t *space_avail,
+             bool *result);
+
 /*
  * mmap shmem module
  */
@@ -115,13 +119,12 @@ opal_shmem_mmap_module_t opal_shmem_mmap_module = {
 static inline void
 shmem_ds_reset(opal_shmem_ds_t *ds_buf)
 {
+    /* don't print ds_buf info here, as we may be printing garbage. */
     OPAL_OUTPUT_VERBOSE(
         (70, opal_shmem_base_framework.framework_output,
-         "%s: %s: shmem_ds_resetting "
-         "(id: %d, size:  %lu, name: %s)\n",
+         "%s: %s: shmem_ds_resetting\n",
          mca_shmem_mmap_component.super.base_version.mca_type_name,
-         mca_shmem_mmap_component.super.base_version.mca_component_name,
-         ds_buf->seg_id, (unsigned long)ds_buf->seg_size, ds_buf->seg_name)
+         mca_shmem_mmap_component.super.base_version.mca_component_name)
     );
 
     ds_buf->seg_cpid = 0;
@@ -132,6 +135,59 @@ shmem_ds_reset(opal_shmem_ds_t *ds_buf)
     ds_buf->seg_base_addr = (unsigned char *)MAP_FAILED;
 }
 
+/* ////////////////////////////////////////////////////////////////////////// */
+static int
+enough_space(const char *filename,
+             size_t space_req,
+             uint64_t *space_avail,
+             bool *result)
+{
+    uint64_t avail = 0;
+    size_t fluff = (size_t)(.05 * space_req);
+    bool enough = false;
+    char *last_sep = NULL;
+    /* the target file name is passed here, but we need to check the parent
+     * directory. store it so we can extract that info later. */
+    char *target_dir = strdup(filename);
+    int rc;
+
+    if (NULL == target_dir) {
+        rc = OPAL_ERR_OUT_OF_RESOURCE;
+        goto out;
+    }
+    /* get the parent directory */
+    last_sep = strrchr(target_dir, OPAL_PATH_SEP[0]);
+    *last_sep = '\0';
+    /* now check space availability */
+    if (OPAL_SUCCESS != (rc = opal_path_df(target_dir, &avail))) {
+        OPAL_OUTPUT_VERBOSE(
+            (70, opal_shmem_base_framework.framework_output,
+             "WARNING: opal_path_df failure!")
+        );
+        goto out;
+    }
+    /* do we have enough space? */
+    if (avail >= space_req + fluff) {
+        enough = true;
+    }
+    else {
+        OPAL_OUTPUT_VERBOSE(
+            (70, opal_shmem_base_framework.framework_output,
+             "WARNING: not enough space on %s to meet request!"
+             "available: %"PRIu64 "requested: %lu", target_dir,
+             avail, (unsigned long)space_req + fluff)
+        );
+    }
+
+out:
+    if (NULL != target_dir) {
+        free(target_dir);
+    }
+    *result = enough;
+    *space_avail = avail;
+    return rc;
+}
+
 /* ////////////////////////////////////////////////////////////////////////// */
 static int
 module_init(void)
@@ -244,6 +300,9 @@ segment_create(opal_shmem_ds_t *ds_buf,
     int rc = OPAL_SUCCESS;
     char *real_file_name = NULL;
     pid_t my_pid = getpid();
+    bool space_available = false;
+    uint64_t amount_space_avail = 0;
+
     /* the real size of the shared memory segment.  this includes enough space
      * to store our segment header.
      */
@@ -305,13 +364,34 @@ segment_create(opal_shmem_ds_t *ds_buf,
      * this is an important check because if the backing store is located on
      * a network filesystem, the user may see a shared memory performance hit.
      */
-    if (opal_shmem_mmap_nfs_warning && opal_path_nfs(real_file_name)) {
+    if (opal_shmem_mmap_nfs_warning && opal_path_nfs(real_file_name, NULL)) {
         char hn[MAXHOSTNAMELEN];
         gethostname(hn, MAXHOSTNAMELEN - 1);
         hn[MAXHOSTNAMELEN - 1] = '\0';
         opal_show_help("help-opal-shmem-mmap.txt", "mmap on nfs", 1, hn,
                        real_file_name);
     }
+    /* let's make sure we have enough space for the backing file */
+    if (OPAL_SUCCESS != (rc = enough_space(real_file_name,
+                                           real_size,
+                                           &amount_space_avail,
+                                           &space_available))) {
+        opal_output(0, "shmem: mmap: an error occurred while determining "
+                    "whether or not %s could be created.", real_file_name);
+        /* rc is set */
+        goto out;
+    }
+    if (!space_available) {
+        char hn[MAXHOSTNAMELEN];
+        gethostname(hn, MAXHOSTNAMELEN - 1);
+        hn[MAXHOSTNAMELEN - 1] = '\0';
+        rc = OPAL_ERR_OUT_OF_RESOURCE;
+        opal_show_help("help-opal-shmem-mmap.txt", "target full", 1,
+                       real_file_name, hn, (unsigned long)real_size,
+                       (unsigned long long)amount_space_avail);
+        goto out;
+    }
+    /* enough space is available, so create the segment */
     if (-1 == (ds_buf->seg_id = open(real_file_name, O_CREAT | O_RDWR, 0600))) {
         int err = errno;
         char hn[MAXHOSTNAMELEN];
