@@ -11,7 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2007-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2010-2013 Los Alamos National Security, LLC.
+ * Copyright (c) 2010-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * Copyright (c) 2014      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
@@ -42,9 +42,7 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif /* HAVE_SYS_TYPES_H */
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif /* HAVE_STRING_H */
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif /* HAVE_NETDB_H */
@@ -112,13 +110,12 @@ opal_shmem_posix_module_t opal_shmem_posix_module = {
 static inline void
 shmem_ds_reset(opal_shmem_ds_t *ds_buf)
 {
+    /* don't print ds_buf info here, as we may be printing garbage. */
     OPAL_OUTPUT_VERBOSE(
         (70, opal_shmem_base_framework.framework_output,
-         "%s: %s: shmem_ds_resetting "
-         "(id: %d, size: %lu, name: %s)\n",
+         "%s: %s: shmem_ds_resetting\n",
          mca_shmem_posix_component.super.base_version.mca_type_name,
-         mca_shmem_posix_component.super.base_version.mca_component_name,
-         ds_buf->seg_id, (unsigned long)ds_buf->seg_size, ds_buf->seg_name)
+         mca_shmem_posix_component.super.base_version.mca_component_name)
     );
 
     ds_buf->seg_cpid = 0;
