@@ -10,12 +10,14 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2016 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2010-2013 Los Alamos National Security, LLC.
+ * Copyright (c) 2010-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2014 Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2016 Intel, Inc. All rights reserved
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -33,8 +35,10 @@
 
 #include "opal/util/malloc.h"
 #include "opal/util/arch.h"
+#include "opal/util/opal_environ.h"
 #include "opal/util/output.h"
 #include "opal/util/show_help.h"
+#include "opal/util/proc.h"
 #include "opal/memoryhooks/memory.h"
 #include "opal/mca/base/base.h"
 #include "opal/runtime/opal.h"
@@ -42,6 +46,7 @@
 #include "opal/datatype/opal_datatype.h"
 #include "opal/mca/installdirs/base/base.h"
 #include "opal/mca/memory/base/base.h"
+#include "opal/mca/patcher/base/base.h"
 #include "opal/mca/memcpy/base/base.h"
 #include "opal/mca/hwloc/base/base.h"
 #include "opal/mca/sec/base/base.h"
@@ -49,12 +54,7 @@
 #include "opal/mca/memchecker/base/base.h"
 #include "opal/dss/dss.h"
 #include "opal/mca/shmem/base/base.h"
-#if OPAL_ENABLE_FT_CR    == 1
-#include "opal/mca/compress/base/base.h"
-#endif
-
-#include "opal/runtime/opal_cr.h"
-#include "opal/mca/crs/base/base.h"
+#include "opal/threads/threads.h"
 
 #include "opal/runtime/opal_progress.h"
 #include "opal/mca/event/base/base.h"
@@ -80,6 +80,7 @@ int opal_util_initialized = 0;
    hwloc is available, this value will be overwritten when the
    hwloc data is loaded. */
 int opal_cache_line_size = 128;
+bool opal_warn_on_fork = true;
 
 static int
 opal_err2str(int errnum, const char **errmsg)
@@ -114,7 +115,7 @@ opal_err2str(int errnum, const char **errmsg)
     case OPAL_ERR_NOT_SUPPORTED:
         retval = "Not supported";
         break;
-    case OPAL_ERR_INTERUPTED:
+    case OPAL_ERR_INTERRUPTED:
         retval = "Interrupted";
         break;
     case OPAL_ERR_WOULD_BLOCK:
@@ -237,6 +238,12 @@ opal_err2str(int errnum, const char **errmsg)
     case OPAL_ERR_AUTHENTICATION_FAILED:
         retval = "Authentication failed";
         break;
+    case OPAL_ERR_COMM_FAILURE:
+        retval = "Comm failure";
+        break;
+    case OPAL_ERR_SERVER_NOT_AVAIL:
+        retval = "Server not available";
+        break;
     default:
         retval = NULL;
     }
@@ -246,11 +253,40 @@ opal_err2str(int errnum, const char **errmsg)
 }
 
 
+int opal_init_psm(void)
+{
+    /* Very early in the init sequence -- before *ANY* MCA components
+       are opened -- we need to disable some behavior from the PSM and
+       PSM2 libraries (by default): at least some old versions of
+       these libraries hijack signal handlers during their library
+       constructors and then do not un-hijack them when the libraries
+       are unloaded.
+
+       It is a bit of an abstraction break that we have to put
+       vendor/transport-specific code in the OPAL core, but we're
+       out of options, unfortunately.
+
+       NOTE: We only disable this behavior if the corresponding
+       environment variables are not already set (i.e., if the
+       user/environment has indicated a preference for this behavior,
+       we won't override it). */
+    if (NULL == getenv("IPATH_NO_BACKTRACE")) {
+        opal_setenv("IPATH_NO_BACKTRACE", "1", true, &environ);
+    }
+    if (NULL == getenv("HFI_NO_BACKTRACE")) {
+        opal_setenv("HFI_NO_BACKTRACE", "1", true, &environ);
+    }
+
+    return OPAL_SUCCESS;
+}
+
+
 int
 opal_init_util(int* pargc, char*** pargv)
 {
     int ret;
     char *error = NULL;
+    char hostname[512];
 
     if( ++opal_util_initialized != 1 ) {
         if( opal_util_initialized < 1 ) {
@@ -271,6 +307,13 @@ opal_init_util(int* pargc, char*** pargv)
 
     opal_init_called = true;
 
+    /* set the nodename right away so anyone who needs it has it. Note
+     * that we don't bother with fqdn and prefix issues here - we let
+     * the RTE later replace this with a modified name if the user
+     * requests it */
+    gethostname(hostname, 512);
+    opal_process_info.nodename = strdup(hostname);
+
     /* initialize the memory allocator */
     opal_malloc_init();
 
@@ -283,12 +326,12 @@ opal_init_util(int* pargc, char*** pargv)
                 __FILE__, __LINE__, ret);
         return ret;
     }
-    
+
     /* initialize the help system */
     opal_show_help_init();
 
     /* register handler for errnum -> string converstion */
-    if (OPAL_SUCCESS != 
+    if (OPAL_SUCCESS !=
         (ret = opal_error_register("OPAL",
                                    OPAL_ERR_BASE, OPAL_ERR_MAX, opal_err2str))) {
         error = "opal_error_register";
@@ -301,9 +344,13 @@ opal_init_util(int* pargc, char*** pargv)
         goto return_error;
     }
 
+    // Disable PSM signal hijacking (see comment in function for more
+    // details)
+    opal_init_psm();
+
     /* Setup the parameter system */
-    if (OPAL_SUCCESS != (ret = mca_base_param_init())) {
-        error = "mca_base_param_init";
+    if (OPAL_SUCCESS != (ret = mca_base_var_init())) {
+        error = "mca_base_var_init";
         goto return_error;
     }
 
@@ -334,6 +381,12 @@ opal_init_util(int* pargc, char*** pargv)
         return OPAL_ERR_SILENT;
     }
 
+    /* initialize the arch string */
+    if (OPAL_SUCCESS != (ret = opal_arch_init ())) {
+        error = "opal_arch_init";
+        goto return_error;
+    }
+
     /* initialize the datatype engine */
     if (OPAL_SUCCESS != (ret = opal_datatype_init ())) {
         error = "opal_datatype_init";
@@ -346,8 +399,9 @@ opal_init_util(int* pargc, char*** pargv)
         goto return_error;
     }
 
-    if (OPAL_SUCCESS != (ret = mca_base_framework_register(&opal_event_base_framework, 0))) {
-        error = "opal_event_register";
+    /* initialize the mca */
+    if (OPAL_SUCCESS != (ret = mca_base_open())) {
+        error = "mca_base_open";
         goto return_error;
     }
 
@@ -381,12 +435,6 @@ opal_init(int* pargc, char*** pargv)
         return ret;
     }
 
-    /* initialize the mca */
-    if (OPAL_SUCCESS != (ret = mca_base_open())) {
-        error = "mca_base_open";
-        goto return_error;
-    }
-
     /* open hwloc - since this is a static framework, no
      * select is required
      */
@@ -442,9 +490,7 @@ opal_init(int* pargc, char*** pargv)
     }
 
     /*
-     * Need to start the event and progress engines if noone else is.
-     * opal_cr_init uses the progress engine, so it is lumped together
-     * into this set as well.
+     * Need to start the event and progress engines if none else is.
      */
     /*
      * Initialize the event library
@@ -453,7 +499,7 @@ opal_init(int* pargc, char*** pargv)
         error = "opal_event_base_open";
         goto return_error;
     }
-            
+
     /*
      * Initialize the general progress engine
      */
@@ -475,34 +521,6 @@ opal_init(int* pargc, char*** pargv)
         goto return_error;
     }
 
-#if OPAL_ENABLE_FT_CR    == 1
-    /*
-     * Initialize the compression framework
-     * Note: Currently only used in C/R so it has been marked to only
-     *       initialize when C/R is enabled. If other places in the code
-     *       wish to use this framework, it is safe to remove the protection.
-     */
-    if( OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_compress_base_framework, 0)) ) {
-        error = "opal_compress_base_open";
-        goto return_error;
-    }
-    if( OPAL_SUCCESS != (ret = opal_compress_base_select()) ) {
-        error = "opal_compress_base_select";
-        goto return_error;
-    }
-#endif
-
-    /*
-     * Initalize the checkpoint/restart functionality
-     * Note: Always do this so we can detect if the user
-     * attempts to checkpoint a non checkpointable job,
-     * otherwise the tools may hang or not clean up properly.
-     */
-    if (OPAL_SUCCESS != (ret = opal_cr_init() ) ) {
-        error = "opal_cr_init";
-        goto return_error;
-    }
-    
     /* initialize the security framework */
     if( OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_sec_base_framework, 0)) ) {
         error = "opal_sec_base_open";
@@ -522,3 +540,106 @@ opal_init(int* pargc, char*** pargv)
     return ret;
 }
 
+int opal_init_test(void)
+{
+    int ret;
+    char *error;
+
+    /* initialize the memory allocator */
+    opal_malloc_init();
+
+    /* initialize the output system */
+    opal_output_init();
+
+    /* initialize install dirs code */
+    if (OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_installdirs_base_framework, 0))) {
+        fprintf(stderr, "opal_installdirs_base_open() failed -- process will likely abort (%s:%d, returned %d instead of OPAL_SUCCESS)\n",
+                __FILE__, __LINE__, ret);
+        return ret;
+    }
+
+    /* initialize the help system */
+    opal_show_help_init();
+
+    /* register handler for errnum -> string converstion */
+    if (OPAL_SUCCESS !=
+        (ret = opal_error_register("OPAL",
+                                   OPAL_ERR_BASE, OPAL_ERR_MAX, opal_err2str))) {
+        error = "opal_error_register";
+        goto return_error;
+    }
+
+    /* keyval lex-based parser */
+    if (OPAL_SUCCESS != (ret = opal_util_keyval_parse_init())) {
+        error = "opal_util_keyval_parse_init";
+        goto return_error;
+    }
+
+    if (OPAL_SUCCESS != (ret = opal_net_init())) {
+        error = "opal_net_init";
+        goto return_error;
+    }
+
+    /* Setup the parameter system */
+    if (OPAL_SUCCESS != (ret = mca_base_var_init())) {
+        error = "mca_base_var_init";
+        goto return_error;
+    }
+
+    /* register params for opal */
+    if (OPAL_SUCCESS != (ret = opal_register_params())) {
+        error = "opal_register_params";
+        goto return_error;
+    }
+
+    /* pretty-print stack handlers */
+    if (OPAL_SUCCESS != (ret = opal_util_register_stackhandlers())) {
+        error = "opal_util_register_stackhandlers";
+        goto return_error;
+    }
+
+    /* Initialize the data storage service. */
+    if (OPAL_SUCCESS != (ret = opal_dss_open())) {
+        error = "opal_dss_open";
+        goto return_error;
+    }
+
+    /* initialize the mca */
+    if (OPAL_SUCCESS != (ret = mca_base_open())) {
+        error = "mca_base_open";
+        goto return_error;
+    }
+
+    if (OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_event_base_framework, 0))) {
+        error = "opal_event_base_open";
+        goto return_error;
+    }
+
+    return OPAL_SUCCESS;
+
+ return_error:
+    opal_show_help( "help-opal-runtime.txt",
+                    "opal_init:startup:internal-failure", true,
+                    error, ret );
+    return ret;
+}
+
+static bool fork_warning_issued = false;
+static bool atfork_called = false;
+
+static void warn_fork_cb(void)
+{
+    if (opal_initialized && !fork_warning_issued) {
+        opal_show_help("help-opal-runtime.txt", "opal_init:warn-fork", true,
+                       OPAL_NAME_PRINT(OPAL_PROC_MY_NAME), getpid());
+        fork_warning_issued = true;
+    }
+}
+
+void opal_warn_fork(void)
+{
+    if (opal_warn_on_fork && !atfork_called) {
+        pthread_atfork(warn_fork_cb, NULL, NULL);
+        atfork_called = true;
+    }
+}
