@@ -6,17 +6,19 @@
  * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -63,17 +65,13 @@ typedef struct {
     bool ldi_syslog;
     int ldi_syslog_priority;
 
-#ifndef __WINDOWS__
     char *ldi_syslog_ident;
-#else
-    HANDLE ldi_syslog_ident;
-#endif
     char *ldi_prefix;
     int ldi_prefix_len;
 
     char *ldi_suffix;
     int ldi_suffix_len;
-    
+
     bool ldi_stdout;
     bool ldi_stderr;
 
@@ -91,13 +89,13 @@ static void construct(opal_object_t *stream);
 static int do_open(int output_id, opal_output_stream_t * lds);
 static int open_file(int i);
 static void free_descriptor(int output_id);
-static int make_string(char **no_newline_string, output_desc_t *ldi, 
+static int make_string(char **no_newline_string, output_desc_t *ldi,
                        const char *format, va_list arglist);
 static int output(int output_id, const char *format, va_list arglist);
 
 
 #define OPAL_OUTPUT_MAX_STREAMS 64
-#if defined(__WINDOWS__) || defined(HAVE_SYSLOG)
+#if defined(HAVE_SYSLOG)
 #define USE_SYSLOG 1
 #else
 #define USE_SYSLOG 0
@@ -105,7 +103,7 @@ static int output(int output_id, const char *format, va_list arglist);
 
 /* global state */
 bool opal_output_redirected_to_syslog = false;
-int opal_output_redirected_syslog_pri;
+int opal_output_redirected_syslog_pri = -1;
 
 /*
  * Local state
@@ -116,7 +114,9 @@ static output_desc_t info[OPAL_OUTPUT_MAX_STREAMS];
 static char *temp_str = 0;
 static size_t temp_str_len = 0;
 static opal_mutex_t mutex;
+#if defined(HAVE_SYSLOG)
 static bool syslog_opened = false;
+#endif
 static char *redirect_syslog_ident = NULL;
 
 OBJ_CLASS_INSTANCE(opal_output_stream_t, opal_object_t, construct, NULL);
@@ -165,12 +165,6 @@ bool opal_output_init(void)
     }
 
     OBJ_CONSTRUCT(&verbose, opal_output_stream_t);
-#if defined(__WINDOWS__)
-    {
-        WSADATA wsaData;
-        WSAStartup( MAKEWORD(2,2), &wsaData );
-    }
-#endif  /* defined(__WINDOWS__) */
     if (opal_output_redirected_to_syslog) {
         verbose.lds_want_syslog = true;
         verbose.lds_syslog_priority = opal_output_redirected_syslog_pri;
@@ -287,7 +281,7 @@ void opal_output_reopen_all(void)
             break;
         }
 
-        /* 
+        /*
          * set this to zero to ensure that opal_output_open will
          * return this same index as the output stream id
          */
@@ -309,7 +303,7 @@ void opal_output_reopen_all(void)
         lds.lds_want_file_append = true;
         lds.lds_file_suffix = info[i].ldi_file_suffix;
 
-        /* 
+        /*
          * call opal_output_open to open the stream. The return value
          * is guaranteed to be i.  So we can ignore it.
          */
@@ -341,7 +335,7 @@ void opal_output_close(int output_id)
         free_descriptor(output_id);
 
         /* If no one has the syslog open, we should close it */
-        
+
         for (i = 0; i < OPAL_OUTPUT_MAX_STREAMS; ++i) {
             if (info[i].ldi_used && info[i].ldi_syslog) {
                 break;
@@ -352,10 +346,6 @@ void opal_output_close(int output_id)
         if (i >= OPAL_OUTPUT_MAX_STREAMS && syslog_opened) {
             closelog();
         }
-#elif defined(__WINDOWS__)
-        if(info[output_id].ldi_syslog_ident != NULL) {
-            DeregisterEventSource(info[output_id].ldi_syslog_ident);
-        }
 #endif
     }
 
@@ -395,7 +385,7 @@ void opal_output_verbose(int level, int output_id, const char *format, ...)
 /*
  * Send a message to a stream if the verbose level is high enough
  */
-void opal_output_vverbose(int level, int output_id, const char *format, 
+void opal_output_vverbose(int level, int output_id, const char *format,
                           va_list arglist)
 {
     if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
@@ -431,7 +421,7 @@ char *opal_output_string(int level, int output_id, const char *format, ...)
 /*
  * Send a message to a string if the verbose level is high enough
  */
-char *opal_output_vstring(int level, int output_id, const char *format,  
+char *opal_output_vstring(int level, int output_id, const char *format,
                           va_list arglist)
 {
     int rc;
@@ -514,9 +504,6 @@ void opal_output_finalize(void)
         OBJ_DESTRUCT(&verbose);
         OBJ_DESTRUCT(&mutex);
     }
-#if defined(__WINDOWS__)
-    WSACleanup();
-#endif  /* defined(__WINDOWS__) */
 
     initialized = false;
 }
@@ -552,6 +539,8 @@ static void construct(opal_object_t *obj)
 static int do_open(int output_id, opal_output_stream_t * lds)
 {
     int i;
+    bool redirect_to_file = false;
+    char *str, *sfx;
 
     /* Setup */
 
@@ -559,6 +548,12 @@ static int do_open(int output_id, opal_output_stream_t * lds)
         opal_output_init();
     }
 
+    str = getenv("OPAL_OUTPUT_REDIRECT");
+    if (NULL != str && 0 == strcasecmp(str, "file")) {
+        redirect_to_file = true;
+    }
+    sfx = getenv("OPAL_OUTPUT_SUFFIX");
+
     /* If output_id == -1, find an available stream, or return
      * OPAL_ERROR */
 
@@ -626,12 +621,6 @@ static int do_open(int output_id, opal_output_stream_t * lds)
                 info[i].ldi_syslog_ident = NULL;
                 openlog("opal", LOG_PID, LOG_USER);
             }
-#elif defined(__WINDOWS__)
-            if (NULL == (info[i].ldi_syslog_ident =
-                         RegisterEventSource(NULL, TEXT("opal: ")))) {
-                /* handle the error */
-                return OPAL_ERROR;
-            }
 #endif
             syslog_opened = true;
             info[i].ldi_syslog_priority = lds->lds_syslog_priority;
@@ -660,7 +649,7 @@ static int do_open(int output_id, opal_output_stream_t * lds)
         info[i].ldi_suffix = NULL;
         info[i].ldi_suffix_len = 0;
     }
-    
+
     if (opal_output_redirected_to_syslog) {
         /* since all is redirected to syslog, ensure
          * we don't duplicate the output to the std places
@@ -670,16 +659,26 @@ static int do_open(int output_id, opal_output_stream_t * lds)
         info[i].ldi_file = false;
         info[i].ldi_fd = -1;
     } else {
-        /* since we aren't redirecting, use what was
+        /* since we aren't redirecting to syslog, use what was
          * given to us
          */
-        info[i].ldi_stdout = lds->lds_want_stdout;
-        info[i].ldi_stderr = lds->lds_want_stderr;
+        if (NULL != str && redirect_to_file) {
+            info[i].ldi_stdout = false;
+            info[i].ldi_stderr = false;
+            info[i].ldi_file = true;
+        } else {
+            info[i].ldi_stdout = lds->lds_want_stdout;
+            info[i].ldi_stderr = lds->lds_want_stderr;
 
-        info[i].ldi_fd = -1;
-        info[i].ldi_file = lds->lds_want_file;
-        info[i].ldi_file_suffix = (NULL == lds->lds_file_suffix) ? NULL :
-            strdup(lds->lds_file_suffix);
+            info[i].ldi_fd = -1;
+            info[i].ldi_file = lds->lds_want_file;
+        }
+        if (NULL != sfx) {
+            info[i].ldi_file_suffix = strdup(sfx);
+        } else {
+            info[i].ldi_file_suffix = (NULL == lds->lds_file_suffix) ? NULL :
+                strdup(lds->lds_file_suffix);
+        }
         info[i].ldi_file_want_append = lds->lds_want_file_append;
         info[i].ldi_file_num_lines_lost = 0;
     }
@@ -695,6 +694,42 @@ static int open_file(int i)
 {
     int flags;
     char *filename;
+    int n;
+
+    /* first check to see if this file is already open
+     * on someone else's stream - if so, we don't want
+     * to open it twice
+     */
+    for (n=0; n < OPAL_OUTPUT_MAX_STREAMS; n++) {
+        if (i == n) {
+            continue;
+        }
+        if (!info[n].ldi_used) {
+            continue;
+        }
+        if (!info[n].ldi_file) {
+            continue;
+        }
+        if (NULL != info[i].ldi_file_suffix &&
+            NULL != info[n].ldi_file_suffix) {
+            if (0 != strcmp(info[i].ldi_file_suffix, info[n].ldi_file_suffix)) {
+                break;
+            }
+        }
+        if (NULL == info[i].ldi_file_suffix &&
+            NULL != info[n].ldi_file_suffix) {
+            break;
+        }
+        if (NULL != info[i].ldi_file_suffix &&
+            NULL == info[n].ldi_file_suffix) {
+            break;
+        }
+        if (info[n].ldi_fd < 0) {
+            break;
+        }
+        info[i].ldi_fd = info[n].ldi_fd;
+        return OPAL_SUCCESS;
+    }
 
     /* Setup the filename and open flags */
 
@@ -729,13 +764,9 @@ static int open_file(int i)
 
         /* Make the file be close-on-exec to prevent child inheritance
          * problems */
-
-#ifndef __WINDOWS__
-        /* TODO: Need to find out the equivalent in windows */
         if (-1 == fcntl(info[i].ldi_fd, F_SETFD, 1)) {
            return OPAL_ERR_IN_ERRNO;
         }
-#endif
 
     }
 
@@ -773,23 +804,21 @@ static void free_descriptor(int output_id)
         free(ldi->ldi_suffix);
     }
     ldi->ldi_suffix = NULL;
-    
+
     if (NULL != ldi->ldi_file_suffix) {
 	    free(ldi->ldi_file_suffix);
 	}
 	ldi->ldi_file_suffix = NULL;
 
-#ifndef __WINDOWS__
 	if (NULL != ldi->ldi_syslog_ident) {
 	    free(ldi->ldi_syslog_ident);
 	}
 	ldi->ldi_syslog_ident = NULL;
-#endif
     }
 }
 
 
-static int make_string(char **no_newline_string, output_desc_t *ldi, 
+static int make_string(char **no_newline_string, output_desc_t *ldi,
                        const char *format, va_list arglist)
 {
     size_t len, total_len;
@@ -833,7 +862,7 @@ static int make_string(char **no_newline_string, output_desc_t *ldi,
             snprintf(temp_str, temp_str_len, "%s%s%s\n",
                      ldi->ldi_prefix, *no_newline_string, ldi->ldi_suffix);
         } else {
-            snprintf(temp_str, temp_str_len, "%s%s%s", ldi->ldi_prefix, 
+            snprintf(temp_str, temp_str_len, "%s%s%s", ldi->ldi_prefix,
                      *no_newline_string, ldi->ldi_suffix);
         }
     } else if (NULL != ldi->ldi_prefix) {
@@ -841,7 +870,7 @@ static int make_string(char **no_newline_string, output_desc_t *ldi,
             snprintf(temp_str, temp_str_len, "%s%s\n",
                      ldi->ldi_prefix, *no_newline_string);
         } else {
-            snprintf(temp_str, temp_str_len, "%s%s", ldi->ldi_prefix, 
+            snprintf(temp_str, temp_str_len, "%s%s", ldi->ldi_prefix,
                      *no_newline_string);
         }
     } else if (NULL != ldi->ldi_suffix) {
@@ -849,7 +878,7 @@ static int make_string(char **no_newline_string, output_desc_t *ldi,
             snprintf(temp_str, temp_str_len, "%s%s\n",
                      *no_newline_string, ldi->ldi_suffix);
         } else {
-            snprintf(temp_str, temp_str_len, "%s%s", 
+            snprintf(temp_str, temp_str_len, "%s%s",
                      *no_newline_string, ldi->ldi_suffix);
         }
     } else {
@@ -859,10 +888,10 @@ static int make_string(char **no_newline_string, output_desc_t *ldi,
             snprintf(temp_str, temp_str_len, "%s", *no_newline_string);
         }
     }
-    
+
     return OPAL_SUCCESS;
 }
-    
+
 /*
  * Do the actual output.  Take a va_list so that we can be called from
  * multiple different places, even functions that took "..." as input
@@ -907,15 +936,15 @@ static int output(int output_id, const char *format, va_list arglist)
 
         /* stdout output */
         if (ldi->ldi_stdout) {
-            write(fileno(stdout), out, (int)strlen(out)); 
+            write(fileno(stdout), out, (int)strlen(out));
             fflush(stdout);
         }
 
         /* stderr output */
         if (ldi->ldi_stderr) {
-            write((-1 == default_stderr_fd) ? 
+            write((-1 == default_stderr_fd) ?
                   fileno(stderr) : default_stderr_fd,
-                  out, (int)strlen(out)); 
+                  out, (int)strlen(out));
             fflush(stderr);
         }
 
