@@ -11,7 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2010-2011 Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2011-2014 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2011-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2013-2015 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
@@ -37,11 +37,10 @@
 #include "opal/hash_string.h"
 #include "opal/class/opal_hash_table.h"
 #include "opal/class/opal_list.h"
-#include "opal/mca/db/base/base.h"
 #include "opal/mca/event/event.h"
 #include "opal/runtime/opal.h"
-#include "opal/runtime/opal_cr.h"
 
+#include "opal/util/arch.h"
 #include "opal/util/argv.h"
 #include "opal/util/if.h"
 #include "opal/util/os_path.h"
@@ -49,6 +48,7 @@
 #include "opal/util/malloc.h"
 #include "opal/util/basename.h"
 #include "opal/util/fd.h"
+#include "opal/mca/pmix/base/base.h"
 #include "opal/mca/pstat/base/base.h"
 #include "opal/mca/hwloc/base/base.h"
 
@@ -57,6 +57,7 @@
 #include "orte/mca/rml/rml_types.h"
 #include "orte/mca/routed/base/base.h"
 #include "orte/mca/routed/routed.h"
+#include "orte/mca/rtc/base/base.h"
 #include "orte/mca/dfs/base/base.h"
 #include "orte/mca/errmgr/base/base.h"
 #include "orte/mca/grpcomm/base/base.h"
@@ -65,16 +66,14 @@
 #include "orte/mca/plm/base/base.h"
 #include "orte/mca/plm/plm.h"
 #include "orte/mca/odls/base/base.h"
-#include "orte/mca/sensor/base/base.h"
-#include "orte/mca/sensor/sensor.h"
 #include "orte/mca/rmaps/base/base.h"
-#if OPAL_ENABLE_FT_CR == 1
-#include "orte/mca/snapc/base/base.h"
-#endif
 #include "orte/mca/filem/base/base.h"
+#include "orte/mca/schizo/base/base.h"
 #include "orte/mca/state/base/base.h"
 #include "orte/mca/state/state.h"
 
+#include "orte/orted/pmix/pmix_server.h"
+
 #include "orte/util/show_help.h"
 #include "orte/util/proc_info.h"
 #include "orte/util/session_dir.h"
@@ -82,13 +81,11 @@
 #include "orte/util/name_fns.h"
 #include "orte/util/show_help.h"
 #include "orte/util/comm/comm.h"
-#include "orte/util/nidmap.h"
 
 #include "orte/runtime/runtime.h"
 #include "orte/runtime/orte_wait.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/runtime/orte_quit.h"
-#include "orte/runtime/orte_cr.h"
 #include "orte/runtime/orte_locks.h"
 #include "orte/runtime/orte_data_server.h"
 
@@ -140,6 +137,11 @@ static int rte_init(void)
     orte_node_t *node;
     orte_proc_t *proc;
     orte_app_context_t *app;
+    char **aliases, *aptr;
+    char *coprocessors, **sns;
+    uint32_t h;
+    int idx;
+    orte_topology_t *t;
 
     /* run the prolog */
     if (ORTE_SUCCESS != (ret = orte_ess_base_std_prolog())) {
@@ -189,52 +191,21 @@ static int rte_init(void)
     setup_sighandler(SIGCONT, &sigcont_handler, signal_forward_callback);
     signals_set = true;
 
-#if OPAL_HAVE_HWLOC
-    {
-        hwloc_obj_t obj;
-        unsigned i, j;
-
-        /* get the local topology */
-        if (NULL == opal_hwloc_topology) {
-            if (OPAL_SUCCESS != (ret = opal_hwloc_base_get_topology())) {
-                error = "topology discovery";
-                goto error;
-            }
-        }
-        /* generate the signature */
-        orte_topo_signature = opal_hwloc_base_get_topo_signature(opal_hwloc_topology);
-
-        /* remove the hostname from the topology. Unfortunately, hwloc
-         * decided to add the source hostname to the "topology", thus
-         * rendering it unusable as a pure topological description. So
-         * we remove that information here.
-         */
-        obj = hwloc_get_root_obj(opal_hwloc_topology);
-        for (i=0; i < obj->infos_count; i++) {
-            if (NULL == obj->infos[i].name ||
-                NULL == obj->infos[i].value) {
-                continue;
-            }
-            if (0 == strncmp(obj->infos[i].name, "HostName", strlen("HostName"))) {
-                free(obj->infos[i].name);
-                free(obj->infos[i].value);
-                /* left justify the array */
-                for (j=i; j < obj->infos_count-1; j++) {
-                    obj->infos[j] = obj->infos[j+1];
-                }
-                obj->infos[obj->infos_count-1].name = NULL;
-                obj->infos[obj->infos_count-1].value = NULL;
-                obj->infos_count--;
-                break;
-            }
-        }
-
-        if (4 < opal_output_get_verbosity(orte_ess_base_framework.framework_output)) {
-            opal_output(0, "%s Topology Info:", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-            opal_dss.dump(0, opal_hwloc_topology, OPAL_HWLOC_TOPO);
+    /* get the local topology */
+    if (NULL == opal_hwloc_topology) {
+        if (OPAL_SUCCESS != (ret = opal_hwloc_base_get_topology())) {
+            error = "topology discovery";
+            goto error;
         }
     }
-#endif
+    /* generate the signature */
+    orte_topo_signature = opal_hwloc_base_get_topo_signature(opal_hwloc_topology);
+
+    if (15 < opal_output_get_verbosity(orte_ess_base_framework.framework_output)) {
+        opal_output(0, "%s Topology Info:", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+        opal_dss.dump(0, opal_hwloc_topology, OPAL_HWLOC_TOPO);
+    }
+
 
     /* if we are using xml for output, put an mpirun start tag */
     if (orte_xml_output) {
@@ -242,41 +213,30 @@ static int rte_init(void)
         fflush(orte_xml_fp);
     }
 
-    /* setup the global nidmap/pidmap object */
-    orte_nidmap.bytes = NULL;
-    orte_nidmap.size = 0;
-    orte_pidmap.bytes = NULL;
-    orte_pidmap.size = 0;
-
     /* open and setup the opal_pstat framework so we can provide
      * process stats if requested
      */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&opal_pstat_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "opal_pstat_base_open";
         goto error;
     }
     if (ORTE_SUCCESS != (ret = opal_pstat_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "opal_pstat_base_select";
         goto error;
     }
 
     /* open and setup the state machine */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_state_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_state_base_open";
         goto error;
     }
     if (ORTE_SUCCESS != (ret = orte_state_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_state_base_select";
         goto error;
     }
 
     /* open the errmgr */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_errmgr_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_errmgr_base_open";
         goto error;
     }
@@ -287,43 +247,81 @@ static int rte_init(void)
      * first and select that component.
      */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_plm_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_plm_base_open";
         goto error;
     }
 
     if (ORTE_SUCCESS != (ret = orte_plm_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_plm_base_select";
+        if (ORTE_ERR_FATAL == ret) {
+            /* we already output a show_help - so keep down the verbage */
+            ret = ORTE_ERR_SILENT;
+        }
         goto error;
     }
     /* if we were spawned by a singleton, our jobid was given to us */
     if (NULL != orte_ess_base_jobid) {
         if (ORTE_SUCCESS != (ret = orte_util_convert_string_to_jobid(&ORTE_PROC_MY_NAME->jobid, orte_ess_base_jobid))) {
-            ORTE_ERROR_LOG(ret);
             error = "convert_string_to_jobid";
             goto error;
         }
         ORTE_PROC_MY_NAME->vpid = 0;
     } else {
         if (ORTE_SUCCESS != (ret = orte_plm.set_hnp_name())) {
-            ORTE_ERROR_LOG(ret);
             error = "orte_plm_set_hnp_name";
             goto error;
         }
     }
+    /* now that my name is set, xfer it to the OPAL layer */
+    orte_process_info.super.proc_name = *(opal_process_name_t*)ORTE_PROC_MY_NAME;
+    orte_process_info.super.proc_hostname = strdup(orte_process_info.nodename);
+    orte_process_info.super.proc_flags = OPAL_PROC_ALL_LOCAL;
+    orte_process_info.super.proc_arch = opal_local_arch;
+    opal_proc_local_set(&orte_process_info.super);
+
+    /* setup my session directory here as the OOB may need it */
+    if (orte_create_session_dirs) {
+        OPAL_OUTPUT_VERBOSE((2, orte_debug_output,
+                             "%s setting up session dir with\n\ttmpdir: %s\n\thost %s",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             (NULL == orte_process_info.tmpdir_base) ? "UNDEF" : orte_process_info.tmpdir_base,
+                             orte_process_info.nodename));
+
+        /* take a pass thru the session directory code to fillin the
+         * tmpdir names - don't create anything yet
+         */
+        if (ORTE_SUCCESS != (ret = orte_session_dir(false,
+                                                    orte_process_info.tmpdir_base,
+                                                    orte_process_info.nodename, NULL,
+                                                    ORTE_PROC_MY_NAME))) {
+            error = "orte_session_dir define";
+            goto error;
+        }
+        /* clear the session directory just in case there are
+         * stale directories laying around
+         */
+        orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
+
+        /* now actually create the directory tree */
+        if (ORTE_SUCCESS != (ret = orte_session_dir(true,
+                                                    orte_process_info.tmpdir_base,
+                                                    orte_process_info.nodename, NULL,
+                                                    ORTE_PROC_MY_NAME))) {
+            error = "orte_session_dir";
+            goto error;
+        }
+    }
+
     /* Setup the communication infrastructure */
 
     /*
      * OOB Layer
      */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_oob_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_oob_base_open";
         goto error;
     }
     if (ORTE_SUCCESS != (ret = orte_oob_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_oob_base_select";
         goto error;
     }
@@ -332,18 +330,15 @@ static int rte_init(void)
      * Runtime Messaging Layer
      */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_rml_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_rml_base_open";
         goto error;
     }
     if (ORTE_SUCCESS != (ret = orte_rml_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_rml_base_select";
         goto error;
     }
 
     if (ORTE_SUCCESS != (ret = orte_errmgr_base_select())) {
-        ORTE_ERROR_LOG(ret);
         error = "orte_errmgr_base_select";
         goto error;
     }
@@ -378,9 +373,6 @@ static int rte_init(void)
         goto error;
     }
 
-    /* init the nidmap - just so we register that verbosity */
-    orte_util_nidmap_init(NULL);
-
     /* Setup the job data object for the daemons */
     /* create and store the job data object */
     jdata = OBJ_NEW(orte_job_t);
@@ -401,16 +393,12 @@ static int rte_init(void)
     node = OBJ_NEW(orte_node_t);
     node->name = strdup(orte_process_info.nodename);
     node->index = opal_pointer_array_set_item(orte_node_pool, 0, node);
-#if OPAL_HAVE_HWLOC
-    {
-        orte_topology_t *t;
-        /* add it to the array of known topologies */
-        t = OBJ_NEW(orte_topology_t);
-        t->topo = opal_hwloc_topology;
-        t->sig = strdup(orte_topo_signature);
-        opal_pointer_array_add(orte_node_topologies, t);
-    }
-#endif
+
+    /* add it to the array of known topologies */
+    t = OBJ_NEW(orte_topology_t);
+    t->topo = opal_hwloc_topology;
+    t->sig = strdup(orte_topo_signature);
+    opal_pointer_array_add(orte_node_topologies, t);
 
     /* create and store a proc object for us */
     proc = OBJ_NEW(orte_proc_t);
@@ -422,7 +410,6 @@ static int rte_init(void)
     proc->state = ORTE_PROC_STATE_RUNNING;
     OBJ_RETAIN(node);  /* keep accounting straight */
     proc->node = node;
-    proc->nodename = node->name;
     opal_pointer_array_set_item(jdata->procs, proc->name.vpid, proc);
 
     /* record that the daemon (i.e., us) is on this node
@@ -433,14 +420,19 @@ static int rte_init(void)
      */
     OBJ_RETAIN(proc);   /* keep accounting straight */
     node->daemon = proc;
-    node->daemon_launched = true;
+    ORTE_FLAG_SET(node, ORTE_NODE_FLAG_DAEMON_LAUNCHED);
     node->state = ORTE_NODE_STATE_UP;
 
     /* if we are to retain aliases, get ours */
     if (orte_retain_aliases) {
-        opal_ifgetaliases(&node->alias);
+        aliases = NULL;
+        opal_ifgetaliases(&aliases);
         /* add our own local name to it */
-        opal_argv_append_nosize(&node->alias, orte_process_info.nodename);
+        opal_argv_append_nosize(&aliases, orte_process_info.nodename);
+        aptr = opal_argv_join(aliases, ',');
+        opal_argv_free(aliases);
+        orte_set_attribute(&node->attributes, ORTE_NODE_ALIAS, ORTE_ATTR_LOCAL, aptr, OPAL_STRING);
+        free(aptr);
     }
 
     /* record that the daemon job is running */
@@ -463,19 +455,6 @@ static int rte_init(void)
         goto error;
     }
 
-    /* database */
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&opal_db_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_db_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = opal_db_base_select(true))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_db_base_select";
-        goto error;
-    }
-    /* set our id */
-    opal_db.set_id((opal_identifier_t*)ORTE_PROC_MY_NAME);
 
     /*
      * Group communications
@@ -529,52 +508,46 @@ static int rte_init(void)
         error = "orte_rmaps_base_find_available";
         goto error;
     }
-#if OPAL_HAVE_HWLOC
-    {
-        char *coprocessors, **sns;
-        uint32_t h;
-        int idx;
 
-        /* if a topology file was given, then the rmaps framework open
-         * will have reset our topology. Ensure we always get the right
-         * one by setting our node topology afterwards
+    /* if a topology file was given, then the rmaps framework open
+     * will have reset our topology. Ensure we always get the right
+     * one by setting our node topology afterwards
+     */
+    node->topology = opal_hwloc_topology;
+
+    /* init the hash table, if necessary */
+    if (NULL == orte_coprocessors) {
+        orte_coprocessors = OBJ_NEW(opal_hash_table_t);
+        opal_hash_table_init(orte_coprocessors, orte_process_info.num_procs);
+    }
+    /* detect and add any coprocessors */
+    coprocessors = opal_hwloc_base_find_coprocessors(opal_hwloc_topology);
+    if (NULL != coprocessors) {
+        /* separate the serial numbers of the coprocessors
+         * on this host
          */
-        node->topology = opal_hwloc_topology;
-
-        /* init the hash table, if necessary */
-        if (NULL == orte_coprocessors) {
-            orte_coprocessors = OBJ_NEW(opal_hash_table_t);
-            opal_hash_table_init(orte_coprocessors, orte_process_info.num_procs);
-        }
-        /* detect and add any coprocessors */
-        coprocessors = opal_hwloc_base_find_coprocessors(opal_hwloc_topology);
-        if (NULL != coprocessors) {
-            /* separate the serial numbers of the coprocessors
-             * on this host
-             */
-            sns = opal_argv_split(coprocessors, ',');
-            for (idx=0; NULL != sns[idx]; idx++) {
-                /* compute the hash */
-                OPAL_HASH_STR(sns[idx], h);
-                /* mark that this coprocessor is hosted by this node */
-                opal_hash_table_set_value_uint32(orte_coprocessors, h, (void*)&(ORTE_PROC_MY_NAME->vpid));
-            }
-            opal_argv_free(sns);
-            free(coprocessors);
-            orte_coprocessors_detected = true;
-        }
-        /* see if I am on a coprocessor */
-        coprocessors = opal_hwloc_base_check_on_coprocessor();
-        if (NULL != coprocessors) {
-            node->serial_number = coprocessors;
-            orte_coprocessors_detected = true;
+        sns = opal_argv_split(coprocessors, ',');
+        for (idx=0; NULL != sns[idx]; idx++) {
             /* compute the hash */
-            OPAL_HASH_STR(coprocessors, h);
-            /* mark that I am on this coprocessor */
+            OPAL_HASH_STR(sns[idx], h);
+            /* mark that this coprocessor is hosted by this node */
             opal_hash_table_set_value_uint32(orte_coprocessors, h, (void*)&(ORTE_PROC_MY_NAME->vpid));
         }
+        opal_argv_free(sns);
+        free(coprocessors);
+        orte_coprocessors_detected = true;
+    }
+    /* see if I am on a coprocessor */
+    coprocessors = opal_hwloc_base_check_on_coprocessor();
+    if (NULL != coprocessors) {
+        /* compute the hash */
+        OPAL_HASH_STR(coprocessors, h);
+        /* mark that I am on this coprocessor */
+        opal_hash_table_set_value_uint32(orte_coprocessors, h, (void*)&(ORTE_PROC_MY_NAME->vpid));
+        orte_set_attribute(&node->attributes, ORTE_NODE_SERIAL_NUMBER, ORTE_ATTR_LOCAL, coprocessors, OPAL_STRING);
+        free(coprocessors);
+        orte_coprocessors_detected = true;
     }
-#endif
 
     /* Open/select the odls */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_odls_base_framework, 0))) {
@@ -588,6 +561,18 @@ static int rte_init(void)
         goto error;
     }
 
+    /* Open/select the rtc */
+    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_rtc_base_framework, 0))) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_rtc_base_open";
+        goto error;
+    }
+    if (ORTE_SUCCESS != (ret = orte_rtc_base_select())) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_rtc_base_select";
+        goto error;
+    }
+
     /* enable communication with the rml */
     if (ORTE_SUCCESS != (ret = orte_rml.enable_comm())) {
         ORTE_ERROR_LOG(ret);
@@ -606,43 +591,16 @@ static int rte_init(void)
     orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, ORTE_RML_TAG_SHOW_HELP,
                             ORTE_RML_PERSISTENT, orte_show_help_recv, NULL);
 
-    /* setup my session directory */
+    /* setup the data server */
+    if (ORTE_SUCCESS != (ret = orte_data_server_init())) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_data_server_init";
+        goto error;
+    }
+
     if (orte_create_session_dirs) {
-        OPAL_OUTPUT_VERBOSE((2, orte_debug_output,
-                             "%s setting up session dir with\n\ttmpdir: %s\n\thost %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             (NULL == orte_process_info.tmpdir_base) ? "UNDEF" : orte_process_info.tmpdir_base,
-                             orte_process_info.nodename));
-
-        /* take a pass thru the session directory code to fillin the
-         * tmpdir names - don't create anything yet
-         */
-        if (ORTE_SUCCESS != (ret = orte_session_dir(false,
-                                                    orte_process_info.tmpdir_base,
-                                                    orte_process_info.nodename, NULL,
-                                                    ORTE_PROC_MY_NAME))) {
-            ORTE_ERROR_LOG(ret);
-            error = "orte_session_dir define";
-            goto error;
-        }
-        /* clear the session directory just in case there are
-         * stale directories laying around
-         */
-        orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
-
-        /* now actually create the directory tree */
-        if (ORTE_SUCCESS != (ret = orte_session_dir(true,
-                                                    orte_process_info.tmpdir_base,
-                                                    orte_process_info.nodename, NULL,
-                                                    ORTE_PROC_MY_NAME))) {
-            ORTE_ERROR_LOG(ret);
-            error = "orte_session_dir";
-            goto error;
-        }
-
-        /* Once the session directory location has been established, set
-           the opal_output hnp file location to be in the
-           proc-specific session directory. */
+        /* set the opal_output hnp file location to be in the
+         * proc-specific session directory. */
         opal_output_set_output_file_info(orte_process_info.proc_session_dir,
                                          "output-", NULL, NULL);
 
@@ -669,6 +627,21 @@ static int rte_init(void)
         free(contact_path);
     }
 
+    /* setup the PMIx framework - ensure it skips all non-PMIx components */
+    putenv("OMPI_MCA_pmix=^s1,s2,cray");
+    if (OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_pmix_base_framework, 0))) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_pmix_base_open";
+        goto error;
+    }
+    if (ORTE_SUCCESS != (ret = opal_pmix_base_select())) {
+        ORTE_ERROR_LOG(ret);
+        error = "opal_pmix_base_select";
+        goto error;
+    }
+    /* set the event base */
+    opal_pmix_base_set_evbase(orte_event_base);
+
     /* setup the routed info - the selected routed component
      * will know what to do.
      */
@@ -678,6 +651,13 @@ static int rte_init(void)
         goto error;
     }
 
+    /* setup the PMIx server */
+    if (ORTE_SUCCESS != (ret = pmix_server_init())) {
+        ORTE_ERROR_LOG(ret);
+        error = "pmix server init";
+        goto error;
+    }
+
     /* setup I/O forwarding system - must come after we init routes */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_iof_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
@@ -702,62 +682,6 @@ static int rte_init(void)
         goto error;
     }
 
-#if OPAL_ENABLE_FT_CR == 1
-    /*
-     * Setup the SnapC
-     */
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_snapc_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_snapc_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_sstore_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sstore_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_snapc_base_select(ORTE_PROC_IS_HNP, !ORTE_PROC_IS_DAEMON))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_snapc_base_select";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_sstore_base_select())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sstore_base_select";
-        goto error;
-    }
-
-    /* For HNP, ORTE doesn't need the OPAL CR stuff */
-    opal_cr_set_enabled(false);
-#else
-    opal_cr_set_enabled(false);
-#endif
-
-    /*
-     * Initalize the CR setup
-     * Note: Always do this, even in non-FT builds.
-     * If we don't some user level tools may hang.
-     */
-    if (ORTE_SUCCESS != (ret = orte_cr_init())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_cr_init";
-        goto error;
-    }
-
-    /* setup the SENSOR framework */
-    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_sensor_base_framework, 0))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sensor_base_open";
-        goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_sensor_base_select())) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_sensor_select";
-        goto error;
-    }
-    /* start the local sensors */
-    orte_sensor.start(ORTE_PROC_MY_NAME->jobid);
-
     /* setup the dfs framework */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_dfs_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
@@ -770,6 +694,18 @@ static int rte_init(void)
         goto error;
     }
 
+    /* setup the schizo framework */
+    if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_schizo_base_framework, 0))) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_schizo_base_open";
+        goto error;
+    }
+    if (ORTE_SUCCESS != (ret = orte_schizo_base_select())) {
+        ORTE_ERROR_LOG(ret);
+        error = "orte_schizo_select";
+        goto error;
+    }
+
     /* if a tool has launched us and is requesting event reports,
      * then set its contact info into the comm system
      */
@@ -809,18 +745,6 @@ static int rte_init(void)
                        true, error, ORTE_ERROR_NAME(ret), ret);
     }
 
-    /* remove my contact info file, if we have session directories */
-    if (NULL != orte_process_info.job_session_dir) {
-        jobfam_dir = opal_dirname(orte_process_info.job_session_dir);
-        contact_path = opal_os_path(false, jobfam_dir, "contact.txt", NULL);
-        free(jobfam_dir);
-        unlink(contact_path);
-        free(contact_path);
-    }
-    /* remove our use of the session directory tree */
-    orte_session_dir_finalize(ORTE_PROC_MY_NAME);
-    /* ensure we scrub the session directory tree */
-    orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
     return ORTE_ERR_SILENT;
 }
 
@@ -844,22 +768,24 @@ static int rte_finalize(void)
         signals_set = false;
     }
 
-    /* stop the local sensors */
-    orte_sensor.stop(ORTE_PROC_MY_NAME->jobid);
-    (void) mca_base_framework_close(&orte_sensor_base_framework);
+    /* shutdown the pmix server */
+    pmix_server_finalize();
+    (void) mca_base_framework_close(&opal_pmix_base_framework);
+    /* cleanup our data server */
+    orte_data_server_finalize();
 
-    /* close the dfs */
+    (void) mca_base_framework_close(&orte_schizo_base_framework);
     (void) mca_base_framework_close(&orte_dfs_base_framework);
     (void) mca_base_framework_close(&orte_filem_base_framework);
     /* output any lingering stdout/err data */
     fflush(stdout);
     fflush(stderr);
     (void) mca_base_framework_close(&orte_iof_base_framework);
+    (void) mca_base_framework_close(&orte_rtc_base_framework);
     (void) mca_base_framework_close(&orte_odls_base_framework);
     (void) mca_base_framework_close(&orte_rmaps_base_framework);
     (void) mca_base_framework_close(&orte_ras_base_framework);
     (void) mca_base_framework_close(&orte_grpcomm_base_framework);
-    (void) mca_base_framework_close(&opal_db_base_framework);
     (void) mca_base_framework_close(&orte_routed_base_framework);
     (void) mca_base_framework_close(&orte_plm_base_framework);
     (void) mca_base_framework_close(&orte_errmgr_base_framework);
@@ -881,8 +807,6 @@ static int rte_finalize(void)
     (void) mca_base_framework_close(&orte_rml_base_framework);
     (void) mca_base_framework_close(&orte_oob_base_framework);
 
-    /* remove our use of the session directory tree */
-    orte_session_dir_finalize(ORTE_PROC_MY_NAME);
     /* ensure we scrub the session directory tree */
     orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
 
@@ -909,9 +833,6 @@ static void rte_abort(int status, bool report)
      * - Assume errmgr cleans up child processes before we exit.
      */
 
-    /* CRS cleanup since it may have a named pipe and thread active */
-    orte_cr_finalize();
-
     /* ensure we scrub the session directory tree */
     orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
 
