@@ -11,7 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2007-2010 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2007      Evergrid, Inc. All rights reserved.
- * Copyright (c) 2008-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2016 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved.
@@ -49,7 +49,7 @@
  * - the child tries to set affinity and do other housekeeping in
  *   preparation of exec'ing the target executable
  * - if the child fails anywhere along the way, it sends a message up
- *   the pipe to the parent indicating what happened -- including a 
+ *   the pipe to the parent indicating what happened -- including a
  *   rendered error message detailing the problem (i.e., human-readable).
  * - it is important that the child renders the error message: there
  *   are so many errors that are possible that the child is really the
@@ -68,9 +68,7 @@
 #include "orte/constants.h"
 #include "orte/types.h"
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -95,15 +93,11 @@
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif  /* HAVE_SYS_STAT_H */
-#ifdef HAVE_STDARG_H
 #include <stdarg.h>
-#endif
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
@@ -127,34 +121,13 @@
 #include "orte/mca/ess/ess.h"
 #include "orte/mca/iof/base/iof_base_setup.h"
 #include "orte/mca/plm/plm.h"
+#include "orte/mca/rtc/rtc.h"
 #include "orte/util/name_fns.h"
 
 #include "orte/mca/odls/base/base.h"
 #include "orte/mca/odls/base/odls_private.h"
 #include "orte/mca/odls/default/odls_default.h"
-
-/*
- * Struct written up the pipe from the child to the parent.
- */
-typedef struct {
-    /* True if the child has died; false if this is just a warning to
-       be printed. */
-    bool fatal;
-    /* Relevant only if fatal==true */
-    int exit_status;
-
-    /* Length of the strings that are written up the pipe after this
-       struct */
-    int file_str_len;
-    int topic_str_len;
-    int msg_str_len;
-} pipe_err_msg_t;
-
-/* 
- * Max length of strings from the pipe_err_msg_t
- */
-#define MAX_FILE_LEN 511
-#define MAX_TOPIC_LEN MAX_FILE_LEN
+#include "orte/orted/pmix/pmix_server.h"
 
 /*
  * Module functions (function pointers used in a struct)
@@ -168,7 +141,7 @@ static int orte_odls_default_restart_proc(orte_proc_t *child);
  * Explicitly declared functions so that we can get the noreturn
  * attribute registered with the compiler.
  */
-static void send_error_show_help(int fd, int exit_status, 
+static void send_error_show_help(int fd, int exit_status,
                                  const char *file, const char *topic, ...)
     __opal_attribute_noreturn__;
 static int do_child(orte_app_context_t* context,
@@ -188,7 +161,6 @@ orte_odls_base_module_t orte_odls_default_module = {
     orte_odls_default_kill_local_procs,
     orte_odls_default_signal_local_procs,
     orte_odls_base_default_deliver_message,
-    orte_odls_base_default_require_sync,
     orte_odls_default_restart_proc
 };
 
@@ -197,20 +169,20 @@ static bool odls_default_child_died(orte_proc_t *child)
 {
     time_t end;
     pid_t ret;
-        
+
     /* Because of rounding in time (which returns whole seconds) we
      * have to add 1 to our wait number: this means that we wait
      * somewhere between (target) and (target)+1 seconds.  Otherwise,
      * the default 1s actually means 'somwhere between 0 and 1s'. */
     end = time(NULL) + orte_odls_globals.timeout_before_sigkill + 1;
     do {
-        OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
+        OPAL_OUTPUT_VERBOSE((20, orte_odls_base_framework.framework_output,
                              "%s odls:default:WAITPID CHECKING PID %d WITH TIMEOUT %d SECONDS",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid),
                              orte_odls_globals.timeout_before_sigkill + 1));
         ret = waitpid(child->pid, &child->exit_code, WNOHANG);
         if (child->pid == ret) {
-            OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
+            OPAL_OUTPUT_VERBOSE((20, orte_odls_base_framework.framework_output,
                                  "%s odls:default:WAITPID INDICATES PROC %d IS DEAD",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid)));
             /* It died -- return success */
@@ -230,19 +202,19 @@ static bool odls_default_child_died(orte_proc_t *child)
 	     * which will occasionally trip the timeout for cases that
 	     * are right on the edge.)
              */
-            OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
+            OPAL_OUTPUT_VERBOSE((20, orte_odls_base_framework.framework_output,
                                  "%s odls:default:WAITPID INDICATES PID %d MAY HAVE ALREADY EXITED",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid)));
 	    /* Do nothing, process still alive */
         } else if (-1 == ret && ECHILD == errno) {
             /* The pid no longer exists, so we'll call this "good
                enough for government work" */
-            OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
+            OPAL_OUTPUT_VERBOSE((20, orte_odls_base_framework.framework_output,
                                  "%s odls:default:WAITPID INDICATES PID %d NO LONGER EXISTS",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid)));
             return true;
         }
-        
+
         /* Bogus delay for 1 msec - let's actually give the CPU some time
          * to quit the other process (sched_yield() -- even if we have it
          * -- changed behavior in 2.6.3x Linux flavors to be undesirable)
@@ -291,7 +263,7 @@ static int odls_default_kill_local(pid_t pid, int signum)
 int orte_odls_default_kill_local_procs(opal_pointer_array_t *procs)
 {
     int rc;
-    
+
     if (ORTE_SUCCESS != (rc = orte_odls_base_default_kill_local_procs(procs,
                                     odls_default_kill_local, odls_default_child_died))) {
         ORTE_ERROR_LOG(rc);
@@ -316,7 +288,7 @@ static void set_handler_default(int sig)
  * Internal function to write a rendered show_help message back up the
  * pipe to the waiting parent.
  */
-static int write_help_msg(int fd, pipe_err_msg_t *msg, const char *file,
+static int write_help_msg(int fd, orte_odls_pipe_err_msg_t *msg, const char *file,
                           const char *topic, va_list ap)
 {
     int ret;
@@ -329,12 +301,12 @@ static int write_help_msg(int fd, pipe_err_msg_t *msg, const char *file,
     str = opal_show_help_vstring(file, topic, true, ap);
 
     msg->file_str_len = (int) strlen(file);
-    if (msg->file_str_len > MAX_FILE_LEN) {
+    if (msg->file_str_len > ORTE_ODLS_MAX_FILE_LEN) {
         ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
         return ORTE_ERR_BAD_PARAM;
     }
     msg->topic_str_len = (int) strlen(topic);
-    if (msg->topic_str_len > MAX_TOPIC_LEN) {
+    if (msg->topic_str_len > ORTE_ODLS_MAX_TOPIC_LEN) {
         ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
         return ORTE_ERR_BAD_PARAM;
     }
@@ -363,35 +335,13 @@ static int write_help_msg(int fd, pipe_err_msg_t *msg, const char *file,
 }
 
 
-/* Called from the child to send a warning show_help message up the
-   pipe to the waiting parent. */
-#if OPAL_HAVE_HWLOC
-static int send_warn_show_help(int fd, const char *file, 
-                               const char *topic, ...)
-{
-    int ret;
-    va_list ap;
-    pipe_err_msg_t msg;
-
-    msg.fatal = false;
-    msg.exit_status = 0; /* ignored */
-
-    /* Send it */
-    va_start(ap, topic);
-    ret = write_help_msg(fd, &msg, file, topic, ap);
-    va_end(ap);
-
-    return ret;
-}
-#endif
-
 /* Called from the child to send an error message up the pipe to the
    waiting parent. */
 static void send_error_show_help(int fd, int exit_status,
                                  const char *file, const char *topic, ...)
 {
     va_list ap;
-    pipe_err_msg_t msg;
+    orte_odls_pipe_err_msg_t msg;
 
     msg.fatal = true;
     msg.exit_status = exit_status;
@@ -448,14 +398,14 @@ static int do_child(orte_app_context_t* context,
            orted. */
         setpgid(0, 0);
     }
-    
+
     /* Setup the pipe to be close-on-exec */
     opal_fd_set_cloexec(write_fd);
 
     if (NULL != child) {
         /* setup stdout/stderr so that any error messages that we
            may print out will get displayed back at orterun.
-           
+
            NOTE: Definitely do this AFTER we check contexts so
            that any error message from those two functions doesn't
            come out to the user. IF we didn't do it in this order,
@@ -466,185 +416,22 @@ static int do_child(orte_app_context_t* context,
            always outputs a nice, single message indicating what
            happened
         */
-        if (ORTE_SUCCESS != (i = orte_iof_base_setup_child(&opts, 
-                                                           &environ_copy))) {
-            ORTE_ERROR_LOG(i);
-            send_error_show_help(write_fd, 1, 
-                                 "help-orte-odls-default.txt", 
-                                 "iof setup failed",
-                                 orte_process_info.nodename, context->app);
-            /* Does not return */
-        }
-
-#if OPAL_HAVE_HWLOC
-        {
-            hwloc_cpuset_t cpuset;
-            hwloc_obj_t root;
-            opal_hwloc_topo_data_t *sum;
-
-            /* Set process affinity, if given */
-            if (NULL == child->cpu_bitmap) {
-                /* if the daemon is bound, then we need to "free" this proc */
-                if (NULL != orte_daemon_cores) {
-                    root = hwloc_get_root_obj(opal_hwloc_topology);
-                    if (NULL == root->userdata) {
-                        send_warn_show_help(write_fd,
-                                            "help-orte-odls-default.txt", "incorrectly bound",
-                                            orte_process_info.nodename, context->app,
-                                            __FILE__, __LINE__);
-                    }
-                    sum = (opal_hwloc_topo_data_t*)root->userdata;
-                    /* bind this proc to all available processors */
-                    hwloc_set_cpubind(opal_hwloc_topology, sum->available, 0);
-                }
-                if (opal_hwloc_report_bindings) {
-                    opal_output(0, "MCW rank %d is not bound (or bound to all available processors)", child->name.vpid);
-                    /* avoid reporting it twice */
-                    (void) mca_base_var_env_name ("hwloc_base_report_bindings", &param);
-                    opal_unsetenv(param, &environ_copy);
-                    free(param);
-                }
-            } else {
-                if (0 == strlen(child->cpu_bitmap)) {
-                    /* this proc is not bound */
-                    if (opal_hwloc_report_bindings) {
-                        opal_output(0, "MCW rank %d is not bound (or bound to all available processors)", child->name.vpid);
-                        /* avoid reporting it twice */
-                        (void) mca_base_var_env_name ("hwloc_base_report_bindings", &param);
-                        opal_unsetenv(param, &environ_copy);
-                        free(param);
-                    }
-                    /* if the daemon is bound, then we need to "free" this proc */
-                    if (NULL != orte_daemon_cores) {
-                        root = hwloc_get_root_obj(opal_hwloc_topology);
-                        if (NULL == root->userdata) {
-                            send_warn_show_help(write_fd,
-                                                "help-orte-odls-default.txt", "incorrectly bound",
-                                                orte_process_info.nodename, context->app,
-                                                __FILE__, __LINE__);
-                        }
-                        sum = (opal_hwloc_topo_data_t*)root->userdata;
-                        /* bind this proc to all available processors */
-                        hwloc_set_cpubind(opal_hwloc_topology, sum->available, 0);
-                    }
-                    /* provide a nice string representation of what we bound to */
-                    (void) mca_base_var_env_name ("orte_base_applied_binding", &param);
-                    opal_setenv(param, child->cpu_bitmap, true, &environ_copy);
-                    free(param);
-                    goto PROCEED;
-                }
-                /* convert the list to a cpu bitmap */
-                cpuset = hwloc_bitmap_alloc();
-                if (0 != (rc = hwloc_bitmap_list_sscanf(cpuset, child->cpu_bitmap))) {
-                    /* See comment above about "This may be a small memory leak" */
-                    asprintf(&msg, "hwloc_bitmap_sscanf returned \"%s\" for the string \"%s\"",
-                             opal_strerror(rc), child->cpu_bitmap);
-                    if (NULL == msg) {
-                        msg = "failed to convert bitmap list to hwloc bitmap";
-                    }
-                    if (OPAL_BINDING_REQUIRED(jobdat->map->binding) &&
-                        OPAL_BINDING_POLICY_IS_SET(jobdat->map->binding)) {
-                        /* If binding is required and a binding directive was explicitly
-                         * given (i.e., we are not binding due to a default policy),
-                         * send an error up the pipe (which exits -- it doesn't return).
-                         */
-                        send_error_show_help(write_fd, 1, "help-orte-odls-default.txt",
-                                             "binding generic error",
-                                             orte_process_info.nodename, 
-                                             context->app, msg, 
-                                             __FILE__, __LINE__);
-                    } else {
-                        send_warn_show_help(write_fd,
-                                            "help-orte-odls-default.txt", "not bound",
-                                            orte_process_info.nodename, context->app, msg,
-                                            __FILE__, __LINE__);
-                        goto PROCEED;
-                    }
-                }
-                /* bind as specified */
-                rc = hwloc_set_cpubind(opal_hwloc_topology, cpuset, 0);
-                /* if we got an error and this wasn't a default binding policy, then report it */
-                if (rc < 0  && OPAL_BINDING_POLICY_IS_SET(jobdat->map->binding)) {
-                    if (errno == ENOSYS) {
-                        msg = "hwloc indicates cpu binding not supported";
-                    } else if (errno == EXDEV) {
-                        msg = "hwloc indicates cpu binding cannot be enforced";
-                    } else {
-                        asprintf(&msg, "hwloc_set_cpubind returned \"%s\" for bitmap \"%s\"",
-                                 opal_strerror(rc), child->cpu_bitmap);
-                    }
-                    if (OPAL_BINDING_REQUIRED(jobdat->map->binding)) {
-                        /* If binding is required, send an error up the pipe (which exits
-                           -- it doesn't return). */
-                        send_error_show_help(write_fd, 1, "help-orte-odls-default.txt",
-                                             "binding generic error",
-                                             orte_process_info.nodename, context->app, msg, 
-                                             __FILE__, __LINE__);
-                    } else {
-                        send_warn_show_help(write_fd,
-                                            "help-orte-odls-default.txt", "not bound",
-                                            orte_process_info.nodename, context->app, msg,
-                                            __FILE__, __LINE__);
-                        goto PROCEED;
-                    }
-                }
-                if (0 == rc && opal_hwloc_report_bindings) {
-                    char tmp1[1024], tmp2[1024];
-                    hwloc_cpuset_t mycpus;
-                    /* get the cpus we are bound to */
-                    mycpus = hwloc_bitmap_alloc();
-                    if (hwloc_get_cpubind(opal_hwloc_topology, 
-                                          mycpus, 
-                                          HWLOC_CPUBIND_PROCESS) < 0) {
-                        opal_output(0, "MCW rank %d is not bound",
-                                    child->name.vpid);
-                    } else {
-                        if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), opal_hwloc_topology, mycpus)) {
-                            opal_output(0, "MCW rank %d is not bound (or bound to all available processors)", child->name.vpid);
-                        } else {
-                            opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), opal_hwloc_topology, mycpus);
-                            opal_output(0, "MCW rank %d bound to %s: %s",
-                                        child->name.vpid, tmp1, tmp2);
-                        }
-                    }
-                    hwloc_bitmap_free(mycpus);
-                    /* avoid reporting it twice */
-                    (void) mca_base_var_env_name ("hwloc_base_report_bindings", &param);
-                    opal_unsetenv(param, &environ_copy);
-                    free(param);
-                }
-                /* set memory affinity policy - if we get an error, don't report
-                 * anything unless the user actually specified the binding policy
-                 */
-                rc = opal_hwloc_base_set_process_membind_policy();
-                if (ORTE_SUCCESS != rc  && OPAL_BINDING_POLICY_IS_SET(jobdat->map->binding)) {
-                    if (errno == ENOSYS) {
-                        msg = "hwloc indicates memory binding not supported";
-                    } else if (errno == EXDEV) {
-                        msg = "hwloc indicates memory binding cannot be enforced";
-                    } else {
-                        msg = "failed to bind memory";
-                    }
-                    if (OPAL_HWLOC_BASE_MBFA_ERROR == opal_hwloc_base_mbfa) {
-                        /* If binding is required, send an error up the pipe (which exits
-                           -- it doesn't return). */
-                        send_error_show_help(write_fd, 1, "help-orte-odls-default.txt",
-                                             "memory binding error",
-                                             orte_process_info.nodename, context->app, msg, 
-                                             __FILE__, __LINE__);
-                    } else {
-                        send_warn_show_help(write_fd,
-                                            "help-orte-odls-default.txt", "memory not bound",
-                                            orte_process_info.nodename, context->app, msg,
-                                            __FILE__, __LINE__);
-                        goto PROCEED;
-                    }
-                }
+        if (ORTE_FLAG_TEST(jobdat, ORTE_JOB_FLAG_FORWARD_OUTPUT)) {
+            if (ORTE_SUCCESS != (i = orte_iof_base_setup_child(&opts,
+                                                               &environ_copy))) {
+                ORTE_ERROR_LOG(i);
+                send_error_show_help(write_fd, 1,
+                                     "help-orte-odls-default.txt",
+                                     "iof setup failed",
+                                     orte_process_info.nodename, context->app);
+                /* Does not return */
             }
         }
-#endif
 
-    } else if (!(ORTE_JOB_CONTROL_FORWARD_OUTPUT & jobdat->controls)) {
+        /* now set any child-level controls such as binding */
+        orte_rtc.set(jobdat, child, &environ_copy, write_fd);
+
+    } else if (!ORTE_FLAG_TEST(jobdat, ORTE_JOB_FLAG_FORWARD_OUTPUT)) {
         /* tie stdin/out/err/internal to /dev/null */
         int fdnull;
         for (i=0; i < 3; i++) {
@@ -661,15 +448,11 @@ static int do_child(orte_app_context_t* context,
         close(fdnull);
     }
 
-#if OPAL_HAVE_HWLOC
- PROCEED:
-#endif
-
     /* if the user requested it, set the system resource limits */
     if (OPAL_SUCCESS != (rc = opal_util_init_sys_limits(&msg))) {
         send_error_show_help(write_fd, 1, "help-orte-odls-default.txt",
                              "set limit",
-                             orte_process_info.nodename, context->app, 
+                             orte_process_info.nodename, context->app,
                              __FILE__, __LINE__, msg);
     }
     /* ensure we only do this once */
@@ -688,48 +471,37 @@ static int do_child(orte_app_context_t* context,
             }
         }
     }
-    
+
     if (context->argv == NULL) {
         context->argv = malloc(sizeof(char*)*2);
         context->argv[0] = strdup(context->app);
         context->argv[1] = NULL;
     }
-    
+
     /* Set signal handlers back to the default.  Do this close to
        the exev() because the event library may (and likely will)
        reset them.  If we don't do this, the event library may
        have left some set that, at least on some OS's, don't get
        reset via fork() or exec().  Hence, the launched process
        could be unkillable (for example). */
-    
+
     set_handler_default(SIGTERM);
     set_handler_default(SIGINT);
     set_handler_default(SIGHUP);
     set_handler_default(SIGPIPE);
     set_handler_default(SIGCHLD);
-    
+
     /* Unblock all signals, for many of the same reasons that we
        set the default handlers, above.  This is noticable on
        Linux where the event library blocks SIGTERM, but we don't
        want that blocked by the launched process. */
     sigprocmask(0, 0, &sigs);
     sigprocmask(SIG_UNBLOCK, &sigs, 0);
-    
+
     /* Exec the new executable */
-    
-    if (10 < opal_output_get_verbosity(orte_odls_base_framework.framework_output)) {
-        int jout;
-        opal_output(0, "%s STARTING %s", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), context->app);
-        for (jout=0; NULL != context->argv[jout]; jout++) {
-            opal_output(0, "%s\tARGV[%d]: %s", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), jout, context->argv[jout]);
-        }
-        for (jout=0; NULL != environ_copy[jout]; jout++) {
-            opal_output(0, "%s\tENVIRON[%d]: %s", ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), jout, environ_copy[jout]);
-        }
-    }
-    
+
     execve(context->app, context->argv, environ_copy);
-    send_error_show_help(write_fd, 1, 
+    send_error_show_help(write_fd, 1,
                          "help-orte-odls-default.txt", "execve error",
                          orte_process_info.nodename, context->app, strerror(errno));
     /* Does not return */
@@ -743,10 +515,10 @@ static int do_parent(orte_app_context_t* context,
                      orte_iof_base_io_conf_t opts)
 {
     int rc;
-    pipe_err_msg_t msg;
-    char file[MAX_FILE_LEN + 1], topic[MAX_TOPIC_LEN + 1], *str = NULL;
+    orte_odls_pipe_err_msg_t msg;
+    char file[ORTE_ODLS_MAX_FILE_LEN + 1], topic[ORTE_ODLS_MAX_TOPIC_LEN + 1], *str = NULL;
 
-    if (NULL != child && (ORTE_JOB_CONTROL_FORWARD_OUTPUT & jobdat->controls)) {
+    if (NULL != child && ORTE_FLAG_TEST(jobdat, ORTE_JOB_FLAG_FORWARD_OUTPUT)) {
         /* connect endpoints IOF */
         rc = orte_iof_base_setup_parent(&child->name, &opts);
         if (ORTE_SUCCESS != rc) {
@@ -768,12 +540,12 @@ static int do_parent(orte_app_context_t* context,
         if (OPAL_ERR_TIMEOUT == rc) {
             break;
         }
-        
+
         /* If Something Bad happened in the read, error out */
         if (OPAL_SUCCESS != rc) {
             ORTE_ERROR_LOG(rc);
             close(read_fd);
-            
+
             if (NULL != child) {
                 child->state = ORTE_PROC_STATE_UNDEF;
             }
@@ -782,14 +554,18 @@ static int do_parent(orte_app_context_t* context,
 
         /* Otherwise, we got a warning or error message from the child */
         if (NULL != child) {
-            child->alive = msg.fatal ? 0 : 1;
+            if (msg.fatal) {
+                ORTE_FLAG_UNSET(child, ORTE_PROC_FLAG_ALIVE);
+            } else {
+                ORTE_FLAG_SET(child, ORTE_PROC_FLAG_ALIVE);
+            }
         }
 
         /* Read in the strings; ensure to terminate them with \0 */
         if (msg.file_str_len > 0) {
             rc = opal_fd_read(read_fd, msg.file_str_len, file);
             if (OPAL_SUCCESS != rc) {
-                orte_show_help("help-orte-odls-default.txt", "syscall fail", 
+                orte_show_help("help-orte-odls-default.txt", "syscall fail",
                                true,
                                orte_process_info.nodename, context->app,
                                "opal_fd_read", __FILE__, __LINE__);
@@ -803,7 +579,7 @@ static int do_parent(orte_app_context_t* context,
         if (msg.topic_str_len > 0) {
             rc = opal_fd_read(read_fd, msg.topic_str_len, topic);
             if (OPAL_SUCCESS != rc) {
-                orte_show_help("help-orte-odls-default.txt", "syscall fail", 
+                orte_show_help("help-orte-odls-default.txt", "syscall fail",
                                true,
                                orte_process_info.nodename, context->app,
                                "opal_fd_read", __FILE__, __LINE__);
@@ -817,7 +593,7 @@ static int do_parent(orte_app_context_t* context,
         if (msg.msg_str_len > 0) {
             str = calloc(1, msg.msg_str_len + 1);
             if (NULL == str) {
-                orte_show_help("help-orte-odls-default.txt", "syscall fail", 
+                orte_show_help("help-orte-odls-default.txt", "syscall fail",
                                true,
                                orte_process_info.nodename, context->app,
                                "opal_fd_read", __FILE__, __LINE__);
@@ -845,7 +621,7 @@ static int do_parent(orte_app_context_t* context,
         if (msg.fatal) {
             if (NULL != child) {
                 child->state = ORTE_PROC_STATE_FAILED_TO_START;
-                child->alive = false;
+                ORTE_FLAG_UNSET(child, ORTE_PROC_FLAG_ALIVE);
             }
             close(read_fd);
             return ORTE_ERR_FAILED_TO_START;
@@ -857,10 +633,10 @@ static int do_parent(orte_app_context_t* context,
        launched successfully. */
     if (NULL != child) {
         child->state = ORTE_PROC_STATE_RUNNING;
-        child->alive = true;
+        ORTE_FLAG_SET(child, ORTE_PROC_FLAG_ALIVE);
     }
     close(read_fd);
-    
+
     return ORTE_SUCCESS;
 }
 
@@ -876,12 +652,12 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
     orte_iof_base_io_conf_t opts;
     int rc, p[2];
     pid_t pid;
-    
+
     if (NULL != child) {
         /* should pull this information from MPIRUN instead of going with
          default */
         opts.usepty = OPAL_ENABLE_PTY_SUPPORT;
-        
+
         /* do we want to setup stdin? */
         if (NULL != child &&
             (jobdat->stdin_target == ORTE_VPID_WILDCARD ||
@@ -890,7 +666,7 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
         } else {
             opts.connect_stdin = false;
         }
-        
+
         if (ORTE_SUCCESS != (rc = orte_iof_base_setup_prefork(&opts))) {
             ORTE_ERROR_LOG(rc);
             if (NULL != child) {
@@ -917,13 +693,13 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
         }
         return ORTE_ERR_SYS_LIMITS_PIPES;
     }
-    
+
     /* Fork off the child */
     pid = fork();
     if (NULL != child) {
         child->pid = pid;
     }
-    
+
     if (pid < 0) {
         ORTE_ERROR_LOG(ORTE_ERR_SYS_LIMITS_CHILDREN);
         if (NULL != child) {
@@ -932,7 +708,7 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
         }
         return ORTE_ERR_SYS_LIMITS_CHILDREN;
     }
-    
+
     if (pid == 0) {
 	close(p[0]);
 #if HAVE_SETPGID
@@ -940,7 +716,7 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
 #endif
         do_child(context, child, environ_copy, jobdat, p[1], opts);
         /* Does not return */
-    } 
+    }
 
     close(p[1]);
     return do_parent(context, child, environ_copy, jobdat, p[0], opts);
@@ -963,22 +739,22 @@ int orte_odls_default_launch_local_procs(opal_buffer_t *data)
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), ORTE_ERROR_NAME(rc)));
         return rc;
     }
-    
+
     /* launch the local procs */
     ORTE_ACTIVATE_LOCAL_LAUNCH(job, odls_default_fork_local_proc);
-    
+
     return ORTE_SUCCESS;
 }
 
 
 /**
  * Send a signal to a pid.  Note that if we get an error, we set the
- * return value and let the upper layer print out the message.  
+ * return value and let the upper layer print out the message.
  */
 static int send_signal(pid_t pid, int signal)
 {
     int rc = ORTE_SUCCESS;
-    
+
     OPAL_OUTPUT_VERBOSE((1, orte_odls_base_framework.framework_output,
                          "%s sending signal %d to pid %ld",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -997,8 +773,8 @@ static int send_signal(pid_t pid, int signal)
             case ESRCH:
                 /* This case can occur when we deliver a signal to a
                    process that is no longer there.  This can happen if
-                   we deliver a signal while the job is shutting down. 
-                   This does not indicate a real problem, so just 
+                   we deliver a signal while the job is shutting down.
+                   This does not indicate a real problem, so just
                    ignore the error.  */
                 break;
             case EPERM:
@@ -1008,14 +784,14 @@ static int send_signal(pid_t pid, int signal)
                 rc = ORTE_ERROR;
         }
     }
-    
+
     return rc;
 }
 
 static int orte_odls_default_signal_local_procs(const orte_process_name_t *proc, int32_t signal)
 {
     int rc;
-    
+
     if (ORTE_SUCCESS != (rc = orte_odls_base_default_signal_local_procs(proc, signal, send_signal))) {
         ORTE_ERROR_LOG(rc);
         return rc;
@@ -1026,7 +802,7 @@ static int orte_odls_default_signal_local_procs(const orte_process_name_t *proc,
 static int orte_odls_default_restart_proc(orte_proc_t *child)
 {
     int rc;
-    
+
     /* restart the local proc */
     if (ORTE_SUCCESS != (rc = orte_odls_base_default_restart_proc(child, odls_default_fork_local_proc))) {
         OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
