@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2008 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,22 +6,22 @@
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2011 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
- * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2010      Oracle and/or its affiliates.  All rights 
+ * Copyright (c) 2007-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2010      Oracle and/or its affiliates.  All rights
+ *                         reserved.
+ * Copyright (c) 2009-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011      IBM Corporation.  All rights reserved.
  * Copyright (c) 2015      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  *
  * These symbols are in a file by themselves to provide nice linker
@@ -42,8 +43,10 @@
 #include "opal/util/opal_environ.h"
 #include "opal/util/output.h"
 #include "opal/util/argv.h"
+#include "opal/util/basename.h"
 #include "opal/util/path.h"
 
+#include "orte/mca/state/state.h"
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/util/show_help.h"
@@ -78,25 +81,24 @@ orte_plm_rsh_component_t mca_plm_rsh_component = {
     /* First, the mca_component_t struct containing meta information
        about the component itself */
 
-    {
-        ORTE_PLM_BASE_VERSION_2_0_0,
+        .base_version = {
+            ORTE_PLM_BASE_VERSION_2_0_0,
 
-        /* Component name and version */
-        "rsh",
-        ORTE_MAJOR_VERSION,
-        ORTE_MINOR_VERSION,
-        ORTE_RELEASE_VERSION,
+            /* Component name and version */
+            .mca_component_name = "rsh",
+            MCA_BASE_MAKE_VERSION(component, ORTE_MAJOR_VERSION, ORTE_MINOR_VERSION,
+                                  ORTE_RELEASE_VERSION),
 
-        /* Component open and close functions */
-        rsh_component_open,
-        rsh_component_close,
-        rsh_component_query,
-        rsh_component_register
-    },
-    {
-        /* The component is checkpoint ready */
-        MCA_BASE_METADATA_PARAM_CHECKPOINT
-    }
+            /* Component open and close functions */
+            .mca_open_component = rsh_component_open,
+            .mca_close_component = rsh_component_close,
+            .mca_query_component = rsh_component_query,
+            .mca_register_component_params = rsh_component_register,
+        },
+        .base_data = {
+            /* The component is checkpoint ready */
+            MCA_BASE_METADATA_PARAM_CHECKPOINT
+        },
     }
 };
 
@@ -206,7 +208,7 @@ static int rsh_component_register(void)
     (void) mca_base_component_var_register (c, "args",
                                             "Arguments to add to rsh/ssh",
                                             MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                                            OPAL_INFO_LVL_9,
+                                            OPAL_INFO_LVL_2,
                                             MCA_BASE_VAR_SCOPE_READONLY,
                                             &mca_plm_rsh_component.ssh_args);
 
@@ -214,7 +216,7 @@ static int rsh_component_register(void)
     (void) mca_base_component_var_register (c, "pass_libpath",
                                             "Prepend the specified library path to the remote shell's LD_LIBRARY_PATH",
                                             MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                                            OPAL_INFO_LVL_9,
+                                            OPAL_INFO_LVL_2,
                                             MCA_BASE_VAR_SCOPE_READONLY,
                                             &mca_plm_rsh_component.pass_libpath);
 
@@ -228,6 +230,7 @@ static int rsh_component_open(void)
     /* initialize globals */
     mca_plm_rsh_component.using_qrsh = false;
     mca_plm_rsh_component.using_llspawn = false;
+    mca_plm_rsh_component.agent_argv = NULL;
 
     /* lookup parameters */
     if (mca_plm_rsh_component.num_concurrent <= 0) {
@@ -253,61 +256,72 @@ static int rsh_component_open(void)
 static int rsh_component_query(mca_base_module_t **module, int *priority)
 {
     char *tmp;
-    
+
     /* Check if we are under Grid Engine parallel environment by looking at several
      * environment variables.  If so, setup the path and argv[0]. */
-    if (!mca_plm_rsh_component.disable_qrsh &&
-        NULL != getenv("SGE_ROOT") && NULL != getenv("ARC") &&
-        NULL != getenv("PE_HOSTFILE") && NULL != getenv("JOB_ID")) {
-        /* setup the search path for qrsh */
-        asprintf(&tmp, "%s/bin/%s", getenv("SGE_ROOT"), getenv("ARC"));
-        /* see if the agent is available */
-        if (ORTE_SUCCESS != rsh_launch_agent_lookup("qrsh", tmp)) {
-            /* can't be SGE */
-             opal_output_verbose(1, orte_plm_base_framework.framework_output,
-                                "%s plm:rsh: unable to be used: SGE indicated but cannot find path "
-                                "or execution permissions not set for launching agent qrsh", 
-                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-             free(tmp);
-             *module = NULL;
-             return ORTE_ERROR;
+    if (NULL == mca_plm_rsh_component.agent) {
+        if (!mca_plm_rsh_component.disable_qrsh &&
+            NULL != getenv("SGE_ROOT") && NULL != getenv("ARC") &&
+            NULL != getenv("PE_HOSTFILE") && NULL != getenv("JOB_ID")) {
+            /* setup the search path for qrsh */
+            asprintf(&tmp, "%s/bin/%s", getenv("SGE_ROOT"), getenv("ARC"));
+            /* see if the agent is available */
+            if (ORTE_SUCCESS != rsh_launch_agent_lookup("qrsh", tmp)) {
+                /* can't be SGE */
+                 opal_output_verbose(1, orte_plm_base_framework.framework_output,
+                                    "%s plm:rsh: unable to be used: SGE indicated but cannot find path "
+                                    "or execution permissions not set for launching agent qrsh",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+                 free(tmp);
+                 *module = NULL;
+                 return ORTE_ERROR;
+            }
+            mca_plm_rsh_component.agent = tmp;
+            mca_plm_rsh_component.using_qrsh = true;
+            /* no tree spawn allowed under qrsh */
+            mca_plm_rsh_component.no_tree_spawn = true;
+            goto success;
+        } else if (!mca_plm_rsh_component.disable_llspawn &&
+                   NULL != getenv("LOADL_STEP_ID")) {
+            /* We are running  as a LOADLEVELER job.
+             * Search for llspawn in the users PATH */
+            if (ORTE_SUCCESS != rsh_launch_agent_lookup("llspawn", NULL)) {
+                 opal_output_verbose(1, orte_plm_base_framework.framework_output,
+                                    "%s plm:rsh: unable to be used: LoadLeveler "
+                                    "indicated but cannot find path or execution "
+                                    "permissions not set for launching agent llspawn",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+                *module = NULL;
+                return ORTE_ERROR;
+            }
+            mca_plm_rsh_component.agent = strdup("llspawn");
+            mca_plm_rsh_component.using_llspawn = true;
+            goto success;
         }
-        free(tmp);
-        mca_plm_rsh_component.using_qrsh = true;
-        /* no tree spawn allowed under qrsh */
-        mca_plm_rsh_component.no_tree_spawn = true;
-        goto success; 
-    } else if (!mca_plm_rsh_component.disable_llspawn &&
-               NULL != getenv("LOADL_STEP_ID")) { 
-	/* We are running  as a LOADLEVELER job.
-	   Search for llspawn in the users PATH */
-        if (ORTE_SUCCESS != rsh_launch_agent_lookup("llspawn", NULL)) {
-             opal_output_verbose(1, orte_plm_base_framework.framework_output,
-                                "%s plm:rsh: unable to be used: LoadLeveler "
-                                "indicated but cannot find path or execution "
-                                "permissions not set for launching agent llspawn",
-                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-            *module = NULL;
-            return ORTE_ERROR;
-        }
-        mca_plm_rsh_component.using_llspawn = true;
-        goto success;
     }
-    
-    /* if this isn't an Grid Engine or LoadLeveler environment, 
-       see if MCA-specified agent (default: ssh:rsh) is available */
-    
+
+    /* if this isn't an Grid Engine or LoadLeveler environment, or
+     * if the user specified a launch agent, look for it */
+
     if (ORTE_SUCCESS != rsh_launch_agent_lookup(NULL, NULL)) {
+        /* if the user specified an agent and we couldn't find it,
+         * then we want to error out and not continue */
+        if (NULL != mca_plm_rsh_component.agent) {
+            orte_show_help("help-plm-rsh.txt", "agent-not-found", true,
+                           mca_plm_rsh_component.agent);
+            ORTE_FORCED_TERMINATE(ORTE_ERR_NOT_FOUND);
+            return ORTE_ERR_FATAL;
+        }
         /* this isn't an error - we just cannot be selected */
         OPAL_OUTPUT_VERBOSE((1, orte_plm_base_framework.framework_output,
                              "%s plm:rsh: unable to be used: cannot find path "
-                             "for launching agent \"%s\"\n", 
+                             "for launching agent \"%s\"\n",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              mca_plm_rsh_component.agent));
         *module = NULL;
         return ORTE_ERROR;
     }
-success: 
+success:
     /* we are good - make ourselves available */
     *priority = mca_plm_rsh_component.priority;
     *module = (mca_base_module_t *) &orte_plm_rsh_module;
@@ -331,11 +345,12 @@ char **orte_plm_rsh_search(const char* agent_list, const char *path)
     char *line, **lines;
     char **tokens, *tmp;
     char cwd[OPAL_PATH_MAX];
-    
+
     if (NULL == path) {
         getcwd(cwd, OPAL_PATH_MAX);
     } else {
-        strncpy(cwd, path, OPAL_PATH_MAX);
+        strncpy(cwd, path, OPAL_PATH_MAX - 1);
+        cwd[OPAL_PATH_MAX - 1] = '\0';
     }
     if (NULL == agent_list) {
         lines = opal_argv_split(mca_plm_rsh_component.agent, ':');
@@ -344,7 +359,7 @@ char **orte_plm_rsh_search(const char* agent_list, const char *path)
     }
     for (i = 0; NULL != lines[i]; ++i) {
         line = lines[i];
-        
+
         /* Trim whitespace at the beginning and end of the line */
         for (j = 0; '\0' != line[j] && isspace(line[j]); ++line) {
             continue;
@@ -355,10 +370,10 @@ char **orte_plm_rsh_search(const char* agent_list, const char *path)
         if (strlen(line) <= 0) {
             continue;
         }
-        
+
         /* Split it */
         tokens = opal_argv_split(line, ' ');
-        
+
         /* Look for the first token in the PATH */
         tmp = opal_path_findv(tokens[0], X_OK, environ, cwd);
         if (NULL != tmp) {
@@ -367,11 +382,11 @@ char **orte_plm_rsh_search(const char* agent_list, const char *path)
             opal_argv_free(lines);
             return tokens;
         }
-        
+
         /* Didn't find it */
         opal_argv_free(tokens);
     }
-    
+
     /* Doh -- didn't find anything */
     opal_argv_free(lines);
     return NULL;
@@ -379,19 +394,48 @@ char **orte_plm_rsh_search(const char* agent_list, const char *path)
 
 static int rsh_launch_agent_lookup(const char *agent_list, char *path)
 {
-    char **tmp;
+    char *bname;
+    int i;
 
     OPAL_OUTPUT_VERBOSE((5, orte_plm_base_framework.framework_output,
                          "%s plm:rsh_lookup on agent %s path %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          (NULL == agent_list) ? mca_plm_rsh_component.agent : agent_list,
                          (NULL == path) ? "NULL" : path));
-    if (NULL == (tmp = orte_plm_rsh_search(agent_list, path))) {
+    if (NULL == (mca_plm_rsh_component.agent_argv = orte_plm_rsh_search(agent_list, path))) {
         return ORTE_ERR_NOT_FOUND;
     }
 
-    /* if we got here, then one of the given agents could be found */
-    opal_argv_free(tmp);
+    /* if we got here, then one of the given agents could be found - the
+     * complete path is in the argv[0] position */
+    mca_plm_rsh_component.agent_path = strdup(mca_plm_rsh_component.agent_argv[0]);
+    bname = opal_basename(mca_plm_rsh_component.agent_argv[0]);
+    if (NULL == bname) {
+        return ORTE_SUCCESS;
+    }
+    /* replace the initial position with the basename */
+    free(mca_plm_rsh_component.agent_argv[0]);
+    mca_plm_rsh_component.agent_argv[0] = bname;
+    /* see if we need to add an xterm argument */
+    if (0 == strcmp(bname, "ssh")) {
+        /* if xterm option was given, add '-X', ensuring we don't do it twice */
+        if (NULL != orte_xterm) {
+            opal_argv_append_unique_nosize(&mca_plm_rsh_component.agent_argv, "-X", false);
+        } else if (0 >= opal_output_get_verbosity(orte_plm_base_framework.framework_output)) {
+            /* if debug was not specified, and the user didn't explicitly
+             * specify X11 forwarding/non-forwarding, add "-x" if it
+             * isn't already there (check either case)
+             */
+            for (i = 1; NULL != mca_plm_rsh_component.agent_argv[i]; ++i) {
+                if (0 == strcasecmp("-x", mca_plm_rsh_component.agent_argv[i])) {
+                    break;
+                }
+            }
+            if (NULL == mca_plm_rsh_component.agent_argv[i]) {
+                opal_argv_append_nosize(&mca_plm_rsh_component.agent_argv, "-x");
+            }
+        }
+    }
     return ORTE_SUCCESS;
 }
 
