@@ -9,6 +9,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2011-2012 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
+ * Copyright (c) 2014-2015 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -65,18 +68,22 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
     orte_std_cntr_t num_nodes;
     int rc, i;
     orte_node_t *node, *hnp_node;
+    char *ptr;
+    bool hnp_alone = true;
+    orte_attribute_t *kv;
+    char **alias=NULL, **nalias;
 
     /* get the number of nodes */
     num_nodes = (orte_std_cntr_t)opal_list_get_size(nodes);
     if (0 == num_nodes) {
         return ORTE_SUCCESS;  /* nothing to do */
     }
-    
-    OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+
+    OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
                          "%s ras:base:node_insert inserting %ld nodes",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          (long)num_nodes));
-    
+
     /* set the size of the global array - this helps minimize time
      * spent doing realloc's
      */
@@ -84,37 +91,44 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    
+
     /* get the hnp node's info */
-    hnp_node = (orte_node_t*)(orte_node_pool->addr[0]);
-    
+    hnp_node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, 0);
+
     /* cycle through the list */
     while (NULL != (item = opal_list_remove_first(nodes))) {
         node = (orte_node_t*)item;
-        
+
         /* the HNP had to already enter its node on the array - that entry is in the
          * first position since it is the first one entered. We need to check to see
          * if this node is the same as the HNP's node so we don't double-enter it
          */
-        if (0 == strcmp(node->name, hnp_node->name) ||
-            opal_ifislocal(node->name)) {
-            OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
-                                 "%s ras:base:node_insert updating HNP info to %ld slots",
+        if (NULL != hnp_node && orte_ifislocal(node->name)) {
+            OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
+                                 "%s ras:base:node_insert updating HNP [%s] info to %ld slots",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                 node->name,
                                  (long)node->slots));
-            
+
             /* flag that hnp has been allocated */
             orte_hnp_is_allocated = true;
-            /* adjust the total slots in the job */
-            jdata->total_slots_alloc -= hnp_node->slots;
+            /* update the total slots in the job */
+            orte_ras_base.total_slots_alloc = node->slots;
             /* copy the allocation data to that node's info */
             hnp_node->slots = node->slots;
             hnp_node->slots_max = node->slots_max;
-            hnp_node->launch_id = node->launch_id;
-            /* default allocate all the slots - may be modified later
-             * as a result of filtering actions in mapper
-             */
-            hnp_node->slots_alloc = node->slots;
+            /* copy across any attributes */
+            OPAL_LIST_FOREACH(kv, &node->attributes, orte_attribute_t) {
+                orte_set_attribute(&node->attributes, kv->key, ORTE_ATTR_LOCAL, &kv->data, kv->type);
+            }
+            if (orte_managed_allocation || ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
+                /* the slots are always treated as sacred
+                 * in managed allocations
+                 */
+                ORTE_FLAG_SET(hnp_node, ORTE_NODE_FLAG_SLOTS_GIVEN);
+            } else {
+                ORTE_FLAG_UNSET(hnp_node, ORTE_NODE_FLAG_SLOTS_GIVEN);
+            }
             /* use the local name for our node - don't trust what
              * we got from an RM. If requested, store the resolved
              * nodename info
@@ -122,30 +136,45 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
             if (orte_show_resolved_nodenames) {
                 /* if the node name is different, store it as an alias */
                 if (0 != strcmp(node->name, hnp_node->name)) {
-                    /* add to list of aliases for this node - only add if unique */
-                    opal_argv_append_unique_nosize(&hnp_node->alias, node->name, false);
-                }
-                if (NULL != node->alias) {
-                    /* now copy over any aliases that are unique */
-                    for (i=0; NULL != node->alias[i]; i++) {
-                        opal_argv_append_unique_nosize(&hnp_node->alias, node->alias[i], false);
+                    /* get any current list of aliases */
+                    ptr = NULL;
+                    orte_get_attribute(&hnp_node->attributes, ORTE_NODE_ALIAS, (void**)&ptr, OPAL_STRING);
+                    if (NULL != ptr) {
+                        alias = opal_argv_split(ptr, ',');
+                        free(ptr);
                     }
+                    /* add to list of aliases for this node - only add if unique */
+                    opal_argv_append_unique_nosize(&alias, node->name, false);
                 }
+                if (orte_get_attribute(&node->attributes, ORTE_NODE_ALIAS, (void**)&ptr, OPAL_STRING)) {
+                    nalias = opal_argv_split(ptr, ',');
+                    /* now copy over any aliases that are unique */
+                    for (i=0; NULL != nalias[i]; i++) {
+                        opal_argv_append_unique_nosize(&alias, nalias[i], false);
+                    }
+                    opal_argv_free(nalias);
+                }
+                /* and store the result */
+                ptr = opal_argv_join(alias, ',');
+                opal_argv_free(alias);
+                orte_set_attribute(&hnp_node->attributes, ORTE_NODE_ALIAS, ORTE_ATTR_LOCAL, ptr, OPAL_STRING);
+                free(ptr);
             }
-            /* update the total slots in the job */
-            jdata->total_slots_alloc += hnp_node->slots;
             /* don't keep duplicate copy */
             OBJ_RELEASE(node);
         } else {
             /* insert the object onto the orte_nodes global array */
-            OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
-                                 "%s ras:base:node_insert node %s",
+            OPAL_OUTPUT_VERBOSE((5, orte_ras_base_framework.framework_output,
+                                 "%s ras:base:node_insert node %s slots %d",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                 (NULL == node->name) ? "NULL" : node->name));
-            /* default allocate all the slots - may be modified later
-             * as a result of filtering actions in mapper
-             */
-            node->slots_alloc = node->slots;
+                                 (NULL == node->name) ? "NULL" : node->name,
+                                 node->slots));
+            if (orte_managed_allocation) {
+                /* the slots are always treated as sacred
+                 * in managed allocations
+                 */
+                ORTE_FLAG_SET(node, ORTE_NODE_FLAG_SLOTS_GIVEN);
+            }
             /* insert it into the array */
             node->index = opal_pointer_array_add(orte_node_pool, (void*)node);
             if (ORTE_SUCCESS > (rc = node->index)) {
@@ -153,9 +182,25 @@ int orte_ras_base_node_insert(opal_list_t* nodes, orte_job_t *jdata)
                 return rc;
             }
             /* update the total slots in the job */
-            jdata->total_slots_alloc += node->slots;
+            orte_ras_base.total_slots_alloc += node->slots;
+            /* check if we have fqdn names in the allocation */
+            if (NULL != strchr(node->name, '.')) {
+                orte_have_fqdn_allocation = true;
+            }
+            /* indicate the HNP is not alone */
+            hnp_alone = false;
         }
     }
-    
+
+    /* if we didn't find any fqdn names in the allocation, then
+     * ensure we don't have any domain info in the node record
+     * for the hnp
+     */
+    if (!orte_have_fqdn_allocation && !hnp_alone) {
+        if (NULL != (ptr = strchr(hnp_node->name, '.'))) {
+            *ptr = '\0';
+        }
+    }
+
     return ORTE_SUCCESS;
 }
