@@ -5,7 +5,7 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -13,10 +13,12 @@
  * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * Copyright (c) 2014-2015 Intel, Inc. All rights reserved.
+ * Copyright (c) 2016      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -25,7 +27,7 @@
 
 #include <string.h>
 
-#include "opal/mca/mca.h"
+#include "orte/mca/mca.h"
 #include "opal/util/output.h"
 #include "opal/mca/base/base.h"
 #include "opal/mca/hwloc/base/base.h"
@@ -50,7 +52,6 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
     orte_job_map_t *map;
     int rc, i;
     bool did_map;
-    opal_list_item_t *item;
     orte_rmaps_base_selected_module_t *mod;
     orte_job_t *parent;
     orte_state_caddy_t *caddy = (orte_state_caddy_t*)cbdata;
@@ -65,7 +66,7 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
      * ALLOWED TO CALL RMAPS INDEPENDENTLY. ONLY THE PLM CAN
      * DO SO, AND ALL PLM COMMANDS ARE RELAYED TO HNP
      */
-    
+
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                         "mca:rmaps: mapping job %s",
                         ORTE_JOBID_PRINT(jdata->jobid));
@@ -77,7 +78,7 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
      * NULL MAP FIELD
      * LONE EXCEPTION - WE COPY DISPLAY MAP ACROSS IF THEY
      * DIDN'T SET IT
-     */        
+     */
     if (NULL == jdata->map) {
         opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                             "mca:rmaps: creating new map for job %s",
@@ -114,7 +115,6 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
                                 "mca:rmaps mapping given - using default");
             map->mapping = orte_rmaps_base.mapping;
         } else {
-#if OPAL_HAVE_HWLOC
             /* default based on number of procs */
             if (nprocs <= 2) {
                 if (1 < orte_rmaps_base.cpus_per_rank) {
@@ -123,26 +123,22 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
                      * to byslot if nothing else was specified by the user.
                      */
                     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] mapping not given - using byslot", __LINE__);
+                                        "mca:rmaps mapping not given - using byslot");
                     ORTE_SET_MAPPING_POLICY(map->mapping, ORTE_MAPPING_BYSLOT);
                 } else if (opal_hwloc_use_hwthreads_as_cpus) {
                     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] mapping not given - using byhwthread", __LINE__);
+                                        "mca:rmaps mapping not given - using byhwthread");
                     ORTE_SET_MAPPING_POLICY(map->mapping, ORTE_MAPPING_BYHWTHREAD);
                 } else {
                     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] mapping not given - using bycore", __LINE__);
+                                        "mca:rmaps mapping not given - using bycore");
                     ORTE_SET_MAPPING_POLICY(map->mapping, ORTE_MAPPING_BYCORE);
                 }
             } else {
                 opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                    "mca:rmaps[%d] mapping not given - using bysocket", __LINE__);
+                                    "mca:rmaps mapping not given - using bysocket");
                 ORTE_SET_MAPPING_POLICY(map->mapping, ORTE_MAPPING_BYSOCKET);
             }
-#else
-            /* in the absence of hwloc, default to map-by slot */
-            ORTE_SET_MAPPING_POLICY(map->mapping, ORTE_MAPPING_BYSLOT);
-#endif
             /* check for oversubscribe directives */
             if (!(ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
                 if (orte_managed_allocation) {
@@ -191,7 +187,6 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
         }
         /* set the default mapping policy IFF it wasn't provided */
         if (!ORTE_MAPPING_POLICY_IS_SET(jdata->map->mapping)) {
-#if OPAL_HAVE_HWLOC
             /* default based on number of procs */
             if (nprocs <= 2) {
                 if (1 < orte_rmaps_base.cpus_per_rank) {
@@ -200,26 +195,22 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
                      * to byslot if nothing else was specified by the user.
                      */
                     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] mapping not given - using byslot", __LINE__);
+                                        "mca:rmaps mapping not given - using byslot");
                     ORTE_SET_MAPPING_POLICY(jdata->map->mapping, ORTE_MAPPING_BYSLOT);
                 } else if (opal_hwloc_use_hwthreads_as_cpus) {
                     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] mapping not given - using byhwthread", __LINE__);
+                                        "mca:rmaps mapping not given - using byhwthread");
                     ORTE_SET_MAPPING_POLICY(jdata->map->mapping, ORTE_MAPPING_BYHWTHREAD);
                 } else {
                     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] mapping not given - using bycore", __LINE__);
+                                        "mca:rmaps mapping not given - using bycore");
                     ORTE_SET_MAPPING_POLICY(jdata->map->mapping, ORTE_MAPPING_BYCORE);
                 }
             } else {
                 opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                    "mca:rmaps[%d] mapping not set by user - using bysocket", __LINE__);
+                                    "mca:rmaps mapping not set by user - using bysocket");
                 ORTE_SET_MAPPING_POLICY(jdata->map->mapping, ORTE_MAPPING_BYSOCKET);
             }
-#else
-            /* in the absence of hwloc, default to map-by slot */
-            ORTE_SET_MAPPING_POLICY(jdata->map->mapping, ORTE_MAPPING_BYSLOT);
-#endif
         }
         /* check for oversubscribe directives */
         if (!(ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
@@ -247,72 +238,53 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
         }
     }
 
-#if OPAL_HAVE_HWLOC
     /* define the binding policy for this job - if the user specified one
      * already (e.g., during the call to comm_spawn), then we don't
      * override it */
     if (!OPAL_BINDING_POLICY_IS_SET(jdata->map->binding)) {
-        /* if the user specified a default binding policy via
-         * MCA param, then we use it */
-        if (OPAL_BINDING_POLICY_IS_SET(opal_hwloc_binding_policy)) {
+        /* if the user specified that we allow oversubscription, then do not bind.
+         * otherwise, if the user explicitly mapped-by some object, then we default
+         * to binding to that object */
+        if ((ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping)) &&
+            !(ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
+            OPAL_SET_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_NONE);
+        } else if (OPAL_BINDING_POLICY_IS_SET(opal_hwloc_binding_policy)) {
+            /* if the user specified a default binding policy via
+             * MCA param, then we use it */
             jdata->map->binding = opal_hwloc_binding_policy;
         } else {
             orte_mapping_policy_t mpol;
-            mpol = ORTE_GET_MAPPING_POLICY(jdata->map->mapping);
-            /* if the user explicitly mapped-by some object, then we default
-             * to binding to that object */
+            mpol = ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping);
             if (ORTE_MAPPING_POLICY_IS_SET(jdata->map->mapping) &&
                 ORTE_MAPPING_BYBOARD < mpol && mpol < ORTE_MAPPING_BYSLOT) {
                 if (ORTE_MAPPING_BYHWTHREAD == mpol) {
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using byhwthread", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_HWTHREAD);
                 } else if (ORTE_MAPPING_BYCORE == mpol) {
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using bycore", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_CORE);
                 } else if (ORTE_MAPPING_BYL1CACHE == mpol) {
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using byl1cache", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_L1CACHE);
                 } else if (ORTE_MAPPING_BYL2CACHE == mpol) {
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using byl2cache", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_L2CACHE);
                 } else if (ORTE_MAPPING_BYL3CACHE == mpol) {
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using byl3cache", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_L3CACHE);
                 } else if (ORTE_MAPPING_BYSOCKET == mpol) {
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using bysocket", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_SOCKET);
                 } else if (ORTE_MAPPING_BYNUMA == mpol) {
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using bynuma", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_NUMA);
                 }
-            } else if (ORTE_MAPPING_BYNODE == mpol || ORTE_MAPPING_BYBOARD == mpol) {
-                jdata->map->binding = OPAL_BIND_TO_NONE;
             } else if (nprocs <= 2) {
                 /* if nothing was specified, then we default to a policy
                  * based on number of procs and cpus_per_rank */
                 if (1 < orte_rmaps_base.cpus_per_rank) {
                     /* assigning multiple cpus to a rank implies threading,
                      * so we only bind to the NUMA level */
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using bynuma", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_NUMA);
                 } else {
                     if (opal_hwloc_use_hwthreads_as_cpus) {
                         /* if we are using hwthread cpus, then bind to those */
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using byhwthread", __LINE__);
                         OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_HWTHREAD);
                     } else {
                         /* for performance, bind to core */
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using bycore", __LINE__);
                         OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_CORE);
                     }
                 }
@@ -320,19 +292,18 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
                 if (1 < orte_rmaps_base.cpus_per_rank) {
                     /* assigning multiple cpus to a rank implies threading,
                      * so we only bind to the NUMA level */
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using bynuma", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_NUMA);
                 } else {
                     /* for performance, bind to socket */
-                    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                        "mca:rmaps[%d] binding not given - using bysocket", __LINE__);
                     OPAL_SET_DEFAULT_BINDING_POLICY(jdata->map->binding, OPAL_BIND_TO_SOCKET);
                 }
             }
+            if (OPAL_BIND_OVERLOAD_ALLOWED(opal_hwloc_binding_policy)) {
+                jdata->map->binding |= OPAL_BIND_ALLOW_OVERLOAD;
+            }
         }
     }
-    
+
     /* if we are not going to launch, then we need to set any
      * undefined topologies to match our own so the mapper
      * can operate
@@ -341,7 +312,12 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
         orte_node_t *node;
         hwloc_topology_t t0;
         int i;
-        node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, 0);
+        if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, 0))) {
+            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+            OBJ_RELEASE(caddy);
+            ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_MAP_FAILED);
+            return;
+        }
         t0 = node->topology;
         for (i=1; i < orte_node_pool->size; i++) {
             if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, i))) {
@@ -352,7 +328,6 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
             }
         }
     }
-#endif
 
     /* cycle thru the available mappers until one agrees to map
      * the job
@@ -363,10 +338,7 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
         mod = (orte_rmaps_base_selected_module_t*)opal_list_get_first(&orte_rmaps_base.selected_modules);
         jdata->map->req_mapper = strdup(mod->component->mca_component_name);
     }
-    for (item = opal_list_get_first(&orte_rmaps_base.selected_modules);
-         item != opal_list_get_end(&orte_rmaps_base.selected_modules);
-         item = opal_list_get_next(item)) {
-        mod = (orte_rmaps_base_selected_module_t*)item;
+    OPAL_LIST_FOREACH(mod, &orte_rmaps_base.selected_modules, orte_rmaps_base_selected_module_t) {
         if (ORTE_SUCCESS == (rc = mod->module->map_job(jdata)) ||
             ORTE_ERR_RESOURCE_BUSY == rc) {
             did_map = true;
@@ -411,8 +383,7 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
         OBJ_RELEASE(caddy);
         return;
     }
-    
-#if OPAL_HAVE_HWLOC
+
     /* compute and save bindings */
     if (ORTE_SUCCESS != (rc = orte_rmaps_base_compute_bindings(jdata))) {
         ORTE_ERROR_LOG(rc);
@@ -420,8 +391,7 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
         OBJ_RELEASE(caddy);
         return;
     }
-#endif
-    
+
     /* set the offset so shared memory components can potentially
      * connect to any spawned jobs
      */
@@ -441,9 +411,14 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
      */
     if (jdata->map->display_map) {
         char *output=NULL;
-        int i, j;
+        int i, j, cnt;
         orte_node_t *node;
         orte_proc_t *proc;
+        char tmp1[1024];
+        hwloc_obj_t bd=NULL;;
+        opal_hwloc_locality_t locality;
+        orte_proc_t *p0;
+        char *p0bitmap, *procbitmap;
 
         if (orte_display_diffable_output) {
             /* intended solely to test mapping methods, this output
@@ -453,72 +428,67 @@ void orte_rmaps_base_map_job(int fd, short args, void *cbdata)
              * the output a line at a time here
              */
             /* display just the procs in a diffable format */
-            opal_output(orte_clean_output, "<map>\n\t<jobid=%s>\n\t<offset=%s>",
-                        ORTE_JOBID_PRINT(jdata->jobid), ORTE_VPID_PRINT(jdata->offset));
+            opal_output(orte_clean_output, "<map>\n");
             fflush(stderr);
             /* loop through nodes */
+            cnt = 0;
             for (i=0; i < jdata->map->nodes->size; i++) {
                 if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(jdata->map->nodes, i))) {
                     continue;
                 }
-                opal_output(orte_clean_output, "\t<host name=%s>", (NULL == node->name) ? "UNKNOWN" : node->name);
+                opal_output(orte_clean_output, "\t<host num=%d>", cnt);
                 fflush(stderr);
+                cnt++;
                 for (j=0; j < node->procs->size; j++) {
                     if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
                         continue;
                     }
-#if OPAL_HAVE_HWLOC
-                    {
-                        char locale[64];
-
-                        if (NULL != proc->locale) {
-                            hwloc_bitmap_list_snprintf(locale, 64, proc->locale->cpuset);
+                    memset(tmp1, 0, 1024);
+                    orte_get_attribute(&proc->attributes, ORTE_PROC_HWLOC_BOUND, (void**)&bd, OPAL_PTR);
+                    if (NULL == bd) {
+                        (void)strncpy(tmp1, "UNBOUND", strlen("UNBOUND"));
+                    } else {
+                        if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2mapstr(tmp1, sizeof(tmp1), node->topology, bd->cpuset)) {
+                            (void)strncpy(tmp1, "UNBOUND", strlen("UNBOUND"));
                         }
-                        opal_output(orte_clean_output, "\t\t<process rank=%s app_idx=%ld local_rank=%lu node_rank=%lu locale=%s binding=%s>",
-                                    ORTE_VPID_PRINT(proc->name.vpid),  (long)proc->app_idx,
-                                    (unsigned long)proc->local_rank,
-                                    (unsigned long)proc->node_rank, locale,
-                                    (NULL == proc->cpu_bitmap) ? "NULL" : proc->cpu_bitmap);
                     }
-#else
-                    opal_output(orte_clean_output, "\t\t<process rank=%s app_idx=%ld local_rank=%lu node_rank=%lu>",
+                    opal_output(orte_clean_output, "\t\t<process rank=%s app_idx=%ld local_rank=%lu node_rank=%lu binding=%s>",
                                 ORTE_VPID_PRINT(proc->name.vpid),  (long)proc->app_idx,
                                 (unsigned long)proc->local_rank,
-                                (unsigned long)proc->node_rank);
-#endif
-                    fflush(stderr);
+                                (unsigned long)proc->node_rank, tmp1);
                 }
                 opal_output(orte_clean_output, "\t</host>");
                 fflush(stderr);
             }
-#if OPAL_HAVE_HWLOC
-            {
-                opal_hwloc_locality_t locality;
-                orte_proc_t *p0;
 
-                /* test locality - for the first node, print the locality of each proc relative to the first one */
-                node = (orte_node_t*)opal_pointer_array_get_item(jdata->map->nodes, 0);
-                p0 = (orte_proc_t*)opal_pointer_array_get_item(node->procs, 0);
-                opal_output(orte_clean_output, "\t<locality>");
-                for (j=1; j < node->procs->size; j++) {
-                    if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
-                        continue;
-                    }
-                    locality = opal_hwloc_base_get_relative_locality(node->topology,
-                                                                     p0->cpu_bitmap,
-                                                                     proc->cpu_bitmap);
-                    opal_output(orte_clean_output, "\t\t<rank=%s rank=%s locality=%s>",
-                                ORTE_VPID_PRINT(p0->name.vpid),
-                                ORTE_VPID_PRINT(proc->name.vpid),
-                                opal_hwloc_base_print_locality(locality));
+             /* test locality - for the first node, print the locality of each proc relative to the first one */
+            node = (orte_node_t*)opal_pointer_array_get_item(jdata->map->nodes, 0);
+            p0 = (orte_proc_t*)opal_pointer_array_get_item(node->procs, 0);
+            p0bitmap = NULL;
+            orte_get_attribute(&p0->attributes, ORTE_PROC_CPU_BITMAP, (void**)&p0bitmap, OPAL_STRING);
+            opal_output(orte_clean_output, "\t<locality>");
+            for (j=1; j < node->procs->size; j++) {
+                if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
+                    continue;
                 }
-                opal_output(orte_clean_output, "\t</locality>\n</map>");
-                fflush(stderr);
+                procbitmap = NULL;
+                orte_get_attribute(&proc->attributes, ORTE_PROC_CPU_BITMAP, (void**)&procbitmap, OPAL_STRING);
+                locality = opal_hwloc_base_get_relative_locality(node->topology,
+                                                                 p0bitmap,
+                                                                 procbitmap);
+                opal_output(orte_clean_output, "\t\t<rank=%s rank=%s locality=%s>",
+                            ORTE_VPID_PRINT(p0->name.vpid),
+                            ORTE_VPID_PRINT(proc->name.vpid),
+                            opal_hwloc_base_print_locality(locality));
             }
-#else
-            opal_output(orte_clean_output, "\n</map>");
+            opal_output(orte_clean_output, "\t</locality>\n</map>");
             fflush(stderr);
-#endif
+            if (NULL != p0bitmap) {
+                free(p0bitmap);
+            }
+            if (NULL != procbitmap) {
+                free(procbitmap);
+            }
         } else {
             opal_output(orte_clean_output, " Data for JOB %s offset %s", ORTE_JOBID_PRINT(jdata->jobid), ORTE_VPID_PRINT(jdata->offset));
             opal_dss.print(&output, NULL, jdata->map, ORTE_JOB_MAP);
