@@ -5,17 +5,20 @@
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
+ * Copyright (c) 2014-2016 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -27,9 +30,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif  /* HAVE_UNISTD_H */
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif  /* HAVE_STRING_H */
 
 #include "opal/mca/base/mca_base_var.h"
 
@@ -47,52 +48,6 @@ orte_rmaps_base_module_t orte_rmaps_mindist_module = {
     mindist_map
 };
 
-static int num_devices_in_list(char *list)
-{
-    int count = 0;
-    list = strtok(list, ",");
-    while (NULL != list) {
-        ++count;
-        list = strtok(NULL, ",");
-    }
-    return count;
-}
-
-static char* get_hca_name(orte_app_context_t *app)
-{
-    int found_ind = -1;
-    char** env = app->env;
-    int i;
-    for (i = 0; env[i]; i++) {
-        if (strstr(env[i], "OMPI_MCA_btl_openib_if_include") != NULL) {
-            found_ind = i;
-            break;
-        }
-    }
-    if (found_ind == -1) {
-        for (i = 0; env[i]; i++) {
-            if (strstr(env[i], "MXM_RDMA_PORTS") != NULL) {
-                found_ind = i;
-                break;
-            }
-        }
-    }
-    if (found_ind != -1) {
-        char* start = strstr(env[found_ind], "=");
-        if (start != NULL) {
-            start = strdup(start+sizeof(char));
-            if (num_devices_in_list(start) == 1) {
-                return strtok(start, ":");
-            }
-            else {
-                free(start);
-                return NULL;
-            }
-        }
-    }
-    return NULL;
-}
-
 /*
  * Create a round-robin mapping for the job.
  */
@@ -106,7 +61,7 @@ static int mindist_map(orte_job_t *jdata)
     opal_list_t numa_list;
     opal_list_item_t *item;
     opal_list_item_t *numa_item;
-    orte_rmaps_numa_node_t *numa;
+    opal_rmaps_numa_node_t *numa;
     orte_node_t *node;
     orte_proc_t *proc;
     int nprocs_mapped;
@@ -118,12 +73,11 @@ static int mindist_map(orte_job_t *jdata)
     bool initial_map=true;
     bool bynode = false;
     int ret;
-    char* tmp = NULL;
 
     /* this mapper can only handle initial launch
      * when mindist mapping is desired
      */
-    if (ORTE_JOB_CONTROL_RESTART & jdata->controls) {
+    if (ORTE_FLAG_TEST(jdata, ORTE_JOB_FLAG_RESTART)) {
         opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                             "mca:rmaps:mindist: job %s is being restarted - mindist cannot map",
                             ORTE_JOBID_PRINT(jdata->jobid));
@@ -171,7 +125,7 @@ static int mindist_map(orte_job_t *jdata)
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                         "mca:rmaps:mindist: mapping job %s",
                         ORTE_JOBID_PRINT(jdata->jobid));
- 
+
     /* flag that I did the mapping */
     if (NULL != jdata->map->last_mapper) {
         free(jdata->map->last_mapper);
@@ -180,13 +134,13 @@ static int mindist_map(orte_job_t *jdata)
 
     /* start at the beginning... */
     jdata->num_procs = 0;
-    
+
     /* cycle through the app_contexts, mapping them sequentially */
     for(i=0; i < jdata->apps->size; i++) {
         if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, i))) {
             continue;
         }
-        
+
         /* setup the nodelist here in case we jump to error */
         OBJ_CONSTRUCT(&node_list, opal_list_t);
 
@@ -195,7 +149,7 @@ static int mindist_map(orte_job_t *jdata)
          * all available slots. We'll double-check the single app_context rule first
          */
         if (0 == app->num_procs && 1 < jdata->num_apps) {
-            orte_show_help("help-orte-rmaps-mindist.txt", "multi-apps-and-zero-np",
+            orte_show_help("help-orte-rmaps-md.txt", "multi-apps-and-zero-np",
                            true, jdata->num_apps, NULL);
             rc = ORTE_ERR_SILENT;
             goto error;
@@ -216,17 +170,17 @@ static int mindist_map(orte_job_t *jdata)
 
         /* if a bookmark exists from some prior mapping, set us to start there */
         jdata->bookmark = orte_rmaps_base_get_starting_point(&node_list, jdata);
-       
+
         if (0 == app->num_procs) {
             /* set the num_procs to equal the number of slots on these mapped nodes */
             app->num_procs = num_slots;
         }
-        
+
         nprocs_mapped = 0;
         if (!num_nodes) {
             rc = ORTE_ERR_SILENT;
             goto error;
-        } 
+        }
         if (bynode) {
             /* calculate num_procs_to_assign for bynode case */
             navg = app->num_procs / num_nodes;
@@ -241,7 +195,7 @@ static int mindist_map(orte_job_t *jdata)
                 item != opal_list_get_end(&node_list);
                 item = opal_list_get_next(item)) {
             node = (orte_node_t*)item;
-       
+
             if (NULL == node->topology) {
                 orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-topology",
                         true, node->name);
@@ -258,7 +212,7 @@ static int mindist_map(orte_job_t *jdata)
                 rc = ORTE_ERR_SILENT;
                 goto error;
             }
-            
+
             /* get the number of available pus */
             if (opal_hwloc_use_hwthreads_as_cpus) {
                 total_npus = opal_hwloc_base_get_nbobjs_by_type(node->topology, HWLOC_OBJ_PU, 0, OPAL_HWLOC_AVAILABLE);
@@ -272,26 +226,17 @@ static int mindist_map(orte_job_t *jdata)
                         orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
                                 true, app->num_procs, app->app);
                         rc = ORTE_ERR_SILENT;
+                        ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
                         goto error;
                     } else {
-                        node->oversubscribed = true;
+                        ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
                     }
                 }
             }
-            /* first we need to fill summary object for root with information about nodes 
+            /* first we need to fill summary object for root with information about nodes
              * so we call opal_hwloc_base_get_nbobjs_by_type */
             opal_hwloc_base_get_nbobjs_by_type(node->topology, HWLOC_OBJ_NODE, 0, OPAL_HWLOC_AVAILABLE);
             OBJ_CONSTRUCT(&numa_list, opal_list_t);
-            /* check if hca device is specified */
-            if (NULL != orte_rmaps_base.device && (0 == strcasecmp("auto",orte_rmaps_base.device))) {
-                /* check if hca device is specified via openib or mxm parameter */
-                tmp = get_hca_name(app);
-                if (NULL != tmp) {
-                    free(orte_rmaps_base.device);
-                    orte_rmaps_base.device = tmp;
-                }
-            }
-            /* if it's not specified and auto mode is on, mapper will use the first available device from the list */
             ret = opal_hwloc_get_sorted_numa_list(node->topology, orte_rmaps_base.device, &numa_list);
             if (ret > 1) {
                 orte_show_help("help-orte-rmaps-md.txt", "orte-rmaps-mindist:several-devices",
@@ -309,7 +254,7 @@ static int mindist_map(orte_job_t *jdata)
             if (opal_list_get_size(&numa_list) > 0) {
                 j = 0;
                 required = 0;
-                OPAL_LIST_FOREACH(numa, &numa_list, orte_rmaps_numa_node_t) {
+                OPAL_LIST_FOREACH(numa, &numa_list, opal_rmaps_numa_node_t) {
                     /* get the hwloc object for this numa */
                     if (NULL == (obj = opal_hwloc_base_get_obj_by_type(node->topology, HWLOC_OBJ_NODE, 0, numa->index, OPAL_HWLOC_AVAILABLE))) {
                         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
@@ -328,7 +273,7 @@ static int mindist_map(orte_job_t *jdata)
                         }
                         nprocs_mapped++;
                         j++;
-                        proc->locale = obj;
+                        orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, ORTE_ATTR_LOCAL, obj, OPAL_PTR);
                     }
                     if ((nprocs_mapped == (int)app->num_procs) || (bynode && ((int)num_procs_to_assign == j))) {
                         break;
@@ -358,14 +303,14 @@ static int mindist_map(orte_job_t *jdata)
                     num_procs_to_assign--;
                 }
             }
-            
+
             /* add the node to the map, if needed */
-            if (!node->mapped) {
+            if (!ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_MAPPED)) {
                 if (ORTE_SUCCESS > (rc = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
                     ORTE_ERROR_LOG(rc);
                     goto error;
                 }
-                node->mapped = true;
+                ORTE_FLAG_SET(node, ORTE_NODE_FLAG_MAPPED);
                 OBJ_RETAIN(node);  /* maintain accounting on object */
                 jdata->map->num_nodes++;
             }
@@ -379,12 +324,13 @@ static int mindist_map(orte_job_t *jdata)
          */
 
         extra_procs = app->num_procs - nprocs_mapped;
-        if (extra_procs > 0) { 
+        if (extra_procs > 0) {
             /* check if oversubscribing is allowed */
             if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
                 orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
                         true, app->num_procs, app->app);
                 rc = ORTE_ERR_SILENT;
+                ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
                 goto error;
             }
             opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
@@ -402,7 +348,7 @@ static int mindist_map(orte_job_t *jdata)
 
                 if (nprocs_mapped == app->num_procs)
                     break;
-                node->oversubscribed = true;
+                ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
                 opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
                         "mca:rmaps:mindist: second pass assigning %d extra procs to node %s",
                         (int)num_procs_to_assign, node->name);
@@ -411,7 +357,7 @@ static int mindist_map(orte_job_t *jdata)
                 if (opal_list_get_size(&numa_list) > 0) {
                     numa_item = opal_list_get_first(&numa_list);
                     k = 0;
-                    obj = hwloc_get_obj_by_type(node->topology, HWLOC_OBJ_NODE,((orte_rmaps_numa_node_t*)numa_item)->index);
+                    obj = hwloc_get_obj_by_type(node->topology, HWLOC_OBJ_NODE,((opal_rmaps_numa_node_t*)numa_item)->index);
                     npus = opal_hwloc_base_get_npus(node->topology, obj);
                     for (j = 0; j < (int)num_procs_to_assign && nprocs_mapped < (int)app->num_procs; j++) {
                         if (NULL == (proc = orte_rmaps_base_setup_proc(jdata, node, i))) {
@@ -420,13 +366,13 @@ static int mindist_map(orte_job_t *jdata)
                         }
                         nprocs_mapped++;
                         k++;
-                        proc->locale = obj;
+                        orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, ORTE_ATTR_LOCAL, obj, OPAL_PTR);
                         if (k > npus/orte_rmaps_base.cpus_per_rank-1) {
                             numa_item = opal_list_get_next(numa_item);
-                            if (numa_item == opal_list_get_end(&numa_list)) { 
+                            if (numa_item == opal_list_get_end(&numa_list)) {
                                 numa_item = opal_list_get_first(&numa_list);
                             }
-                            obj = hwloc_get_obj_by_type(node->topology, HWLOC_OBJ_NODE,((orte_rmaps_numa_node_t*)numa_item)->index);
+                            obj = hwloc_get_obj_by_type(node->topology, HWLOC_OBJ_NODE,((opal_rmaps_numa_node_t*)numa_item)->index);
                             npus = opal_hwloc_base_get_npus(node->topology, obj);
                             k = 0;
                         }
@@ -466,9 +412,7 @@ static int mindist_map(orte_job_t *jdata)
         }
         OBJ_DESTRUCT(&node_list);
     }
-    if (orte_rmaps_base.device != NULL) {
     free(orte_rmaps_base.device);
-    }
     return ORTE_SUCCESS;
 
 error:
