@@ -5,7 +5,7 @@
  * Copyright (c) 2004-2011 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -14,12 +14,14 @@
  *                         All rights reserved.
  * Copyright (c) 2008      Voltaire. All rights reserved
  * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
- * Copyright (c) 2014      Intel, Inc. All rights reserved.
- *  
+ * Copyright (c) 2014-2016 Intel, Inc. All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ *
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -31,9 +33,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif  /* HAVE_UNISTD_H */
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif  /* HAVE_STRING_H */
 
 #include "opal/util/argv.h"
 #include "opal/util/if.h"
@@ -53,7 +53,7 @@
 static int orte_rmaps_rank_file_parse(const char *);
 static char *orte_rmaps_rank_file_parse_string_or_int(void);
 static const char *orte_rmaps_rank_file_name_cur = NULL;
-char *orte_rmaps_rank_file_slot_list;
+char *orte_rmaps_rank_file_slot_list = NULL;
 
 /*
  * Local variable
@@ -81,12 +81,10 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
     mca_base_component_t *c = &mca_rmaps_rank_file_component.super.base_version;
     char *slots;
     bool initial_map=true;
-#if OPAL_HAVE_HWLOC
     opal_hwloc_resource_type_t rtype;
-#endif
-    
+
     /* only handle initial launch of rf job */
-    if (ORTE_JOB_CONTROL_RESTART & jdata->controls) {
+    if (ORTE_FLAG_TEST(jdata, ORTE_JOB_FLAG_RESTART)) {
         opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                             "mca:rmaps:rf: job %s being restarted - rank_file cannot map",
                             ORTE_JOBID_PRINT(jdata->jobid));
@@ -107,7 +105,7 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                         "mca:rmaps:rank_file: mapping job %s",
                         ORTE_JOBID_PRINT(jdata->jobid));
- 
+
     /* flag that I did the mapping */
     if (NULL != jdata->map->last_mapper) {
         free(jdata->map->last_mapper);
@@ -117,7 +115,6 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
     /* convenience def */
     map = jdata->map;
 
-#if OPAL_HAVE_HWLOC
     /* default to LOGICAL processors */
     if (mca_rmaps_rank_file_component.physical) {
         opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
@@ -128,8 +125,7 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
                             "mca:rmaps:rank_file: using LOGICAL processors");
         rtype = OPAL_HWLOC_LOGICAL;
     }
-#endif
-    
+
     /* setup the node list */
     OBJ_CONSTRUCT(&node_list, opal_list_t);
 
@@ -138,9 +134,9 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
         rc = ORTE_ERR_SILENT;
         goto error;
     }
-    
+
     /* SANITY CHECKS */
-    
+
     /* if the number of processes wasn't specified, then we know there can be only
      * one app_context allowed in the launch, and that we are to launch it across
      * all available slots.
@@ -151,14 +147,14 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
         rc = ORTE_ERR_SILENT;
         goto error;
     }
-    
+
     /* END SANITY CHECKS */
-    
+
     /* start at the beginning... */
     vpid_start = 0;
     jdata->num_procs = 0;
     OBJ_CONSTRUCT(&rankmap, opal_pointer_array_t);
-    
+
     /* parse the rankfile, storing its results in the rankmap */
     if ( NULL != orte_rankfile ) {
         if ( ORTE_SUCCESS != (rc = orte_rmaps_rank_file_parse(orte_rankfile))) {
@@ -166,13 +162,13 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
             goto error;
         }
     }
-    
+
     /* cycle through the app_contexts, mapping them sequentially */
     for(i=0; i < jdata->apps->size; i++) {
         if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, i))) {
             continue;
         }
-        
+
         /* for each app_context, we have to get the list of nodes that it can
          * use since that can now be modified with a hostfile and/or -host
          * option
@@ -220,7 +216,7 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
                         /* all would be oversubscribed, so take the least loaded one */
                         k = UINT32_MAX;
                         OPAL_LIST_FOREACH(nd, &node_list, orte_node_t) {
-                            if (nd->num_procs < k) {
+                            if (nd->num_procs < (orte_vpid_t)k) {
                                 k = nd->num_procs;
                                 node = nd;
                             }
@@ -278,10 +274,10 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
                 goto error;
             }
             /* ensure the node is in the map */
-            if (!node->mapped) {
+            if (!ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_MAPPED)) {
                 OBJ_RETAIN(node);
                 opal_pointer_array_add(map->nodes, node);
-                node->mapped = true;
+                ORTE_FLAG_SET(node, ORTE_NODE_FLAG_MAPPED);
                 ++(jdata->map->num_nodes);
             }
             if (NULL == (proc = orte_rmaps_base_setup_proc(jdata, node, i))) {
@@ -294,21 +290,22 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
                 if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
                     orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
                                    true, node->num_procs, app->app);
+                    ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
                     rc = ORTE_ERR_SILENT;
                     goto error;
                 }
                 /* flag the node as oversubscribed so that sched-yield gets
                  * properly set
                  */
-                node->oversubscribed = true;
+                ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
             }
             /* set the vpid */
             proc->name.vpid = rank;
-            
-#if OPAL_HAVE_HWLOC
+
             if (NULL != slots) {
                 /* setup the bitmap */
                 hwloc_cpuset_t bitmap;
+                char *cpu_bitmap;
                 if (NULL == node->topology) {
                     /* not allowed - for rank-file, we must have
                      * the topology info
@@ -321,6 +318,7 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
                 /* parse the slot_list to find the socket and core */
                 if (ORTE_SUCCESS != (rc = opal_hwloc_base_slot_list_parse(slots, node->topology, rtype, bitmap))) {
                     ORTE_ERROR_LOG(rc);
+                    hwloc_bitmap_free(bitmap);
                     goto error;
                 }
                 /* note that we cannot set the proc locale to any specific object
@@ -328,21 +326,12 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
                  * leave that field NULL
                  */
                 /* set the proc to the specified map */
-                hwloc_bitmap_list_asprintf(&proc->cpu_bitmap, bitmap);
+                hwloc_bitmap_list_asprintf(&cpu_bitmap, bitmap);
+                orte_set_attribute(&proc->attributes, ORTE_PROC_CPU_BITMAP, ORTE_ATTR_GLOBAL, cpu_bitmap, OPAL_STRING);
                 /* cleanup */
+                free(cpu_bitmap);
                 hwloc_bitmap_free(bitmap);
             }
-#else
-            /* if we don't have hwloc, then all the rank_file can contain
-             * is the node assignment - it cannot contain any directives
-             * for socket, cores, etc. as we cannot honor them
-             */
-            if (NULL != slots) {
-                orte_show_help("help-rmaps_rank_file.txt", "no-hwloc", true, rank, slots);
-                rc = ORTE_ERR_SILENT;
-                goto error;
-            }
-#endif
 
             /* insert the proc into the proper place */
             if (ORTE_SUCCESS != (rc = opal_pointer_array_set_item(jdata->procs,
@@ -364,7 +353,7 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
         OBJ_CONSTRUCT(&node_list, opal_list_t);
     }
     OBJ_DESTRUCT(&node_list);
-    
+
     /* cleanup the rankmap */
     for (i=0; i < rankmap.size; i++) {
         if (NULL != (rfmap = opal_pointer_array_get_item(&rankmap, i))) {
@@ -376,7 +365,7 @@ static int orte_rmaps_rf_map(orte_job_t *jdata)
 
  error:
     OPAL_LIST_DESTRUCT(&node_list);
-    
+
     return rc;
 }
 
@@ -400,27 +389,27 @@ static int orte_rmaps_rank_file_parse(const char *rankfile)
     orte_rmaps_rank_file_map_t *rfmap=NULL;
     opal_pointer_array_t *assigned_ranks_array;
     char tmp_rank_assignment[64];
-        
+
     /* keep track of rank assignments */
     assigned_ranks_array = OBJ_NEW(opal_pointer_array_t);
 
     /* get the hnp node's info */
     hnp_node = (orte_node_t*)(orte_node_pool->addr[0]);
-    
+
     orte_rmaps_rank_file_name_cur = rankfile;
     orte_rmaps_rank_file_done = false;
     orte_rmaps_rank_file_in = fopen(rankfile, "r");
-    
+
     if (NULL == orte_rmaps_rank_file_in) {
         orte_show_help("help-rmaps_rank_file.txt", "no-rankfile", true, rankfile);
         rc = OPAL_ERR_NOT_FOUND;
         ORTE_ERROR_LOG(rc);
         goto unlock;
     }
-    
+
     while (!orte_rmaps_rank_file_done) {
         token = orte_rmaps_rank_file_lex();
-        
+
         switch (token) {
             case ORTE_RANKFILE_ERROR:
                 orte_show_help("help-rmaps_rank_file.txt", "bad-syntax", true, rankfile);
@@ -484,6 +473,9 @@ static int orte_rmaps_rank_file_parse(const char *rankfile)
                         }
                         argv = opal_argv_split (value, '@');
                         cnt = opal_argv_count (argv);
+                        if (NULL != node_name) {
+                            free(node_name);
+                        }
                         if (1 == cnt) {
                             node_name = strdup(argv[0]);
                         } else if (2 == cnt) {
@@ -493,6 +485,7 @@ static int orte_rmaps_rank_file_parse(const char *rankfile)
                             rc = ORTE_ERR_BAD_PARAM;
                             ORTE_ERROR_LOG(rc);
                             opal_argv_free(argv);
+                            node_name = NULL;
                             goto unlock;
                         }
                         opal_argv_free (argv);
@@ -501,13 +494,10 @@ static int orte_rmaps_rank_file_parse(const char *rankfile)
                             orte_show_help("help-rmaps_rank_file.txt", "bad-syntax", true, rankfile);
                             rc = ORTE_ERR_BAD_PARAM;
                             ORTE_ERROR_LOG(rc);
-                            free(node_name);
-                            node_name = NULL;
                             goto unlock;
                         }
                         /* check if this is the local node */
-                        if (0 == strcmp(node_name, hnp_node->name) ||
-                            opal_ifislocal(node_name)) {
+                        if (orte_ifislocal(node_name)) {
                             rfmap->node_name = strdup(hnp_node->name);
                         } else {
                             rfmap->node_name = strdup(node_name);
@@ -525,7 +515,7 @@ static int orte_rmaps_rank_file_parse(const char *rankfile)
 
                 /* check for a duplicate rank assignment */
                 if (NULL != opal_pointer_array_get_item(assigned_ranks_array, rank)) {
-                    orte_show_help("help-rmaps_rank_file.txt", "bad-assign", true, rank, 
+                    orte_show_help("help-rmaps_rank_file.txt", "bad-assign", true, rank,
                                    opal_pointer_array_get_item(assigned_ranks_array, rank), rankfile);
                     rc = ORTE_ERR_BAD_PARAM;
                     free(value);
@@ -568,11 +558,11 @@ static char *orte_rmaps_rank_file_parse_string_or_int(void)
 {
     int rc;
     char tmp_str[64];
-    
+
     if (ORTE_RANKFILE_EQUAL != orte_rmaps_rank_file_lex()){
         return NULL;
     }
-    
+
     rc = orte_rmaps_rank_file_lex();
     switch (rc) {
         case ORTE_RANKFILE_STRING:
@@ -582,7 +572,7 @@ static char *orte_rmaps_rank_file_parse_string_or_int(void)
             return strdup(tmp_str);
         default:
             return NULL;
-            
+
     }
-    
+
 }
