@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
+ * Copyright (c) 2014      Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,7 +22,6 @@
 #include "orte/mca/rmaps/rmaps_types.h"
 #include "orte/mca/plm/plm.h"
 #include "orte/mca/routed/routed.h"
-#include "orte/mca/sensor/sensor.h"
 #include "orte/util/session_dir.h"
 
 #include "orte/mca/state/base/base.h"
@@ -394,7 +394,7 @@ static void cleanup_node(orte_proc_t *proc)
                          "%s state:base:cleanup_node on proc %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          ORTE_NAME_PRINT(&proc->name)));
-    
+
     if (NULL == (node = proc->node)) {
         return;
     }
@@ -462,6 +462,7 @@ void orte_state_base_track_procs(int fd, short argc, void *cbdata)
     orte_proc_state_t state = caddy->proc_state;
     orte_job_t *jdata;
     orte_proc_t *pdata;
+    int i;
 
     opal_output_verbose(5, orte_state_base_framework.framework_output,
                         "%s state:base:track_procs called for proc %s state %s",
@@ -481,7 +482,7 @@ void orte_state_base_track_procs(int fd, short argc, void *cbdata)
         pdata->state = state;
         jdata->num_launched++;
         if (jdata->num_launched == jdata->num_procs) {
-            if (jdata->controls & ORTE_JOB_CONTROL_DEBUGGER_DAEMON) {
+            if (ORTE_FLAG_TEST(jdata, ORTE_JOB_FLAG_DEBUGGER_DAEMON)) {
                 ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_READY_FOR_DEBUGGERS);
             } else {
                 ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_RUNNING);
@@ -505,28 +506,48 @@ void orte_state_base_track_procs(int fd, short argc, void *cbdata)
         if (NULL != orte_iof.close) {
             orte_iof.close(proc, ORTE_IOF_STDIN);
         }
-        pdata->iof_complete = true;
-        if (pdata->waitpid_recvd) {
+        ORTE_FLAG_SET(pdata, ORTE_PROC_FLAG_IOF_COMPLETE);
+        if (ORTE_FLAG_TEST(pdata, ORTE_PROC_FLAG_WAITPID)) {
             ORTE_ACTIVATE_PROC_STATE(proc, ORTE_PROC_STATE_TERMINATED);
         }
     } else if (ORTE_PROC_STATE_WAITPID_FIRED == state) {
         /* update the proc state */
         pdata->state = state;
-        pdata->waitpid_recvd = true;
-        if (pdata->iof_complete) {
+        ORTE_FLAG_SET(pdata, ORTE_PROC_FLAG_WAITPID);
+        if (ORTE_FLAG_TEST(pdata, ORTE_PROC_FLAG_IOF_COMPLETE)) {
             ORTE_ACTIVATE_PROC_STATE(proc, ORTE_PROC_STATE_TERMINATED);
         }
     } else if (ORTE_PROC_STATE_TERMINATED == state) {
         /* update the proc state */
-        pdata->alive = false;
+        ORTE_FLAG_UNSET(pdata, ORTE_PROC_FLAG_ALIVE);
         pdata->state = state;
-	if (pdata->local_proc) {
+	if (ORTE_FLAG_TEST(pdata, ORTE_PROC_FLAG_LOCAL)) {
             /* Clean up the session directory as if we were the process
              * itself.  This covers the case where the process died abnormally
              * and didn't cleanup its own session directory.
              */
             orte_session_dir_finalize(proc);
 	}
+        /* if we are trying to terminate and our routes are
+         * gone, then terminate ourselves IF no local procs
+         * remain (might be some from another job)
+         */
+        if (orte_orteds_term_ordered &&
+            0 == orte_routed.num_routes()) {
+            for (i=0; i < orte_local_children->size; i++) {
+                if (NULL != (pdata = (orte_proc_t*)opal_pointer_array_get_item(orte_local_children, i)) &&
+                    ORTE_FLAG_TEST(pdata, ORTE_PROC_FLAG_ALIVE)) {
+                    /* at least one is still alive */
+                    goto cleanup;
+                }
+            }
+            /* call our appropriate exit procedure */
+            OPAL_OUTPUT_VERBOSE((5, orte_state_base_framework.framework_output,
+                                 "%s state:base all routes and children gone - exiting",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+            ORTE_ACTIVATE_JOB_STATE(NULL, ORTE_JOB_STATE_DAEMONS_TERMINATED);
+            goto cleanup;
+        }
         /* return the allocated slot for reuse */
         cleanup_node(pdata);
 	/* track job status */
@@ -554,6 +575,7 @@ void orte_state_base_check_all_complete(int fd, short args, void *cbdata)
     orte_std_cntr_t index;
     bool one_still_alive;
     orte_vpid_t lowest=0;
+    int32_t i32, *i32ptr;
 
     opal_output_verbose(2, orte_state_base_framework.framework_output,
                         "%s state:base:check_job_complete on job %s",
@@ -575,15 +597,13 @@ void orte_state_base_check_all_complete(int fd, short args, void *cbdata)
         }
     }
 
-    /* turn off any sensor monitors on this job */
-    orte_sensor.stop(jdata->jobid);
-
     /* tell the IOF that the job is complete */
     if (NULL != orte_iof.complete) {
         orte_iof.complete(jdata);
     }
 
-    if (0 < jdata->num_non_zero_exit && !orte_abort_non_zero_exit) {
+    i32ptr = &i32;
+    if (orte_get_attribute(&jdata->attributes, ORTE_JOB_NUM_NONZERO_EXIT, (void**)&i32ptr, OPAL_INT32) && !orte_abort_non_zero_exit) {
         if (!orte_report_child_jobs_separately || 1 == ORTE_LOCAL_JOBID(jdata->jobid)) {
             /* update the exit code */
             ORTE_UPDATE_EXIT_STATUS(lowest);
@@ -596,8 +616,7 @@ void orte_state_base_check_all_complete(int fd, short args, void *cbdata)
                     "-------------------------------------------------------",
                     (1 == ORTE_LOCAL_JOBID(jdata->jobid)) ? "the primary" : "child",
                     (1 == ORTE_LOCAL_JOBID(jdata->jobid)) ? "" : ORTE_LOCAL_JOBID_PRINT(jdata->jobid),
-                    jdata->num_non_zero_exit,
-                    (1 == jdata->num_non_zero_exit) ? "process returned\na non-zero exit code." :
+                    i32, (1 == i32) ? "process returned\na non-zero exit code." :
                     "processes returned\nnon-zero exit codes.");
     }
 
@@ -606,16 +625,16 @@ void orte_state_base_check_all_complete(int fd, short args, void *cbdata)
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          ORTE_JOBID_PRINT(jdata->jobid),
                          orte_job_state_to_str(jdata->state)));
-    
+
     /* if this job is a continuously operating one, then don't do
      * anything further - just return here
      */
     if (NULL != jdata &&
-        (ORTE_JOB_CONTROL_CONTINUOUS_OP & jdata->controls ||
-         ORTE_JOB_CONTROL_RECOVERABLE & jdata->controls)) {
+        (orte_get_attribute(&jdata->attributes, ORTE_JOB_CONTINUOUS_OP, NULL, OPAL_BOOL) ||
+         ORTE_FLAG_TEST(jdata, ORTE_JOB_FLAG_RECOVERABLE))) {
         goto CHECK_ALIVE;
     }
-    
+
     /* if the job that is being checked is the HNP, then we are
      * trying to terminate the orteds. In that situation, we
      * do -not- check all jobs - we simply notify the HNP
@@ -643,7 +662,7 @@ void orte_state_base_check_all_complete(int fd, short args, void *cbdata)
         OBJ_RELEASE(caddy);
         return;
     }
-    
+
     /* Release the resources used by this job. Since some errmgrs may want
      * to continue using resources allocated to the job as part of their
      * fault recovery procedure, we only do this once the job is "complete".
@@ -686,12 +705,12 @@ void orte_state_base_check_all_complete(int fd, short args, void *cbdata)
             /* maintain accounting */
             OBJ_RELEASE(node);
             /* flag that the node is no longer in a map */
-            node->mapped = false;
+            ORTE_FLAG_UNSET(node, ORTE_NODE_FLAG_MAPPED);
         }
         OBJ_RELEASE(map);
         jdata->map = NULL;
     }
-    
+
  CHECK_ALIVE:
     /* now check to see if all jobs are done - trigger notification of this jdata
      * object when we find it
@@ -733,7 +752,7 @@ void orte_state_base_check_all_complete(int fd, short args, void *cbdata)
                  * is maintained!
                  */
                 if (1 < j) {
-		    if (jdata->controls & ORTE_JOB_CONTROL_DEBUGGER_DAEMON) {
+		    if (ORTE_FLAG_TEST(jdata, ORTE_JOB_FLAG_DEBUGGER_DAEMON)) {
 			/* this was a debugger daemon. notify that a debugger has detached */
 			ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_DEBUGGER_DETACH);
 		    }
@@ -744,7 +763,7 @@ void orte_state_base_check_all_complete(int fd, short args, void *cbdata)
             continue;
         }
         /* if the job is flagged to not be monitored, skip it */
-        if (ORTE_JOB_CONTROL_DO_NOT_MONITOR & job->controls) {
+        if (ORTE_FLAG_TEST(job, ORTE_JOB_FLAG_DO_NOT_MONITOR)) {
             continue;
         }
         /* when checking for job termination, we must be sure to NOT check
