@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2010-2011 Cisco Systems, Inc.  All rights reserved. 
+ * Copyright (c) 2010-2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * Copyright (c) 2014      Intel, Inc.  All rights reserved.
@@ -29,10 +29,8 @@
 #ifdef HAVE_SYS_SIGNAL_H
 #include <sys/signal.h>
 #else
-#ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif
-#endif
 
 #include "orte/mca/plm/plm_types.h"
 #include "orte/util/error_strings.h"
@@ -63,12 +61,6 @@ int orte_err2str(int errnum, const char **errmsg)
     case ORTE_ERR_CONNECTION_REFUSED:
         retval = "Connection refused";
         break;
-    case ORTE_ERR_CONNECTION_FAILED:
-        retval = "Connection failed";
-        break;
-    case ORTE_ERR_COMM_FAILURE:
-        retval = "Communication failure";
-        break;
     case ORTE_ERR_TYPE_MISMATCH:
         retval = "Type mismatch";
         break;
@@ -200,6 +192,9 @@ int orte_err2str(int errnum, const char **errmsg)
     case ORTE_ERR_NO_PATH_TO_TARGET:
         retval = "No OOB path to target";
         break;
+    case ORTE_ERR_OP_IN_PROGRESS:
+        retval = "Operation in progress";
+        break;
     default:
         if (orte_report_silent_errors) {
             retval = "Unknown error";
@@ -303,6 +298,12 @@ const char *orte_job_state_to_str(orte_job_state_t state)
         return "MAP FAILED";
     case ORTE_JOB_STATE_CANNOT_LAUNCH:
         return "CANNOT LAUNCH";
+    case ORTE_JOB_STATE_FT_CHECKPOINT:
+        return "FAULT TOLERANCE CHECKPOINT";
+    case ORTE_JOB_STATE_FT_CONTINUE:
+        return "FAULT TOLERANCE CONTINUE";
+    case ORTE_JOB_STATE_FT_RESTART:
+        return "FAULT TOLERANCE RESTART";
     case ORTE_JOB_STATE_ANY:
         return "ANY";
     default:
@@ -393,6 +394,8 @@ const char *orte_proc_state_to_str(orte_proc_state_t state)
 const char *orte_node_state_to_str(orte_node_state_t state)
 {
     switch(state) {
+    case ORTE_NODE_STATE_UNDEF:
+        return "UNDEF";
     case ORTE_NODE_STATE_UNKNOWN:
         return "UNKNOWN";
     case ORTE_NODE_STATE_DOWN:
