@@ -9,8 +9,12 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2012 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
+ * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013-2014 Intel, Inc. All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -39,7 +43,7 @@
 #include "orte/util/proc_info.h"
 #include "orte/util/name_fns.h"
 #include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/ras/ras_types.h"
+#include "orte/mca/ras/base/base.h"
 #include "orte/runtime/orte_globals.h"
 
 #include "orte/util/hostfile/hostfile_lex.h"
@@ -54,9 +58,9 @@ static void hostfile_parse_error(int token)
     case ORTE_HOSTFILE_STRING:
         orte_show_help("help-hostfile.txt", "parse_error_string",
                        true,
-                       cur_hostfile_name, 
-                       orte_util_hostfile_line, 
-                       token, 
+                       cur_hostfile_name,
+                       orte_util_hostfile_line,
+                       token,
                        orte_util_hostfile_value.sval);
         break;
     case ORTE_HOSTFILE_IPV4:
@@ -64,16 +68,16 @@ static void hostfile_parse_error(int token)
     case ORTE_HOSTFILE_INT:
         orte_show_help("help-hostfile.txt", "parse_error_int",
                        true,
-                       cur_hostfile_name, 
-                       orte_util_hostfile_line, 
-                       token, 
+                       cur_hostfile_name,
+                       orte_util_hostfile_line,
+                       token,
                        orte_util_hostfile_value.ival);
         break;
      default:
         orte_show_help("help-hostfile.txt", "parse_error",
                        true,
-                       cur_hostfile_name, 
-                       orte_util_hostfile_line, 
+                       cur_hostfile_name,
+                       orte_util_hostfile_line,
                        token );
         break;
     }
@@ -114,24 +118,22 @@ static orte_node_t* hostfile_lookup(opal_list_t* nodes, const char* name)
         item != opal_list_get_end(nodes);
         item =  opal_list_get_next(item)) {
         orte_node_t* node = (orte_node_t*)item;
-        if(strcmp(node->name, name) == 0) {
-            opal_list_remove_item(nodes, item);
+        if (strcmp(node->name, name) == 0) {
             return node;
         }
     }
     return NULL;
 }
 
-static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exclude, bool keep_all)
+static int hostfile_parse_line(int token, opal_list_t* updates,
+                               opal_list_t* exclude, bool keep_all)
 {
     int rc;
     orte_node_t* node;
-    bool got_count = false;
     bool got_max = false;
     char* value;
-    char** argv;
+    char **argv;
     char* node_name = NULL;
-    char* node_alias = NULL;
     char* username = NULL;
     int cnt;
     int number_of_slots = 0;
@@ -150,12 +152,12 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
             value = orte_util_hostfile_value.sval;
         }
         argv = opal_argv_split (value, '@');
-        
+
         cnt = opal_argv_count (argv);
         if (1 == cnt) {
             node_name = strdup(argv[0]);
         } else if (2 == cnt) {
-            username = strdup(argv[0]);
+            username = argv[0];
             node_name = strdup(argv[1]);
         } else {
             opal_output(0, "WARNING: Unhandled user@host-combination\n"); /* XXX */
@@ -173,80 +175,67 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
                 node_name[i-1] = node_name[i];
             }
             node_name[len-1] = '\0';  /* truncate */
-            
-            OPAL_OUTPUT_VERBOSE((3, orte_debug_output,
+
+            OPAL_OUTPUT_VERBOSE((3, orte_ras_base_framework.framework_output,
                                  "%s hostfile: node %s is being excluded",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), node_name));
-            
-            /* convert this into something globally unique */
-            if (strcmp(node_name, "localhost") == 0 || opal_ifislocal(node_name)) {
+
+            /* see if this is another name for us */
+            if (orte_ifislocal(node_name)) {
                 /* Nodename has been allocated, that is for sure */
-                if (orte_show_resolved_nodenames &&
-                    0 != strcmp(node_name, orte_process_info.nodename)) {
-                    node_alias = strdup(node_name);
-                }
                 free (node_name);
                 node_name = strdup(orte_process_info.nodename);
             }
-            
+
             /* Do we need to make a new node object?  First check to see
-             if it's already in the exclude list */
+               if it's already in the exclude list */
             if (NULL == (node = hostfile_lookup(exclude, node_name))) {
                 node = OBJ_NEW(orte_node_t);
                 node->name = node_name;
                 if (NULL != username) {
-                    node->username = strdup(username);
+                    orte_set_attribute(&node->attributes, ORTE_NODE_USERNAME, ORTE_ATTR_LOCAL, username, OPAL_STRING);
                 }
+                opal_list_append(exclude, &node->super);
+            } else {
+                free(node_name);
             }
-            /* Note that we need to add this back to the exclude list.
-             If it was found, we just removed it (in hostfile_lookup()),
-             so this puts it back. If it was not found, then we have to
-             add it to the exclude list anyway. */
-            opal_list_append(exclude, &node->super);
             return ORTE_SUCCESS;
         }
-        
+
         /* this is not a node to be excluded, so we need to process it and
          * add it to the "include" list. See if this host is actually us.
          */
-        if (strcmp(node_name, "localhost") == 0 || opal_ifislocal(node_name)) {
+        if (orte_ifislocal(node_name)) {
             /* Nodename has been allocated, that is for sure */
-            if (orte_show_resolved_nodenames &&
-                0 != strcmp(node_name, orte_process_info.nodename)) {
-                node_alias = strdup(node_name);
-            }
             free (node_name);
             node_name = strdup(orte_process_info.nodename);
         }
 
-        OPAL_OUTPUT_VERBOSE((3, orte_debug_output,
+        OPAL_OUTPUT_VERBOSE((3, orte_ras_base_framework.framework_output,
                              "%s hostfile: node %s is being included - keep all is %s",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), node_name,
                              keep_all ? "TRUE" : "FALSE"));
 
-        /* Do we need to make a new node object?  First check to see
-         * if we are keeping everything or if it's already in the updates
-         * list. Because we check keep_all first, if that is set we will
-         * not do the hostfile_lookup call, and thus won't remove the
-         * pre-existing node from the updates list
-         */
+        /* Do we need to make a new node object? */
         if (keep_all || NULL == (node = hostfile_lookup(updates, node_name))) {
             node = OBJ_NEW(orte_node_t);
             node->name = node_name;
+            node->slots = 1;
             if (NULL != username) {
-                node->username = strdup(username);
+                orte_set_attribute(&node->attributes, ORTE_NODE_USERNAME, ORTE_ATTR_LOCAL, username, OPAL_STRING);
             }
-        }
-        /* do we need to record an alias for this node? */
-        if (NULL != node_alias) {
-            /* add to list of aliases for this node - only add if unique */
-            opal_argv_append_unique_nosize(&node->alias, node_alias, false);
-            free(node_alias);
+            opal_list_append(updates, &node->super);
+        } else {
+            /* this node was already found once - add a slot and mark slots as "given" */
+            node->slots++;
+            ORTE_FLAG_SET(node, ORTE_NODE_FLAG_SLOTS_GIVEN);
+            free(node_name);
         }
     } else if (ORTE_HOSTFILE_RELATIVE == token) {
         /* store this for later processing */
         node = OBJ_NEW(orte_node_t);
         node->name = strdup(orte_util_hostfile_value.sval);
+        opal_list_append(updates, &node->super);
     } else if (ORTE_HOSTFILE_RANK == token) {
         /* we can ignore the rank, but we need to extract the node name. we
          * first need to shift over to the other side of the equal sign as
@@ -268,57 +257,56 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
         } else {
             value = orte_util_hostfile_value.sval;
         }
-        
+
         argv = opal_argv_split (value, '@');
-        
+
         cnt = opal_argv_count (argv);
         if (1 == cnt) {
             node_name = strdup(argv[0]);
         } else if (2 == cnt) {
-            username = strdup(argv[0]);
+            username = argv[0];
             node_name = strdup(argv[1]);
         } else {
             opal_output(0, "WARNING: Unhandled user@host-combination\n"); /* XXX */
         }
         opal_argv_free (argv);
-        /* Do we need to make a new node object?  First check to see
-         * if we are keeping everything or if it's already in the updates
-         * list. Because we check keep_all first, if that is set we will
-         * not do the hostfile_lookup call, and thus won't remove the
-         * pre-existing node from the updates list
-         */
-        if (keep_all || NULL == (node = hostfile_lookup(updates, node_name))) {
+        /* Do we need to make a new node object? */
+        if (NULL == (node = hostfile_lookup(updates, node_name))) {
             node = OBJ_NEW(orte_node_t);
             node->name = node_name;
+            node->slots = 1;
             if (NULL != username) {
-                node->username = strdup(username);
+                orte_set_attribute(&node->attributes, ORTE_NODE_USERNAME, ORTE_ATTR_LOCAL, username, OPAL_STRING);
             }
+            opal_list_append(updates, &node->super);
+        } else {
+            /* add a slot */
+            node->slots++;
+            free(node_name);
         }
-        /* add a slot */
-        node->slots++;
-        /* do we need to record an alias for this node? */
-        if (NULL != node_alias) {
-            /* add to list of aliases for this node - only add if unique */
-            opal_argv_append_unique_nosize(&node->alias, node_alias, false);
-            free(node_alias);
-        }
+        OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
+                             "%s hostfile: node %s slots %d",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), node->name, node->slots));
+        /* mark the slots as "given" since we take them as being the
+         * number specified via the rankfile
+         */
+        ORTE_FLAG_SET(node, ORTE_NODE_FLAG_SLOTS_GIVEN);
         /* skip to end of line */
         while (!orte_util_hostfile_done &&
                ORTE_HOSTFILE_NEWLINE != token) {
             token = orte_util_hostfile_lex();
         }
-        opal_list_append(updates, &node->super);
         return ORTE_SUCCESS;
     } else {
         hostfile_parse_error(token);
         return ORTE_ERROR;
     }
-    
-    got_count = false;
+    free(username);
+
     while (!orte_util_hostfile_done) {
         token = orte_util_hostfile_lex();
-        
-        switch (token) {            
+
+        switch (token) {
         case ORTE_HOSTFILE_DONE:
             goto done;
 
@@ -326,43 +314,11 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
             goto done;
 
         case ORTE_HOSTFILE_USERNAME:
-            node->username = hostfile_parse_string();
-            break;
-
-        case ORTE_HOSTFILE_BOARDS:
-            rc = hostfile_parse_int();
-            if (rc < 0) {
-                orte_show_help("help-hostfile.txt", "boards",
-                               true,
-                               cur_hostfile_name, rc);
-                OBJ_RELEASE(node);
-                return ORTE_ERROR;
+            username = hostfile_parse_string();
+            if (NULL != username) {
+                orte_set_attribute(&node->attributes, ORTE_NODE_USERNAME, ORTE_ATTR_LOCAL, username, OPAL_STRING);
+                free(username);
             }
-            node->boards = rc;
-            break;
-
-        case ORTE_HOSTFILE_SOCKETS_PER_BOARD:
-            rc = hostfile_parse_int();
-            if (rc < 0) {
-                orte_show_help("help-hostfile.txt", "sockets",
-                               true,
-                               cur_hostfile_name, rc);
-                OBJ_RELEASE(node);
-                return ORTE_ERROR;
-            }
-            node->sockets_per_board = rc;
-            break;
-
-        case ORTE_HOSTFILE_CORES_PER_SOCKET:
-            rc = hostfile_parse_int();
-            if (rc < 0) {
-                orte_show_help("help-hostfile.txt", "cores",
-                               true,
-                               cur_hostfile_name, rc);
-                OBJ_RELEASE(node);
-                return ORTE_ERROR;
-            }
-            node->cores_per_socket = rc;
             break;
 
         case ORTE_HOSTFILE_COUNT:
@@ -373,11 +329,23 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
                 orte_show_help("help-hostfile.txt", "slots",
                                true,
                                cur_hostfile_name, rc);
+                opal_list_remove_item(updates, &node->super);
                 OBJ_RELEASE(node);
                 return ORTE_ERROR;
             }
-            node->slots += rc;
-            got_count = true;
+            if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
+                /* multiple definitions were given for the
+                 * slot count - this is not allowed
+                 */
+                orte_show_help("help-hostfile.txt", "slots-given",
+                               true,
+                               cur_hostfile_name, node->name);
+                opal_list_remove_item(updates, &node->super);
+                OBJ_RELEASE(node);
+                return ORTE_ERROR;
+            }
+            node->slots = rc;
+            ORTE_FLAG_SET(node, ORTE_NODE_FLAG_SLOTS_GIVEN);
 
             /* Ensure that slots_max >= slots */
             if (node->slots_max != 0 && node->slots_max < node->slots) {
@@ -391,6 +359,7 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
                 orte_show_help("help-hostfile.txt", "max_slots",
                                true,
                                cur_hostfile_name, ((size_t) rc));
+                opal_list_remove_item(updates, &node->super);
                 OBJ_RELEASE(node);
                 return ORTE_ERROR;
             }
@@ -405,32 +374,36 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
                                true,
                                cur_hostfile_name, node->slots, rc);
                 ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
+                opal_list_remove_item(updates, &node->super);
                 OBJ_RELEASE(node);
                 return ORTE_ERROR;
             }
             break;
 
+        case ORTE_HOSTFILE_STRING:
+        case ORTE_HOSTFILE_INT:
+            /* just ignore it */
+            break;
+
         default:
             hostfile_parse_error(token);
+            opal_list_remove_item(updates, &node->super);
             OBJ_RELEASE(node);
             return ORTE_ERROR;
         }
         if (number_of_slots > node->slots) {
             ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
+            opal_list_remove_item(updates, &node->super);
             OBJ_RELEASE(node);
             return ORTE_ERROR;
         }
     }
 
-done:
-    if (!got_count) {
-        if (got_max) {
-            node->slots = node->slots_max;
-        } else {
-            ++node->slots;
-        }
+ done:
+    if (got_max && !ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
+        node->slots = node->slots_max;
+        ORTE_FLAG_SET(node, ORTE_NODE_FLAG_SLOTS_GIVEN);
     }
-    opal_list_append(updates, &node->super);
 
     return ORTE_SUCCESS;
 }
@@ -440,19 +413,37 @@ done:
  * Parse the specified file into a node list.
  */
 
-static int hostfile_parse(const char *hostfile, opal_list_t* updates, opal_list_t* exclude, bool keep_all)
+static int hostfile_parse(const char *hostfile, opal_list_t* updates,
+                          opal_list_t* exclude, bool keep_all)
 {
     int token;
     int rc = ORTE_SUCCESS;
 
 
     cur_hostfile_name = hostfile;
-    
+
     orte_util_hostfile_done = false;
     orte_util_hostfile_in = fopen(hostfile, "r");
     if (NULL == orte_util_hostfile_in) {
-        orte_show_help("help-hostfile.txt", "no-hostfile", true, hostfile);
-        rc = ORTE_ERR_NOT_FOUND;
+        if (NULL == orte_default_hostfile ||
+            0 != strcmp(orte_default_hostfile, hostfile)) {
+            /* not the default hostfile, so not finding it
+             * is an error
+             */
+            orte_show_help("help-hostfile.txt", "no-hostfile", true, hostfile);
+            rc = ORTE_ERR_SILENT;
+            goto unlock;
+        }
+        /* if this is the default hostfile and it was given,
+         * then it's an error
+         */
+        if (orte_default_hostfile_given) {
+            orte_show_help("help-hostfile.txt", "no-hostfile", true, hostfile);
+            rc = ORTE_ERR_NOT_FOUND;
+            goto unlock;
+        }
+        /* otherwise, not finding it is okay */
+        rc = ORTE_SUCCESS;
         goto unlock;
     }
 
@@ -506,31 +497,32 @@ unlock:
  */
 
 int orte_util_add_hostfile_nodes(opal_list_t *nodes,
-                                 bool *override_oversubscribed,
                                  char *hostfile)
 {
-    opal_list_t exclude;
+    opal_list_t exclude, adds;
     opal_list_item_t *item, *itm;
     int rc;
+    orte_node_t *nd, *node;
+    bool found;
 
-    
-    OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
+    OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                          "%s hostfile: checking hostfile %s for nodes",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), hostfile));
 
     OBJ_CONSTRUCT(&exclude, opal_list_t);
-    
-    /* parse the hostfile and add the contents to the list */
-    if (ORTE_SUCCESS != (rc = hostfile_parse(hostfile, nodes, &exclude, false))) {
+    OBJ_CONSTRUCT(&adds, opal_list_t);
+
+    /* parse the hostfile and add any new contents to the list */
+    if (ORTE_SUCCESS != (rc = hostfile_parse(hostfile, &adds, &exclude, false))) {
         goto cleanup;
     }
-    
-    /* parse the nodes to check for any relative node directives */
-    for (item = opal_list_get_first(nodes);
-         item != opal_list_get_end(nodes);
+
+    /* check for any relative node directives */
+    for (item = opal_list_get_first(&adds);
+         item != opal_list_get_end(&adds);
          item = opal_list_get_next(item)) {
-        orte_node_t *node=(orte_node_t*)item;
-        
+        node=(orte_node_t*)item;
+
         if ('+' == node->name[0]) {
             orte_show_help("help-hostfile.txt", "hostfile:relative-syntax",
                            true, node->name);
@@ -538,36 +530,51 @@ int orte_util_add_hostfile_nodes(opal_list_t *nodes,
             goto cleanup;
         }
     }
-    
+
     /* remove from the list of nodes those that are in the exclude list */
-    while(NULL != (item = opal_list_remove_first(&exclude))) {
-        orte_node_t *exnode = (orte_node_t*)item;
+    while (NULL != (item = opal_list_remove_first(&exclude))) {
+        nd = (orte_node_t*)item;
         /* check for matches on nodes */
-        for (itm = opal_list_get_first(nodes);
-             itm != opal_list_get_end(nodes);
+        for (itm = opal_list_get_first(&adds);
+             itm != opal_list_get_end(&adds);
              itm = opal_list_get_next(itm)) {
-            orte_node_t *node=(orte_node_t*)itm;
-            if (0 == strcmp(exnode->name, node->name)) {
+            node = (orte_node_t*)itm;
+            if (0 == strcmp(nd->name, node->name)) {
                 /* match - remove it */
-                opal_list_remove_item(nodes, itm);
+                opal_list_remove_item(&adds, itm);
                 OBJ_RELEASE(itm);
                 break;
             }
         }
         OBJ_RELEASE(item);
     }
-    
-    /* indicate that ORTE should override any oversubscribed conditions
-     * based on local hardware limits since the user (a) might not have
-     * provided us any info on the #slots for a node, and (b) the user
-     * might have been wrong! If we don't check the number of local physical
-     * processors, then we could be too aggressive on our sched_yield setting
-     * and cause performance problems.
-     */
-    *override_oversubscribed = true;
+
+    /* transfer across all unique nodes */
+    while (NULL != (item = opal_list_remove_first(&adds))) {
+        nd = (orte_node_t*)item;
+        found = false;
+        for (itm = opal_list_get_first(nodes);
+             itm != opal_list_get_end(nodes);
+             itm = opal_list_get_next(itm)) {
+            node = (orte_node_t*)itm;
+            if (0 == strcmp(nd->name, node->name)) {
+                found = true;
+                break;
+            }
+        }
+        if (!found) {
+            opal_list_append(nodes, &nd->super);
+            OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
+                                 "%s hostfile: adding node %s slots %d",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), nd->name, nd->slots));
+        } else {
+            OBJ_RELEASE(item);
+        }
+    }
 
 cleanup:
-    OBJ_DESTRUCT(&exclude);
+    OPAL_LIST_DESTRUCT(&exclude);
+    OPAL_LIST_DESTRUCT(&adds);
 
     return rc;
 }
@@ -577,7 +584,8 @@ cleanup:
  * are not found in the hostfile
  */
 int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
-                                    char *hostfile)
+                                    char *hostfile,
+                                    bool remove)
 {
     opal_list_t newnodes, exclude;
     opal_list_item_t *item1, *item2, *next, *item3;
@@ -589,7 +597,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
     opal_list_t keep;
     bool found;
 
-    OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
+    OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                         "%s hostfile: filtering nodes through hostfile %s",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), hostfile));
 
@@ -601,12 +609,13 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
         OBJ_DESTRUCT(&exclude);
         return rc;
     }
-    
+
     /* if the hostfile was empty, then treat it as a no-op filter */
     if (0 == opal_list_get_size(&newnodes)) {
         OBJ_DESTRUCT(&newnodes);
         OBJ_DESTRUCT(&exclude);
-        return ORTE_SUCCESS;
+        /* indicate that the hostfile was empty */
+        return ORTE_ERR_TAKE_NEXT_OPTION;
     }
 
     /* remove from the list of newnodes those that are in the exclude list
@@ -620,6 +629,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
             orte_node_t *node = (orte_node_t*)item2;
             if (0 == strcmp(node_from_file->name, node->name)) {
                 /* match - remove it */
+                opal_output(0, "HOST %s ON EXCLUDE LIST - REMOVING", node->name);
                 opal_list_remove_item(&newnodes, item2);
                 OBJ_RELEASE(item2);
                 break;
@@ -627,16 +637,16 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
         }
         OBJ_RELEASE(item1);
     }
-    
-    /* now check our nodes and keep those that match. We can
+
+    /* now check our nodes and keep or mark those that match. We can
      * destruct our hostfile list as we go since this won't be needed
      */
     OBJ_CONSTRUCT(&keep, opal_list_t);
     while (NULL != (item2 = opal_list_remove_first(&newnodes))) {
         node_from_file = (orte_node_t*)item2;
-        
+
         next = opal_list_get_next(item2);
-        
+
         /* see if this is a relative node syntax */
         if ('+' == node_from_file->name[0]) {
             /* see if we specified empty nodes */
@@ -674,10 +684,15 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                                 goto skipnode;
                             }
                         }
-                        /* remove item from list */
-                        opal_list_remove_item(nodes, item1);
-                        /* xfer to keep list */
-                        opal_list_append(&keep, item1);
+                        if (remove) {
+                            /* remove item from list */
+                            opal_list_remove_item(nodes, item1);
+                            /* xfer to keep list */
+                            opal_list_append(&keep, item1);
+                        } else {
+                            /* mark as included */
+                            ORTE_FLAG_SET(node_from_list, ORTE_NODE_FLAG_MAPPED);
+                        }
                         --num_empty;
                     }
                 skipnode:
@@ -689,7 +704,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                                    true, num_empty);
                     rc = ORTE_ERR_SILENT;
                     goto cleanup;
-                }            
+                }
             } else if ('n' == node_from_file->name[1] ||
                        'N' == node_from_file->name[1]) {
                 /* they want a specific relative node #, so
@@ -709,10 +724,15 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                      item1 = opal_list_get_next(nodes)) {
                     node_from_list = (orte_node_t*)item1;
                     if (0 == strcmp(node_from_list->name, node_from_pool->name)) {
-                        /* match - remove item from list */
-                        opal_list_remove_item(nodes, item1);
-                        /* xfer to keep list */
-                        opal_list_append(&keep, item1);
+                        if (remove) {
+                            /* match - remove item from list */
+                            opal_list_remove_item(nodes, item1);
+                            /* xfer to keep list */
+                            opal_list_append(&keep, item1);
+                        } else {
+                            /* mark as included */
+                            ORTE_FLAG_SET(node_from_list, ORTE_NODE_FLAG_MAPPED);
+                        }
                         break;
                     }
                 }
@@ -733,26 +753,28 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                  item1 != opal_list_get_end(nodes);
                  item1 = opal_list_get_next(item1)) {
                 node_from_list = (orte_node_t*)item1;
-                /* since the name in the hostfile might not match
-                 * our local name, and yet still be intended to match,
-                 * we have to check for local interfaces
-                 */
-                if (0 == strcmp(node_from_file->name, node_from_list->name) ||
-                    (opal_ifislocal(node_from_list->name) &&
-                     opal_ifislocal(node_from_file->name))) {
+                /* we have converted all aliases for ourself
+                 * to our own detected nodename, so no need
+                 * to check for interfaces again - a simple
+                 * strcmp will suffice */
+                if (0 == strcmp(node_from_file->name, node_from_list->name)) {
                     /* if the slot count here is less than the
                      * total slots avail on this node, set it
                      * to the specified count - this allows people
                      * to subdivide an allocation
                      */
                     if (node_from_file->slots < node_from_list->slots) {
-                        node_from_list->slots_alloc = node_from_file->slots;
+                        node_from_list->slots = node_from_file->slots;
+                    }
+                    if (remove) {
+                        /* remove the node from the list */
+                        opal_list_remove_item(nodes, item1);
+                        /* xfer it to keep list */
+                        opal_list_append(&keep, item1);
+                    } else {
+                        /* mark as included */
+                        ORTE_FLAG_SET(node_from_list, ORTE_NODE_FLAG_MAPPED);
                     }
-                    /* remove the node from the list */
-                    opal_list_remove_item(nodes, item1);
-                    /* xfer it to keep list */
-                    opal_list_append(&keep, item1);
-                    /* mark as found */
                     found = true;
                     break;
                 }
@@ -771,7 +793,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
         /* cleanup the newnode list */
         OBJ_RELEASE(item2);
     }
-    
+
     /* if we still have entries on our hostfile list, then
      * there were requested hosts that were not in our allocation.
      * This is an error - report it to the user and return an error
@@ -786,16 +808,22 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
         return ORTE_ERR_SILENT;
     }
 
+    if (!remove) {
+        /* all done */
+        OBJ_DESTRUCT(&newnodes);
+        return ORTE_SUCCESS;
+    }
+
     /* clear the rest of the nodes list */
     while (NULL != (item1 = opal_list_remove_first(nodes))) {
         OBJ_RELEASE(item1);
     }
-    
+
     /* the nodes list has been cleared - rebuild it in order */
     while (NULL != (item1 = opal_list_remove_first(&keep))) {
         opal_list_append(nodes, item1);
     }
-    
+
 cleanup:
     OBJ_DESTRUCT(&newnodes);
 
@@ -812,31 +840,31 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
     bool want_all_empty=false;
     orte_node_t *node_from_pool, *newnode;
     int rc;
-    
-    OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
+
+    OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                          "%s hostfile: creating ordered list of hosts from hostfile %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), hostfile));
-    
+
     OBJ_CONSTRUCT(&exclude, opal_list_t);
-    
+
     /* parse the hostfile and add the contents to the list, keeping duplicates */
     if (ORTE_SUCCESS != (rc = hostfile_parse(hostfile, nodes, &exclude, true))) {
         goto cleanup;
     }
-    
+
     /* parse the nodes to process any relative node directives */
     item2 = opal_list_get_first(nodes);
     while (item2 != opal_list_get_end(nodes)) {
         orte_node_t *node=(orte_node_t*)item2;
-        
+
         /* save the next location in case this one gets removed */
         item1 = opal_list_get_next(item2);
-        
+
         if ('+' != node->name[0]) {
             item2 = item1;
             continue;
         }
-        
+
         /* see if we specified empty nodes */
         if ('e' == node->name[1] ||
             'E' == node->name[1]) {
@@ -871,9 +899,9 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
                      * to subdivide an allocation
                      */
                     if (node->slots < node_from_pool->slots) {
-                        newnode->slots_alloc = node->slots;
+                        newnode->slots = node->slots;
                     } else {
-                        newnode->slots_alloc = node_from_pool->slots;
+                        newnode->slots = node_from_pool->slots;
                     }
                     opal_list_insert_pos(nodes, item1, &newnode->super);
                     /* track number added */
@@ -888,7 +916,7 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
                                true, num_empty);
                 rc = ORTE_ERR_SILENT;
                 goto cleanup;
-            }            
+            }
             /* since we have expanded the provided node, remove
              * it from list
              */
@@ -924,9 +952,9 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
              * to subdivide an allocation
              */
             if (node->slots < node_from_pool->slots) {
-                newnode->slots_alloc = node->slots;
+                newnode->slots = node->slots;
             } else {
-                newnode->slots_alloc = node_from_pool->slots;
+                newnode->slots = node_from_pool->slots;
             }
             /* insert it before item1 */
             opal_list_insert_pos(nodes, item1, &newnode->super);
@@ -942,7 +970,7 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
             rc = ORTE_ERR_SILENT;
             goto cleanup;
         }
-        
+
         /* move to next */
         item2 = item1;
     }
@@ -966,9 +994,9 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
         }
         OBJ_RELEASE(item);
     }
-    
+
 cleanup:
     OBJ_DESTRUCT(&exclude);
-    
+
     return rc;
 }
