@@ -1,8 +1,11 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2013-2015 Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2014      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -119,8 +122,8 @@ static int mca_spml_ikrit_put_request_free(struct oshmem_request_t** request)
     OPAL_THREAD_LOCK(&oshmem_request_lock);
     assert(false == put_req->req_put.req_base.req_free_called);
     put_req->req_put.req_base.req_free_called = true;
-    OMPI_FREE_LIST_RETURN_MT( &mca_spml_base_put_requests,
-                          (ompi_free_list_item_t*)put_req);
+    opal_free_list_return (&mca_spml_base_put_requests,
+                           (opal_free_list_item_t*)put_req);
     opal_memchecker_base_mem_noaccess(put_req, sizeof(*put_req));
     OPAL_THREAD_UNLOCK(&oshmem_request_lock);
 
@@ -168,8 +171,8 @@ static int mca_spml_ikrit_get_request_free(struct oshmem_request_t** request)
     OPAL_THREAD_LOCK(&oshmem_request_lock);
     assert(false == get_req->req_get.req_base.req_free_called);
     get_req->req_get.req_base.req_free_called = true;
-    OMPI_FREE_LIST_RETURN_MT( &mca_spml_base_get_requests,
-                          (ompi_free_list_item_t*)get_req);
+    opal_free_list_return (&mca_spml_base_get_requests,
+                           (opal_free_list_item_t*)get_req);
     opal_memchecker_base_mem_noaccess(get_req, sizeof(*get_req));
     OPAL_THREAD_UNLOCK(&oshmem_request_lock);
 
@@ -259,9 +262,9 @@ void mca_spml_ikrit_dump_stats()
 static inline mca_spml_ikrit_put_request_t *alloc_put_req(void)
 {
     mca_spml_ikrit_put_request_t *req;
-    ompi_free_list_item_t* item;
+    opal_free_list_item_t* item;
 
-    OMPI_FREE_LIST_WAIT_MT(&mca_spml_base_put_requests, item);
+    item = opal_free_list_wait (&mca_spml_base_put_requests);
 
     req = (mca_spml_ikrit_put_request_t *) item;
     opal_memchecker_base_mem_undefined(req, sizeof(*req));
@@ -277,9 +280,9 @@ static inline mca_spml_ikrit_put_request_t *alloc_put_req(void)
 static inline mca_spml_ikrit_get_request_t *alloc_get_req(void)
 {
     mca_spml_ikrit_get_request_t *req;
-    ompi_free_list_item_t* item;
+    opal_free_list_item_t* item;
 
-    OMPI_FREE_LIST_WAIT_MT(&mca_spml_base_get_requests, item);
+    item = opal_free_list_wait (&mca_spml_base_get_requests);
 
     req = (mca_spml_ikrit_get_request_t *) item;
     opal_memchecker_base_mem_undefined(req, sizeof(*req));
@@ -299,27 +302,27 @@ int mca_spml_ikrit_enable(bool enable)
         return OSHMEM_SUCCESS;
     }
 
-    ompi_free_list_init_new(&mca_spml_base_put_requests,
-                            sizeof(mca_spml_ikrit_put_request_t),
-                            opal_cache_line_size,
-                            OBJ_CLASS(mca_spml_ikrit_put_request_t),
-                            0,
-                            opal_cache_line_size,
-                            mca_spml_ikrit.free_list_num,
-                            mca_spml_ikrit.free_list_max,
-                            mca_spml_ikrit.free_list_inc,
-                            NULL );
+    opal_free_list_init (&mca_spml_base_put_requests,
+                         sizeof(mca_spml_ikrit_put_request_t),
+                         opal_cache_line_size,
+                         OBJ_CLASS(mca_spml_ikrit_put_request_t),
+                         0,
+                         opal_cache_line_size,
+                         mca_spml_ikrit.free_list_num,
+                         mca_spml_ikrit.free_list_max,
+                         mca_spml_ikrit.free_list_inc,
+                         NULL, 0, NULL, NULL, NULL);
 
-    ompi_free_list_init_new(&mca_spml_base_get_requests,
-                            sizeof(mca_spml_ikrit_get_request_t),
-                            opal_cache_line_size,
-                            OBJ_CLASS(mca_spml_ikrit_get_request_t),
-                            0,
-                            opal_cache_line_size,
-                            mca_spml_ikrit.free_list_num,
-                            mca_spml_ikrit.free_list_max,
-                            mca_spml_ikrit.free_list_inc,
-                            NULL );
+    opal_free_list_init (&mca_spml_base_get_requests,
+                         sizeof(mca_spml_ikrit_get_request_t),
+                         opal_cache_line_size,
+                         OBJ_CLASS(mca_spml_ikrit_get_request_t),
+                         0,
+                         opal_cache_line_size,
+                         mca_spml_ikrit.free_list_num,
+                         mca_spml_ikrit.free_list_max,
+                         mca_spml_ikrit.free_list_inc,
+                         NULL, 0, NULL, NULL, NULL);
 
     mca_spml_ikrit.enabled = true;
 
@@ -558,10 +561,13 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
         mxm_conn_ctx_set(conn_reqs[i].conn, mca_spml_ikrit.mxm_peers[i]);
     }
 
-    free(conn_reqs);
+    if (conn_reqs)
+        free(conn_reqs);
 #endif
-    free(ep_info);
-    free(ep_hw_rdma_info);
+    if (ep_info)
+        free(ep_info);
+    if (ep_hw_rdma_info)
+        free(ep_hw_rdma_info);
 
 #if MXM_API >= MXM_VERSION(2,0)
     if (mca_spml_ikrit.bulk_connect) {
@@ -574,8 +580,8 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
     proc_self = oshmem_proc_group_find(oshmem_group_all, my_rank);
     /* identify local processes and change transport to SHM */
     for (i = 0; i < nprocs; i++) {
-        if (procs[i]->proc_name.jobid != proc_self->proc_name.jobid||
-        !OPAL_PROC_ON_LOCAL_NODE(procs[i]->proc_flags)) {
+        if (procs[i]->super.proc_name.jobid != proc_self->super.proc_name.jobid ||
+            !OPAL_PROC_ON_LOCAL_NODE(procs[i]->super.proc_flags)) {
             continue;
         }
         if (procs[i] == proc_self)
@@ -592,10 +598,13 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
 
 bail:
 #if MXM_API < MXM_VERSION(2,0)
-    free(conn_reqs);
+	if (conn_reqs)
+		free(conn_reqs);
 #endif
-    free(ep_info);
-    free(ep_hw_rdma_info);
+	if (ep_info)
+		free(ep_info);
+	if (ep_hw_rdma_info)
+		free(ep_hw_rdma_info);
     SPML_ERROR("add procs FAILED rc=%d", rc);
 
     return rc;
@@ -673,6 +682,10 @@ sshmem_mkey_t *mca_spml_ikrit_register(void* addr,
             }
 #endif
             break;
+
+        default:
+            SPML_ERROR("unsupported PTL: %d", i);
+            goto error_out;
         }
         SPML_VERBOSE(5,
                      "rank %d ptl %d addr %p size %llu %s",
@@ -687,7 +700,7 @@ sshmem_mkey_t *mca_spml_ikrit_register(void* addr,
 error_out:
     mca_spml_ikrit_deregister(mkeys);
 
-    return NULL ;
+    return NULL;
 }
 
 int mca_spml_ikrit_deregister(sshmem_mkey_t *mkeys)
@@ -766,7 +779,7 @@ int mca_spml_ikrit_oob_get_mkeys(int pe, uint32_t seg, sshmem_mkey_t *mkeys)
         mkeys[ptl].len = 0;
         mkeys[ptl].u.key = MAP_SEGMENT_SHM_INVALID;
         return OSHMEM_SUCCESS;
-    } 
+    }
 
     return OSHMEM_ERROR;
 #endif
@@ -1181,7 +1194,7 @@ static inline int mca_spml_ikrit_put_internal(void* dst_addr,
  * Problems:
  * - slighly worse performance than impl based on non buffered put
  * - fence complexity is O(n_active_connections) instead of O(n_connections_with_outstanding_puts).
- *   Later is bounded by the network RTT & mxm ack timer. 
+ *   Later is bounded by the network RTT & mxm ack timer.
  */
 int mca_spml_ikrit_put_simple(void* dst_addr,
                               size_t size,
@@ -1336,7 +1349,7 @@ int mca_spml_ikrit_fence(void)
                  "Into fence with %d active puts on %d pes",
                  mca_spml_ikrit.n_active_puts, (int)opal_list_get_size(&mca_spml_ikrit.active_peers));
 
-    /* puts(unless are send sync) are completed by remote side lazily. That is either when remote decides to 
+    /* puts(unless are send sync) are completed by remote side lazily. That is either when remote decides to
      * ack window which can take hundreds of ms. So speed things up by doing fence */
     while (NULL != (item = opal_list_remove_first(&mca_spml_ikrit.active_peers))) {
         peer = (mxm_peer_t *) item;
