@@ -1,8 +1,11 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2013-2015 Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2014      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -20,9 +23,9 @@
 
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/mca/pml/pml.h"
-#include "ompi/mca/btl/btl.h"
-#include "ompi/mca/btl/base/base.h"
-#include "ompi/mca/btl/sm/btl_sm_frag.h"
+#include "opal/mca/btl/btl.h"
+#include "opal/mca/btl/base/base.h"
+#include "opal/mca/btl/sm/btl_sm_frag.h"
 
 #include "oshmem/proc/proc.h"
 #include "oshmem/mca/memheap/memheap.h"
@@ -105,17 +108,32 @@ static char *btl_type2str(int btl_type)
     return "bad_btl_type";
 }
 
-static inline void calc_nfrags(mca_bml_base_btl_t* bml_btl,
-                               size_t size,
-                               unsigned int *frag_size,
-                               int *nfrags,
-                               int use_send)
+static inline void calc_nfrags_put (mca_bml_base_btl_t* bml_btl,
+                                    size_t size,
+                                    unsigned int *frag_size,
+                                    int *nfrags,
+                                    int use_send)
 {
     if (use_send) {
         *frag_size = bml_btl->btl->btl_max_send_size - SPML_YODA_SEND_CONTEXT_SIZE;
     }
     else {
-        *frag_size = bml_btl->btl->btl_max_send_size;
+        *frag_size = bml_btl->btl->btl_put_limit;
+    }
+    *nfrags = 1 + (size - 1) / (*frag_size);
+}
+
+static inline void calc_nfrags_get (mca_bml_base_btl_t* bml_btl,
+                                    size_t size,
+                                    unsigned int *frag_size,
+                                    int *nfrags,
+                                    int use_send)
+{
+    if (use_send) {
+        *frag_size = bml_btl->btl->btl_max_send_size - SPML_YODA_SEND_CONTEXT_SIZE;
+    }
+    else {
+        *frag_size = bml_btl->btl->btl_get_limit;
     }
     *nfrags = 1 + (size - 1) / (*frag_size);
 }
@@ -162,7 +180,7 @@ static inline void mca_spml_yoda_bml_alloc( mca_bml_base_btl_t* bml_btl,
                        size,
                        flags);
 
-        if (OPAL_UNLIKELY(!(*des) || !(*des)->des_src ) && !is_fence_complete) {
+        if (OPAL_UNLIKELY(!(*des) || !(*des)->des_segments ) && !is_fence_complete) {
             mca_spml_yoda_fence_internal(mca_spml_yoda.bml_alloc_threshold);
 
             is_fence_complete = true;
@@ -181,7 +199,7 @@ static inline void spml_yoda_prepare_for_put(void* buffer, size_t size, void* p_
         memcpy((void*) (((char*) buffer) + sizeof(size) + sizeof(void *)), p_src, size);
     }
     else {
-        memcpy((void*) (  (unsigned char*) buffer), p_src, size);
+        memcpy((void*) ((unsigned char*) buffer), p_src, size);
     }
 }
 
@@ -215,7 +233,7 @@ static void mca_yoda_put_callback(mca_btl_base_module_t* btl,
     size_t* size;
     void** l_addr;
 
-    size = (size_t *) des->des_dst->seg_addr.pval;
+    size = (size_t *) des->des_segments->seg_addr.pval;
     l_addr = (void**) ( ((char*)size) + sizeof(*size));
     memcpy(*l_addr, ((char*)l_addr) + sizeof(*l_addr), *size);
 }
@@ -241,7 +259,7 @@ static void mca_yoda_get_callback(mca_btl_base_module_t* btl,
     putreq = NULL;
 
     /* Unpack data */
-    p = (void **)des->des_dst->seg_addr.pval;
+    p = (void **)des->des_segments->seg_addr.pval;
     p_src = (void*) p;
 
     size = (size_t*)((char*)p_src + sizeof(*p_src) );
@@ -262,18 +280,18 @@ static void mca_yoda_get_callback(mca_btl_base_module_t* btl,
                             MCA_BTL_DES_SEND_ALWAYS_CALLBACK,
                             1);
 
-    if (OPAL_UNLIKELY(!des_loc || !des_loc->des_src)) {
+    if (OPAL_UNLIKELY(!des_loc || !des_loc->des_segments)) {
         SPML_ERROR("shmem OOM error need %d bytes", (int)*size);
         oshmem_shmem_abort(-1);
     }
-    spml_yoda_prepare_for_get_response((void*)des_loc->des_src->seg_addr.pval, *size, (void*)*p_src, (void*) *p_dst,(void*)*p_getreq,1);
+    spml_yoda_prepare_for_get_response((void*)des_loc->des_segments->seg_addr.pval, *size, (void*)*p_src, (void*) *p_dst,(void*)*p_getreq,1);
 
     frag->rdma_req = putreq;
 
     /* Initialize callback data for put*/
     des_loc->des_cbdata = frag;
     des_loc->des_cbfunc = mca_spml_yoda_put_completion;
-    des_loc->des_src_cnt = 1;
+    des_loc->des_segment_count = 1;
 
     OPAL_THREAD_ADD32(&mca_spml_yoda.n_active_puts, 1);
 
@@ -309,7 +327,7 @@ static void mca_yoda_get_response_callback(mca_btl_base_module_t* btl,
     mca_spml_yoda_get_request_t* getreq;
 
     /* unpacking data*/
-    size = (size_t *) ( ((char*)des->des_dst->seg_addr.pval) );
+    size = (size_t *) ( ((char*)des->des_segments->seg_addr.pval) );
     l_addr = (void**)( ((char*)size) + sizeof(*size));
     getreq = (mca_spml_yoda_get_request_t*)*(void**)((char*)l_addr + sizeof(*l_addr) + *size);
 
@@ -349,8 +367,7 @@ int mca_spml_yoda_deregister(sshmem_mkey_t *mkeys)
             yoda_context->btl_src_descriptor = NULL;
         }
         if (yoda_context->registration) {
-            ybtl->btl->btl_mpool->mpool_deregister(ybtl->btl->btl_mpool,
-                                                   yoda_context->registration);
+            ybtl->btl->btl_deregister_mem (ybtl->btl, yoda_context->registration);
         }
 
     }
@@ -365,16 +382,9 @@ sshmem_mkey_t *mca_spml_yoda_register(void* addr,
                                         int *count)
 {
     int i;
-    mca_btl_base_descriptor_t* des = NULL;
-    const opal_datatype_t *datatype = &opal_datatype_wchar;
-    opal_convertor_t convertor;
     sshmem_mkey_t *mkeys;
     struct yoda_btl *ybtl;
-    oshmem_proc_t *proc_self;
     mca_spml_yoda_context_t* yoda_context;
-    struct iovec iov;
-    uint32_t iov_count = 1;
-
 
     SPML_VERBOSE(10, "address %p len %llu", addr, (unsigned long long)size);
     *count = 0;
@@ -385,10 +395,6 @@ sshmem_mkey_t *mca_spml_yoda_register(void* addr,
         return NULL ;
     }
 
-    proc_self = oshmem_proc_group_find(oshmem_group_all, oshmem_my_proc_id());
-    /* create convertor */
-    OBJ_CONSTRUCT(&convertor, opal_convertor_t);
-
     mca_bml.bml_register( MCA_SPML_YODA_PUT,
                           mca_yoda_put_callback,
                           NULL );
@@ -412,7 +418,7 @@ sshmem_mkey_t *mca_spml_yoda_register(void* addr,
             continue;
         }
 
-        /* If we have shared memory just save its id*/
+        /* If we have shared memory just save its id */
         if ((YODA_BTL_SM == ybtl->btl_type || YODA_BTL_VADER == ybtl->btl_type)
                 && MAP_SEGMENT_SHM_INVALID != (int)shmid) {
             mkeys[i].u.key = shmid;
@@ -424,58 +430,26 @@ sshmem_mkey_t *mca_spml_yoda_register(void* addr,
         mkeys[i].spml_context = yoda_context;
 
         yoda_context->registration = NULL;
-
         if (ybtl->btl->btl_flags & MCA_BTL_FLAGS_RDMA) {
-
-            /* initialize convertor for source descriptor*/
-            opal_convertor_copy_and_prepare_for_recv(proc_self->proc_convertor,
-                                                     datatype,
-                                                     size,
-                                                     addr,
-                                                     0,
-                                                     &convertor);
-
-            if (NULL != ybtl->btl->btl_mpool && NULL != ybtl->btl->btl_mpool->mpool_register) {
-                iov.iov_len = size;
-                iov.iov_base = NULL;
-
-                opal_convertor_pack(&convertor, &iov, &iov_count, &size);
-                ybtl->btl->btl_mpool->mpool_register(ybtl->btl->btl_mpool,
-                                                     iov.iov_base, size, 0, &yoda_context->registration);
-            }
-            /* initialize convertor for source descriptor*/
-            opal_convertor_copy_and_prepare_for_recv(proc_self->proc_convertor,
-                                                     datatype,
-                                                     size,
-                                                     addr,
-                                                     0,
-                                                     &convertor);
-
-            /* register source memory */
-            des = ybtl->btl->btl_prepare_src(ybtl->btl,
-                                             ybtl->bml_btl->btl_endpoint,
-                                             yoda_context->registration,
-                                             &convertor,
-                                             MCA_BTL_NO_ORDER,
-                                             0,
-                                             &size,
-                                             0);
-            if (NULL == des) {
-                SPML_ERROR("%s: failed to register source memory. ",
-                           btl_type2str(ybtl->btl_type));
-                /* FIXME some cleanup might be needed here
-                 * yoda_context->btl_src_descriptor = NULL;
-                 * OBJ_DESTRUCT(&convertor);
-                 * *count = ???;
-                 * free(spml_context);
-                 */
-                free(mkeys);
-                return NULL;
+            if (NULL != ybtl->btl->btl_register_mem) {
+                yoda_context->registration = ybtl->btl->btl_register_mem (ybtl->btl, MCA_BTL_ENDPOINT_ANY,
+                                                                          addr, size, MCA_BTL_REG_FLAG_ACCESS_ANY);
+                if (NULL == yoda_context->registration) {
+                    SPML_ERROR("%s: failed to register source memory: addr: %p, size: %u",
+                               btl_type2str(ybtl->btl_type), addr, size);
+                    /* FIXME some cleanup might be needed here
+                     * yoda_context->btl_src_descriptor = NULL;
+                     * *count = ???;
+                     * free(spml_context);
+                     */
+                    free(mkeys);
+                    return NULL;
+                }
             }
 
-            yoda_context->btl_src_descriptor = des;
-            mkeys[i].u.data = des->des_src;
-            mkeys[i].len  = ybtl->btl->btl_seg_size;
+            yoda_context->btl_src_descriptor = NULL;
+            mkeys[i].u.data = yoda_context->registration;
+            mkeys[i].len    = yoda_context->registration ? ybtl->btl->btl_registration_handle_size : 0;
         }
 
         SPML_VERBOSE(5,
@@ -483,7 +457,6 @@ sshmem_mkey_t *mca_spml_yoda_register(void* addr,
 		     oshmem_proc_pe(oshmem_proc_local()), btl_type2str(ybtl->btl_type),
                      mkeys[i].va_base, mkeys[i].len, (unsigned long long)mkeys[i].u.key, (unsigned long long)size);
     }
-    OBJ_DESTRUCT(&convertor);
     *count = mca_spml_yoda.n_btls;
     return mkeys;
 }
@@ -494,7 +467,7 @@ sshmem_mkey_t *mca_spml_yoda_register(void* addr,
  */
 static void mca_spml_yoda_error_handler(struct mca_btl_base_module_t* btl,
                                         int32_t flags,
-                                        ompi_proc_t* errproc,
+                                        opal_proc_t* errproc,
                                         char* btlinfo)
 {
     oshmem_shmem_abort(-1);
@@ -770,6 +743,7 @@ static inline int mca_spml_yoda_put_internal(void *dst_addr,
     int btl_id = 0;
     struct yoda_btl *ybtl;
     int put_via_send;
+    mca_btl_base_registration_handle_t *local_handle = NULL, *remote_handle = NULL;
 
     /* If nothing to put its OK.*/
     if (0 >= size) {
@@ -802,6 +776,11 @@ static inline int mca_spml_yoda_put_internal(void *dst_addr,
 
     ybtl = &mca_spml_yoda.btl_type_map[btl_id];
 
+    if (ybtl->btl->btl_register_mem) {
+        assert (r_mkey->len == ybtl->btl->btl_registration_handle_size);
+        remote_handle = (mca_btl_base_registration_handle_t *) r_mkey->u.data;
+    }
+
     /* check if we doing put into shm attached segment and if so
      * just do memcpy
      */
@@ -812,7 +791,7 @@ static inline int mca_spml_yoda_put_internal(void *dst_addr,
     }
 
     /* We support only blocking PUT now => we always need copy for src buffer*/
-    calc_nfrags(bml_btl, size, &frag_size, &nfrags, put_via_send);
+    calc_nfrags_put (bml_btl, size, &frag_size, &nfrags, put_via_send);
 
     p_src = (char*) src_addr;
     p_dst = (char*) (unsigned long) rva;
@@ -832,7 +811,7 @@ static inline int mca_spml_yoda_put_internal(void *dst_addr,
                                 MCA_BTL_DES_SEND_ALWAYS_CALLBACK,
                                 put_via_send);
 
-        if (OPAL_UNLIKELY(!des || !des->des_src )) {
+        if (OPAL_UNLIKELY(!des || !des->des_segments )) {
             SPML_ERROR("src=%p nfrags = %d frag_size=%d",
                        src_addr, nfrags, frag_size);
             SPML_ERROR("shmem OOM error need %d bytes", ncopied);
@@ -845,35 +824,36 @@ static inline int mca_spml_yoda_put_internal(void *dst_addr,
         }
 
         /* copy data to allocated buffer*/
-        segment = des->des_src;
+        segment = des->des_segments;
         spml_yoda_prepare_for_put((void*)segment->seg_addr.pval, ncopied,
                                   (void*)p_src, (void*)p_dst, put_via_send);
 
-        /* Preparing destination buffer */
-
-        /*assert( NULL != r_mkey->u.data && 0 != r_mkey->len);*/
-
-        memcpy(&frag->rdma_segs[0].base_seg,
-                r_mkey->u.data,
-                r_mkey->len);
+        if (!put_via_send && ybtl->btl->btl_register_mem) {
+            local_handle = ybtl->btl->btl_register_mem (ybtl->btl, bml_btl->btl_endpoint,
+                                                        segment->seg_addr.pval, ncopied, 0);
+            if (NULL == local_handle) {
+                /* No free resources, Block on completion here */
+                SPML_ERROR("shmem error: OSHMEM_ERR_OUT_OF_RESOURCE");
+                oshmem_request_wait_completion(&putreq->req_put.req_base.req_oshmem);
+            }
+        }
 
         frag->rdma_segs[0].base_seg.seg_addr.lval = (uintptr_t) p_dst;
         frag->rdma_segs[0].base_seg.seg_len = (put_via_send ?
                                                    ncopied + SPML_YODA_SEND_CONTEXT_SIZE :
                                                    ncopied);
-        des->des_dst = &frag->rdma_segs[0].base_seg;
-
         frag->rdma_req = putreq;
 
         /* initialize callback data for put*/
         des->des_cbdata = frag;
         des->des_cbfunc = mca_spml_yoda_put_completion;
-        des->des_dst_cnt = 1;
 
         OPAL_THREAD_ADD32(&mca_spml_yoda.n_active_puts, 1);
         /* put the data to remote side */
         if (!put_via_send) {
-            rc = mca_bml_base_put(bml_btl, des);
+            rc = mca_bml_base_put (bml_btl, segment->seg_addr.pval, (uint64_t) (intptr_t) p_dst,
+                                   local_handle, remote_handle, ncopied, 0, 0, mca_spml_yoda_put_completion_rdma,
+                                   des);
         } else {
             rc = mca_bml_base_send(bml_btl, des, MCA_SPML_YODA_PUT);
             if (1 == rc)
@@ -953,27 +933,27 @@ int mca_spml_yoda_enable(bool enable)
      * initialize them with the size of our own requests.
      */
 
-    ompi_free_list_init_new(&mca_spml_base_put_requests,
-                            sizeof(mca_spml_yoda_put_request_t),
-                            opal_cache_line_size,
-                            OBJ_CLASS(mca_spml_yoda_put_request_t),
-                            0,
-                            opal_cache_line_size,
-                            mca_spml_yoda.free_list_num,
-                            mca_spml_yoda.free_list_max,
-                            mca_spml_yoda.free_list_inc,
-                            NULL );
+    opal_free_list_init (&mca_spml_base_put_requests,
+                         sizeof(mca_spml_yoda_put_request_t),
+                         opal_cache_line_size,
+                         OBJ_CLASS(mca_spml_yoda_put_request_t),
+                         0,
+                         opal_cache_line_size,
+                         mca_spml_yoda.free_list_num,
+                         mca_spml_yoda.free_list_max,
+                         mca_spml_yoda.free_list_inc,
+                         NULL, 0, NULL, NULL, NULL);
 
-    ompi_free_list_init_new(&mca_spml_base_get_requests,
-                            sizeof(mca_spml_yoda_get_request_t),
-                            opal_cache_line_size,
-                            OBJ_CLASS(mca_spml_yoda_get_request_t),
-                            0,
-                            opal_cache_line_size,
-                            mca_spml_yoda.free_list_num,
-                            mca_spml_yoda.free_list_max,
-                            mca_spml_yoda.free_list_inc,
-                            NULL );
+    opal_free_list_init (&mca_spml_base_get_requests,
+                         sizeof(mca_spml_yoda_get_request_t),
+                         opal_cache_line_size,
+                         OBJ_CLASS(mca_spml_yoda_get_request_t),
+                         0,
+                         opal_cache_line_size,
+                         mca_spml_yoda.free_list_num,
+                         mca_spml_yoda.free_list_max,
+                         mca_spml_yoda.free_list_inc,
+                         NULL, 0, NULL, NULL, NULL);
 
     mca_spml_yoda.enabled = true;
 
@@ -1032,11 +1012,7 @@ int mca_spml_yoda_get(void* src_addr, size_t size, void* dst_addr, int src)
     struct yoda_btl *ybtl;
     int btl_id = 0;
     int get_via_send;
-    const opal_datatype_t *datatype = &opal_datatype_wchar;
-    opal_convertor_t convertor;
-    oshmem_proc_t *proc_self;
-    size_t prepare_size;
-    mca_mpool_base_registration_t* registration;
+    mca_btl_base_registration_handle_t *local_handle, *remote_handle = NULL;
     mca_spml_yoda_get_request_t* getreq = NULL;
 
     /*If nothing to get its OK.*/
@@ -1071,6 +1047,11 @@ int mca_spml_yoda_get(void* src_addr, size_t size, void* dst_addr, int src)
 
     ybtl = &mca_spml_yoda.btl_type_map[btl_id];
 
+    if (ybtl->btl->btl_register_mem) {
+        assert(ybtl->btl->btl_registration_handle_size == r_mkey->len);
+        remote_handle = (mca_btl_base_registration_handle_t *) r_mkey->u.data;
+    }
+
     nfrags = 1;
 
     /* check if we doing get into shm attached segment and if so
@@ -1096,7 +1077,7 @@ int mca_spml_yoda_get(void* src_addr, size_t size, void* dst_addr, int src)
      */
     frag_size = ncopied;
     if ((NULL == l_mkey) || get_via_send) {
-        calc_nfrags(bml_btl, size, &frag_size, &nfrags, get_via_send);
+        calc_nfrags_get (bml_btl, size, &frag_size, &nfrags, get_via_send);
     }
 
     p_src = (char*) (unsigned long) rva;
@@ -1136,7 +1117,7 @@ int mca_spml_yoda_get(void* src_addr, size_t size, void* dst_addr, int src)
                                     (int)frag_size,
                                     MCA_BTL_DES_SEND_ALWAYS_CALLBACK,
                                     get_via_send);
-            if (OPAL_UNLIKELY(!des || !des->des_src)) {
+            if (OPAL_UNLIKELY(!des || !des->des_segments)) {
                 SPML_ERROR("shmem OOM error need %d bytes", ncopied);
                 SPML_ERROR("src=%p nfrags = %d frag_size=%d",
                            src_addr, nfrags, frag_size);
@@ -1144,9 +1125,10 @@ int mca_spml_yoda_get(void* src_addr, size_t size, void* dst_addr, int src)
                 goto exit_fatal;
             }
 
-            segment = des->des_src;
+            segment = des->des_segments;
             spml_yoda_prepare_for_get((void*)segment->seg_addr.pval, ncopied, (void*)p_src, oshmem_my_proc_id(), (void*)p_dst, (void*) getreq);
             des->des_cbfunc = mca_spml_yoda_get_response_completion;
+            des->des_cbdata = frag;
 
             OPAL_THREAD_ADD32(&mca_spml_yoda.n_active_gets, 1);
         }
@@ -1154,38 +1136,24 @@ int mca_spml_yoda_get(void* src_addr, size_t size, void* dst_addr, int src)
             /*
              * Register src memory if do GET via GET
              */
-            proc_self = oshmem_proc_group_find(oshmem_group_all, oshmem_my_proc_id());
-            OBJ_CONSTRUCT(&convertor, opal_convertor_t);
+            if (NULL == l_mkey && ybtl->btl->btl_register_mem) {
+                local_handle = ybtl->btl->btl_register_mem (ybtl->btl, bml_btl->btl_endpoint, p_dst, ncopied,
+                                                            MCA_BTL_REG_FLAG_LOCAL_WRITE);
 
-            prepare_size = ncopied;
-            opal_convertor_copy_and_prepare_for_recv(proc_self->proc_convertor,
-                                                     datatype,
-                                                     prepare_size,
-                                                     p_dst,
-                                                     0,
-                                                     &convertor);
+                if (NULL == local_handle) {
+                    SPML_ERROR("%s: failed to register destination memory %p.",
+                               btl_type2str(ybtl->btl_type), p_dst);
+                }
 
-            registration = (NULL == l_mkey ? NULL : ((mca_spml_yoda_context_t*)l_mkey->spml_context)->registration);
-            des = ybtl->btl->btl_prepare_dst(ybtl->btl,
-                                             bml_btl->btl_endpoint,
-                                             registration,
-                                             &convertor,
-                                             MCA_BTL_NO_ORDER,
-                                             0,
-                                             &prepare_size,
-                                             0);
-            OBJ_DESTRUCT(&convertor);
-            if (NULL == des) {
-                SPML_ERROR("%s: failed to register destination memory %p.",
-                           btl_type2str(ybtl->btl_type), p_dst);
-                rc = OSHMEM_ERR_FATAL;
-                goto exit_fatal;
+                frag->local_handle = local_handle;
+            } else {
+                local_handle = ((mca_spml_yoda_context_t*)l_mkey->spml_context)->registration;
+                frag->local_handle = NULL;
             }
+
             frag->rdma_segs[0].base_seg.seg_addr.lval = (uintptr_t) p_src;
             getreq->p_dst = (uint64_t*) p_dst;
             frag->size = ncopied;
-            des->des_cbfunc = mca_spml_yoda_get_completion;
-            des->des_src = &frag->rdma_segs[0].base_seg;
 
             OPAL_THREAD_ADD32(&mca_spml_yoda.n_active_gets, 1);
         }
@@ -1198,12 +1166,6 @@ int mca_spml_yoda_get(void* src_addr, size_t size, void* dst_addr, int src)
 
         frag->rdma_req = getreq;
 
-        /**
-         * Init remote side descriptor.
-         */
-        des->des_src_cnt = 1;
-        des->des_cbdata = frag;
-
         /**
          *  Do GET operation
          */
@@ -1212,7 +1174,8 @@ int mca_spml_yoda_get(void* src_addr, size_t size, void* dst_addr, int src)
             if (1 == rc)
                 rc = OSHMEM_SUCCESS;
         } else {
-            rc = mca_bml_base_get(bml_btl, des);
+            rc = mca_bml_base_get(bml_btl, p_dst, (uint64_t) (intptr_t) p_src, local_handle,
+                                  remote_handle, ncopied, 0, 0, mca_spml_yoda_get_completion, frag);
         }
 
         if (OPAL_UNLIKELY(OSHMEM_SUCCESS != rc)) {
