@@ -1,6 +1,8 @@
 /*
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -14,11 +16,17 @@
 #include "oshmem/runtime/runtime.h"
 #include "oshmem/runtime/params.h"
 
+#if OMPI_ENABLE_MPI_PROFILING
+#define MPI_Allgather PMPI_Allgather
+#define MPI_Allgatherv PMPI_Allgatherv
+#define MPI_Barrier PMPI_Barrier
+#endif
+
 int oshmem_shmem_allgather(void *send_buf, void *rcv_buf, int elem_size)
 {
     int rc;
 
-    rc = MPI_Allgather(send_buf, elem_size, MPI_BYTE,
+    rc = PMPI_Allgather(send_buf, elem_size, MPI_BYTE,
                         rcv_buf, elem_size, MPI_BYTE, oshmem_comm_world);
 
     return rc;
@@ -29,7 +37,7 @@ int oshmem_shmem_allgatherv(void *send_buf, void* rcv_buf, int send_count,
 {
     int rc;
 
-    rc = MPI_Allgatherv(send_buf, send_count, MPI_BYTE,
+    rc = PMPI_Allgatherv(send_buf, send_count, MPI_BYTE,
                          rcv_buf, rcv_size, displs, MPI_BYTE, oshmem_comm_world);
 
     return rc;
@@ -37,5 +45,5 @@ int oshmem_shmem_allgatherv(void *send_buf, void* rcv_buf, int send_count,
 
 void oshmem_shmem_barrier(void)
 {
-    MPI_Barrier(oshmem_comm_world);
+    PMPI_Barrier(oshmem_comm_world);
 }
